@@ -17,13 +17,13 @@
  *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
-#include <dbus/dbus.h>
 #include <limits.h>
+#include <dbus/dbus.h>
 
 #include "fcitx/fcitx.h"
 #include "fcitx/frontend.h"
 #include "fcitx-utils/utils.h"
-#include "module/dbus/dbusstuff.h"
+#include "module/dbus/fcitx-dbus.h"
 #include "fcitx/instance.h"
 #include "fcitx/module.h"
 #include "fcitx-utils/log.h"
@@ -93,7 +93,6 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
 static void IPCICFocusIn(FcitxIPCFrontend* ipc, FcitxInputContext* ic);
 static void IPCICFocusOut(FcitxIPCFrontend* ipc, FcitxInputContext* ic);
 static void IPCICReset(FcitxIPCFrontend* ipc, FcitxInputContext* ic);
-static void IPCICCommitPreedit(FcitxIPCFrontend* ipc, FcitxInputContext* ic);
 static void IPCICSetCursorRect(FcitxIPCFrontend* ipc, FcitxInputContext* ic, int x, int y, int w, int h);
 static int IPCProcessKey(FcitxIPCFrontend* ipc, FcitxInputContext* callic, const uint32_t originsym, const uint32_t keycode, const uint32_t originstate, uint32_t t, FcitxKeyEventType type);
 static boolean IPCCheckICFromSameApplication(void* arg, FcitxInputContext* icToCheck, FcitxInputContext* ic);
@@ -102,9 +101,9 @@ static void IPCSetPropertyIMList(void* arg, DBusMessageIter* iter);
 static void IPCUpdateIMList(void* arg);
 static pid_t IPCGetPid(void* arg, FcitxInputContext* ic);
 
-static void FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessage* message);
-static void FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusConnection* coon, DBusMessage* message);
-static void FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessage* message);
+static DBusMessage* FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusMessage* message);
+static DBusMessage* FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusMessage* message);
+static DBusMessage* FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusMessage* message);
 
 const FcitxDBusPropertyTable propertTable[] = {
     { FCITX_IM_DBUS_INTERFACE, "IMList", "a(sssb)", IPCGetPropertyIMList, IPCSetPropertyIMList },
@@ -178,6 +177,9 @@ const char * im_introspection_xml =
     "    </method>\n"
     "    <method name=\"ReloadConfig\">\n"
     "    </method>\n"
+    "    <method name=\"ReloadAddonConfig\">\n"
+    "      <arg name=\"addon\" direction=\"in\" type=\"s\"/>\n"
+    "    </method>\n"
     "    <method name=\"Restart\">\n"
     "    </method>\n"
     "    <method name=\"Configure\">\n"
@@ -253,6 +255,10 @@ const char * ic_introspection_xml =
     "      <arg name=\"cursor\" direction=\"in\" type=\"u\"/>\n"
     "      <arg name=\"anchor\" direction=\"in\" type=\"u\"/>\n"
     "    </method>\n"
+    "    <method name=\"SetSurroundingTextPosition\">\n"
+    "      <arg name=\"cursor\" direction=\"in\" type=\"u\"/>\n"
+    "      <arg name=\"anchor\" direction=\"in\" type=\"u\"/>\n"
+    "    </method>\n"
     "    <method name=\"DestroyIC\">\n"
     "    </method>\n"
     "    <method name=\"ProcessKeyEvent\">\n"
@@ -325,8 +331,8 @@ void* IPCCreate(FcitxInstance* instance, int frontendid)
     ipc->frontendid = frontendid;
     ipc->owner = instance;
 
-    ipc->_conn = InvokeVaArgs(instance, FCITX_DBUS, GETCONNECTION);
-    ipc->_privconn = InvokeVaArgs(instance, FCITX_DBUS, GETPRIVCONNECTION);
+    ipc->_conn = FcitxDBusGetConnection(instance);
+    ipc->_privconn = FcitxDBusGetPrivConnection(instance);
 
     if (ipc->_conn == NULL && ipc->_privconn == NULL) {
         FcitxLog(ERROR, "DBus Not initialized");
@@ -355,6 +361,7 @@ void* IPCCreate(FcitxInstance* instance, int frontendid)
 
 boolean IPCDestroy(void* arg)
 {
+    FCITX_UNUSED(arg);
     return true;
 }
 
@@ -400,10 +407,11 @@ void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
         if (!dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &appname, DBUS_TYPE_INVALID))
             ipcic->appname = NULL;
         else {
-            if (strlen(appname) == 0)
+            if (strlen(appname) == 0) {
                 ipcic->appname = NULL;
-            else
+            } else {
                 ipcic->appname = strdup(appname);
+            }
         }
 
         if (config->shareState == ShareState_PerProgram)
@@ -424,15 +432,16 @@ void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
 
         DBusError error;
         dbus_error_init(&error);
-        char* appname;
-        int icpid;
+        char* appname = NULL;
+        int icpid = 0;
         if (!dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &appname, DBUS_TYPE_INT32, &icpid, DBUS_TYPE_INVALID))
             ipcic->appname = NULL;
         else {
-            if (strlen(appname) == 0)
+            if (strlen(appname) == 0) {
                 ipcic->appname = NULL;
-            else
+            } else {
                 ipcic->appname = strdup(appname);
+            }
         }
         ipcic->pid = icpid;
 
@@ -471,7 +480,8 @@ void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
 
 boolean IPCCheckIC(void* arg, FcitxInputContext* context, void* priv)
 {
-    int *id = (int*) priv;
+    FCITX_UNUSED(arg);
+    int *id = (int*)priv;
     if (GetIPCIC(context)->id == *id)
         return true;
     return false;
@@ -490,10 +500,8 @@ void IPCDestroyIC(void* arg, FcitxInputContext* context)
         if (ipc->_privconn)
             dbus_connection_unregister_object_path(ipc->_privconn, GetIPCIC(context)->path);
     }
-    if (ipcic->appname)
-        free(ipcic->appname);
-    if (ipcic->surroundingText)
-        free(ipcic->surroundingText);
+    fcitx_utils_free(ipcic->appname);
+    fcitx_utils_free(ipcic->surroundingText);
     free(context->privateic);
     context->privateic = NULL;
 }
@@ -566,43 +574,52 @@ void IPCForwardKey(void* arg, FcitxInputContext* ic, FcitxKeyEventType event, Fc
 
 void IPCSetWindowOffset(void* arg, FcitxInputContext* ic, int x, int y)
 {
+    FCITX_UNUSED(arg);
     ic->offset_x = x;
     ic->offset_y = y;
 }
 
 void IPCGetWindowRect(void* arg, FcitxInputContext* ic, int* x, int* y, int* w, int* h)
 {
+    FCITX_UNUSED(arg);
     *x = ic->offset_x;
     *y = ic->offset_y;
     *w = GetIPCIC(ic)->width;
     *h = GetIPCIC(ic)->height;
 }
 
+static inline DBusMessage* IPCDBusUnknownMethod(DBusMessage *msg)
+{
+    DBusMessage* reply = dbus_message_new_error_printf(msg,
+                                                       DBUS_ERROR_UNKNOWN_METHOD,
+                                                       "No such method with signature (%s)",
+                                                       dbus_message_get_signature(msg));
+    return reply;
+}
 
 static DBusHandlerResult IPCDBusEventHandler(DBusConnection *connection, DBusMessage *msg, void *user_data)
 {
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) user_data;
     FcitxInstance* instance = ipc->owner;
+    DBusMessage* reply = NULL;
+    boolean flush = true;
+    DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
     if (dbus_message_is_method_call(msg, DBUS_INTERFACE_INTROSPECTABLE, "Introspect")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        reply = dbus_message_new_method_return(msg);
 
         dbus_message_append_args(reply, DBUS_TYPE_STRING, &im_introspection_xml, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, DBUS_INTERFACE_PROPERTIES, "Get")) {
-        FcitxDBusPropertyGet(ipc, connection, msg);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = FcitxDBusPropertyGet(ipc, msg);
     } else if (dbus_message_is_method_call(msg, DBUS_INTERFACE_PROPERTIES, "Set")) {
-        FcitxDBusPropertySet(ipc, connection, msg);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = FcitxDBusPropertySet(ipc, msg);
     } else if (dbus_message_is_method_call(msg, DBUS_INTERFACE_PROPERTIES, "GetAll")) {
-        FcitxDBusPropertyGetAll(ipc, connection, msg);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = FcitxDBusPropertyGetAll(ipc, msg);
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "CreateIC")
             || dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "CreateICv2")
             || dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "CreateICv3")
             ) {
+        /* we have no choice here, so just return */
         FcitxIPCCreateICPriv ipcpriv;
         ipcpriv.message = msg;
         ipcpriv.conn = connection;
@@ -610,35 +627,31 @@ static DBusHandlerResult IPCDBusEventHandler(DBusConnection *connection, DBusMes
         return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "Exit")) {
         FcitxLog(INFO, "Receive message ask for quit");
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        reply = dbus_message_new_method_return(msg);
         dbus_connection_send(connection, reply, NULL);
         dbus_message_unref(reply);
         dbus_connection_flush(connection);
-        FcitxInstanceEnd(ipc->owner);
+        FcitxInstanceEnd(instance);
         return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "GetCurrentIM")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        reply = dbus_message_new_method_return(msg);
         FcitxIM* im = FcitxInstanceGetCurrentIM(ipc->owner);
         const char* name = "";
         if (im) {
             name = im->uniqueName;
         }
         dbus_message_append_args(reply, DBUS_TYPE_STRING, &name, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "SetCurrentIM")) {
         DBusError error;
         dbus_error_init(&error);
         char* imname = NULL;
         if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &imname, DBUS_TYPE_INVALID)) {
             FcitxInstanceSwitchIMByName(instance, imname);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
+            reply = dbus_message_new_method_return(msg);
+        } else {
+            reply = IPCDBusUnknownMethod(msg);
         }
         dbus_error_free(&error);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "GetIMAddon")) {
         DBusError error;
         dbus_error_init(&error);
@@ -648,175 +661,144 @@ static DBusHandlerResult IPCDBusEventHandler(DBusConnection *connection, DBusMes
             const char* name = "";
             if (im && im->owner)
                 name = im->owner->name;
-            DBusMessage *reply = dbus_message_new_method_return(msg);
+            reply = dbus_message_new_method_return(msg);
             dbus_message_append_args(reply, DBUS_TYPE_STRING, &name, DBUS_TYPE_INVALID);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
+        } else {
+            reply = IPCDBusUnknownMethod(msg);
         }
         dbus_error_free(&error);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "GetCurrentUI")) {
         const char* name = "";
         FcitxAddon* uiaddon  = FcitxInstanceGetCurrentUI(instance);
         if (uiaddon) {
             name = uiaddon->name;
         }
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        reply = dbus_message_new_method_return(msg);
         dbus_message_append_args(reply, DBUS_TYPE_STRING, &name, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "Configure")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
+        reply = dbus_message_new_method_return(msg);
         fcitx_utils_launch_configure_tool();
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ActivateIM")) {
         FcitxInstanceEnableIM(ipc->owner, FcitxInstanceGetCurrentIC(ipc->owner), false);
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = dbus_message_new_method_return(msg);
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "InactivateIM")) {
         FcitxInstanceCloseIM(ipc->owner, FcitxInstanceGetCurrentIC(ipc->owner));
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = dbus_message_new_method_return(msg);
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ToggleIM")) {
         FcitxInstanceChangeIMState(ipc->owner, FcitxInstanceGetCurrentIC(ipc->owner));
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = dbus_message_new_method_return(msg);
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "GetCurrentState")) {
         int r = FcitxInstanceGetCurrentState(ipc->owner);
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        reply = dbus_message_new_method_return(msg);
         dbus_message_append_args(reply,
                                  DBUS_TYPE_INT32, &r,
                                  DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ConfigureAddon")) {
         DBusError error;
         dbus_error_init(&error);
         char* addonname = NULL;
         if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &addonname, DBUS_TYPE_INVALID)) {
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
             if (addonname) {
                 fcitx_utils_launch_configure_tool_for_addon(addonname);
             }
+            reply = dbus_message_new_method_return(msg);
+        } else {
+            reply = IPCDBusUnknownMethod(msg);
         }
         dbus_error_free(&error);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ConfigureIM")) {
         DBusError error;
         dbus_error_init(&error);
         char* imname = NULL;
         if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &imname, DBUS_TYPE_INVALID)) {
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
             if (imname) {
                 fcitx_utils_launch_configure_tool_for_addon(imname);
             }
+            reply = dbus_message_new_method_return(msg);
+        } else {
+            reply = IPCDBusUnknownMethod(msg);
+        }
+        dbus_error_free(&error);
+    } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ReloadAddonConfig")) {
+        DBusError error;
+        dbus_error_init(&error);
+        char* addonname = NULL;
+        if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &addonname, DBUS_TYPE_INVALID)) {
+            if (addonname) {
+                FcitxInstanceReloadAddonConfig(instance, addonname);
+            }
+            reply = dbus_message_new_method_return(msg);
+        } else {
+            reply = IPCDBusUnknownMethod(msg);
         }
         dbus_error_free(&error);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "ReloadConfig")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        dbus_connection_flush(connection);
         FcitxInstanceReloadConfig(instance);
-        return DBUS_HANDLER_RESULT_HANDLED;
+        reply = dbus_message_new_method_return(msg);
+        flush = true;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "Restart")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
+        fcitx_utils_launch_restart();
+        reply = dbus_message_new_method_return(msg);
+        flush = true;
+    }
+
+    if (reply) {
         dbus_connection_send(connection, reply, NULL);
         dbus_message_unref(reply);
-        dbus_connection_flush(connection);
-        fcitx_utils_launch_restart();
-        return DBUS_HANDLER_RESULT_HANDLED;
+        if (flush) {
+            dbus_connection_flush(connection);
+        }
+        result = DBUS_HANDLER_RESULT_HANDLED;
     }
-    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+    return result;
 }
 
 
 static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusMessage *msg, void *user_data)
 {
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) user_data;
-    int id;
+    int id = -1;
     sscanf(dbus_message_get_path(msg), FCITX_IC_DBUS_PATH, &id);
     FcitxInputContext* ic = FcitxInstanceFindIC(ipc->owner, ipc->frontendid, &id);
     DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    DBusMessage *reply = NULL;
+    boolean flush = false;
 
     if (dbus_message_is_method_call(msg, DBUS_INTERFACE_INTROSPECTABLE, "Introspect")) {
-        DBusMessage *reply = dbus_message_new_method_return(msg);
-
+        reply = dbus_message_new_method_return(msg);
         dbus_message_append_args(reply, DBUS_TYPE_STRING, &ic_introspection_xml, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        result = DBUS_HANDLER_RESULT_HANDLED;
     }
 
     /* ic is not NULL */
-    if (result == DBUS_HANDLER_RESULT_NOT_YET_HANDLED && ic) {
+    if (!reply && ic) {
         DBusError error;
         dbus_error_init(&error);
         if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "EnableIC")) {
             FcitxInstanceEnableIM(ipc->owner, ic, false);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "CloseIC")) {
             FcitxInstanceCloseIM(ipc->owner, ic);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "FocusIn")) {
             IPCICFocusIn(ipc, ic);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "FocusOut")) {
             IPCICFocusOut(ipc, ic);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "Reset")) {
             IPCICReset(ipc, ic);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
-        } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "CommitPreedit")) {
-            IPCICCommitPreedit(ipc, ic);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "MouseEvent")) {
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
-            result = DBUS_HANDLER_RESULT_HANDLED;
+            reply = dbus_message_new_method_return(msg);
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "SetCursorLocation")) {
             int x, y;
             if (dbus_message_get_args(msg, &error, DBUS_TYPE_INT32, &x, DBUS_TYPE_INT32, &y, DBUS_TYPE_INVALID)) {
                 IPCICSetCursorRect(ipc, ic, x, y, 0, 0);
-                DBusMessage *reply = dbus_message_new_method_return(msg);
-                dbus_connection_send(connection, reply, NULL);
-                dbus_message_unref(reply);
+                reply = dbus_message_new_method_return(msg);
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
             }
-            result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "SetCursorRect")) {
             int x, y, w, h;
             if (dbus_message_get_args(msg, &error,
@@ -824,11 +806,10 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
                 DBUS_TYPE_INT32, &w, DBUS_TYPE_INT32, &h,
                 DBUS_TYPE_INVALID)) {
                 IPCICSetCursorRect(ipc, ic, x, y, w, h);
-                DBusMessage *reply = dbus_message_new_method_return(msg);
-                dbus_connection_send(connection, reply, NULL);
-                dbus_message_unref(reply);
+                reply = dbus_message_new_method_return(msg);
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
             }
-            result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "SetCapacity")) {
             uint32_t flags;
             if (dbus_message_get_args(msg, &error, DBUS_TYPE_UINT32, &flags, DBUS_TYPE_INVALID)) {
@@ -838,9 +819,9 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
                         free(GetIPCIC(ic)->surroundingText);
                     GetIPCIC(ic)->surroundingText = NULL;
                 }
-                DBusMessage *reply = dbus_message_new_method_return(msg);
-                dbus_connection_send(connection, reply, NULL);
-                dbus_message_unref(reply);
+                reply = dbus_message_new_method_return(msg);
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
             }
             result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "SetSurroundingText")) {
@@ -850,24 +831,35 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
                 FcitxIPCIC* ipcic = GetIPCIC(ic);
                 if (!ipcic->surroundingText || strcmp(ipcic->surroundingText, text) != 0 || cursor != ipcic->cursor || anchor != ipcic->anchor)
                 {
-                    if (ipcic->surroundingText) {
-                        free(ipcic->surroundingText);
-                    }
+                    fcitx_utils_free(ipcic->surroundingText);
                     ipcic->surroundingText = strdup(text);
                     ipcic->cursor = cursor;
                     ipcic->anchor = anchor;
                     FcitxInstanceNotifyUpdateSurroundingText(ipc->owner, ic);
                 }
-                DBusMessage *reply = dbus_message_new_method_return(msg);
-                dbus_connection_send(connection, reply, NULL);
-                dbus_message_unref(reply);
+                reply = dbus_message_new_method_return(msg);
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
+            }
+            result = DBUS_HANDLER_RESULT_HANDLED;
+        } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "SetSurroundingTextPosition")) {
+            uint32_t cursor, anchor;
+            if (dbus_message_get_args(msg, &error,  DBUS_TYPE_UINT32, &cursor, DBUS_TYPE_UINT32, &anchor, DBUS_TYPE_INVALID)) {
+                FcitxIPCIC* ipcic = GetIPCIC(ic);
+                if (cursor != ipcic->cursor || anchor != ipcic->anchor)
+                {
+                    ipcic->cursor = cursor;
+                    ipcic->anchor = anchor;
+                    FcitxInstanceNotifyUpdateSurroundingText(ipc->owner, ic);
+                }
+                reply = dbus_message_new_method_return(msg);
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
             }
             result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "DestroyIC")) {
             FcitxInstanceDestroyIC(ipc->owner, ipc->frontendid, &id);
-            DBusMessage *reply = dbus_message_new_method_return(msg);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
+            reply = dbus_message_new_method_return(msg);
             result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "ProcessKeyEvent")) {
             uint32_t keyval, keycode, state, t;
@@ -883,18 +875,26 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
                 type = itype;
                 ret = IPCProcessKey(ipc, ic, keyval, keycode, state, t, type);
 
-                DBusMessage *reply = dbus_message_new_method_return(msg);
+                reply = dbus_message_new_method_return(msg);
 
                 dbus_message_append_args(reply, DBUS_TYPE_INT32, &ret, DBUS_TYPE_INVALID);
-                dbus_connection_send(connection, reply, NULL);
-                dbus_message_unref(reply);
-                dbus_connection_flush(connection);
+                flush = true;
+            } else {
+                reply = IPCDBusUnknownMethod(msg);
             }
-
-            result = DBUS_HANDLER_RESULT_HANDLED;
         }
         dbus_error_free(&error);
     }
+
+    if (reply) {
+        dbus_connection_send(connection, reply, NULL);
+        dbus_message_unref(reply);
+        if (flush) {
+            dbus_connection_flush(connection);
+        }
+        result = DBUS_HANDLER_RESULT_HANDLED;
+    }
+
     return result;
 }
 
@@ -994,18 +994,6 @@ static void IPCICReset(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
     return;
 }
 
-static void IPCICCommitPreedit(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
-{
-    FcitxInputContext* currentic = FcitxInstanceGetCurrentIC(ipc->owner);
-    if (ic && ic == currentic) {
-        FcitxUICommitPreedit(ipc->owner);
-        FcitxUICloseInputWindow(ipc->owner);
-        FcitxInstanceResetInput(ipc->owner);
-    }
-
-    return;
-}
-
 static void IPCICSetCursorRect(FcitxIPCFrontend* ipc, FcitxInputContext* ic, int x, int y, int w, int h)
 {
     ic->offset_x = x;
@@ -1092,6 +1080,38 @@ void IPCUpdatePreedit(void* arg, FcitxInputContext* ic)
 void IPCDeleteSurroundingText(void* arg, FcitxInputContext* ic, int offset, unsigned int size)
 {
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) arg;
+    FcitxIPCIC* ipcic = GetIPCIC(ic);
+
+    /*
+     * do the real deletion here, and client might update it, but input method itself
+     * would expect a up to date value after this call.
+     *
+     * Make their life easier.
+     */
+    if (ipcic->surroundingText) {
+        int cursor_pos = ipcic->cursor + offset;
+        size_t len = fcitx_utf8_strlen (ipcic->surroundingText);
+        if (cursor_pos >= 0 && len - cursor_pos >= size) {
+            /*
+             * the original size must be larger, so we can do in-place copy here
+             * without alloc new string
+             */
+            char* start = fcitx_utf8_get_nth_char(ipcic->surroundingText, cursor_pos);
+            char* end = fcitx_utf8_get_nth_char(start, size);
+
+            int copylen = strlen(end);
+
+            memmove (start, end, sizeof(char) * copylen);
+            start[copylen] = 0;
+            ipcic->cursor = cursor_pos;
+        } else {
+            ipcic->surroundingText[0] = '\0';
+            ipcic->cursor = 0;
+        }
+        ipcic->anchor = ipcic->cursor;
+    }
+
+
     DBusMessage* msg = dbus_message_new_signal(GetIPCIC(ic)->path, // object name of the signal
                        FCITX_IC_DBUS_INTERFACE, // interface name of the signal
                        "DeleteSurroundingText"); // name of the signal
@@ -1104,6 +1124,7 @@ void IPCDeleteSurroundingText(void* arg, FcitxInputContext* ic, int offset, unsi
 
 boolean IPCGetSurroundingText(void* arg, FcitxInputContext* ic, char** str, unsigned int *cursor, unsigned int *anchor)
 {
+    FCITX_UNUSED(arg);
     FcitxIPCIC* ipcic = GetIPCIC(ic);
 
     if (!ipcic->surroundingText)
@@ -1182,6 +1203,7 @@ void IPCUpdateClientSideUI(void* arg, FcitxInputContext* ic)
 
 boolean IPCCheckICFromSameApplication(void* arg, FcitxInputContext* icToCheck, FcitxInputContext* ic)
 {
+    FCITX_UNUSED(arg);
     FcitxIPCIC* ipcicToCheck = GetIPCIC(icToCheck);
     FcitxIPCIC* ipcic = GetIPCIC(ic);
     if (ipcic->appname == NULL || ipcicToCheck->appname == NULL)
@@ -1189,13 +1211,13 @@ boolean IPCCheckICFromSameApplication(void* arg, FcitxInputContext* icToCheck, F
     return strcmp(ipcicToCheck->appname, ipcic->appname) == 0;
 }
 
-void FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessage* message)
+DBusMessage* FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusMessage* message)
 {
     DBusError error;
     dbus_error_init(&error);
     char *interface;
     char *property;
-    DBusMessage* reply = dbus_message_new_method_return(message);
+    DBusMessage* reply = NULL;
     if (dbus_message_get_args(message, &error,
                               DBUS_TYPE_STRING, &interface,
                               DBUS_TYPE_STRING, &property,
@@ -1207,24 +1229,34 @@ void FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessa
                 break;
             index ++;
         }
-        DBusMessageIter args, variant;
-        dbus_message_iter_init_append(reply, &args);
-        dbus_message_iter_open_container(&args, DBUS_TYPE_VARIANT, propertTable[index].type, &variant);
-        if (propertTable[index].getfunc)
-            propertTable[index].getfunc(ipc, &variant);
-        dbus_message_iter_close_container(&args, &variant);
+
+        if (propertTable[index].interface) {
+            DBusMessageIter args, variant;
+            reply = dbus_message_new_method_return(message);
+            dbus_message_iter_init_append(reply, &args);
+            dbus_message_iter_open_container(&args, DBUS_TYPE_VARIANT, propertTable[index].type, &variant);
+            if (propertTable[index].getfunc)
+                propertTable[index].getfunc(ipc, &variant);
+            dbus_message_iter_close_container(&args, &variant);
+        }
+        else {
+            reply = dbus_message_new_error_printf(message, DBUS_ERROR_UNKNOWN_PROPERTY, "No such property ('%s.%s')", interface, property);
+        }
     }
-    dbus_connection_send(conn, reply, NULL);
-    dbus_message_unref(reply);
+    else {
+        reply = IPCDBusUnknownMethod(message);
+    }
+
+    return reply;
 }
 
-void FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessage* message)
+DBusMessage* FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusMessage* message)
 {
     DBusError error;
     dbus_error_init(&error);
     char *interface;
     char *property;
-    DBusMessage* reply = dbus_message_new_method_return(message);
+    DBusMessage* reply = NULL;
 
     DBusMessageIter args, variant;
     dbus_message_iter_init(message, &args);
@@ -1251,23 +1283,31 @@ void FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessa
             break;
         index ++;
     }
-    if (propertTable[index].setfunc)
+    if (propertTable[index].setfunc) {
         propertTable[index].setfunc(ipc, &variant);
+        reply = dbus_message_new_method_return(message);
+    }
+    else {
+        reply = dbus_message_new_error_printf(message, DBUS_ERROR_UNKNOWN_PROPERTY, "No such property ('%s.%s')", interface, property);
+    }
 
 dbus_property_set_end:
-    dbus_connection_send(conn, reply, NULL);
-    dbus_message_unref(reply);
+    if (!reply)
+        reply = IPCDBusUnknownMethod(message);
+
+    return reply;
 }
 
-void FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMessage* message)
+DBusMessage* FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusMessage* message)
 {
     DBusError error;
     dbus_error_init(&error);
     char *interface;
-    DBusMessage* reply = dbus_message_new_method_return(message);
+    DBusMessage* reply = NULL;
     if (dbus_message_get_args(message, &error,
                               DBUS_TYPE_STRING, &interface,
                               DBUS_TYPE_INVALID)) {
+        reply = dbus_message_new_method_return(message);
         int index = 0;
         DBusMessageIter args;
         dbus_message_iter_init_append(reply, &args);
@@ -1291,8 +1331,10 @@ void FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusConnection* conn, DBusMe
         }
         dbus_message_iter_close_container(&args, &array);
     }
-    dbus_connection_send(conn, reply, NULL);
-    dbus_message_unref(reply);
+    if (!reply)
+        reply = IPCDBusUnknownMethod(message);
+
+    return reply;
 }
 
 
@@ -1376,10 +1418,13 @@ void IPCSetPropertyIMList(void* arg, DBusMessageIter* args)
         dbus_message_iter_next(&ssub);
 
         char* newresult;
-        if (result == NULL)
-            asprintf(&newresult, "%s:%s", uniqueName, enable ? "True" : "False");
-        else
-            asprintf(&newresult, "%s,%s:%s", result, uniqueName, enable ? "True" : "False");
+        if (!result) {
+            fcitx_utils_alloc_cat_str(newresult, uniqueName, ":",
+                                      enable ? "True" : "False");
+        } else {
+            fcitx_utils_alloc_cat_str(newresult, result, ",", uniqueName, ":",
+                                      enable ? "True" : "False");
+        }
         if (result)
             free(result);
         result = newresult;
@@ -1425,6 +1470,7 @@ void IPCUpdateIMList(void* arg)
 
 pid_t IPCGetPid(void* arg, FcitxInputContext* ic)
 {
+    FCITX_UNUSED(arg);
     return GetIPCIC(ic)->pid;
 }
 
