@@ -40,7 +40,7 @@
 #include "xim.h"
 #include "ximhandler.h"
 #include "ximqueue.h"
-#include "module/x11/x11stuff.h"
+#include "module/x11/fcitx-x11.h"
 #include "fcitx-config/xdg.h"
 #include "fcitx/hook.h"
 
@@ -57,6 +57,7 @@ static void XimUpdatePreedit(void* arg, FcitxInputContext* ic);
 // static pid_t XimFindApplicationPid(FcitxXimFrontend* xim, Window w);
 
 static Bool XimProtocolHandler(XIMS _ims, IMProtocol * call_data);
+DECLARE_ADDFUNCTIONS(Xim)
 
 static XIMStyle OverTheSpot_Styles[] = {
     XIMPreeditPosition | XIMStatusArea, //OverTheSpot
@@ -112,39 +113,36 @@ static XIMEncoding zhEncodings[] = {
     NULL
 };
 
-char strLocale[201] = "zh_CN.GB18030,zh_CN.GB2312,zh_CN,zh_CN.GBK,zh_CN.UTF-8,zh_CN.UTF8,en_US.UTF-8,en_US.UTF8";
+static char strLocale[LOCALES_BUFSIZE + 1] = LOCALES_STRING;
 
-void* XimCreate(FcitxInstance* instance, int frontendid)
+static void*
+XimCreate(FcitxInstance* instance, int frontendid)
 {
     if (ximfrontend != NULL)
         return NULL;
-    FcitxXimFrontend* xim = fcitx_utils_malloc0(sizeof(FcitxXimFrontend));
+    FcitxXimFrontend *xim = fcitx_utils_new(FcitxXimFrontend);
     if (xim == NULL)
         return NULL;
 
     ximfrontend = xim;
 
-    XIMStyles *input_styles;
-    XIMEncodings *encodings;
     char *imname = NULL;
     char *p;
 
-    xim->display = InvokeVaArgs(instance, FCITX_X11, GETDISPLAY);
-
+    xim->display = FcitxX11GetDisplay(instance);
     if (xim->display == NULL) {
         FcitxLog(FATAL, _("X11 not initialized"));
         free(xim);
         return NULL;
     }
 
-    FcitxAddon* ximaddon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), "fcitx-xim");
-    xim->x11addon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), "fcitx-x11");
     xim->iScreen = DefaultScreen(xim->display);
     xim->owner = instance;
     xim->frontendid = frontendid;
-
-    xim->ximWindow = XCreateSimpleWindow(xim->display, DefaultRootWindow(xim->display), 0, 0, 1, 1, 1, 0, 0);
-    if (xim->ximWindow == (Window) NULL) {
+    xim->xim_window = XCreateWindow(xim->display, DefaultRootWindow(xim->display),
+                                    0, 0, 1, 1, 0, 0, InputOnly,
+                                    CopyFromParent, 0, NULL);
+    if (!xim->xim_window) {
         FcitxLog(FATAL, _("Can't Create imWindow"));
         free(xim);
         return NULL;
@@ -153,9 +151,9 @@ void* XimCreate(FcitxInstance* instance, int frontendid)
     if (!imname) {
         imname = getenv("XMODIFIERS");
         if (imname) {
-            if (strstr(imname, "@im="))
+            if (!strncmp(imname, "@im=", strlen("@im="))) {
                 imname += 4;
-            else {
+            } else {
                 FcitxLog(WARNING, _("XMODIFIERS Error."));
                 imname = DEFAULT_IMNAME;
             }
@@ -179,12 +177,15 @@ void* XimCreate(FcitxInstance* instance, int frontendid)
         if (!fp) {
             if (errno == ENOENT) {
                 char *file;
-                FILE *fp2 = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-xim.config", "w", &file);
+                FILE *fp2 = FcitxXDGGetFileUserWithPrefix("conf",
+                                                          "fcitx-xim.config",
+                                                          "w", &file);
                 FcitxLog(DEBUG, "Save Config to %s", file);
                 FcitxConfigSaveConfigFileFp(fp2, &xim->gconfig, configDesc);
                 free(file);
-                if (fp2)
+                if (fp2) {
                     fclose(fp2);
+                }
             }
         }
 
@@ -193,22 +194,26 @@ void* XimCreate(FcitxInstance* instance, int frontendid)
         FcitxXimFrontendConfigBind(xim, cfile, configDesc);
         FcitxConfigBindSync((FcitxGenericConfig*)xim);
 
-        if (fp)
+        if (fp) {
             fclose(fp);
+        }
     }
 
-    input_styles = (XIMStyles *) malloc(sizeof(XIMStyles));
+    XIMStyles input_styles;
     if (xim->bUseOnTheSpotStyle) {
-        input_styles->count_styles = sizeof(OnTheSpot_Styles) / sizeof(XIMStyle) - 1;
-        input_styles->supported_styles = OnTheSpot_Styles;
+        input_styles.count_styles =
+            sizeof(OnTheSpot_Styles) / sizeof(XIMStyle) - 1;
+        input_styles.supported_styles = OnTheSpot_Styles;
     } else {
-        input_styles->count_styles = sizeof(OverTheSpot_Styles) / sizeof(XIMStyle) - 1;
-        input_styles->supported_styles = OverTheSpot_Styles;
+        input_styles.count_styles =
+            sizeof(OverTheSpot_Styles) / sizeof(XIMStyle) - 1;
+        input_styles.supported_styles = OverTheSpot_Styles;
     }
 
-    encodings = (XIMEncodings *) malloc(sizeof(XIMEncodings));
-    encodings->count_encodings = sizeof(zhEncodings) / sizeof(XIMEncoding) - 1;
-    encodings->supported_encodings = zhEncodings;
+    XIMEncodings encodings = {
+        .count_encodings = sizeof(zhEncodings) / sizeof(XIMEncoding) - 1,
+        .supported_encodings = zhEncodings
+    };
 
     p = getenv("LC_CTYPE");
     if (!p) {
@@ -217,41 +222,39 @@ void* XimCreate(FcitxInstance* instance, int frontendid)
             p = getenv("LANG");
     }
     if (p) {
-        if (!strcasestr(strLocale, p)) {
-            strcat(strLocale, ",");
-            strcat(strLocale, p);
+        int p_l = strlen(p);
+        if (strlen(LOCALES_STRING) + p_l + 1 < LOCALES_BUFSIZE) {
+            strLocale[strlen(LOCALES_STRING)] = ',';
+            memcpy(strLocale + strlen(LOCALES_STRING) + 1, p, p_l + 1);
         }
     }
 
     xim->ims = IMOpenIM(xim->display,
                         IMModifiers, "Xi18n",
-                        IMServerWindow, xim->ximWindow,
+                        IMServerWindow, xim->xim_window,
                         IMServerName, imname,
                         IMLocale, strLocale,
                         IMServerTransport, "X/",
-                        IMInputStyles, input_styles,
-                        IMEncodingList, encodings,
+                        IMInputStyles, &input_styles,
+                        IMEncodingList, &encodings,
                         IMProtocolHandler, XimProtocolHandler,
                         IMFilterEventMask, KeyPressMask | KeyReleaseMask,
                         NULL);
 
-    free(input_styles);
-    free(encodings);
-
     if (xim->ims == (XIMS) NULL) {
-        FcitxLog(ERROR, _("Start XIM error. Another XIM daemon named %s is running?"), imname);
-        free(xim);
+        FcitxLog(ERROR, _("Start XIM error. Another XIM daemon named %s "
+                          "is running?"), imname);
+        XimDestroy(xim);
         FcitxInstanceEnd(instance);
         return NULL;
     }
-
-    AddFunction(ximaddon, XimConsumeQueue);
-
+    FcitxXimAddFunctions(instance);
     return xim;
 }
 
 Bool XimProtocolHandler(XIMS _ims, IMProtocol * call_data)
 {
+    FCITX_UNUSED(_ims);
     switch (call_data->major_code) {
     case XIM_OPEN:
         FcitxLog(DEBUG, "XIM_OPEN:\t\ticid=%d\tconnect_id=%d", ((IMForwardEventStruct *) call_data)->icid,
@@ -339,12 +342,21 @@ Bool XimProtocolHandler(XIMS _ims, IMProtocol * call_data)
 boolean XimDestroy(void* arg)
 {
     FcitxXimFrontend* xim = (FcitxXimFrontend*) arg;
-
+    /**
+     * Destroy the window BEFORE(!!!!!) CloseIM!!!
+     * Work arround for a bug in libX11. See wengxt's commit log:
+     * f773dd4f7152a4b4b7f406fe01bff466e0de3dc2
+     * [xim, x11] correctly shutdown xim, destroy x11 with error handling
+     **/
+    if (xim->xim_window) {
+        XDestroyWindow(xim->display, xim->xim_window);
+    }
     if (xim->ims) {
         IMCloseIM(xim->ims);
         xim->ims = NULL;
     }
-
+    XimQueueDestroy(xim);
+    free(xim);
     return true;
 }
 
@@ -393,24 +405,23 @@ void XimCommitString(void* arg, FcitxInputContext* ic, const char* str)
 void XimForwardKey(void *arg, FcitxInputContext* ic, FcitxKeyEventType event, FcitxKeySym sym, unsigned int state)
 {
     FcitxXimFrontend* xim = (FcitxXimFrontend*) arg;
+    Window win;
+    if (!(win = GetXimIC(ic)->focus_win))
+        win = GetXimIC(ic)->client_win;
     XEvent xEvent;
-
+    memset(&xEvent, 0, sizeof(xEvent));
     xEvent.xkey.type = (event == FCITX_PRESS_KEY) ? KeyPress : KeyRelease;
-    xEvent.xkey.display = xim->display;
     xEvent.xkey.serial = xim->currentSerialNumberKey;
     xEvent.xkey.send_event = False;
-    xEvent.xkey.x = xEvent.xkey.y = xEvent.xkey.x_root = xEvent.xkey.y_root = 0;
-    xEvent.xkey.same_screen = False;
-    xEvent.xkey.subwindow = None;
-    xEvent.xkey.window = None;
+    xEvent.xkey.display = xim->display;
+    xEvent.xkey.window = win;
     xEvent.xkey.root = DefaultRootWindow(xim->display);
+    xEvent.xkey.subwindow = None;
+    xEvent.xkey.time = CurrentTime;
     xEvent.xkey.state = state;
-    if (GetXimIC(ic)->focus_win)
-        xEvent.xkey.window = GetXimIC(ic)->focus_win;
-    else if (GetXimIC(ic)->client_win)
-        xEvent.xkey.window = GetXimIC(ic)->client_win;
-
     xEvent.xkey.keycode = XKeysymToKeycode(xim->display, sym);
+    xEvent.xkey.same_screen = False;
+
     XimForwardKeyInternal(xim, GetXimIC(ic), &xEvent);
 }
 
@@ -418,10 +429,8 @@ void XimSetWindowOffset(void* arg, FcitxInputContext* ic, int x, int y)
 {
     FcitxXimFrontend* xim = (FcitxXimFrontend*) arg;
     FcitxXimIC* ximic = GetXimIC(ic);
-    Window window = None, dst;
-    if (ximic->focus_win)
-        window = ximic->focus_win;
-    else if (ximic->client_win)
+    Window window, dst;
+    if (!(window = ximic->focus_win))
         window = ximic->client_win;
 
     if (window != None) {
@@ -435,6 +444,7 @@ void XimSetWindowOffset(void* arg, FcitxInputContext* ic, int x, int y)
 
 void XimGetWindowRect(void* arg, FcitxInputContext* ic, int* x, int* y, int* w, int* h)
 {
+    FCITX_UNUSED(arg);
     *x = ic->offset_x;
     *y = ic->offset_y;
     *w = 0;
@@ -473,11 +483,9 @@ void XimUpdatePreedit(void* arg, FcitxInputContext* ic)
 pid_t XimGetPid(void* arg, FcitxInputContext* ic)
 {
     FcitxXimFrontend* xim = (FcitxXimFrontend*) arg;
-    Window w = None;
+    Window w;
     FcitxXimIC* ximic = GetXimIC(ic);
-    if (ximic->focus_win)
-        w = ximic->focus_win;
-    else if (ximic->client_win)
+    if (!(w = ximic->focus_win))
         w = ximic->client_win;
 
     return XimFindApplicationPid(xim, w);
@@ -522,4 +530,4 @@ pid_t XimFindApplicationPid(FcitxXimFrontend* xim, Window w) {
 }
 #endif
 
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-xim-addfunctions.h"
