@@ -40,9 +40,8 @@
 #include "fcitx/hook.h"
 #include "fcitx-config/xdg.h"
 #include "fcitx-utils/log.h"
-#include "module/spell/spell.h"
+#include "module/spell/fcitx-spell.h"
 
-#include "xkb.h"
 #include "keyboard.h"
 #if defined(ENABLE_LIBXML2)
 #include "isocodes.h"
@@ -57,10 +56,13 @@
 static void* FcitxKeyboardCreate(FcitxInstance* instance);
 static boolean FcitxKeyboardInit(void *arg);
 static void  FcitxKeyboardResetIM(void *arg);
-static INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym, unsigned int);
+static void FcitxKeyboardOnClose(void* arg, FcitxIMCloseEventType event);
+static INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym,
+                                               unsigned int);
 static void  FcitxKeyboardSave(void *arg);
 static void  FcitxKeyboardReloadConfig(void *arg);
-INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg);
+static INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg);
+static void FcitxKeyboardCommitBuffer(FcitxKeyboard* keyboard);
 static void FcitxKeyboardLayoutCreate(FcitxKeyboard* keyboard,
                                       const char* name,
                                       const char* langCode,
@@ -195,13 +197,13 @@ static const char* FcitxKeyboardGetFutureStream(void* arg)
             break;
         int len = surlen - cursor + 1;
         if (keyboard->lastLength < len) {
-            free(keyboard->tempBuffer);
             while (keyboard->lastLength < len)
                 keyboard->lastLength *= 2;
-            keyboard->tempBuffer = fcitx_utils_malloc0(keyboard->lastLength);
+            keyboard->tempBuffer = realloc(keyboard->tempBuffer,
+                                           keyboard->lastLength);
         }
 
-        strcpy(keyboard->tempBuffer, &surrounding[cursor]);
+        memcpy(keyboard->tempBuffer, &surrounding[cursor], len);
         result = keyboard->tempBuffer;
     } while(0);
 
@@ -226,48 +228,58 @@ void FcitxKeyboardLayoutCreate(FcitxKeyboard* keyboard,
         layout->variantString = strdup(variantString);
     layout->owner = keyboard;
 
-    char* uniqueName;
-    if (variantString)
-        asprintf(&uniqueName, "fcitx-keyboard-%s-%s", layoutString, variantString);
-    else
-        asprintf(&uniqueName, "fcitx-keyboard-%s", layoutString);
+    if (fcitx_utils_strcmp0(langCode, "en") == 0
+        && fcitx_utils_strcmp0(layoutString, "us") == 0
+        && fcitx_utils_strcmp0(variantString, NULL) == 0)
+        keyboard->enUSRegistered = true;
 
     int iPriority = 100;
-    if (strcmp(keyboard->initialLayout, layoutString) == 0
-        && fcitx_utils_strcmp0(keyboard->initialVariant, variantString) == 0) {
+    if (strcmp(keyboard->initialLayout, layoutString) == 0 &&
+        fcitx_utils_strcmp0(keyboard->initialVariant, variantString) == 0) {
         iPriority = PRIORITY_MAGIC_FIRST;
     }
     else {
         boolean result = false;
-        InvokeVaArgs(keyboard->owner, FCITX_XKB, LAYOUTEXISTS,
-                     (void*)layoutString, (void*)variantString, &result);
+        FcitxXkbLayoutExists(keyboard->owner, layoutString,
+                             variantString, &result);
         if (result)
             iPriority = 50;
     }
 
-    FcitxInstanceRegisterIM(
+    char *uniqueName;
+    if (variantString) {
+        fcitx_utils_alloc_cat_str(uniqueName, "fcitx-keyboard-", layoutString,
+                                  "-", variantString);
+    } else {
+        fcitx_utils_alloc_cat_str(uniqueName, "fcitx-keyboard-", layoutString);
+    }
+
+    FcitxIMIFace iface;
+    memset(&iface, 0, sizeof(FcitxIMIFace));
+    iface.Init = FcitxKeyboardInit;
+    iface.ResetIM = FcitxKeyboardResetIM;
+    iface.DoInput = FcitxKeyboardDoInput;
+    iface.GetCandWords = FcitxKeyboardGetCandWords;
+    iface.Save = FcitxKeyboardSave;
+    iface.ReloadConfig = FcitxKeyboardReloadConfig;
+    iface.OnClose = FcitxKeyboardOnClose;
+
+    FcitxInstanceRegisterIMv2(
         keyboard->owner,
         layout,
         uniqueName,
         name,
         "kbd",
-        FcitxKeyboardInit,
-        FcitxKeyboardResetIM,
-        FcitxKeyboardDoInput,
-        FcitxKeyboardGetCandWords,
-        NULL,
-        FcitxKeyboardSave,
-        FcitxKeyboardReloadConfig,
-        NULL,
+        iface,
         iPriority,
-        langCode
-    );
-
+        langCode);
     free(uniqueName);
 }
 
 void* SimpleCopy(void* arg, void* dest, void* src)
 {
+    FCITX_UNUSED(arg);
+    FCITX_UNUSED(dest);
     return src;
 }
 
@@ -301,21 +313,31 @@ void* FcitxKeyboardCreate(FcitxInstance* instance)
 
     FcitxInstanceRegisterHotkeyFilter(instance, hk);
 
-    FcitxXkbRules* rules = InvokeVaArgs(instance, FCITX_XKB, GETRULES);
+    FcitxXkbRules* rules = FcitxXkbGetRules(instance);
     keyboard->rules = rules;
-#if defined(ENABLE_LIBXML2)
-#endif
 
     keyboard->initialLayout = NULL;
     keyboard->initialVariant = NULL;
 
-    InvokeVaArgs(instance, FCITX_XKB, GETCURRENTLAYOUT,
-                 &keyboard->initialLayout, &keyboard->initialVariant);
+    FcitxXkbGetCurrentLayout(instance, &keyboard->initialLayout,
+                             &keyboard->initialVariant);
     if (!keyboard->initialLayout)
         keyboard->initialLayout = strdup("us");
 
 #if defined(ENABLE_LIBXML2)
     if (rules && utarray_len(rules->layoutInfos)) {
+        char* tempfile = NULL;
+        FcitxXDGGetFileUserWithPrefix("", "", "w", NULL);
+        FcitxXDGGetFileUserWithPrefix("", "cached_layout_XXXXXX", NULL, &tempfile);
+        int fd = mkstemp(tempfile);
+
+        FILE* fp = NULL;
+        if (fd > 0)
+            fp = fdopen(fd, "w");
+        else {
+            free(tempfile);
+        }
+
         FcitxIsoCodes* isocodes = FcitxXkbReadIsoCodes(ISOCODES_ISO639_XML, ISOCODES_ISO3166_XML);
         FcitxXkbLayoutInfo* layoutInfo;
         for (layoutInfo = (FcitxXkbLayoutInfo*) utarray_front(rules->layoutInfos);
@@ -332,12 +354,15 @@ void* FcitxKeyboardCreate(FcitxInstance* instance)
                         lang = entry->iso_639_1_code;
                     }
                 }
-                char* description;
-                asprintf(&description, _("Keyboard - %s"),
-                        dgettext("xkeyboard-config", layoutInfo->description));
-
-                FcitxKeyboardLayoutCreate(keyboard, description, lang, layoutInfo->name, NULL);
+                char *description;
+                fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
+                                          dgettext("xkeyboard-config",
+                                                   layoutInfo->description));
+                FcitxKeyboardLayoutCreate(keyboard, description,
+                                          lang, layoutInfo->name, NULL);
                 free(description);
+                if (fp)
+                    fprintf(fp, "%s:%s:%s\n", layoutInfo->description, layoutInfo->name, lang ? lang : "");
             }
             FcitxXkbVariantInfo* variantInfo;
             for (variantInfo = (FcitxXkbVariantInfo*) utarray_front(layoutInfo->variantInfos);
@@ -355,31 +380,101 @@ void* FcitxKeyboardCreate(FcitxInstance* instance)
                         lang = entry->iso_639_1_code;
                     }
                 }
-                char* description;
-                asprintf(&description, _("Keyboard - %s - %s"),
-                        dgettext("xkeyboard-config", layoutInfo->description),
-                        dgettext("xkeyboard-config", variantInfo->description));
-
-                FcitxKeyboardLayoutCreate(keyboard, description, lang, layoutInfo->name, variantInfo->name);
+                char *description;
+                fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
+                                          dgettext("xkeyboard-config",
+                                                   layoutInfo->description),
+                                          " - ",
+                                          dgettext("xkeyboard-config",
+                                                   variantInfo->description));
+                FcitxKeyboardLayoutCreate(keyboard, description, lang,
+                                          layoutInfo->name, variantInfo->name);
+                if (fp)
+                    fprintf(fp, "%s:%s:%s:%s:%s\n",
+                            layoutInfo->description, layoutInfo->name,
+                            variantInfo->description, variantInfo->name,
+                            lang ? lang : "");
                 free(description);
             }
         }
         FcitxIsoCodesFree(isocodes);
+
+        if (fp) {
+            fclose(fp);
+            char* layoutFileName = 0;
+            FcitxXDGGetFileUserWithPrefix("", "cached_layout", NULL, &layoutFileName);
+            if (access(layoutFileName, 0))
+                unlink(layoutFileName);
+            rename(tempfile, layoutFileName);
+
+            free(tempfile);
+            free(layoutFileName);
+        }
     }
     else
 #endif
     {
-        char* description;
-        asprintf(&description, _("Keyboard"));
+        /* though this is unrelated to libxml2, but can only generated with libxml2 enabled */
+#if defined(ENABLE_LIBXML2)
+        FILE* fp = FcitxXDGGetFileUserWithPrefix("", "cached_layout", "r", NULL);
+        if (fp) {
+            char *buf = NULL, *buf1 = NULL;
+            size_t len = 0;
+            while (getline(&buf, &len, fp) != -1) {
+                fcitx_utils_free(buf1);
+                buf1 = fcitx_utils_trim(buf);
+                UT_array* list = fcitx_utils_split_string(buf1, ':');
+                if (utarray_len(list) == 3) {
+                    char** layoutDescription = (char**)utarray_eltptr(list, 0);
+                    char** layoutName = (char**)utarray_eltptr(list, 1);
+                    char** layoutLang = (char**)utarray_eltptr(list, 2);
+                    char *description;
+                    fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
+                                              dgettext("xkeyboard-config",
+                                                     *layoutDescription), " ", _("(Unavailable)"));
+                    FcitxKeyboardLayoutCreate(keyboard, description,
+                                            *layoutLang, *layoutName, NULL);
+                }
+                else if (utarray_len(list) == 5) {
+                    char** layoutDescription = (char**)utarray_eltptr(list, 0);
+                    char** layoutName = (char**)utarray_eltptr(list, 1);
+                    char** variantDescription = (char**)utarray_eltptr(list, 2);
+                    char** variantName = (char**)utarray_eltptr(list, 3);
+                    char** variantLang = (char**)utarray_eltptr(list, 4);
+                    char *description;
+                    fcitx_utils_alloc_cat_str(description, _("Keyboard"), " - ",
+                                              dgettext("xkeyboard-config",
+                                                       *layoutDescription),
+                                              " - ",
+                                              dgettext("xkeyboard-config",
+                                                       *variantDescription),
+                                              " ", _("(Unavailable)"));
+                    FcitxKeyboardLayoutCreate(keyboard, description, *variantLang,
+                                              *layoutName, *variantName);
+                }
+                fcitx_utils_free_string_list(list);
+            }
 
-        fcitx_utils_free(keyboard->initialLayout);
-        keyboard->initialLayout = strdup("us");
-        fcitx_utils_free(keyboard->initialVariant);
-        keyboard->initialVariant = NULL;
-        FcitxKeyboardLayoutCreate(keyboard, description, "en", "us", NULL);
-        free(description);
+            fcitx_utils_free(buf);
+            fcitx_utils_free(buf1);
+
+            fclose(fp);
+        }
+        else
+#endif
+        {
+            fcitx_utils_free(keyboard->initialLayout);
+            keyboard->initialLayout = strdup("us");
+            fcitx_utils_free(keyboard->initialVariant);
+            keyboard->initialVariant = NULL;
+            FcitxKeyboardLayoutCreate(keyboard, _("Keyboard"), "en", "us", NULL);
+        }
     }
 
+    /* always have en us is better */
+    if (!keyboard->enUSRegistered)
+        FcitxKeyboardLayoutCreate(keyboard, _("Keyboard"), "en", "us", NULL);
+
     keyboard->lastLength = 10;
     keyboard->tempBuffer = fcitx_utils_malloc0(keyboard->lastLength);
 
@@ -399,8 +494,9 @@ boolean FcitxKeyboardInit(void *arg)
     FcitxInstanceSetContext(layout->owner->owner,
                             CONTEXT_DISABLE_FULLWIDTH, &flag);
     if (layout->variantString) {
-        char* string;
-        asprintf(&string, "%s,%s", layout->layoutString, layout->variantString);
+        char *string;
+        fcitx_utils_alloc_cat_str(string, layout->layoutString, ",",
+                                  layout->variantString);
         FcitxInstanceSetContext(layout->owner->owner,
                                 CONTEXT_IM_KEYBOARD_LAYOUT, string);
         free(string);
@@ -412,6 +508,15 @@ boolean FcitxKeyboardInit(void *arg)
     return true;
 }
 
+void FcitxKeyboardCommitBuffer(FcitxKeyboard* keyboard)
+{
+    if (keyboard->buffer[0]) {
+        FcitxInstanceCommitString(keyboard->owner, FcitxInstanceGetCurrentIC(keyboard->owner),
+                                    keyboard->buffer);
+        FcitxKeyboardResetIM(keyboard);
+    }
+}
+
 void  FcitxKeyboardResetIM(void *arg)
 {
     FcitxKeyboardLayout *layout = (FcitxKeyboardLayout*) arg;
@@ -422,10 +527,21 @@ void  FcitxKeyboardResetIM(void *arg)
     keyboard->n_compose = 0;
 }
 
-boolean IsDictAvailable(FcitxKeyboard* keyboard)
+void FcitxKeyboardOnClose(void* arg, FcitxIMCloseEventType event)
 {
-    return InvokeVaArgs(keyboard->owner, FCITX_SPELL, DICT_AVAILABLE,
-                        keyboard->dictLang, NULL);
+    FcitxKeyboardLayout *layout = (FcitxKeyboardLayout*) arg;
+    FcitxKeyboard *keyboard = layout->owner;
+    if (event == CET_LostFocus) {
+    } else if (event == CET_ChangeByInactivate) {
+        FcitxKeyboardCommitBuffer(keyboard);
+    } else if (event == CET_ChangeByUser) {
+        FcitxKeyboardCommitBuffer(keyboard);
+    }
+}
+
+static boolean IsDictAvailable(FcitxKeyboard* keyboard)
+{
+    return FcitxSpellDictAvailable(keyboard->owner, keyboard->dictLang, NULL);
 }
 
 /* a little bit funny here LOL.... */
@@ -434,21 +550,92 @@ static const FcitxHotkey FCITX_HYPHEN_APOS[2] = {
     {NULL, FcitxKey_apostrophe, FcitxKeyState_None}
 };
 
+static void
+FcitxKeyboardSetBuff(FcitxKeyboard *keyboard, const char *str)
+{
+    int len = strlen(str);
+    if (len > FCITX_KEYBOARD_MAX_BUFFER)
+        len = FCITX_KEYBOARD_MAX_BUFFER;
+    memcpy(keyboard->buffer, str, len);
+    keyboard->cursorPos = len;
+    keyboard->buffer[len] = '\0';
+}
+
+static INPUT_RETURN_VALUE
+FcitxKeyboardHandleFocus(FcitxKeyboard *keyboard, FcitxKeySym sym,
+                         unsigned int state)
+{
+    FcitxInstance *instance = keyboard->owner;
+    FcitxInputState *input = FcitxInstanceGetInputState(instance);
+    FcitxCandidateWordList *cand_list = FcitxInputStateGetCandidateList(input);
+    FcitxCandidateWord *cand_word;
+    if (!FcitxCandidateWordGetListSize(cand_list))
+        return IRV_TO_PROCESS;
+    FcitxGlobalConfig *fc = FcitxInstanceGetGlobalConfig(instance);
+    if (FcitxHotkeyIsHotKey(sym, state, fc->nextWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetNext(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(cand_list, 0);
+            cand_word = FcitxCandidateWordGetFirst(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state, fc->prevWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetPrev(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(
+                cand_list, FcitxCandidateWordPageCount(cand_list) - 1);
+            cand_word = FcitxCandidateWordGetLast(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigPrevPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (FcitxCandidateWordGoPrevPage(cand_list)) {
+            cand_word = FcitxCandidateWordGetCurrentWindow(cand_list) +
+                FcitxCandidateWordGetCurrentWindowSize(cand_list) - 1;
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigNextPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (FcitxCandidateWordGoNextPage(cand_list)) {
+            cand_word = FcitxCandidateWordGetCurrentWindow(cand_list);
+        }
+    } else {
+        return IRV_TO_PROCESS;
+    }
+    FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+    FcitxKeyboardSetBuff(keyboard, cand_word->strWord);
+    FcitxMessages *client_preedit = FcitxInputStateGetClientPreedit(input);
+    FcitxMessagesSetMessageCount(client_preedit, 0);
+    FcitxMessagesAddMessageStringsAtLast(client_preedit, MSG_INPUT,
+                                         keyboard->buffer);
+    FcitxInputStateSetClientCursorPos(input, keyboard->cursorPos);
+    return IRV_FLAG_UPDATE_INPUT_WINDOW;
+}
+
 INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int state)
 {
     FcitxKeyboardLayout* layout = (FcitxKeyboardLayout*) arg;
     FcitxKeyboard* keyboard = layout->owner;
     FcitxInstance* instance = keyboard->owner;
+    INPUT_RETURN_VALUE res = FcitxKeyboardHandleFocus(keyboard, sym, state);
+    if (res != IRV_TO_PROCESS)
+        return res;
     const char* currentLang = FcitxInstanceGetContextString(instance,
                                                             CONTEXT_IM_LANGUAGE);
     if (currentLang == NULL)
         currentLang = "";
 
-    if (sym == FcitxKey_Shift_L || sym == FcitxKey_Shift_R
-        || sym == FcitxKey_Alt_L || sym == FcitxKey_Alt_R
-        || sym == FcitxKey_Control_L || sym == FcitxKey_Control_R
-        || sym == FcitxKey_Super_L || sym == FcitxKey_Super_R
-    )
+    if (sym == FcitxKey_Shift_L || sym == FcitxKey_Shift_R ||
+        sym == FcitxKey_Alt_L || sym == FcitxKey_Alt_R ||
+        sym == FcitxKey_Control_L || sym == FcitxKey_Control_R ||
+        sym == FcitxKey_Super_L || sym == FcitxKey_Super_R)
         return IRV_TO_PROCESS;
 
     uint32_t result = processCompose(layout, sym, state);
@@ -472,16 +659,18 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
 
         if (FcitxHotkeyIsHotKey(sym, state,
                                 keyboard->config.hkAddToUserDict)) {
-            if (InvokeVaArgs(instance, FCITX_SPELL, ADD_PERSONAL,
-                             keyboard->buffer, keyboard->dictLang))
+            if (FcitxSpellAddPersonal(instance, keyboard->buffer,
+                                      keyboard->dictLang)) {
                 return IRV_DO_NOTHING;
+            }
         }
 
         if (IsValidChar(result) || FcitxHotkeyIsHotKeySimple(sym, state) ||
             IsValidSym(sym, state)) {
             if (IsValidChar(result) || FcitxHotkeyIsHotKeyLAZ(sym, state) ||
                 FcitxHotkeyIsHotKeyUAZ(sym, state) || IsValidSym(sym, state) ||
-                FcitxHotkeyIsHotKey(sym, state, FCITX_HYPHEN_APOS)) {
+                (*keyboard->buffer &&
+                 FcitxHotkeyIsHotKey(sym, state, FCITX_HYPHEN_APOS))) {
                 char buf[UTF8_MAX_LENGTH + 1];
                 memset(buf, 0, sizeof(buf));
                 if (result)
@@ -490,13 +679,14 @@ INPUT_RETURN_VALUE FcitxKeyboardDoInput(void *arg, FcitxKeySym sym, unsigned int
                     Ucs4ToUtf8(keyboard->iconv, FcitxKeySymToUnicode(sym), buf);
                 size_t charlen = strlen(buf);
 
-                if (strlen(keyboard->buffer) >= FCITX_KEYBOARD_MAX_BUFFER) {
+                size_t len = strlen(keyboard->buffer);
+                if (len >= FCITX_KEYBOARD_MAX_BUFFER) {
                     FcitxInstanceCommitString(instance, currentIC,
                                               keyboard->buffer);
                     keyboard->cursorPos = 0;
                     keyboard->buffer[0] = '\0';
+                    len = 0;
                 }
-                size_t len = strlen(keyboard->buffer);
                 if (keyboard->buffer[keyboard->cursorPos] != 0) {
                     memmove(keyboard->buffer + keyboard->cursorPos + charlen,
                             keyboard->buffer + keyboard->cursorPos,
@@ -589,39 +779,40 @@ INPUT_RETURN_VALUE FcitxKeyboardGetCandWords(void* arg)
     if (keyboard->buffer[0] == '\0')
         return IRV_CLEAN;
 
-    unsigned int cmodtable[] = {FcitxKeyState_None, FcitxKeyState_Alt,
-                                FcitxKeyState_Ctrl, FcitxKeyState_Shift};
-    if (keyboard->config.chooseModifier > CM_CTRL)
-        keyboard->config.chooseModifier = CM_CTRL;
-    FcitxCandidateWordSetPageSize(FcitxInputStateGetCandidateList(input),
-                                  keyboard->config.maximumHintLength);
+    static const unsigned int cmodtable[] = {
+        FcitxKeyState_None, FcitxKeyState_Alt,
+        FcitxKeyState_Ctrl, FcitxKeyState_Shift};
+    if (fcitx_unlikely(keyboard->config.chooseModifier >= _CM_COUNT))
+        keyboard->config.chooseModifier = _CM_COUNT - 1;
+    FcitxCandidateWordList *mainList = FcitxInputStateGetCandidateList(input);
+    FcitxCandidateWordSetPageSize(mainList, keyboard->config.maximumHintLength);
     FcitxCandidateWordSetChooseAndModifier(
-        FcitxInputStateGetCandidateList(input), DIGIT_STR_CHOOSE,
-        cmodtable[keyboard->config.chooseModifier]);
-    ssize_t bufferlen = strlen(keyboard->buffer);
-    strcpy(FcitxInputStateGetRawInputBuffer(input), keyboard->buffer);
+        mainList, DIGIT_STR_CHOOSE, cmodtable[keyboard->config.chooseModifier]);
+    size_t bufferlen = strlen(keyboard->buffer);
+    memcpy(FcitxInputStateGetRawInputBuffer(input),
+           keyboard->buffer, bufferlen + 1);
     FcitxInputStateSetRawInputBufferSize(input, bufferlen);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetClientPreedit(input),
-                                  MSG_INPUT, "%s", keyboard->buffer);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetPreedit(input), MSG_INPUT,
-                                  "%s", keyboard->buffer);
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetClientPreedit(input),
+                                         MSG_INPUT, keyboard->buffer);
     FcitxInputStateSetClientCursorPos(input, keyboard->cursorPos);
-    FcitxInputStateSetCursorPos(input, keyboard->cursorPos);
+    if (!FcitxInstanceICSupportPreedit(instance, FcitxInstanceGetCurrentIC(instance))) {
+        FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
+                                             MSG_INPUT, keyboard->buffer);
+        FcitxInputStateSetCursorPos(input, keyboard->cursorPos);
+    }
 
     if (bufferlen < keyboard->config.minimumHintLength)
         return IRV_DISPLAY_CANDWORDS;
 
-    FcitxCandidateWordList *candList;
-    candList = InvokeVaArgs(instance, FCITX_SPELL, GET_CANDWORDS,
-                            NULL, keyboard->buffer, NULL,
-                            (void*)(long)keyboard->config.maximumHintLength,
-                            keyboard->dictLang, NULL,
-                            FcitxKeyboardGetCandWordCb, layout);
-    if (!candList)
+    FcitxCandidateWordList *newList;
+    newList = FcitxSpellGetCandWords(instance, NULL, keyboard->buffer, NULL,
+                                     keyboard->config.maximumHintLength,
+                                     keyboard->dictLang, NULL,
+                                     FcitxKeyboardGetCandWordCb, layout);
+    if (!newList)
         return IRV_DISPLAY_CANDWORDS;
-    FcitxCandidateWordMerge(FcitxInputStateGetCandidateList(input),
-                            candList, -1);
-    FcitxCandidateWordFreeList(candList);
+    FcitxCandidateWordMerge(mainList, newList, -1);
+    FcitxCandidateWordFreeList(newList);
     return IRV_DISPLAY_CANDWORDS;
 }
 
@@ -643,8 +834,7 @@ boolean LoadKeyboardConfig(FcitxKeyboard* keyboard, FcitxKeyboardConfig* fs)
 
     FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-keyboard.config", "r", NULL);
 
-    if (!fp)
-    {
+    if (!fp) {
         if (errno == ENOENT)
             SaveKeyboardConfig(fs);
     }
@@ -661,7 +851,8 @@ boolean LoadKeyboardConfig(FcitxKeyboard* keyboard, FcitxKeyboardConfig* fs)
 void SaveKeyboardConfig(FcitxKeyboardConfig* fs)
 {
     FcitxConfigFileDesc *configDesc = GetKeyboardConfigDesc();
-    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-keyboard.config", "w", NULL);
+    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-keyboard.config",
+                                             "w", NULL);
     FcitxConfigSaveConfigFileFp(fp, &fs->gconfig, configDesc);
     if (fp)
         fclose(fp);
@@ -2455,7 +2646,7 @@ static const uint16_t validChar[] =
 0x30f3
 };
 
-inline
+static inline
 boolean IsValidSym(FcitxKeySym keysym, unsigned int state) {
     if (state != 0)
         return false;
@@ -2477,7 +2668,7 @@ boolean IsValidSym(FcitxKeySym keysym, unsigned int state) {
     return false;
 }
 
-inline
+static inline
 boolean IsValidChar(uint32_t c) {
     if (c == 0 || c == INVALID_COMPOSE_RESULT)
         return false;
