@@ -28,7 +28,7 @@
 #include <fcitx/ime.h>
 #include <fcitx-utils/utarray.h>
 #include <fcitx-config/fcitx-config.h>
-#include "rules.h"
+#include "module/xkb/fcitx-xkb.h"
 
 #define FCITX_KEYBOARD_MAX_BUFFER 20
 #define FCITX_MAX_COMPOSE_LEN 7
@@ -38,6 +38,7 @@ typedef enum _ChooseModifier {
     CM_ALT,
     CM_CTRL,
     CM_SHIFT,
+    _CM_COUNT
 } ChooseModifier;
 
 typedef struct _FcitxKeyboardConfig {
@@ -66,6 +67,7 @@ typedef struct _FcitxKeyboard {
     char *tempBuffer;
     int lastLength;
     int dataSlot;
+    int enUSRegistered;
 } FcitxKeyboard;
 
 typedef struct _FcitxKeyboardLayout {
