@@ -48,45 +48,35 @@
 #include "sp.h"
 #include "pyconfig.h"
 #include "spdata.h"
-#include "module/quickphrase/quickphrase.h"
+#include "module/quickphrase/fcitx-quickphrase.h"
 
 #define PY_INDEX_MAGIC_NUMBER 0xf7462e34
 #define PINYIN_TEMP_FILE "pinyin_XXXXXX"
 
 FCITX_DEFINE_PLUGIN(fcitx_pinyin, ime, FcitxIMClass) = {
     PYCreate,
-    NULL
+    PYDestroy
 };
 
-FCITX_EXPORT_API
-int ABI_VERSION = FCITX_ABI_VERSION;
-
-const UT_icd pycand_icd = {sizeof(PYCandWord*) , NULL, NULL, NULL };
-
 typedef struct {
     PY_CAND_WORD_TYPE type;
     ADJUSTORDER order;
     FcitxPinyinState* pystate;
 } PYCandWordSortContext;
 
-static void LoadPYPhraseDict(FcitxPinyinState* pystate, FILE* fp, boolean isSystem, boolean stripDup);
-static void * LoadPYBaseDictWrapper(void* arg, FcitxModuleFunctionArg args);
-static void * PYGetPYByHZWrapper(void* arg, FcitxModuleFunctionArg args);
-static void * DoPYInputWrapper(void* arg, FcitxModuleFunctionArg args);
-static void * PYGetCandWordsWrapper(void* arg, FcitxModuleFunctionArg args);
-static void * PYGetFindStringWrapper(void* arg, FcitxModuleFunctionArg args);
-static void * PYResetWrapper(void* arg, FcitxModuleFunctionArg args);
+static void LoadPYPhraseDict(FcitxPinyinState* pystate, FILE* fp,
+                             boolean isSystem, boolean stripDup);
 static void ReloadConfigPY(void* arg);
 static void PinyinMigration();
 static int PYCandWordCmp(const void* b, const void* a, void* arg);
-static void* PYSP2QP(void* arg, FcitxModuleFunctionArg args);
-static boolean PYGetPYMapByHZ(FcitxPinyinState*pystate, char *strHZ, char* mapHint, char *strMap);
-static void PYAddUserPhraseFromCString(void* arg, FcitxModuleFunctionArg args);
+static boolean PYGetPYMapByHZ(FcitxPinyinState*pystate, char *strHZ,
+                              char* mapHint, char *strMap);
+
+DECLARE_ADDFUNCTIONS(Pinyin)
 
 void *PYCreate(FcitxInstance* instance)
 {
     FcitxPinyinState *pystate = fcitx_utils_new(FcitxPinyinState);
-    FcitxAddon *pyaddon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_PINYIN_NAME);
     InitMHPY(&pystate->pyconfig.MHPY_C, MHPY_C_TEMPLATE);
     InitMHPY(&pystate->pyconfig.MHPY_S, MHPY_S_TEMPLATE);
     InitPYTable(&pystate->pyconfig);
@@ -95,6 +85,7 @@ void *PYCreate(FcitxInstance* instance)
         free(pystate->pyconfig.MHPY_C);
         free(pystate->pyconfig.MHPY_S);
         free(pystate->pyconfig.PYTable);
+        FreePYSplitData(&pystate->pyconfig);
         free(pystate);
         return NULL;
     }
@@ -104,51 +95,85 @@ void *PYCreate(FcitxInstance* instance)
     pystate->pool = fcitx_memory_pool_create();
 
     FcitxInstanceRegisterIM(instance,
-                      pystate,
-                      "pinyin",
-                      _("Pinyin"),
-                      "pinyin",
-                      PYInit,
-                      ResetPYStatus,
-                      DoPYInput,
-                      PYGetCandWords,
-                      NULL,
-                      SavePY,
-                      ReloadConfigPY,
-                      NULL,
-                      5,
-                      "zh_CN"
-                     );
+                            pystate,
+                            "pinyin",
+                            _("Pinyin"),
+                            "pinyin",
+                            PYInit,
+                            ResetPYStatus,
+                            DoPYInput,
+                            PYGetCandWords,
+                            NULL,
+                            SavePY,
+                            ReloadConfigPY,
+                            NULL,
+                            5,
+                            "zh_CN");
     FcitxInstanceRegisterIM(instance,
-                      pystate,
-                      "shuangpin",
-                      _("Shuangpin"),
-                      "shuangpin",
-                      SPInit,
-                      ResetPYStatus,
-                      DoPYInput,
-                      PYGetCandWords,
-                      NULL,
-                      SavePY,
-                      ReloadConfigPY,
-                      NULL,
-                      5,
-                      "zh_CN"
-                     );
+                            pystate,
+                            "shuangpin",
+                            _("Shuangpin"),
+                            "shuangpin",
+                            SPInit,
+                            ResetPYStatus,
+                            DoPYInput,
+                            PYGetCandWords,
+                            NULL,
+                            SavePY,
+                            ReloadConfigPY,
+                            NULL,
+                            5,
+                            "zh_CN");
     pystate->owner = instance;
 
-    /* ensure the order! */
-    AddFunction(pyaddon, LoadPYBaseDictWrapper); // 0
-    AddFunction(pyaddon, PYGetPYByHZWrapper); // 1
-    AddFunction(pyaddon, DoPYInputWrapper); // 2
-    AddFunction(pyaddon, PYGetCandWordsWrapper); // 3
-    AddFunction(pyaddon, PYGetFindStringWrapper); // 4
-    AddFunction(pyaddon, PYResetWrapper); // 5
-    AddFunction(pyaddon, PYSP2QP); // 6
-    AddFunction(pyaddon, PYAddUserPhraseFromCString); // 6
+    FcitxPinyinAddFunctions(instance);
     return pystate;
 }
 
+void PYDestroy(void* arg)
+{
+    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
+    free(pystate->pyconfig.MHPY_C);
+    free(pystate->pyconfig.MHPY_S);
+    free(pystate->pyconfig.PYTable);
+    FreePYSplitData(&pystate->pyconfig);
+    FcitxConfigFree(&pystate->pyconfig.gconfig);
+    fcitx_memory_pool_destroy(pystate->pool);
+
+    int i, j, k;
+    PYFA *PYFAList = pystate->PYFAList;
+    for (i = 0; i < pystate->iPYFACount; i++) {
+        for (j = 0; j < PYFAList[i].iBase; j++) {
+            PyPhrase* phrase = USER_PHRASE_NEXT(PYFAList[i].pyBase[j].userPhrase);
+            for (k = 0; k < PYFAList[i].pyBase[j].iUserPhrase; k++) {
+                PyPhrase* cur = phrase;
+                fcitx_utils_free(cur->strPhrase);
+                fcitx_utils_free(cur->strMap);
+                phrase = USER_PHRASE_NEXT(phrase);
+                free(cur);
+            }
+
+            free(PYFAList[i].pyBase[j].userPhrase);
+            fcitx_utils_free(PYFAList[i].pyBase[j].phrase);
+        }
+        free(PYFAList[i].pyBase);
+    }
+    free(PYFAList);
+
+    while(pystate->pyFreq) {
+        PyFreq* pCurFreq = pystate->pyFreq;
+        pystate->pyFreq = pCurFreq->next;
+        while (pCurFreq->HZList) {
+            HZ* pHZ = pCurFreq->HZList;
+            pCurFreq->HZList = pHZ->next;
+            free(pHZ);
+        }
+        free(pCurFreq);
+    }
+
+    free(pystate);
+}
+
 boolean PYInit(void *arg)
 {
     FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
@@ -541,8 +566,8 @@ INPUT_RETURN_VALUE DoPYInput(void* arg, FcitxKeySym sym, unsigned int state)
                 int key = sym;
                 boolean useDup = false;
                 boolean append = true;
-                if (InvokeVaArgs(pystate->owner, FCITX_QUICKPHRASE,
-                                 LAUNCHQUICKPHRASE, &key, &useDup, &append))
+                if (FcitxQuickPhraseLaunch(pystate->owner, &key,
+                                           &useDup, &append))
                     return IRV_DISPLAY_MESSAGE;
             }
 
@@ -686,7 +711,9 @@ INPUT_RETURN_VALUE DoPYInput(void* arg, FcitxKeySym sym, unsigned int state)
                 FcitxInputStateSetIsDoInputOnly(input, true);
 
                 FcitxInstanceCleanInputWindowUp(pystate->owner);
-                FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Press index to delete user phrase (ESC for cancel)"));
+                FcitxMessagesAddMessageStringsAtLast(
+                    FcitxInputStateGetAuxUp(input), MSG_TIPS,
+                    _("Press index to delete user phrase (ESC for cancel)"));
                 FcitxInputStateSetShowCursor(input, false);
 
                 return IRV_DISPLAY_MESSAGE;
@@ -697,7 +724,9 @@ INPUT_RETURN_VALUE DoPYInput(void* arg, FcitxKeySym sym, unsigned int state)
                 FcitxInputStateSetIsDoInputOnly(input, true);
 
                 FcitxInstanceCleanInputWindowUp(pystate->owner);
-                FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Press number to make word in frequent list"), pystate->strFindString);
+                FcitxMessagesAddMessageStringsAtLast(
+                    FcitxInputStateGetAuxUp(input), MSG_TIPS,
+                    _("Press number to make word in frequent list"));
                 FcitxInputStateSetShowCursor(input, false);
 
                 return IRV_DISPLAY_MESSAGE;
@@ -724,10 +753,11 @@ INPUT_RETURN_VALUE DoPYInput(void* arg, FcitxKeySym sym, unsigned int state)
                     return IRV_DO_NOTHING;
 
                 FcitxInstanceCleanInputWindowUp(pystate->owner);
-                if (val == 1)
+                if (val == 1) {
                     FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Press 1 to delete %s in frequent list (ESC for cancel)"), pystate->strFindString);
-                else
+                } else {
                     FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Press 1-%d to delete %s in frequent list (ESC for cancel)"), val, pystate->strFindString);
+                }
 
                 pystate->bIsPYDelFreq = true;
                 FcitxInputStateSetIsDoInputOnly(input, true);
@@ -881,8 +911,8 @@ INPUT_RETURN_VALUE PYGetCandWords(void* arg)
     FcitxMessagesSetMessageCount(msgPreedit, 0);
     FcitxMessagesSetMessageCount(msgClientPreedit, 0);
     if (pystate->iPYSelected) {
-        FcitxMessagesAddMessageAtLast(msgPreedit, MSG_OTHER, "");
-        FcitxMessagesAddMessageAtLast(msgClientPreedit, MSG_OTHER, "");
+        FcitxMessagesAddMessageStringsAtLast(msgPreedit, MSG_OTHER, "");
+        FcitxMessagesAddMessageStringsAtLast(msgClientPreedit, MSG_OTHER, "");
         for (i = 0; i < pystate->iPYSelected; i++) {
             FcitxMessagesMessageConcat(msgPreedit, FcitxMessagesGetMessageCount(msgPreedit) - 1, pystate->pySelected[i].strHZ);
             FcitxMessagesMessageConcat(msgClientPreedit, FcitxMessagesGetMessageCount(msgClientPreedit) - 1, pystate->pySelected[i].strHZ);
@@ -890,14 +920,16 @@ INPUT_RETURN_VALUE PYGetCandWords(void* arg)
     }
 
     for (i = 0; i < pystate->findMap.iHZCount; i++) {
-        FcitxMessagesAddMessageAtLast(msgPreedit, MSG_CODE, "%s", pystate->findMap.strPYParsed[i]);
+        FcitxMessagesAddMessageStringsAtLast(msgPreedit, MSG_CODE,
+                                             pystate->findMap.strPYParsed[i]);
         if (i < pystate->findMap.iHZCount - 1)
             FcitxMessagesMessageConcat(msgPreedit, FcitxMessagesGetMessageCount(msgPreedit) - 1, " ");
     }
 
     if (pystate->findMap.iMode == PARSE_ERROR) {
         for (i = 0; i < pystate->findMap.iHZCount; i++) {
-            FcitxMessagesAddMessageAtLast(msgClientPreedit, MSG_CODE, "%s", pystate->findMap.strPYParsed[i]);
+            FcitxMessagesAddMessageStringsAtLast(
+                msgClientPreedit, MSG_CODE, pystate->findMap.strPYParsed[i]);
         }
         char* errorAuto = FcitxUIMessagesToCString(msgClientPreedit);
         FcitxInstanceCleanInputWindowDown(pystate->owner);
@@ -946,8 +978,9 @@ INPUT_RETURN_VALUE PYGetCandWords(void* arg)
     PYGetBaseCandWords(pystate, pCurFreq);
 
     if (FcitxCandidateWordPageCount(candList) != 0) {
-        FcitxCandidateWord* candWord = FcitxCandidateWordGetCurrentWindow(candList);
-        FcitxMessagesAddMessageAtLast(msgClientPreedit, MSG_INPUT, "%s", candWord->strWord);
+        FcitxCandidateWord *candWord = FcitxCandidateWordGetCurrentWindow(candList);
+        FcitxMessagesAddMessageStringsAtLast(msgClientPreedit, MSG_INPUT,
+                                             candWord->strWord);
     }
 
     return IRV_DISPLAY_CANDWORDS;
@@ -1257,7 +1290,7 @@ void PYGetPhraseCandWords(FcitxPinyinState* pystate)
         return;
 
     UT_array candtemp;
-    utarray_init(&candtemp, &pycand_icd);
+    utarray_init(&candtemp, fcitx_ptr_icd);
 
     str[0] = pystate->findMap.strMap[0][0];
     str[1] = pystate->findMap.strMap[0][1];
@@ -1328,8 +1361,9 @@ void PYGetPhraseCandWords(FcitxPinyinState* pystate)
             candWord.wordType = MSG_OTHER;
         const char* pBase = PYFAList[(*pcand)->cand.phrase.iPYFA].pyBase[(*pcand)->cand.phrase.iBase].strHZ;
         const char* pPhrase = (*pcand)->cand.phrase.phrase->strPhrase;
-        asprintf(&candWord.strWord, "%s%s", pBase, pPhrase);
-        FcitxCandidateWordAppend(FcitxInputStateGetCandidateList(input), &candWord);
+        fcitx_utils_alloc_cat_str(candWord.strWord, pBase, pPhrase);
+        FcitxCandidateWordAppend(FcitxInputStateGetCandidateList(input),
+                                 &candWord);
     }
 
     utarray_done(&candtemp);
@@ -1371,7 +1405,7 @@ void PYGetBaseCandWords(FcitxPinyinState* pystate, PyFreq* pCurFreq)
     FcitxPinyinConfig* pyconfig = &pystate->pyconfig;
     FcitxInputState* input = FcitxInstanceGetInputState(pystate->owner);
     UT_array candtemp;
-    utarray_init(&candtemp, &pycand_icd);
+    utarray_init(&candtemp, fcitx_ptr_icd);
 
     str[0] = pystate->findMap.strMap[0][0];
     str[1] = pystate->findMap.strMap[0][1];
@@ -1429,7 +1463,7 @@ void PYGetFreqCandWords(FcitxPinyinState* pystate, PyFreq* pCurFreq)
     HZ *hz;
     UT_array candtemp;
     FcitxInputState* input = FcitxInstanceGetInputState(pystate->owner);
-    utarray_init(&candtemp, &pycand_icd);
+    utarray_init(&candtemp, fcitx_ptr_icd);
 
     if (pCurFreq) {
         hz = pCurFreq->HZList->next;
@@ -1481,7 +1515,7 @@ void PYAddFreqCandWord(PyFreq* pyFreq, HZ * hz, char *strPY, PYCandWord* pycandW
  * 将一个词组保存到用户词组库中
  * 返回true表示是新词组
  */
-boolean PYAddUserPhrase(FcitxPinyinState* pystate, char *phrase, char *map, boolean incHit)
+boolean PYAddUserPhrase(FcitxPinyinState* pystate, const char *phrase, const char *map, boolean incHit)
 {
     PyUsrPhrase *userPhrase, *newPhrase, *temp;
     char str[UTF8_MAX_LENGTH + 1];
@@ -1943,7 +1977,7 @@ _HIT:
         return IRV_TO_PROCESS;
 
     UT_array candtemp;
-    utarray_init(&candtemp, &pycand_icd);
+    utarray_init(&candtemp, fcitx_ptr_icd);
 
     for (i = 0; i < pyBaseForRemind->iPhrase; i++) {
 
@@ -1992,9 +2026,11 @@ _HIT:
         phrase = USER_PHRASE_NEXT(phrase);
     }
 
-    FcitxMessagesSetMessageCount(FcitxInputStateGetAuxUp(input), 0);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Remind: "));
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_INPUT, "%s", pystate->strPYRemindSource);
+    FcitxMessages *msg_up = FcitxInputStateGetAuxUp(input);
+    FcitxMessagesSetMessageCount(msg_up, 0);
+    FcitxMessagesAddMessageStringsAtLast(msg_up, MSG_TIPS, _("Remind: "));
+    FcitxMessagesAddMessageStringsAtLast(msg_up, MSG_INPUT,
+                                         pystate->strPYRemindSource);
 
     PYCandWord** pcand = NULL;
     for (pcand = (PYCandWord**) utarray_front(&candtemp);
@@ -2026,7 +2062,7 @@ void PYAddRemindCandWord(FcitxPinyinState* pystate, PyPhrase * phrase, PYCandWor
     pyRemindCandWords->iLength = strlen(pystate->strPYRemindSource) - fcitx_utf8_char_len(pystate->strPYRemindSource);
 }
 
-void PYGetPYByHZ(FcitxPinyinState*pystate, char *strHZ, char *strPY)
+void PYGetPYByHZ(FcitxPinyinState*pystate, const char *strHZ, char *strPY)
 {
     int i, j;
     char str_PY[MAX_PY_LENGTH + 1];
@@ -2057,55 +2093,6 @@ void SavePY(void *arg)
         SavePYFreq(pystate);
 }
 
-void* LoadPYBaseDictWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    if (!pystate->bPYBaseDictLoaded)
-        LoadPYBaseDict(pystate);
-    return NULL;
-}
-
-void* PYGetPYByHZWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    char *a = args.args[0];
-    char *b = args.args[1];
-    PYGetPYByHZ(pystate, a, b);
-    return NULL;
-
-}
-void* DoPYInputWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    FcitxKeySym *a = args.args[0];
-    unsigned int *b = args.args[1];
-    DoPYInput(pystate, *a, *b);
-    return NULL;
-
-}
-void* PYGetCandWordsWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    PYGetCandWords(arg);
-    return NULL;
-}
-
-void* PYGetFindStringWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    return pystate->strFindString;
-
-}
-void* PYResetWrapper(void * arg, FcitxModuleFunctionArg args)
-{
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-
-    pystate->bSP = false;
-    pystate->strPYAuto[0] = '\0';
-    ResetPYStatus(pystate);
-
-    return NULL;
-}
-
 void ReloadConfigPY(void* arg)
 {
     FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
@@ -2230,20 +2217,17 @@ int PYCandWordCmp(const void* b, const void *a, void* arg)
     return 0;
 }
 
-
-void* PYSP2QP(void* arg, FcitxModuleFunctionArg args)
+static char*
+PYSP2QP(FcitxPinyinState *pystate, const char* strSP)
 {
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    char* strSP = args.args[0];
-    char strQP[MAX_PY_LENGTH + 1];
-    strQP[0] = 0;
-
+    char strQP[MAX_PY_LENGTH + 1] = "";
     SP2QP(&pystate->pyconfig, strSP, strQP);
-
     return strdup(strQP);
 }
 
-boolean PYGetPYMapByHZ(FcitxPinyinState* pystate, char* strHZ, char* mapHint, char* strMap)
+static boolean
+PYGetPYMapByHZ(FcitxPinyinState* pystate, char* strHZ,
+               char* mapHint, char* strMap)
 {
     int i, j;
     PYFA* PYFAList = pystate->PYFAList;
@@ -2262,10 +2246,11 @@ boolean PYGetPYMapByHZ(FcitxPinyinState* pystate, char* strHZ, char* mapHint, ch
     return false;
 }
 
-void PYAddUserPhraseFromCString(void* arg, FcitxModuleFunctionArg args)
+static void
+PYAddUserPhraseFromCString(FcitxPinyinState *pystate, const char *strHZ)
 {
-    FcitxPinyinState *pystate = (FcitxPinyinState*)arg;
-    char* strHZ = args.args[0], *sp, *pivot;
+    const char *pivot;
+    char *sp;
     char singleHZ[UTF8_MAX_LENGTH + 1];
     char strMap[3];
     if (!fcitx_utf8_check_string(strHZ))
@@ -2276,19 +2261,20 @@ void PYAddUserPhraseFromCString(void* arg, FcitxModuleFunctionArg args)
     size_t hzCountLocal = 0;
 
     if (pystate->iPYSelected) {
-        int i = 0;
-        for (i = 0 ; i < pystate->iPYSelected; i ++)
+        int i;
+        for (i = 0 ; i < pystate->iPYSelected; i ++) {
             hzCountLocal += strlen(pystate->pySelected[i].strMap) / 2;
+        }
     }
     hzCountLocal += pystate->findMap.iHZCount;
 
     /* in order not to get a wrong one, use strict check */
     if (hzCountLocal != hzCount || hzCount > MAX_PY_PHRASE_LENGTH)
         return;
-    char *totalMap = fcitx_utils_malloc0(sizeof(char) * (1 + 2 * hzCount));
+    char *totalMap = fcitx_utils_malloc0(1 + 2 * hzCount);
 
     if (pystate->iPYSelected) {
-        int i = 0;
+        int i;
         for (i = 0 ; i < pystate->iPYSelected; i ++)
             strcat(totalMap, pystate->pySelected[i].strMap);
         strHZ = fcitx_utf8_get_nth_char(strHZ, strlen(totalMap) / 2);
@@ -2303,21 +2289,19 @@ void PYAddUserPhraseFromCString(void* arg, FcitxModuleFunctionArg args)
         strncpy(singleHZ, strHZ, len);
         singleHZ[len] = '\0';
 
-        if (!PYGetPYMapByHZ(pystate, singleHZ, pystate->findMap.strMap[i], strMap)) {
+        if (!PYGetPYMapByHZ(pystate, singleHZ, pystate->findMap.strMap[i],
+                            strMap)) {
             free(totalMap);
             return;
         }
 
         strncat(totalMap, strMap, 2);
-
         strHZ = sp;
         i ++;
     }
 
     PYAddUserPhrase(pystate, pivot, totalMap, true);
     free(totalMap);
-
-    return;
 }
 
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-pinyin-addfunctions.h"
