@@ -497,7 +497,8 @@ int Cmp2Map(FcitxPinyinConfig* pyconfig, char map1[3], char map2[3], boolean bSP
  * 否 返回值不为0
  * *iMatchedLength 记录了二者能够匹配的长度
  */
-int CmpMap(FcitxPinyinConfig* pyconfig, char *strMap1, char *strMap2, int *iMatchedLength, boolean bSP)
+int CmpMap(FcitxPinyinConfig* pyconfig, const char *strMap1,
+           const char *strMap2, int *iMatchedLength, boolean bSP)
 {
     int             val;
 
@@ -529,15 +530,15 @@ struct _PYMappedSplitData {
     UT_hash_handle hh;
 };
 
-const UT_icd splitData_icd = { sizeof(PYMappedSplitData), 0, 0, 0 };
-
 void InitPYSplitData(FcitxPinyinConfig* pyconfig)
 {
     size_t size = sizeof(pySplitData) / sizeof(pySplitData[0]);
-    int i = 0;
-    for (i = 0; i < size; i ++) {
-        PYMappedSplitData* data = fcitx_utils_malloc0(sizeof(PYMappedSplitData));
-        sprintf(data->py, "%s %s", pySplitData[i].py1, pySplitData[i].py2);
+    unsigned int i = 0;
+    for (i = 0;i < size;i++) {
+        PYMappedSplitData *data = fcitx_utils_new(PYMappedSplitData);
+        const char *strs[] = {pySplitData[i].py1, " ", pySplitData[i].py2};
+        fcitx_utils_cat_str_simple_with_len(data->py, MAX_PY_LENGTH * 2 + 2,
+                                            3, strs);
         data->freq = pySplitData[i].freq;
         HASH_ADD_STR(pyconfig->splitData, py, data);
     }
@@ -545,13 +546,11 @@ void InitPYSplitData(FcitxPinyinConfig* pyconfig)
 
 double LookupPYFreq(FcitxPinyinConfig* pyconfig, int index1, int index2)
 {
-    char py[MAX_PY_LENGTH * 2 + 2];
     if (index1 < 0 || index2 < 0)
         return 0;
-
-    sprintf(py, "%s %s", pyconfig->PYTable[index1].strPY, pyconfig->PYTable[index2].strPY);
-
-    PYMappedSplitData* s = NULL;
+    fcitx_utils_local_cat_str(py, 6 * 2 + 2, pyconfig->PYTable[index1].strPY,
+                              " ", pyconfig->PYTable[index2].strPY);
+    PYMappedSplitData *s = NULL;
     HASH_FIND_STR(pyconfig->splitData, py, s);
 
     if (s == NULL)
@@ -559,4 +558,13 @@ double LookupPYFreq(FcitxPinyinConfig* pyconfig, int index1, int index2)
     return s->freq;
 }
 
+void FreePYSplitData(FcitxPinyinConfig* pyconfig)
+{
+    while (pyconfig->splitData) {
+        PYMappedSplitData* data = pyconfig->splitData;
+        HASH_DEL(pyconfig->splitData, data);
+        free(data);
+    }
+}
+
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
