@@ -21,6 +21,15 @@
 #include "fcitx/fcitx.h"
 
 #include "tabledict.h"
+#include "table.h"
+
+CONFIG_BINDING_BEGIN(TableConfig)
+CONFIG_BINDING_REGISTER("Key", "AddPhrase", hkTableAddPhrase)
+CONFIG_BINDING_REGISTER("Key", "DeletePhrase", hkTableDelPhrase)
+CONFIG_BINDING_REGISTER("Key", "AdjustOrder", hkTableAdjustOrder)
+CONFIG_BINDING_REGISTER("Key", "ClearFreq", hkTableClearFreq)
+CONFIG_BINDING_REGISTER("Key", "LookupPinyin", hkLookupPinyin)
+CONFIG_BINDING_END()
 
 CONFIG_BINDING_BEGIN(TableMetaData)
 CONFIG_BINDING_REGISTER("CodeTable", "UniqueName", uniqueName)
@@ -28,6 +37,7 @@ CONFIG_BINDING_REGISTER("CodeTable", "Name", strName)
 CONFIG_BINDING_REGISTER("CodeTable", "IconName", strIconName)
 CONFIG_BINDING_REGISTER("CodeTable", "File", strPath)
 CONFIG_BINDING_REGISTER("CodeTable", "AdjustOrder", tableOrder)
+CONFIG_BINDING_REGISTER("CodeTable", "SimpleCodeOrderLevel", iSimpleLevel)
 CONFIG_BINDING_REGISTER("CodeTable", "Priority", iPriority)
 CONFIG_BINDING_REGISTER("CodeTable", "UsePY", bUsePY)
 CONFIG_BINDING_REGISTER("CodeTable", "PYKey", cPinyin)
@@ -48,6 +58,7 @@ CONFIG_BINDING_REGISTER("CodeTable", "PromptTableCode", bPromptTableCode)
 CONFIG_BINDING_REGISTER("CodeTable", "Symbol", strSymbol)
 CONFIG_BINDING_REGISTER("CodeTable", "SymbolFile", strSymbolFile)
 CONFIG_BINDING_REGISTER("CodeTable", "Choose", strChoose)
+CONFIG_BINDING_REGISTER("CodeTable", "ChooseModifier", chooseModifier)
 CONFIG_BINDING_REGISTER("CodeTable", "LangCode", langCode)
 CONFIG_BINDING_REGISTER("CodeTable", "KeyboardLayout", kbdlayout)
 CONFIG_BINDING_REGISTER("CodeTable", "UseCustomPrompt", customPrompt)
