@@ -89,7 +89,6 @@ FcitxConfigFile *FcitxConfigParseMultiConfigFile(char **filename, int len, Fcitx
     int i = 0;
 
     for (i = 0 ; i < len ; i++) {
-        fp[i] = NULL;
         fp[i] = fopen(filename[i], "r");
     }
 
@@ -248,31 +247,23 @@ FcitxConfigFileDesc *FcitxConfigParseConfigFileDescFp(FILE *fp)
         if (p == NULL)
             continue;
 
-        int groupNameLen = p - group->groupName;
+        unsigned int groupNameLen = p - group->groupName;
 
-        int optionNameLen = strlen(p + 1);
+        unsigned int optionNameLen = strlen(p + 1);
 
         if (groupNameLen == 0 || optionNameLen == 0)
             continue;
-
-        char *groupName = malloc(sizeof(char) * (groupNameLen + 1));
-
-        strncpy(groupName, group->groupName, groupNameLen);
-
-        groupName[groupNameLen] = '\0';
-
-        char *optionName = strdup(p + 1);
-
-        HASH_FIND_STR(cfdesc->groupsDesc, groupName, cgdesc);
-
+        HASH_FIND(hh, cfdesc->groupsDesc, group->groupName,
+                  groupNameLen, cgdesc);
         if (!cgdesc) {
-            cgdesc = fcitx_utils_malloc0(sizeof(FcitxConfigGroupDesc));
-            cgdesc->groupName = groupName;
+            cgdesc = fcitx_utils_new(FcitxConfigGroupDesc);
+            cgdesc->groupName = fcitx_utils_set_str_with_len(
+                NULL, group->groupName, groupNameLen);
             cgdesc->optionsDesc = NULL;
-            HASH_ADD_KEYPTR(hh, cfdesc->groupsDesc, cgdesc->groupName, groupNameLen, cgdesc);
-        } else
-            free(groupName);
-
+            HASH_ADD_KEYPTR(hh, cfdesc->groupsDesc, cgdesc->groupName,
+                            groupNameLen, cgdesc);
+        }
+        char *optionName = strdup(p + 1);
         FcitxConfigOptionDesc2 *codesc2 = fcitx_utils_new(FcitxConfigOptionDesc2);
         FcitxConfigOptionDesc *codesc = (FcitxConfigOptionDesc*) codesc2;
 
@@ -359,18 +350,13 @@ FcitxConfigFileDesc *FcitxConfigParseConfigFileDescFp(FILE *fp)
                     int ecount = atoi(eoption->rawValue);
 
                     if (ecount > 0) {
-                        char *enumname;
-                        /* trick, max length is enough */
-                        asprintf(&enumname, "Enum%d", ecount);
-
+                        char enumname[FCITX_INT_LEN + strlen("Enum") + 1];
+                        memcpy(enumname, "Enum", strlen("Enum"));
                         codesc->configEnum.enumDesc = malloc(sizeof(char*) * ecount);
-
                         codesc->configEnum.enumCount = ecount;
-
                         size_t nel = 0;
-
                         for (i = 0; i < ecount; i++) {
-                            sprintf(enumname, "Enum%d", i);
+                            sprintf(enumname + strlen("Enum"), "%d", i);
                             HASH_FIND_STR(options, enumname, eoption);
 
                             if (eoption) {
@@ -383,12 +369,9 @@ FcitxConfigFileDesc *FcitxConfigParseConfigFileDescFp(FILE *fp)
                                 codesc->configEnum.enumDesc[i] = strdup(eoption->rawValue);
                             } else {
                                 enumError = true;
-                                free(enumname);
                                 goto config_enum_final;
                             }
                         }
-
-                        free(enumname);
                     } else {
                         FcitxLog(WARNING, _("Enum option number must larger than 0"));
                         enumError = true;
@@ -674,22 +657,14 @@ FcitxConfigSyncResult FcitxConfigOptionChar(FcitxConfigOption *option, FcitxConf
         return SyncNoBinding;
 
     switch (sync) {
-
     case Raw2Value:
-
-        if (strlen(option->rawValue) == 0)
-            *option->value.chr = '\0';
-        else
-            *option->value.chr = option->rawValue[0];
-
+        *option->value.chr = *option->rawValue;
         return SyncSuccess;
 
     case Value2Raw:
-        if (option->rawValue)
-            free(option->rawValue);
-
-        asprintf(&option->rawValue, "%c", *option->value.chr);
-
+        option->rawValue = realloc(option->rawValue, 2);
+        option->rawValue[0] = *option->value.chr;
+        option->rawValue[1] = '\0';
         return SyncSuccess;
 
     case ValueFree:
@@ -728,13 +703,15 @@ FcitxConfigSyncResult FcitxConfigOptionHotkey(FcitxConfigOption *option, FcitxCo
         if (option->rawValue)
             free(option->rawValue);
 
-        if (option->value.hotkey[1].desc)
-            asprintf(&option->rawValue, "%s %s", option->value.hotkey[0].desc, option->value.hotkey[1].desc);
-        else if (option->value.hotkey[0].desc) {
+        if (option->value.hotkey[1].desc) {
+            fcitx_utils_alloc_cat_str(option->rawValue,
+                                      option->value.hotkey[0].desc,
+                                      " ", option->value.hotkey[1].desc);
+        } else if (option->value.hotkey[0].desc) {
             option->rawValue = strdup(option->value.hotkey[0].desc);
-        } else
+        } else {
             option->rawValue = strdup("");
-
+        }
         return SyncSuccess;
 
     case ValueFree:
@@ -874,24 +851,17 @@ FcitxConfigFile* FcitxConfigParseIni(char* filename, FcitxConfigFile* reuse)
 }
 
 FCITX_EXPORT_API
-FcitxConfigFile* FcitxConfigParseIniFp(FILE *fp, FcitxConfigFile* reuse)
+FcitxConfigFile* FcitxConfigParseIniFp(FILE *fp, FcitxConfigFile *cfile)
 {
     char *line = NULL, *buf = NULL;
     size_t len = 0;
     int lineLen = 0;
-    FcitxConfigFile* cfile;
 
-    if (!fp) {
-        if (reuse)
-            return reuse;
-        else
-            return NULL;
-    }
+    if (!fp)
+        return cfile;
 
-    if (reuse)
-        cfile = reuse;
-    else
-        cfile = fcitx_utils_malloc0(sizeof(FcitxConfigFile));
+    if (!cfile)
+        cfile = fcitx_utils_new(FcitxConfigFile);
 
     FcitxConfigGroup* curGroup = NULL;
 
@@ -915,25 +885,23 @@ FcitxConfigFile* FcitxConfigParseIniFp(FILE *fp, FcitxConfigFile* reuse)
                 return NULL;
             }
 
-            char *groupName;
-
-            groupName = malloc(sizeof(char) * (lineLen - 2 + 1));
-            strncpy(groupName, line + 1, lineLen - 2);
-            groupName[lineLen - 2] = '\0';
-            HASH_FIND_STR(cfile->groups, groupName, curGroup);
-
+            size_t grp_len = lineLen - 2;
+            HASH_FIND(hh, cfile->groups, line + 1, grp_len, curGroup);
             if (curGroup) {
-                FcitxLog(DEBUG, _("Duplicate group name, merge with the previous: %s :line %d"), groupName, lineNo);
-                free(groupName);
+                FcitxLog(DEBUG, _("Duplicate group name, "
+                                  "merge with the previous: %s :line %d"),
+                         curGroup->groupName, lineNo);
                 continue;
             }
 
+            char *groupName;
+            groupName = fcitx_utils_set_str_with_len(NULL, line + 1, grp_len);
             curGroup = fcitx_utils_malloc0(sizeof(FcitxConfigGroup));
-
             curGroup->groupName = groupName;
             curGroup->options = NULL;
             curGroup->groupDesc = NULL;
-            HASH_ADD_KEYPTR(hh, cfile->groups, curGroup->groupName, strlen(curGroup->groupName), curGroup);
+            HASH_ADD_KEYPTR(hh, cfile->groups, curGroup->groupName,
+                            grp_len, curGroup);
         } else {
             if (curGroup == NULL)
                 continue;
@@ -1155,11 +1123,9 @@ void FcitxConfigSyncValue(FcitxGenericConfig* config, FcitxConfigGroup* group, F
 
     if (r == SyncInvalid) {
         if (codesc->rawDefaultValue) {
-            FcitxLog(WARNING, _("Option %s is Invalid, Use Default Value %s"), option->optionName, codesc->rawDefaultValue);
-
-            if (option->rawValue)
-                free(option->rawValue);
-
+            FcitxLog(WARNING, _("Option %s is Invalid, Use Default Value %s"),
+                     option->optionName, codesc->rawDefaultValue);
+            fcitx_utils_free(option->rawValue);
             option->rawValue = strdup(codesc->rawDefaultValue);
 
             if (sync == Raw2Value)
@@ -1220,11 +1186,16 @@ boolean FcitxConfigSaveConfigFileFp(FILE* fp, FcitxGenericConfig *config, FcitxC
 
             if (!option) {
                 if (optiondesc->rawDefaultValue)
-                    fprintf(fp, "%s=%s\n", optiondesc->optionName, optiondesc->rawDefaultValue);
+                    fprintf(fp, "#%s=%s\n", optiondesc->optionName,
+                            optiondesc->rawDefaultValue);
                 else
-                    FcitxLog(FATAL, _("no default option for %s/%s"), groupdesc->groupName, optiondesc->optionName);
+                    FcitxLog(FATAL, _("no default option for %s/%s"),
+                             groupdesc->groupName, optiondesc->optionName);
             } else {
                 FcitxConfigSyncValue(config, group, option, Value2Raw);
+                /* comment out the default value, for future automatical change */
+                if (optiondesc->rawDefaultValue && strcmp(option->rawValue, optiondesc->rawDefaultValue) == 0)
+                    fprintf(fp, "#");
                 fprintf(fp, "%s=%s\n", option->optionName, option->rawValue);
                 HASH_FOREACH(subkey, option->subkey, FcitxConfigOptionSubkey) {
                     fprintf(fp, "%s[%s]=%s\n", option->optionName, subkey->subkeyName, subkey->rawValue);
@@ -1389,7 +1360,8 @@ void FcitxConfigResetConfigToDefaultValue(FcitxGenericConfig* config)
             }
 
             if (!codesc->rawDefaultValue) {
-                /* ignore it, actually if it doesn't have a default value, the reset is meaning less */
+                /* ignore it, actually the reset is meaningless
+                 * if it doesn't have a default value. */
                 continue;
             }
 
