@@ -51,6 +51,14 @@ extern "C"
      **/
     FILE *FcitxXDGGetLibFile(const char *filename, const char *mode, char **retFile);
 
+    /**
+     * Get library path
+     *
+     * @param len length
+     * @return char**
+     **/
+    char** FcitxXDGGetLibPath(size_t* len);
+
 #ifdef FCITX_CONFIG_XDG_DEPRECATED
     /**
      * get a xdg file pointer with given path, if mode contains "w", it will create necessary parent folder,
@@ -141,8 +149,21 @@ extern "C"
      *
      **/
     FILE *FcitxXDGGetFileUserWithPrefix(const char* prefix, const char *fileName, const char *mode, char **retFile);
+
+
     /**
-     * free xdg path return by FcitxXDGGetPath
+     * make a dir with prefix under user, no return value indicates it's successful created or not.
+     *
+     * @param prefix
+     * @return void
+     *
+     * @since 4.2.7
+     **/
+    void FcitxXDGMakeDirUser(const char* prefix);
+
+    /**
+     * free xdg path return by FcitxXDGGetPath, FcitxXDGGetLibPath, FcitxXDGGetPathUserWithPrefix,
+     * and FcitxXDGGetPathWithPrefix
      *
      * @param path path array
      * @return void
@@ -150,20 +171,25 @@ extern "C"
     void FcitxXDGFreePath(char **path);
 
     /**
-     * Get All files under directory with a suffix
+     * Get All files under directory with a suffix, the path usually be
+     * ~/.config/fcitx/path/prefix*suffix and
+     * [prefix]/share/fcitx/path/prefix*suffix
+     *
+     * It will only return the file name in form of path/prefix*suffix,
+     * since for file in fcitx, if user file and system file both exists,
+     * user file will override system file. This will make life easier when
+     * you want to load a bunch of sub-configuration, and want to have a
+     * unique set of file names.
      *
      * @param path xdg subpath
-     * @param suffix filename suffix
      * @param prefix filename prefix
+     * @param suffix filename suffix
      * @return StringHashSet*
      *
      * @since 4.2.0
      **/
-    FcitxStringHashSet* FcitxXDGGetFiles(
-        char* path,
-        char* suffix,
-        char* prefix
-    );
+    FcitxStringHashSet *FcitxXDGGetFiles(char *path, char *prefix,
+                                         char *suffix);
 
 #ifdef __cplusplus
 }
