@@ -25,17 +25,17 @@
 #define CONT(i)   ISUTF8_CB(in[i])
 #define VAL(i, s) ((in[i]&0x3f) << s)
 
-#define UTF8_LENGTH(Char)              \
-    ((Char) < 0x80 ? 1 :                 \
-     ((Char) < 0x800 ? 2 :               \
-      ((Char) < 0x10000 ? 3 :            \
-       ((Char) < 0x200000 ? 4 :          \
+#define UTF8_LENGTH(Char)                       \
+    ((Char) < 0x80 ? 1 :                        \
+     ((Char) < 0x800 ? 2 :                      \
+      ((Char) < 0x10000 ? 3 :                   \
+       ((Char) < 0x200000 ? 4 :                 \
         ((Char) < 0x4000000 ? 5 : 6)))))
 
-#define UNICODE_VALID(Char)                   \
-    ((Char) < 0x110000 &&                     \
-     (((Char) & 0xFFFFF800) != 0xD800) &&     \
-     ((Char) < 0xFDD0 || (Char) > 0xFDEF) &&  \
+#define UNICODE_VALID(Char)                     \
+    ((Char) < 0x110000 &&                       \
+     (((Char) & 0xFFFFF800) != 0xD800) &&       \
+     ((Char) < 0xFDD0 || (Char) > 0xFDEF) &&    \
      ((Char) & 0xFFFE) != 0xFFFE)
 
 FCITX_EXPORT_API
@@ -45,7 +45,7 @@ fcitx_utf8_strlen(const char *s)
     unsigned int l = 0;
 
     while (*s) {
-        int chr;
+        uint32_t chr;
 
         s = fcitx_utf8_get_char(s, &chr);
         l++;
@@ -84,9 +84,76 @@ int fcitx_utf8_char_len(const char *in)
 }
 
 FCITX_EXPORT_API
-char *
-fcitx_utf8_get_char(const char *in, int *chr)
+int fcitx_ucs4_char_len(uint32_t c)
 {
+    if (c < 0x00080) {
+        return 1;
+    } else if (c < 0x00800) {
+        return 2;
+    } else if (c < 0x10000) {
+        return 3;
+    } else if (c < 0x200000) {
+        return 4;
+        // below is not in UCS4 but in 32bit int.
+    } else if (c < 0x8000000) {
+        return 5;
+    } else {
+        return 6;
+    }
+}
+
+FCITX_EXPORT_API
+int fcitx_ucs4_to_utf8(uint32_t c, char* output)
+{
+    if (c < 0x00080) {
+        output[0] = (char) (c & 0xFF);
+        output[1] = '\0';
+        return 1;
+    } else if (c < 0x00800) {
+        output[0] = (char) (0xC0 + ((c >> 6) & 0x1F));
+        output[1] = (char) (0x80 + (c & 0x3F));
+        output[2] = '\0';
+        return 2;
+    } else if (c < 0x10000) {
+        output[0] = (char) (0xE0 + ((c >> 12) & 0x0F));
+        output[1] = (char) (0x80 + ((c >> 6) & 0x3F));
+        output[2] = (char) (0x80 + (c & 0x3F));
+        output[3] = '\0';
+        return 3;
+    } else if (c < 0x200000) {
+        output[0] = (char) (0xF0 + ((c >> 18) & 0x07));
+        output[1] = (char) (0x80 + ((c >> 12) & 0x3F));
+        output[2] = (char) (0x80 + ((c >> 6)  & 0x3F));
+        output[3] = (char) (0x80 + (c & 0x3F));
+        output[4] = '\0';
+        return 4;
+        // below is not in UCS4 but in 32bit int.
+    } else if (c < 0x8000000) {
+        output[0] = (char) (0xF8 + ((c >> 24) & 0x03));
+        output[1] = (char) (0x80 + ((c >> 18) & 0x3F));
+        output[2] = (char) (0x80 + ((c >> 12) & 0x3F));
+        output[3] = (char) (0x80 + ((c >> 6)  & 0x3F));
+        output[4] = (char) (0x80 + (c & 0x3F));
+        output[5] = '\0';
+        return 5;
+    } else {
+        output[0] = (char) (0xFC + ((c >> 30) & 0x01));
+        output[1] = (char) (0x80 + ((c >> 24) & 0x3F));
+        output[2] = (char) (0x80 + ((c >> 18) & 0x3F));
+        output[3] = (char) (0x80 + ((c >> 12) & 0x3F));
+        output[4] = (char) (0x80 + ((c >> 6)  & 0x3F));
+        output[5] = (char) (0x80 + (c & 0x3F));
+        output[6] = '\0';
+        return 6;
+    }
+    return 6;
+}
+
+FCITX_EXPORT_API
+char *
+fcitx_utf8_get_char(const char *i, uint32_t *chr)
+{
+    const unsigned char* in = (const unsigned char *)i;
     if (!(in[0] & 0x80)) {
         *(chr) = *(in);
         return (char *)in + 1;
@@ -131,7 +198,7 @@ FCITX_EXPORT_API
 int fcitx_utf8_strncmp(const char *s1, const char *s2, int n)
 {
     // Seems to work.
-    int c1, c2;
+    uint32_t c1, c2;
     int i;
 
     for (i = 0; i < n; i++) {
@@ -158,24 +225,24 @@ int fcitx_utf8_strncmp(const char *s1, const char *s2, int n)
 }
 
 FCITX_EXPORT_API
-char* fcitx_utf8_get_nth_char(char* s, unsigned int n)
+char* fcitx_utf8_get_nth_char(const char* s, uint32_t n)
 {
     unsigned int l = 0;
 
     while (*s && l < n) {
-        int chr;
+        uint32_t chr;
 
         s = fcitx_utf8_get_char(s, &chr);
         l++;
     }
 
-    return s;
+    return (char*)s;
 }
 
 FCITX_EXPORT_API
 int
 fcitx_utf8_get_char_extended(const char *s,
-                       int max_len)
+                             int max_len)
 {
     const unsigned char*p = (const unsigned char*)s;
     int i, len;
@@ -236,7 +303,7 @@ fcitx_utf8_get_char_extended(const char *s,
 
 FCITX_EXPORT_API
 int fcitx_utf8_get_char_validated(const char *p,
-                            int max_len)
+                                  int max_len)
 {
     int result;
 
@@ -257,7 +324,7 @@ FCITX_EXPORT_API
 int fcitx_utf8_check_string(const char *s)
 {
     while (*s) {
-        int chr;
+        uint32_t chr;
 
         if (fcitx_utf8_get_char_validated(s, 6) < 0)
             return 0;
@@ -272,7 +339,7 @@ FCITX_EXPORT_API
 void fcitx_utf8_strncpy(char* str, const char* s, size_t byte)
 {
     while (*s) {
-        int chr;
+        uint32_t chr;
 
         const char* next = fcitx_utf8_get_char(s, &chr);
         size_t diff = next - s;
@@ -291,13 +358,12 @@ void fcitx_utf8_strncpy(char* str, const char* s, size_t byte)
     }
 }
 
-
 FCITX_EXPORT_API
 size_t fcitx_utf8_strnlen(const char* str, size_t byte)
 {
     size_t len = 0;
     while (*str && byte > 0) {
-        int chr;
+        uint32_t chr;
 
         const char* next = fcitx_utf8_get_char(str, &chr);
         size_t diff = next - str;
@@ -312,4 +378,55 @@ size_t fcitx_utf8_strnlen(const char* str, size_t byte)
     return len;
 }
 
+FCITX_EXPORT_API
+char*
+fcitx_utils_get_ascii_partn(char *string, size_t len)
+{
+    if (!string)
+        return NULL;
+
+    char *s = string + len;
+    while ((--s) >= string && !(*s & 0x80)) {
+    }
+    return s + 1;
+}
+
+FCITX_EXPORT_API
+char*
+fcitx_utils_get_ascii_part(char *string)
+{
+    if (!string)
+        return NULL;
+    return fcitx_utils_get_ascii_partn(string, strlen(string));
+}
+
+static inline int
+is_valid_ascii(char c)
+{
+    return (!(c & 0x80)) && c;
+}
+
+FCITX_EXPORT_API
+char*
+fcitx_utils_get_ascii_endn(const char *string, size_t len)
+{
+    if (!string)
+        return NULL;
+    const char *end = string + len;
+    for (;string < end && is_valid_ascii(*string);string++) {
+    }
+    return (char*)string;
+}
+
+FCITX_EXPORT_API
+char*
+fcitx_utils_get_ascii_end(const char *string)
+{
+    if (!string)
+        return NULL;
+    for (;is_valid_ascii(*string);string++) {
+    }
+    return (char*)string;
+}
+
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
