@@ -35,22 +35,119 @@
 #include "instance-internal.h"
 #include "addon-internal.h"
 
-static const UT_icd frontend_icd = {sizeof(FcitxAddon*), NULL, NULL, NULL };
 static void FcitxInstanceCleanUpIC(FcitxInstance* instance);
+static void NewICData(FcitxInstance* instance, FcitxInputContext* ic);
+static void FreeICData(FcitxInstance* instance, FcitxInputContext* ic);
+static void FillICData(FcitxInstance* instance, FcitxInputContext* ic);
+
+void FillICData(FcitxInstance* instance, FcitxInputContext* ic)
+{
+    FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
+    unsigned int i = utarray_len(ic2->data);
+    for (;i < utarray_len(&instance->icdata);i++) {
+        FcitxICDataInfo *info = (FcitxICDataInfo*)utarray_eltptr(&instance->icdata, i);
+        void *data = NULL;
+        if (info->allocCallback)
+            data = info->allocCallback(info->arg);
+        utarray_push_back(ic2->data, &data);
+    }
+}
+
+
+void NewICData(FcitxInstance* instance, FcitxInputContext* ic)
+{
+    FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
+    utarray_new(ic2->data, fcitx_ptr_icd);
+    FillICData(instance, ic);
+}
+
+void FreeICData(FcitxInstance* instance, FcitxInputContext* ic)
+{
+    FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
+    unsigned int i = 0;
+    for (;i < utarray_len(ic2->data);i++) {
+        void** data = (void**)utarray_eltptr(ic2->data, i);
+        FcitxICDataInfo* info = (FcitxICDataInfo*)utarray_eltptr(&instance->icdata, i);
+        if (info->freeCallback) {
+            info->freeCallback(info->arg, *data);
+        }
+    }
+    utarray_free(ic2->data);
+}
+
+FCITX_EXPORT_API void*
+FcitxInstanceGetICData(FcitxInstance *instance, FcitxInputContext *ic,
+                       int icdataid)
+{
+    FCITX_UNUSED(instance);
+    if (!ic)
+        return NULL;
+
+    FcitxInputContext2* ic2 = (FcitxInputContext2*)ic;
+    void **data = fcitx_array_eltptr(ic2->data, icdataid);
+    if (!data)
+        return NULL;
+    return *data;
+}
+
+static
+void FcitxInstanceSetICDataInternal(struct _FcitxInstance* instance, FcitxInputContext* ic, int icdataid, void* newdata, boolean copy)
+{
+    FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
+    FcitxICDataInfo *info = fcitx_array_eltptr(&instance->icdata, icdataid);
+    void **data = fcitx_array_eltptr(ic2->data, icdataid);
+    if (!data || !info)
+        return;
+    if (copy) {
+        if (info->copyCallback) {
+            *data = info->copyCallback(info->arg, *data, newdata);
+        }
+    }
+    else
+        *data = newdata;
+}
+
+FCITX_EXPORT_API
+void FcitxInstanceSetICData(struct _FcitxInstance* instance, FcitxInputContext* ic, int icdataid, void* newdata)
+{
+    if (!ic)
+        return;
+    switch (instance->config->shareState) {
+    case ShareState_All:
+    case ShareState_PerProgram: {
+        FcitxInputContext *rec = instance->ic_list;
+        while (rec != NULL) {
+            boolean flag = false;
+            if (instance->config->shareState == ShareState_All)
+                flag = true;
+            else {
+                flag = FcitxInstanceCheckICFromSameApplication(instance, rec, ic);
+            }
+
+            if (flag)
+                FcitxInstanceSetICDataInternal(instance, rec, icdataid, newdata, (rec != ic));
+            rec = rec->next;
+        }
+    }
+    break;
+    case ShareState_None:
+        FcitxInstanceSetICDataInternal(instance, ic, icdataid, newdata, false);
+        break;
+    }
+}
 
 FCITX_EXPORT_API
 void FcitxFrontendsInit(UT_array* frontends)
 {
-    utarray_init(frontends, &frontend_icd);
+    utarray_init(frontends, fcitx_ptr_icd);
 }
 
-FCITX_EXPORT_API
-FcitxInputContext* FcitxInstanceCreateIC(FcitxInstance* instance, int frontendid, void * priv)
+FCITX_EXPORT_API FcitxInputContext*
+FcitxInstanceCreateIC(FcitxInstance* instance, int frontendid, void * priv)
 {
     /* clean up invalid ic here */
     FcitxInstanceCleanUpIC(instance);
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, frontendid);
     if (pfrontend == NULL)
         return NULL;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -66,6 +163,8 @@ FcitxInputContext* FcitxInstanceCreateIC(FcitxInstance* instance, int frontendid
     rec->frontendid = frontendid;
     rec->offset_x = -1;
     rec->offset_y = -1;
+
+    NewICData(instance, rec);
     switch (instance->config->shareState) {
     case ShareState_All:
         rec->state = instance->globalState;
@@ -85,14 +184,35 @@ FcitxInputContext* FcitxInstanceCreateIC(FcitxInstance* instance, int frontendid
     return rec;
 }
 
+FCITX_EXPORT_API
+int FcitxInstanceAllocDataForIC(FcitxInstance* instance,
+                                FcitxICDataAllocCallback allocCallback,
+                                FcitxICDataCopyCallback copyCallback,
+                                FcitxICDataFreeCallback freeCallback, void* arg)
+{
+    FcitxICDataInfo info;
+    info.allocCallback = allocCallback;
+    info.copyCallback = copyCallback;
+    info.freeCallback = freeCallback;
+    info.arg = arg;
+
+    utarray_push_back(&instance->icdata, &info);
+    FcitxInputContext *rec = instance->ic_list;
+    while (rec) {
+        FillICData(instance, rec);
+        rec = rec->next;
+    }
+    return utarray_len(&instance->icdata) - 1;
+}
+
 void FcitxInstanceCleanUpIC(FcitxInstance* instance)
 {
-    UT_array* frontends = &instance->frontends;
     FcitxInputContext *rec = instance->ic_list, *last = NULL, *todel;
 
     while (rec) {
-        FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, rec->frontendid);
-        FcitxFrontend* frontend = (*pfrontend)->frontend;
+        FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance,
+                                                           rec->frontendid);
+        FcitxFrontend *frontend = (*pfrontend)->frontend;
         pid_t pid = 0;
         if (frontend->GetPid)
             pid = frontend->GetPid((*pfrontend)->addonInstance, rec);
@@ -106,6 +226,7 @@ void FcitxInstanceCleanUpIC(FcitxInstance* instance)
             todel->next = instance->free_list;
             instance->free_list = todel;
             frontend->DestroyIC((*pfrontend)->addonInstance, todel);
+            FreeICData(instance, todel);
             if (todel == instance->CurrentIC) {
                 instance->CurrentIC = NULL;
                 FcitxUICloseInputWindow(instance);
@@ -123,8 +244,7 @@ void FcitxInstanceCleanUpIC(FcitxInstance* instance)
 FCITX_EXPORT_API
 FcitxInputContext* FcitxInstanceFindIC(FcitxInstance* instance, int frontendid, void *filter)
 {
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, frontendid);
     if (pfrontend == NULL)
         return NULL;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -140,8 +260,7 @@ FcitxInputContext* FcitxInstanceFindIC(FcitxInstance* instance, int frontendid,
 FCITX_EXPORT_API
 void FcitxInstanceSetICStateFromSameApplication(FcitxInstance* instance, int frontendid, FcitxInputContext *ic)
 {
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -160,9 +279,8 @@ void FcitxInstanceSetICStateFromSameApplication(FcitxInstance* instance, int fro
 FCITX_EXPORT_API
 void FcitxInstanceDestroyIC(FcitxInstance* instance, int frontendid, void* filter)
 {
-    FcitxInputContext             *rec, *last;
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, frontendid);
+    FcitxInputContext *rec, *last;
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -185,6 +303,7 @@ void FcitxInstanceDestroyIC(FcitxInstance* instance, int frontendid, void* filte
                 FcitxInstanceSetCurrentIC(instance, NULL);
             }
 
+            FreeICData(instance, rec);
             frontend->DestroyIC((*pfrontend)->addonInstance, rec);
             return;
         }
@@ -232,13 +351,11 @@ void FcitxInstanceCommitString(FcitxInstance* instance, FcitxInputContext* ic, c
     if (ic == NULL)
         return;
 
-    UT_array* frontends = &instance->frontends;
-
     char *pstr = FcitxInstanceProcessCommitFilter(instance, str);
     if (pstr != NULL)
         str = pstr;
 
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -261,9 +378,7 @@ boolean FcitxInstanceGetSurroundingText(FcitxInstance* instance, FcitxInputConte
 
     if (!(ic->contextCaps & CAPACITY_SURROUNDING_TEXT))
         return false;
-
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return false;
 
@@ -279,12 +394,10 @@ void FcitxInstanceDeleteSurroundingText(FcitxInstance* instance, FcitxInputConte
 {
     if (ic == NULL)
         return;
-
-    UT_array* frontends = &instance->frontends;
-
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
+
     FcitxFrontend* frontend = (*pfrontend)->frontend;
     if (frontend->DeleteSurroundingText) {
         frontend->DeleteSurroundingText((*pfrontend)->addonInstance, ic, offset, size);
@@ -302,10 +415,7 @@ void FcitxInstanceUpdatePreedit(FcitxInstance* instance, FcitxInputContext* ic)
 
     if (!(ic->contextCaps & CAPACITY_PREEDIT))
         return;
-
-    UT_array* frontends = &instance->frontends;
-
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -320,10 +430,7 @@ void FcitxInstanceUpdateClientSideUI(FcitxInstance* instance, FcitxInputContext*
 
     if (!(ic->contextCaps & CAPACITY_CLIENT_SIDE_UI))
         return;
-
-    UT_array* frontends = &instance->frontends;
-
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -334,8 +441,7 @@ void FcitxInstanceUpdateClientSideUI(FcitxInstance* instance, FcitxInputContext*
 FCITX_EXPORT_API
 void FcitxInstanceSetWindowOffset(FcitxInstance* instance, FcitxInputContext *ic, int x, int y)
 {
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -349,14 +455,14 @@ void FcitxInstanceGetWindowPosition(FcitxInstance* instance, FcitxInputContext*
     if (ic == NULL)
         return;
 
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
     int rx, ry, rw, rh;
     if (frontend->GetWindowRect) {
-        frontend->GetWindowRect((*pfrontend)->addonInstance, ic, &rx, &ry, &rw, &rh);
+        frontend->GetWindowRect((*pfrontend)->addonInstance, ic,
+                                &rx, &ry, &rw, &rh);
         *x = rx;
         *y = ry + rh;
     }
@@ -368,8 +474,7 @@ void FcitxInstanceGetWindowRect(FcitxInstance* instance, FcitxInputContext* ic,
     if (ic == NULL)
         return;
 
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -399,19 +504,19 @@ boolean FcitxInstanceLoadFrontend(FcitxInstance* instance)
                 if (!fp)
                     break;
                 fclose(fp);
-                handle = dlopen(modulePath, RTLD_LAZY | RTLD_GLOBAL);
+                handle = dlopen(modulePath, RTLD_NOW | RTLD_GLOBAL);
                 if (!handle) {
                     FcitxLog(ERROR, _("Frontend: open %s fail %s") , modulePath , dlerror());
                     break;
                 }
 
-                if (!CheckABIVersion(handle)) {
+                if (!FcitxCheckABIVersion(handle, addon->name)) {
                     FcitxLog(ERROR, "%s ABI Version Error", addon->name);
                     dlclose(handle);
                     break;
                 }
 
-                frontend = dlsym(handle, "frontend");
+                frontend = FcitxGetSymbol(handle, addon->name, "frontend");
                 if (!frontend || !frontend->Create) {
                     FcitxLog(ERROR, _("Frontend: bad frontend"));
                     dlclose(handle);
@@ -421,6 +526,8 @@ boolean FcitxInstanceLoadFrontend(FcitxInstance* instance)
                     dlclose(handle);
                     break;
                 }
+                if (instance->loadingFatalError)
+                    return false;
                 addon->frontend = frontend;
                 frontendindex ++;
                 utarray_push_back(frontends, &addon);
@@ -440,4 +547,13 @@ boolean FcitxInstanceLoadFrontend(FcitxInstance* instance)
     return true;
 }
 
+FCITX_EXPORT_API
+boolean FcitxInstanceICSupportPreedit(FcitxInstance* instance, FcitxInputContext* ic)
+{
+    if (!ic || ((ic->contextCaps & CAPACITY_PREEDIT) == 0 || !instance->profile->bUsePreedit))
+        return false;
+    return true;
+}
+
+
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
