@@ -30,6 +30,7 @@
 
 #include "fcitx-config/hotkey.h"
 #include "ime.h"
+#include "frontend.h"
 #include "fcitx-utils/utarray.h"
 
 struct _FcitxInputContext;
@@ -123,7 +124,11 @@ FcitxConfigFileDesc* GetIMConfigDesc();
 
 int IMPriorityCmp(const void *a, const void *b);
 
-void FcitxInstanceUpdateCurrentIM(struct _FcitxInstance* instance, boolean force);
+boolean FcitxInstanceUpdateCurrentIM(struct _FcitxInstance* instance, boolean force);
+
+void HideInputSpeed(void* arg);
+
+boolean FcitxInstanceCheckICFromSameApplication (struct  _FcitxInstance* instance, FcitxInputContext* rec, FcitxInputContext* ic);
 
 #endif
 
