@@ -112,8 +112,7 @@ static const FcitxHotkey* imSWPrevKey2[] = {
 };
 
 
-static const UT_icd ime_icd = {sizeof(FcitxIM), NULL , NULL, NULL};
-static const UT_icd imclass_icd = {sizeof(FcitxAddon*), NULL , NULL, NULL};
+static const UT_icd ime_icd = { sizeof(FcitxIM), NULL , NULL, NULL };
 static boolean IMMenuAction(FcitxUIMenu* menu, int index);
 static void UpdateIMMenuItem(FcitxUIMenu *menu);
 static void FcitxInstanceEnableIMInternal(FcitxInstance* instance, FcitxInputContext* ic, boolean keepState);
@@ -188,7 +187,7 @@ int IMPriorityCmp(const void *a, const void *b)
 void FcitxInstanceInitBuiltInHotkey(FcitxInstance *instance)
 {
     FcitxHotkeyHook hk;
-    hk.hotkey = FCITX_CTRL_5;
+    hk.hotkey = instance->config->hkReloadConfig;
     hk.hotkeyhandle = ImProcessReload;
     hk.arg = instance;
     FcitxInstanceRegisterHotkeyFilter(instance, hk);
@@ -223,7 +222,7 @@ void FcitxInstanceInitIM(FcitxInstance* instance)
 {
     utarray_init(&instance->imes, &ime_icd);
     utarray_init(&instance->availimes, &ime_icd);
-    utarray_init(&instance->imeclasses, &imclass_icd);
+    utarray_init(&instance->imeclasses, fcitx_ptr_icd);
 }
 
 FCITX_EXPORT_API
@@ -231,11 +230,12 @@ void FcitxInstanceSaveAllIM(FcitxInstance* instance)
 {
     UT_array* imes = &instance->availimes;
     FcitxIM *pim;
-    for (pim = (FcitxIM *) utarray_front(imes);
-            pim != NULL;
-            pim = (FcitxIM *) utarray_next(imes, pim)) {
-        if ((pim)->Save)
-            (pim)->Save(pim->klass);
+    for (pim = (FcitxIM*)utarray_front(imes);
+         pim != NULL;
+         pim = (FcitxIM*)utarray_next(imes, pim)) {
+        if (pim->Save) {
+            pim->Save(pim->klass);
+        }
     }
 }
 
@@ -248,13 +248,14 @@ boolean FcitxInstanceCheckICFromSameApplication (FcitxInstance* instance, FcitxI
     FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
     if (rec2->imname || ic2->imname)
         return false;
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend) {
-        FcitxFrontend* frontend = (*pfrontend)->frontend;
-        if (frontend->CheckICFromSameApplication
-            && frontend->CheckICFromSameApplication((*pfrontend)->addonInstance, rec, ic))
+        FcitxFrontend *frontend = (*pfrontend)->frontend;
+        if (frontend->CheckICFromSameApplication &&
+            frontend->CheckICFromSameApplication((*pfrontend)->addonInstance,
+                                                 rec, ic)) {
             return true;
+        }
     }
 
     return false;
@@ -417,9 +418,12 @@ void FcitxInstanceRegisterIMv2(FcitxInstance *instance,
     if (entry == NULL)
         return;
 
-    entry->uniqueName = strdup(uniqueName);
-    entry->strName = strdup(name);
-    entry->strIconName = strdup(iconName);
+    if (!entry->uniqueName)
+        entry->uniqueName = strdup(uniqueName);
+    if (!entry->strName)
+        entry->strName = strdup(name);
+    if (!entry->strIconName)
+        entry->strIconName = strdup(iconName);
     entry->Init = iface.Init;
     entry->ResetIM = iface.ResetIM;
     entry->DoInput = iface.DoInput;
@@ -430,6 +434,7 @@ void FcitxInstanceRegisterIMv2(FcitxInstance *instance,
     entry->KeyBlocker = iface.KeyBlocker;
     entry->UpdateSurroundingText = iface.UpdateSurroundingText;
     entry->DoReleaseInput = iface.DoReleaseInput;
+    entry->OnClose = iface.OnClose;
     entry->klass = imclass;
     entry->iPriority = priority;
     if (langCode)
@@ -603,10 +608,15 @@ INPUT_RETURN_VALUE FcitxInstanceProcessKey(
                     }
                 } else if ((FcitxHotkeyIsHotKey(sym, state, switchKey1[fc->iSwitchKey]) || FcitxHotkeyIsHotKey(sym, state, switchKey2[fc->iSwitchKey])) && input->keyReleased == KR_SWITCH && !fc->bDoubleSwitchKey) {
                     retVal = IRV_DONOT_PROCESS;
-                    if (fc->bSendTextWhenSwitchEng) {
-                        if (input->iCodeInputCount != 0) {
-                            strcpy(FcitxInputStateGetOutputString(input), FcitxInputStateGetRawInputBuffer(input));
-                            retVal = IRV_ENG;
+                    if (currentIM && currentIM->OnClose) {
+                        currentIM->OnClose(currentIM->klass, CET_ChangeByInactivate);
+                    }
+                    else {
+                        if (fc->bSendTextWhenSwitchEng) {
+                            if (input->iCodeInputCount != 0) {
+                                strcpy(FcitxInputStateGetOutputString(input), FcitxInputStateGetRawInputBuffer(input));
+                                retVal = IRV_ENG;
+                            }
                         }
                     }
                     input->keyReleased = KR_OTHER;
@@ -619,9 +629,8 @@ INPUT_RETURN_VALUE FcitxInstanceProcessKey(
 
 #if 0
     /* Added by hubert_star AT forum.ubuntu.com.cn */
-    if (event == FCITX_RELEASE_KEY
-            && IsHotKeySimple(sym, state)
-            && retVal == IRV_TO_PROCESS)
+    if (event == FCITX_RELEASE_KEY && IsHotKeySimple(sym, state) &&
+        retVal == IRV_TO_PROCESS)
         return IRV_DO_NOTHING;
 #endif
 
@@ -640,12 +649,14 @@ INPUT_RETURN_VALUE FcitxInstanceProcessKey(
                     return IRV_DO_NOTHING;
                 }
             }
-            if (!(FcitxHotkeyIsHotKey(sym, state, switchKey1[fc->iSwitchKey]) || FcitxHotkeyIsHotKey(sym, state, switchKey2[fc->iSwitchKey])))
+            if (!(FcitxHotkeyIsHotKey(sym, state, switchKey1[fc->iSwitchKey]) ||
+                  FcitxHotkeyIsHotKey(sym, state, switchKey2[fc->iSwitchKey])))
                 input->keyReleased = KR_OTHER;
             else {
-                if ((input->keyReleased == KR_SWITCH)
-                        && (timestamp - input->lastKeyPressedTime < fc->iTimeInterval)
-                        && fc->bDoubleSwitchKey) {
+                if (input->keyReleased == KR_SWITCH &&
+                    (timestamp - input->lastKeyPressedTime <
+                     (unsigned)fc->iTimeInterval) &&
+                    fc->bDoubleSwitchKey) {
                     FcitxInstanceCommitString(instance, instance->CurrentIC, FcitxInputStateGetRawInputBuffer(input));
                     FcitxInstanceChangeIMStateWithKey(instance, instance->CurrentIC, true);
                 }
@@ -698,10 +709,10 @@ INPUT_RETURN_VALUE FcitxInstanceProcessKey(
             retVal = IRV_DONOT_PROCESS;
     }
 
-    /* the key processed before this phase is very important, we don't let any interrupt */
-    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE
-        && retVal == IRV_TO_PROCESS
-       ) {
+    /* the key processed before this phase is very important,
+     * we don't let any interrupt */
+    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE &&
+        retVal == IRV_TO_PROCESS) {
         if (!input->bIsDoInputOnly) {
             FcitxInstanceProcessPreInputFilter(instance, sym, state, &retVal);
         }
@@ -722,13 +733,8 @@ INPUT_RETURN_VALUE FcitxInstanceProcessKey(
         }
     }
 
-    return FcitxInstanceDoInputCallback(
-                instance,
-                retVal,
-                event,
-                timestamp,
-                sym,
-                state);
+    return FcitxInstanceDoInputCallback(instance, retVal, event, timestamp,
+                                        sym, state);
 }
 
 
@@ -763,12 +769,15 @@ INPUT_RETURN_VALUE FcitxInstanceDoInputCallback(
     FcitxKeySym sym,
     unsigned int state)
 {
+    FCITX_UNUSED(event);
+    FCITX_UNUSED(timestamp);
     FcitxIM* currentIM = FcitxInstanceGetCurrentIM(instance);
     FcitxInputState *input = instance->input;
 
     FcitxGlobalConfig *fc = instance->config;
 
-    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE && currentIM && (retVal & IRV_FLAG_UPDATE_CANDIDATE_WORDS)) {
+    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE && currentIM &&
+        (retVal & IRV_FLAG_UPDATE_CANDIDATE_WORDS)) {
         if (currentIM->GetCandWords) {
             FcitxInstanceCleanInputWindow(instance);
             retVal = currentIM->GetCandWords(currentIM->klass);
@@ -780,25 +789,21 @@ INPUT_RETURN_VALUE FcitxInstanceDoInputCallback(
      * since all candidate word are cached in candList, so we don't need to trigger
      * GetCandWords after go for another page, simply update input window is ok.
      */
-    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE && !input->bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
-        const FcitxHotkey* hkPrevPage = FcitxInstanceGetContextHotkey(instance, CONTEXT_ALTERNATIVE_PREVPAGE_KEY);
-        if (hkPrevPage == NULL)
-            hkPrevPage = fc->hkPrevPage;
-
-        const FcitxHotkey* hkNextPage = FcitxInstanceGetContextHotkey(instance, CONTEXT_ALTERNATIVE_NEXTPAGE_KEY);
-        if (hkNextPage == NULL)
-            hkNextPage = fc->hkNextPage;
-
-        if (FcitxHotkeyIsHotKey(sym, state, hkPrevPage)) {
+    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE &&
+        !input->bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
+        if (FcitxHotkeyIsHotKey(sym, state,
+                                FcitxConfigPrevPageKey(instance, fc))) {
             if (FcitxCandidateWordGoPrevPage(input->candList))
                 retVal = IRV_DISPLAY_CANDWORDS;
-        } else if (FcitxHotkeyIsHotKey(sym, state, hkNextPage)) {
+        } else if (FcitxHotkeyIsHotKey(sym, state,
+                                       FcitxConfigNextPageKey(instance, fc))) {
             if (FcitxCandidateWordGoNextPage(input->candList))
                 retVal = IRV_DISPLAY_CANDWORDS;
         }
     }
 
-    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE && !input->bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
+    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE &&
+        !input->bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
         FcitxInstanceProcessPostInputFilter(instance, sym, state, &retVal);
 
         if (retVal == IRV_TO_PROCESS) {
@@ -849,8 +854,11 @@ void FcitxInstanceProcessInputReturnValue(
 
     if (retVal & IRV_FLAG_DISPLAY_LAST) {
         FcitxInstanceCleanInputWindow(instance);
-        FcitxMessagesAddMessageAtLast(input->msgAuxUp, MSG_INPUT, "%c", FcitxInputStateGetRawInputBuffer(input)[0]);
-        FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_TIPS, "%s", FcitxInputStateGetLastCommitString(input));
+        char str[2] = {FcitxInputStateGetRawInputBuffer(input)[0], '\0'};
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxUp, MSG_INPUT, str);
+        FcitxMessagesAddMessageStringsAtLast(
+            input->msgAuxDown, MSG_TIPS,
+            FcitxInputStateGetLastCommitString(input));
     }
 
     if (retVal & IRV_FLAG_UPDATE_INPUT_WINDOW)
@@ -862,11 +870,10 @@ void FcitxInstanceForwardKey(FcitxInstance* instance, FcitxInputContext *ic, Fci
 {
     if (ic == NULL)
         return;
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
-    FcitxFrontend* frontend = (*pfrontend)->frontend;
+    FcitxFrontend *frontend = (*pfrontend)->frontend;
     frontend->ForwardKey((*pfrontend)->addonInstance, ic, event, sym, state);
 }
 
@@ -892,7 +899,7 @@ FCITX_EXPORT_API
 void FcitxInstanceSwitchIMByIndex(FcitxInstance* instance, int index)
 {
     UT_array* imes = &instance->imes;
-    int iIMCount = utarray_len(imes);
+    const int iIMCount = utarray_len(imes);
     /* less than -2, invalid, set to zero
      * -2 scroll back
      * -1 scroll forward
@@ -938,14 +945,16 @@ void FcitxInstanceUnregisterIM(FcitxInstance* instance, const char* name)
     if (!im)
         return;
 
-    int index = utarray_eltidx(&instance->availimes, im);
+    unsigned int index = utarray_eltidx(&instance->availimes, im);
     utarray_erase(&instance->availimes, index, 1);
 }
 
-void FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index, boolean skipZero, boolean updateGlobal)
+void
+FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index,
+                              boolean skipZero, boolean updateGlobal)
 {
     UT_array* imes = &instance->imes;
-    int iIMCount = utarray_len(imes);
+    const int iIMCount = utarray_len(imes);
 
     FcitxInstanceCleanInputWindow(instance);
     FcitxInstanceResetInput(instance);
@@ -954,11 +963,7 @@ void FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index, boolean s
     FcitxIM* lastIM, *newIM;
 
     /* set lastIM */
-    if (instance->iIMIndex >= iIMCount || instance->iIMIndex < 0)
-        lastIM = NULL;
-    else {
-        lastIM = (FcitxIM*) utarray_eltptr(imes, instance->iIMIndex);
-    }
+    lastIM = fcitx_array_eltptr(imes, instance->iIMIndex);
 
     /* update instance->iIMIndex start */
     if (index >= iIMCount)
@@ -977,11 +982,7 @@ void FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index, boolean s
     /* update instance->iIMIndex end */
 
     /* set newIM */
-    if (instance->iIMIndex >= iIMCount || instance->iIMIndex < 0)
-        newIM = NULL;
-    else {
-        newIM = (FcitxIM*) utarray_eltptr(imes, instance->iIMIndex);
-    }
+    newIM = fcitx_array_eltptr(imes, instance->iIMIndex);
 
     if (lastIM && lastIM->Save)
         lastIM->Save(lastIM->klass);
@@ -993,13 +994,13 @@ void FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index, boolean s
         FcitxIM* im = FcitxInstanceGetIMFromIMList(instance, IMAS_Disable, name);
         if (!im->initialized) {
             im->initialized = true;
-            int index = utarray_eltidx(&instance->availimes, im);
+            unsigned int index = utarray_eltidx(&instance->availimes, im);
             utarray_erase(&instance->availimes, index, 1);
         }
 
         FcitxInstanceUpdateIMList(instance);
         instance->iIMIndex = FcitxInstanceGetIMIndexByName(instance, name);
-        newIM = (FcitxIM*) utarray_eltptr(imes, instance->iIMIndex);
+        newIM = fcitx_array_eltptr(imes, instance->iIMIndex);
         free(name);
     }
 
@@ -1018,6 +1019,7 @@ void FcitxInstanceSwitchIMInternal(FcitxInstance* instance, int index, boolean s
     }
 
     FcitxInstanceResetInput(instance);
+    FcitxInstanceProcessIMChangedHook(instance);
 }
 
 /**
@@ -1039,7 +1041,7 @@ void FcitxInstanceResetInput(FcitxInstance* instance)
 
     UT_array* ims = &instance->imes;
 
-    FcitxIM* currentIM = (FcitxIM*) utarray_eltptr(ims, instance->iIMIndex);
+    FcitxIM *currentIM = fcitx_array_eltptr(ims, instance->iIMIndex);
 
     if (currentIM && currentIM->ResetIM)
         currentIM->ResetIM(currentIM->klass);
@@ -1049,8 +1051,8 @@ void FcitxInstanceResetInput(FcitxInstance* instance)
 
 void FcitxInstanceDoPhraseTips(FcitxInstance* instance)
 {
-    UT_array* ims = &instance->imes;
-    FcitxIM* currentIM = (FcitxIM*) utarray_eltptr(ims, instance->iIMIndex);
+    UT_array *ims = &instance->imes;
+    FcitxIM *currentIM = fcitx_array_eltptr(ims, instance->iIMIndex);
     FcitxInputState *input = instance->input;
 
     if (currentIM->PhraseTips && currentIM->PhraseTips(currentIM->klass))
@@ -1110,6 +1112,59 @@ INPUT_RETURN_VALUE ImProcessReload(void *arg)
     return IRV_DO_NOTHING;
 }
 
+FCITX_EXPORT_API
+void FcitxInstanceReloadAddonConfig(FcitxInstance *instance, const char* addonname)
+{
+    if (!addonname)
+        return;
+
+    if (strcmp(addonname, "global") == 0) {
+        if (!FcitxGlobalConfigLoad(instance->config))
+            FcitxInstanceEnd(instance);
+
+        FcitxCandidateWordSetPageSize(instance->input->candList, instance->config->iMaxCandWord);
+    } else if (strcmp(addonname, "profile") == 0) {
+        if (!FcitxProfileLoad(instance->profile, instance))
+            FcitxInstanceEnd(instance);
+    } else if (strcmp(addonname, "ui") == 0) {
+        if (instance->ui && instance->ui->ui->ReloadConfig)
+            instance->ui->ui->ReloadConfig(instance->ui->addonInstance);
+    } else if (strcmp(addonname, "addon") == 0) {
+        FcitxAddon* addonHead = FcitxAddonsLoadInternal(&instance->addons, true);
+        FcitxInstanceFillAddonOwner(instance, addonHead);
+        FcitxInstanceResolveAddonDependencyInternal(instance, addonHead);
+        FcitxInstanceLoadAllIM(instance);
+    } else {
+        do {
+            FcitxIM* im = FcitxInstanceGetIMByName(instance, addonname);
+            if (im && im->ReloadConfig) {
+                im->ReloadConfig(im->klass);
+                break;
+            }
+
+            FcitxAddon *addon = FcitxAddonsGetAddonByName(&instance->addons, addonname);
+            if (!addon || !addon->bEnabled || !addon->addonInstance)
+                break;
+            switch (addon->category) {
+                case AC_MODULE:
+                    if (addon->module->ReloadConfig)
+                        addon->module->ReloadConfig(addon->addonInstance);
+                    break;
+                case AC_UI:
+                    if (addon->ui->ReloadConfig)
+                        addon->ui->ReloadConfig(addon->addonInstance);
+                    break;
+                case AC_FRONTEND:
+                    if (addon->frontend->ReloadConfig)
+                        addon->frontend->ReloadConfig(addon->addonInstance);
+                    break;
+                default:
+                    break;
+            }
+        } while(0);
+    }
+}
+
 FCITX_EXPORT_API
 void FcitxInstanceReloadConfig(FcitxInstance *instance)
 {
@@ -1178,14 +1233,6 @@ void FcitxInstanceReloadConfig(FcitxInstance *instance)
     FcitxInstanceLoadAllIM(instance);
 }
 
-FCITX_EXPORT_API
-FcitxIM* FcitxInstanceGetCurrentIM(FcitxInstance* instance)
-{
-    UT_array* imes = &instance->imes;
-    FcitxIM* pcurrentIM = (FcitxIM*) utarray_eltptr(imes, instance->iIMIndex);
-    return pcurrentIM;
-}
-
 static inline void FcitxInstanceSetICStatus(FcitxInstance* instance, FcitxInputContext* ic, FcitxContextState state)
 {
     if (ic->state != state) {
@@ -1210,6 +1257,7 @@ void FcitxInstanceSetLocalIMName(FcitxInstance* instance, FcitxInputContext* ic,
         FcitxInstanceUpdateCurrentIM(instance, false);
 }
 
+/* the "force" to this function is used when the list changed and index is not valid */
 boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force) {
     FcitxInputContext* ic = FcitxInstanceGetCurrentIC(instance);
     if (!ic && !force)
@@ -1217,14 +1265,16 @@ boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force) {
     FcitxInputContext2* ic2 = (FcitxInputContext2*) ic;
     int globalIndex = FcitxInstanceGetIMIndexByName(instance, instance->globalIMName);
     boolean forceSwtich = force;
-    /* fix it here */
+    boolean updateGlobal = false;
+    /* global index is not valid, that's why we need to fix it. */
     if (globalIndex == 0) {
-        UT_array* ime = &instance->imes;
-        FcitxIM* im = (FcitxIM*) utarray_eltptr(ime, 1);
+        UT_array *ime = &instance->imes;
+        FcitxIM *im = (FcitxIM*)utarray_eltptr(ime, 1);
         if (im) {
             fcitx_utils_string_swap(&instance->globalIMName, im->uniqueName);
             globalIndex = 1;
             forceSwtich = true;
+            updateGlobal = true;
         }
     }
     int targetIMIndex = 0;
@@ -1250,7 +1300,7 @@ boolean FcitxInstanceUpdateCurrentIM(FcitxInstance* instance, boolean force) {
     }
 
     if (forceSwtich || targetIMIndex != instance->iIMIndex) {
-        FcitxInstanceSwitchIMInternal(instance, targetIMIndex, skipZero, forceSwtich);
+        FcitxInstanceSwitchIMInternal(instance, targetIMIndex, skipZero, updateGlobal);
         return true;
     }
     else
@@ -1295,8 +1345,7 @@ void FcitxInstanceEnableIMInternal(FcitxInstance* instance, FcitxInputContext* i
 {
     if (ic == NULL)
         return ;
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -1354,8 +1403,7 @@ void FcitxInstanceCloseIMInternal(FcitxInstance* instance, FcitxInputContext* ic
 {
     if (ic == NULL)
         return ;
-    UT_array* frontends = &instance->frontends;
-    FcitxAddon** pfrontend = (FcitxAddon**) utarray_eltptr(frontends, ic->frontendid);
+    FcitxAddon **pfrontend = FcitxInstanceGetPFrontend(instance, ic->frontendid);
     if (pfrontend == NULL)
         return;
     FcitxFrontend* frontend = (*pfrontend)->frontend;
@@ -1513,10 +1561,12 @@ void FcitxInstanceShowInputSpeed(FcitxInstance* instance)
         return;
 
     if (instance->config->bShowVersion) {
-        FcitxMessagesAddMessageAtLast(input->msgAuxUp, MSG_TIPS, "FCITX " VERSION " ");
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxUp, MSG_TIPS,
+                                             "FCITX " VERSION " ");
     }
     if (im) {
-        FcitxMessagesAddMessageAtLast(input->msgAuxUp, MSG_TIPS, "%s", im->strName);
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxUp, MSG_TIPS,
+                                             im->strName);
     }
 
     //显示打字速度
@@ -1528,13 +1578,15 @@ void FcitxInstanceShowInputSpeed(FcitxInstance* instance)
             timePassed = 1.0;
 
         FcitxMessagesSetMessageCount(input->msgAuxDown, 0);
-        FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_OTHER, _("Input Speed: "));
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxDown, MSG_OTHER,
+                                             _("Input Speed: "));
         FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_CODE, "%d", (int)(instance->iHZInputed * 60 / timePassed));
-        FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_OTHER, _("/min  Time Used: "));
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxDown, MSG_OTHER,
+                                             _("/min  Time Used: "));
         FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_CODE, "%d", (int) timePassed / 60);
-        FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_OTHER, _("min Num of Characters: "));
+        FcitxMessagesAddMessageStringsAtLast(input->msgAuxDown, MSG_OTHER,
+                                             _("min Num of Characters: "));
         FcitxMessagesAddMessageAtLast(input->msgAuxDown, MSG_CODE, "%u", instance->iHZInputed);
-
     }
 
     FcitxUIUpdateInputWindow(instance);
@@ -1592,33 +1644,56 @@ int FcitxHotkeyCheckChooseKey(FcitxKeySym sym, unsigned int state, const char* s
 }
 
 FCITX_EXPORT_API
-int FcitxHotkeyCheckChooseKeyAndModifier(FcitxKeySym sym, unsigned int state, const char* strChoose, int candState)
+int FcitxHotkeyCheckChooseKeyAndModifier(FcitxKeySym sym, unsigned int state,
+                                         const char* strChoose, int candState)
 {
-    if (state != candState)
+    if (state != (unsigned)candState)
         return -1;
 
     char *p = strchr(strChoose, FcitxHotkeyPadToMain(sym));
     return p ? p - strChoose : -1;
 }
 
+FCITX_EXPORT_API
+FcitxIM* FcitxInstanceGetCurrentIM(FcitxInstance* instance)
+{
+    return fcitx_array_eltptr(&instance->imes, instance->iIMIndex);
+}
+
+FCITX_EXPORT_API
+FcitxIM* FcitxInstanceGetIMByIndex(FcitxInstance* instance, int index)
+{
+    /* utarray helps us to check the overflow */
+    return fcitx_array_eltptr(&instance->imes, index);
+}
+
 FCITX_EXPORT_API
 int FcitxInstanceGetIMIndexByName(FcitxInstance* instance, const char* imName)
+{
+    UT_array* imes = &instance->imes;
+    FcitxIM *pim = FcitxInstanceGetIMByName(instance, imName);
+    if (!pim)
+        return -1;
+    else
+        return utarray_eltidx(imes, pim);
+}
+
+FCITX_EXPORT_API
+FcitxIM* FcitxInstanceGetIMByName(FcitxInstance* instance, const char* imName)
 {
     UT_array* imes = &instance->imes;
     FcitxIM *pim;
-    int index = -1, i = 0;
     for (pim = (FcitxIM *) utarray_front(imes);
             pim != NULL;
             pim = (FcitxIM *) utarray_next(imes, pim)) {
         if (strcmp(imName, pim->uniqueName) == 0) {
-            index = i;
-            break;
+            return pim;
         }
-        i ++;
     }
-    return index;
+    return NULL;
 }
 
+
 FCITX_EXPORT_API
 void FcitxInstanceNotifyUpdateSurroundingText(FcitxInstance* instance, FcitxInputContext* ic)
 {
@@ -1642,6 +1717,7 @@ void UnusedIMItemFreeAll(UnusedIMItem* item)
         cur = item;
         HASH_DEL(item, cur);
         fcitx_utils_free(cur->name);
+        free(cur);
     }
 }
 
