@@ -33,21 +33,21 @@
  * The input method is key event centric application. When a key event comes to fcitx,
  * the process handling the keyboard event can be separated into 7 phases, PreInput, DoInput, Update Candidates, Prev/Next Page
  * PostInput, Hotkey, and key blocker.
- * 
+ *
  * The input method engine process key event inside "DoInput".
- * 
+ *
  * Each phase will change the INPUT_RETURN_VALUE, if INPUT_RETURN_VALUE is non-zero (non IRV_TO_PROCESS), the following
  * phases will not run.
- * 
+ *
  * If a key event goes through all phase and still the state is IRV_TO_PROCESS, it will be forwarded.
- * 
+ *
  * When it comes to update candidates, if the flag contains IRV_FLAG_UPDATE_CANDIDATE_WORDS, it will trigger the GetCandWords
  * of input method engine and clean up all stiring in the input window, after that, it will trigger update candidates hook.
- * 
- * Key blocker is useful if you want to post input phase to do something, but you don't want forward key if they do nothing.
+ *
+ * Key blocker is useful if you want to do something in the post input phase, but you don't want forward key if they do nothing.
  * There is an default implemention inside fcitx, it will blocks key when raw input buffer is not empty. Those keys
  * contains direction key(left/right..), key will cause something input (a,b,c...), key will cause cursor move (home/end...).
- * 
+ *
  * DoInput, Update Candidates, Key Blocker are belongs to input method engine, other can be registered by other addon.
  */
 #ifndef _FCITX_IME_H_
@@ -150,6 +150,40 @@ extern "C" {
         void  (*padding5)(void *arg); /**< padding */
     } FcitxIMClass2;
 
+    typedef enum _FcitxIMCloseEventType {
+        /**
+         * when user press inactivate key, default behavior is commit raw preedit.
+         * If you want to OVERRIDE this behavior, be sure to implement this function.
+         *
+         * in some case, your implementation of OnClose should respect the value of
+         * [Output/SendTextWhenSwitchEng], when this value is true, commit something you
+         * want.
+         *
+         * And no matter in which case, Reset will be called after that.
+         */
+        CET_ChangeByInactivate,
+        /**
+         * when using lost focus
+         * this might be variance case to case. the default behavior is to commit
+         * the preedit, and resetIM.
+         *
+         * Controlled by [Output/DontCommitPreeditWhenUnfocus], this option will not
+         * work for application switch doesn't support async commit.
+         *
+         * So OnClose is called when preedit IS committed (not like CET_ChangeByInactivate,
+         * this behavior cannot be overrided), it give im a chance to choose remember this
+         * word or not.
+         *
+         * Input method need to notice, that the commit is already DONE, do not to extra commit.
+         */
+        CET_LostFocus,
+        /**
+         * when user switch to a different input method by hand
+         * such as ctrl+shift by default, or by ui,
+         * not implemented yet, default behavior is reset IM.
+         */
+        CET_ChangeByUser,
+    } FcitxIMCloseEventType;
 
     typedef boolean(*FcitxIMInit)(void *arg); /**< FcitxIMInit */
     typedef void (*FcitxIMResetIM)(void *arg); /**< FcitxIMResetIM */
@@ -160,6 +194,7 @@ extern "C" {
     typedef void (*FcitxIMReloadConfig)(void *arg); /**< FcitxIMReloadConfig */
     typedef INPUT_RETURN_VALUE (*FcitxIMKeyBlocker)(void* arg, FcitxKeySym, unsigned int); /**< FcitxIMKeyBlocker */
     typedef void (*FcitxIMUpdateSurroundingText)(void* arg); /**< FcitxIMKeyBlocker */
+    typedef void (*FcitxIMOnClose)(void* arg, FcitxIMCloseEventType);
 
     /**
      * a more fexible interface for input method
@@ -176,8 +211,9 @@ extern "C" {
         FcitxIMReloadConfig ReloadConfig; /**< reload configuration */
         FcitxIMKeyBlocker KeyBlocker; /**< block unused key */
         FcitxIMUpdateSurroundingText UpdateSurroundingText; /**< surrounding text update trigger */
-        FcitxIMDoInput DoReleaseInput;
-        void* padding[63]; /**< padding */
+        FcitxIMDoInput DoReleaseInput; /**< process key release event */
+        FcitxIMOnClose OnClose; /**< process when im being switched away */
+        void* padding[62]; /**< padding */
     } FcitxIMIFace;
 
     /**
@@ -258,6 +294,8 @@ extern "C" {
 
         FcitxIMDoInput DoReleaseInput;
 
+        FcitxIMOnClose OnClose;
+
         void* padding[8]; /**< padding */
     } FcitxIM;
 
@@ -298,13 +336,46 @@ extern "C" {
     const char* FcitxInputStateGetLastCommitString(FcitxInputState * input);
 
     /**
-     * get current input method
+     * get current input method, return result can be NULL.
      *
      * @param instance fcitx instance
      * @return _FcitxIM*
      **/
     struct _FcitxIM* FcitxInstanceGetCurrentIM(struct _FcitxInstance *instance);
 
+    /**
+     * get input method by name
+     *
+     * @param instance fcitx instance
+     * @param index index
+     * @return _FcitxIM*
+     *
+     * @since 4.2.7
+     **/
+    struct _FcitxIM* FcitxInstanceGetIMByIndex(struct _FcitxInstance* instance, int index);
+
+    /**
+     * get im index by im name
+     *
+     * @param instance fcitx instance
+     * @param imName im name
+     * @return int im index
+     *
+     * @since 4.2
+     **/
+    int FcitxInstanceGetIMIndexByName(struct _FcitxInstance* instance, const char* imName);
+
+    /**
+     * get im index by im name
+     *
+     * @param instance fcitx instance
+     * @param imName im name
+     * @return int im index
+     *
+     * @since 4.2.7
+     **/
+    struct _FcitxIM* FcitxInstanceGetIMByName(struct _FcitxInstance* instance, const char* imName);
+
     /**
      * enable im
      *
@@ -504,6 +575,20 @@ extern "C" {
      **/
     void FcitxInstanceSaveAllIM(struct _FcitxInstance* instance);
 
+    /**
+     * reload only an addon's configuration, there are some short hand for reloading
+     * other configuration, "global" for ~/.config/fcitx/config, "profile" for
+     * ~/.config/fcitx/profile, "addon" for addon info. "ui" for current user interface
+     * Input method unique can be also used here.
+     *
+     * @param instance fcitx instance
+     * @param addon addon name
+     * @return void
+     *
+     * @since 4.2.7
+     **/
+    void FcitxInstanceReloadAddonConfig(struct _FcitxInstance* instance, const char* addon);
+
     /**
      * reload all config
      *
@@ -574,17 +659,6 @@ extern "C" {
      **/
     int FcitxHotkeyCheckChooseKeyAndModifier(FcitxKeySym sym, unsigned int state, const char* strChoose, int candState);
 
-    /**
-     * get im index by im name
-     *
-     * @param instance fcitx instance
-     * @param imName im name
-     * @return int im index
-     *
-     * @since 4.2
-     **/
-    int FcitxInstanceGetIMIndexByName(struct _FcitxInstance* instance, const char* imName);
-
     /**
      * ...
      *
