@@ -45,6 +45,7 @@
 #include "instance-internal.h"
 #include "module-internal.h"
 #include "addon-internal.h"
+#include "setjmp.h"
 
 #define CHECK_ENV(env, value, icase) (!getenv(env) \
                                       || (icase ? \
@@ -66,12 +67,20 @@ FCITX_SETTER(FcitxInstance, MaxFD, maxfd, int)
 FCITX_GETTER_VALUE(FcitxInstance, GlobalConfig, config, FcitxGlobalConfig*)
 FCITX_GETTER_VALUE(FcitxInstance, Profile, profile, FcitxProfile*)
 FCITX_GETTER_VALUE(FcitxInstance, InputState, input, FcitxInputState*)
+FCITX_GETTER_VALUE(FcitxInstance, IsDestroying, destroy, boolean)
 
-const UT_icd stat_icd = {sizeof(FcitxUIStatus), 0, 0, 0};
-const UT_icd compstat_icd = {sizeof(FcitxUIComplexStatus), 0, 0, 0};
-const UT_icd menup_icd = {sizeof(FcitxUIMenu*), 0, 0, 0};
-const UT_icd timeout_icd = {sizeof(TimeoutItem), 0, 0, 0};
-const UT_icd icdata_icd = {sizeof(FcitxICDataInfo), NULL, NULL, NULL };
+static const UT_icd stat_icd = {
+    sizeof(FcitxUIStatus), NULL, NULL, NULL
+};
+static const UT_icd compstat_icd = {
+    sizeof(FcitxUIComplexStatus), NULL, NULL, NULL
+};
+static const UT_icd timeout_icd = {
+    sizeof(TimeoutItem), NULL, NULL, NULL
+};
+static const UT_icd icdata_icd = {
+    sizeof(FcitxICDataInfo), NULL, NULL, NULL
+};
 static void FcitxInitThread(FcitxInstance* inst);
 static void ToggleRemindState(void* arg);
 static boolean GetRemindEnabled(void* arg);
@@ -81,6 +90,7 @@ static void Version();
 static void* RunInstance(void* arg);
 static void FcitxInstanceInitBuiltContext(FcitxInstance* instance);
 static void FcitxInstanceShowRemindStatusChanged(void* arg, const void* value);
+static void FcitxInstanceRealEnd(FcitxInstance* instance);
 
 /**
  * 显示命令行参数
@@ -95,7 +105,8 @@ void Usage()
            "\t-v\t\t\tdisplay the version information and exit\n"
            "\t-u, --ui\t\tspecify the user interface to use\n"
            "\t--enable\t\tspecify a comma separated list for addon that will override the enable option\n"
-           "\t--disable\t\tspecify a comma separated list for addon that will explicitly disabled, priority is higher than --enable\n"
+           "\t--disable\t\tspecify a comma separated list for addon that will explicitly disabled,\n"
+           "\t\t\t\t\tpriority is lower than --enable, can use all for disable all module\n"
            "\t-h, --help\t\tdisplay this help and exit\n");
 }
 
@@ -110,11 +121,11 @@ void Version()
 FCITX_EXPORT_API
 FcitxInstance* FcitxInstanceCreate(sem_t *sem, int argc, char* argv[])
 {
-    return FcitxInstanceCreateWithFD(sem, argc, argv, 0);
+    return FcitxInstanceCreateWithFD(sem, argc, argv, -1);
 }
 
 FCITX_EXPORT_API
-FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int fd)
+FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int fd)
 {
     FcitxInstance* instance = fcitx_utils_malloc0(sizeof(FcitxInstance));
     FcitxAddonsInit(&instance->addons);
@@ -124,7 +135,7 @@ FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int
     InitFcitxModules(&instance->eventmodules);
     utarray_init(&instance->uistats, &stat_icd);
     utarray_init(&instance->uicompstats, &compstat_icd);
-    utarray_init(&instance->uimenus, &menup_icd);
+    utarray_init(&instance->uimenus, fcitx_ptr_icd);
     utarray_init(&instance->timeout, &timeout_icd);
     utarray_init(&instance->icdata, &icdata_icd);
     instance->input = FcitxInputStateCreate();
@@ -132,7 +143,8 @@ FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int
     instance->config = fcitx_utils_malloc0(sizeof(FcitxGlobalConfig));
     instance->profile = fcitx_utils_malloc0(sizeof(FcitxProfile));
     instance->globalIMName = strdup("");
-    if (fd > 0) {
+    instance->fd = -1;
+    if (fd >= 0) {
         fcntl(fd, F_SETFL, O_NONBLOCK);
         instance->fd = fd;
     }
@@ -188,14 +200,33 @@ FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int
         return instance;
     }
 
-    /* make in order to use block X, query is not good here */
-    pthread_create(&instance->pid, NULL, RunInstance, instance);
-
     return instance;
 
 error_exit:
     FcitxInstanceEnd(instance);
     return instance;
+}
+
+FCITX_EXPORT_API
+void FcitxInstanceStart(FcitxInstance* instance)
+{
+    if (!instance->loadingFatalError) {
+        instance->initialized = true;
+        /* make in order to use block X, query is not good here */
+        if (pthread_create(&instance->pid, NULL, RunInstance, instance) != 0)
+            instance->initialized = false;
+    }
+}
+
+
+FCITX_EXPORT_API
+FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int fd)
+{
+    FcitxInstance* instance = FcitxInstanceCreatePause(sem, argc, argv, fd);
+
+    FcitxInstanceStart(instance);
+
+    return instance;
 
 }
 
@@ -205,21 +236,25 @@ void FcitxInstanceSetRecheckEvent(FcitxInstance* instance)
     instance->uiflag |= UI_EVENT_CHECK;
 }
 
+FCITX_EXPORT_API
+jmp_buf FcitxRecover;
+
 void* RunInstance(void* arg)
 {
     FcitxInstance* instance = (FcitxInstance*) arg;
-    instance->initialized = true;
-    int64_t curtime = 0;
+    uint64_t curtime = 0;
     while (1) {
         FcitxAddon** pmodule;
         uint8_t signo = 0;
-        while (read(instance->fd, &signo, sizeof(char)) > 0) {
-            if (signo == SIGINT || signo == SIGTERM || signo == SIGQUIT)
-                FcitxInstanceEnd(instance);
-            else if (signo == SIGHUP)
-                fcitx_utils_launch_restart();
-            else if (signo == SIGUSR1)
-                FcitxInstanceReloadConfig(instance);
+        if (instance->fd >= 0) {
+            while (read(instance->fd, &signo, sizeof(char)) > 0) {
+                if (signo == SIGINT || signo == SIGTERM || signo == SIGQUIT || signo == SIGXCPU)
+                    FcitxInstanceEnd(instance);
+                else if (signo == SIGHUP)
+                    fcitx_utils_launch_restart();
+                else if (signo == SIGUSR1)
+                    FcitxInstanceReloadConfig(instance);
+            }
         }
         do {
             instance->uiflag = UI_NONE;
@@ -233,7 +268,7 @@ void* RunInstance(void* arg)
             gettimeofday(&current_time, NULL);
             curtime = (current_time.tv_sec * 1000LL) + (current_time.tv_usec / 1000LL);
 
-            int idx = 0;
+            unsigned int idx = 0;
             while(idx < utarray_len(&instance->timeout))
             {
                 TimeoutItem* ti = (TimeoutItem*) utarray_eltptr(&instance->timeout, idx);
@@ -261,6 +296,13 @@ void* RunInstance(void* arg)
                 FcitxUIUpdateInputWindowReal(instance);
         } while (instance->uiflag != UI_NONE);
 
+        setjmp(FcitxRecover);
+
+        if (instance->destroy) {
+            FcitxInstanceRealEnd(instance);
+            break;
+        }
+
         FD_ZERO(&instance->rfds);
         FD_ZERO(&instance->wfds);
         FD_ZERO(&instance->efds);
@@ -281,12 +323,10 @@ void* RunInstance(void* arg)
         struct timeval tval;
         struct timeval* ptval = NULL;
         if (utarray_len(&instance->timeout) != 0) {
-            long int min_time = LONG_MAX;
+            unsigned long int min_time = LONG_MAX;
             TimeoutItem* ti;
-            for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
-                 ti != NULL;
-                 ti = (TimeoutItem*) utarray_next(&instance->timeout, ti))
-            {
+            for (ti = (TimeoutItem*)utarray_front(&instance->timeout);ti;
+                 ti = (TimeoutItem*)utarray_next(&instance->timeout, ti)) {
                 if (ti->time + ti->milli - curtime < min_time) {
                     min_time = ti->time + ti->milli - curtime;
                 }
@@ -295,7 +335,8 @@ void* RunInstance(void* arg)
             tval.tv_sec = min_time / 1000;
             ptval = &tval;
         }
-        select(instance->maxfd + 1, &instance->rfds, &instance->wfds, &instance->efds, ptval);
+        select(instance->maxfd + 1, &instance->rfds, &instance->wfds,
+               &instance->efds, ptval);
     }
     return NULL;
 }
@@ -303,15 +344,26 @@ void* RunInstance(void* arg)
 FCITX_EXPORT_API
 void FcitxInstanceEnd(FcitxInstance* instance)
 {
+    /* avoid duplicate destroy */
+    if (instance->destroy)
+        return;
+
     if (!instance->initialized) {
         if (!instance->loadingFatalError) {
-            FcitxLog(ERROR, "Exiting.");
+            if (!instance->quietQuit)
+                FcitxLog(ERROR, "Exiting.");
             instance->loadingFatalError = true;
             sem_post(instance->sem);
         }
         return;
     }
 
+    instance->destroy = true;
+}
+
+FCITX_EXPORT_API
+void FcitxInstanceRealEnd(FcitxInstance* instance) {
+
     FcitxProfileSave(instance->profile);
     FcitxInstanceSaveAllIM(instance);
 
@@ -331,29 +383,31 @@ void FcitxInstanceEnd(FcitxInstance* instance)
     FcitxFrontend* frontend;
     FcitxInputContext* rec = NULL;
 
-    for (pimclass = (FcitxAddon**) utarray_front(&instance->imeclasses);
-            pimclass != NULL;
-            pimclass = (FcitxAddon**) utarray_next(&instance->imeclasses, pimclass)
+    for (pimclass = (FcitxAddon**)utarray_front(&instance->imeclasses);
+         pimclass != NULL;
+         pimclass = (FcitxAddon**)utarray_next(&instance->imeclasses, pimclass)
         ) {
         if ((*pimclass)->imclass->Destroy)
             (*pimclass)->imclass->Destroy((*pimclass)->addonInstance);
     }
 
     for (rec = instance->ic_list; rec != NULL; rec = rec->next) {
-        pfrontend = (FcitxAddon**) utarray_eltptr(&instance->frontends, rec->frontendid);
+        pfrontend = (FcitxAddon**)utarray_eltptr(&instance->frontends,
+                                                 (unsigned int)rec->frontendid);
         frontend = (*pfrontend)->frontend;
         frontend->CloseIM((*pfrontend)->addonInstance, rec);
     }
 
     for (rec = instance->ic_list; rec != NULL; rec = rec->next) {
-        pfrontend = (FcitxAddon**) utarray_eltptr(&instance->frontends, rec->frontendid);
+        pfrontend = (FcitxAddon**)utarray_eltptr(&instance->frontends,
+                                                 (unsigned int)rec->frontendid);
         frontend = (*pfrontend)->frontend;
         frontend->DestroyIC((*pfrontend)->addonInstance, rec);
     }
 
-    for (pfrontend = (FcitxAddon**) utarray_front(&instance->frontends);
-            pfrontend != NULL;
-            pfrontend = (FcitxAddon**) utarray_next(&instance->frontends, pfrontend)
+    for (pfrontend = (FcitxAddon**)utarray_front(&instance->frontends);
+         pfrontend != NULL;
+         pfrontend = (FcitxAddon**)utarray_next(&instance->frontends, pfrontend)
         ) {
         if (pfrontend == NULL)
             continue;
@@ -363,9 +417,8 @@ void FcitxInstanceEnd(FcitxInstance* instance)
 
     FcitxAddon** pmodule;
     for (pmodule = (FcitxAddon**) utarray_front(&instance->modules);
-            pmodule != NULL;
-            pmodule = (FcitxAddon**) utarray_next(&instance->modules, pmodule)
-        ) {
+         pmodule != NULL;
+         pmodule = (FcitxAddon**) utarray_next(&instance->modules, pmodule)) {
         if (pmodule == NULL)
             return;
         FcitxModule* module = (*pmodule)->module;
@@ -374,13 +427,6 @@ void FcitxInstanceEnd(FcitxInstance* instance)
     }
 
     sem_post(instance->sem);
-
-    /* don't return to main loop, wait for exit */
-    int countDown = 5;
-    while(countDown--) {
-        sleep(1000);
-    }
-    exit(0);
 }
 
 void FcitxInitThread(FcitxInstance* inst)
@@ -464,6 +510,7 @@ boolean ProcessOption(FcitxInstance* instance, int argc, char* argv[])
                 }
                 break;
             default:
+                instance->quietQuit = true;
                 Usage();
                 return false;
             }
@@ -605,15 +652,27 @@ boolean FcitxInstanceCheckTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCal
 }
 
 FCITX_EXPORT_API
-boolean FcitxInstanceRemoveTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCallback callback)
+boolean FcitxInstanceCheckTimeoutById(FcitxInstance *instance, uint64_t id)
+{
+    TimeoutItem *ti;
+    for (ti = (TimeoutItem*)utarray_front(&instance->timeout);ti;
+         ti = (TimeoutItem*)utarray_next(&instance->timeout, ti)) {
+        if (ti->idx == id)
+            return true;
+    }
+    return false;
+}
+
+FCITX_EXPORT_API boolean
+FcitxInstanceRemoveTimeoutByFunc(FcitxInstance* instance,
+                                 FcitxTimeoutCallback callback)
 {
     TimeoutItem* ti;
     for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
          ti != NULL;
-         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti))
-    {
+         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti)) {
         if (ti->callback == callback) {
-            int idx = utarray_eltidx(&instance->timeout, ti);
+            unsigned int idx = utarray_eltidx(&instance->timeout, ti);
             utarray_remove_quick(&instance->timeout, idx);
             return true;
         }
@@ -629,10 +688,9 @@ boolean FcitxInstanceRemoveTimeoutById(FcitxInstance* instance, uint64_t id)
     TimeoutItem* ti;
     for (ti = (TimeoutItem*) utarray_front(&instance->timeout);
          ti != NULL;
-         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti))
-    {
+         ti = (TimeoutItem*) utarray_next(&instance->timeout, ti)) {
         if (ti->idx == id) {
-            int idx = utarray_eltidx(&instance->timeout, ti);
+            unsigned int idx = utarray_eltidx(&instance->timeout, ti);
             utarray_remove_quick(&instance->timeout, idx);
             return true;
         }
@@ -640,5 +698,12 @@ boolean FcitxInstanceRemoveTimeoutById(FcitxInstance* instance, uint64_t id)
     return false;
 }
 
+FCITX_EXPORT_API
+int FcitxInstanceWaitForEnd(FcitxInstance* instance) {
+    if (instance->initialized) {
+        return pthread_join(instance->pid, NULL);
+    }
+    return 0;
+}
 
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
