@@ -75,6 +75,29 @@ extern "C" {
      **/
     FcitxInstance* FcitxInstanceCreateWithFD(sem_t *sem, int argc, char* argv[], int fd);
 
+    /**
+     * create new fcitx instance, but don't run it.
+     *
+     * @param sem semaphore to notify the instance is end
+     * @param argc argc
+     * @param argv argv
+     * @param signal fd
+     * @return FcitxInstance*
+     *
+     * @see FcitxInstanceCreate
+     *
+     * @since 4.2.5
+     **/
+    FcitxInstance* FcitxInstanceCreatePause(sem_t *sem, int argc, char* argv[], int fd);
+
+    /**
+     * start a paused instance
+     *
+     * @param instance instance
+     * @return void
+     **/
+    void FcitxInstanceStart(FcitxInstance* instance);
+
     /**
      * replace existing fcitx instance
      *
@@ -124,6 +147,16 @@ extern "C" {
      **/
     void FcitxInstanceEnd(FcitxInstance* instance);
 
+    /**
+     * check whether FcitxInstanceEnd is called or not
+     *
+     * @param instance fcitx instance
+     * @return boolean
+     *
+     * @since 4.2.7
+     **/
+    boolean FcitxInstanceGetIsDestroying(FcitxInstance* instance);
+
     /**
      * Get Current Input Context
      *
@@ -180,10 +213,14 @@ extern "C" {
     uint64_t FcitxInstanceAddTimeout(FcitxInstance* instance, long int milli, FcitxTimeoutCallback callback , void* arg);
 
     boolean FcitxInstanceCheckTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCallback callback);
+    boolean FcitxInstanceCheckTimeoutById(FcitxInstance *instance, uint64_t id);
 
     boolean FcitxInstanceRemoveTimeoutByFunc(FcitxInstance* instance, FcitxTimeoutCallback callback);
 
     boolean FcitxInstanceRemoveTimeoutById(FcitxInstance* instance, uint64_t id);
+
+    int FcitxInstanceWaitForEnd(FcitxInstance* instance);
+
 #ifdef __cplusplus
 }
 #endif
