@@ -59,6 +59,9 @@ extern "C" {
     extern FcitxHotkey FCITX_RALT_RSUPER2[2];
     extern FcitxHotkey FCITX_SEMICOLON[2];
     extern FcitxHotkey FCITX_SPACE[2];
+    extern FcitxHotkey FCITX_SHIFT_SPACE[2];
+    extern FcitxHotkey FCITX_SHIFT_ENTER[2];
+    extern FcitxHotkey FCITX_TAB[2];
     extern FcitxHotkey FCITX_COMMA[2];
     extern FcitxHotkey FCITX_PERIOD[2];
     extern FcitxHotkey FCITX_CTRL_5[2];
@@ -70,6 +73,8 @@ extern "C" {
     extern FcitxHotkey FCITX_RSHIFT[2];
     extern FcitxHotkey FCITX_ALT_LSHIFT[2];
     extern FcitxHotkey FCITX_ALT_RSHIFT[2];
+    extern FcitxHotkey FCITX_LALT[2];
+    extern FcitxHotkey FCITX_RALT[2];
     extern FcitxHotkey FCITX_LSUPER[2];
     extern FcitxHotkey FCITX_RSUPER[2];
 
