@@ -85,7 +85,9 @@ static void FcitxUIShowInputWindow(FcitxInstance* instance);
 static boolean FcitxUILoadInternal(FcitxInstance* instance, FcitxAddon* addon);
 static void FcitxMenuItemFree(void* arg);
 
-static const UT_icd menuICD = { sizeof(FcitxMenuItem), NULL, NULL, FcitxMenuItemFree };
+static const UT_icd menuICD = {
+    sizeof(FcitxMenuItem), NULL, NULL, FcitxMenuItemFree
+};
 
 FCITX_EXPORT_API
 FcitxMessages* FcitxMessagesNew()
@@ -268,6 +270,19 @@ void FcitxMessagesAddMessageAtLast(FcitxMessages* message, FcitxMessageType type
     }
 }
 
+FCITX_EXPORT_API void
+FcitxMessagesAddMessageVStringAtLast(FcitxMessages *message,
+                                         FcitxMessageType type, size_t n,
+                                         const char **strs)
+{
+    if (message->msgCount < MAX_MESSAGE_COUNT) {
+        FcitxMessagesSetMessageStringsReal(message, message->msgCount,
+                                           type, n, strs);
+        message->msgCount ++;
+        message->changed = true;
+    }
+}
+
 FCITX_EXPORT_API
 void FcitxMessagesSetMessage(FcitxMessages* message, int position, int type, const char* fmt, ...)
 {
@@ -286,6 +301,14 @@ void FcitxMessagesSetMessageText(FcitxMessages* message, int position, const cha
     va_end(ap);
 }
 
+FCITX_EXPORT_API void
+FcitxMessagesSetMessageTextVString(FcitxMessages *message, int position,
+                                       size_t n, const char **strs)
+{
+    FcitxMessagesSetMessageStringsReal(message, position,
+                                       message->msg[position].type, n, strs);
+}
+
 FCITX_EXPORT_API
 void FcitxMessagesSetMessageV(FcitxMessages* message, int position, int type, const char* fmt, va_list ap)
 {
@@ -296,10 +319,23 @@ void FcitxMessagesSetMessageV(FcitxMessages* message, int position, int type, co
     }
 }
 
+FCITX_EXPORT_API
+void FcitxMessagesSetMessageStringsReal(FcitxMessages *message, int position,
+                                        int type, size_t n, const char **strs)
+{
+    if (position < MAX_MESSAGE_COUNT) {
+        fcitx_utils_cat_str_simple_with_len(message->msg[position].strMsg,
+                                            MESSAGE_MAX_LENGTH + 1, n, strs);
+        message->msg[position].type = type;
+        message->changed = true;
+    }
+}
+
 FCITX_EXPORT_API
 void FcitxMessagesMessageConcatLast(FcitxMessages* message, const char* text)
 {
-    strncat(message->msg[message->msgCount - 1].strMsg, text, MESSAGE_MAX_LENGTH);
+    strncat(message->msg[message->msgCount - 1].strMsg,
+            text, MESSAGE_MAX_LENGTH);
     message->changed = true;
 }
 
@@ -365,6 +401,25 @@ FcitxUIComplexStatus *FcitxUIGetComplexStatusByName(FcitxInstance* instance, con
     return compstatus;
 }
 
+FCITX_EXPORT_API
+void FcitxUIRefreshStatus(FcitxInstance* instance, const char* name)
+{
+    FcitxUIStatus *status = FcitxUIGetStatusByName(instance, name);
+
+    if (status != NULL) {
+        if (UI_FUNC_IS_VALID(UpdateStatus))
+            instance->ui->ui->UpdateStatus(instance->ui->addonInstance , status);
+    }
+    else {
+        FcitxUIComplexStatus *compstatus = FcitxUIGetComplexStatusByName(instance, name);
+        if (!compstatus)
+            return;
+
+        if (UI_FUNC_IS_VALID(UpdateComplexStatus))
+            instance->ui->ui->UpdateComplexStatus(instance->ui->addonInstance , compstatus);
+    }
+}
+
 FCITX_EXPORT_API
 void FcitxUIUpdateStatus(FcitxInstance* instance, const char* name)
 {
@@ -603,9 +658,29 @@ void FcitxUIOnInputUnFocus(struct _FcitxInstance* instance)
 FCITX_EXPORT_API
 void FcitxUICommitPreedit(FcitxInstance* instance)
 {
-    if (instance->CurrentIC
-        && !instance->config->bDontCommitPreeditWhenUnfocus
+    if (!instance->CurrentIC)
+        return;
+
+    boolean callOnClose = false;
+    boolean doServerSideCommit = false;
+    if (!instance->config->bDontCommitPreeditWhenUnfocus
         && !(instance->CurrentIC->contextCaps & CAPACITY_CLIENT_UNFOCUS_COMMIT)) {
+        callOnClose = true;
+        doServerSideCommit = true;
+    }
+
+    if (instance->CurrentIC->contextCaps & CAPACITY_CLIENT_UNFOCUS_COMMIT) {
+        callOnClose = true;
+    }
+
+    if (callOnClose) {
+        FcitxIM* im = FcitxInstanceGetCurrentIM(instance);
+        if (im && im->OnClose) {
+            im->OnClose(im->klass, CET_LostFocus);
+        }
+    }
+
+    if (doServerSideCommit) {
         FcitxInputState* input = FcitxInstanceGetInputState(instance);
         FcitxMessages* clientPreedit = FcitxInputStateGetClientPreedit(input);
 
@@ -618,6 +693,7 @@ void FcitxUICommitPreedit(FcitxInstance* instance)
         }
         FcitxMessagesSetMessageCount(clientPreedit, 0);
     }
+
 }
 
 FCITX_EXPORT_API
@@ -698,15 +774,15 @@ int FcitxUINewMessageToOldStyleMessage(FcitxInstance* instance, FcitxMessages* m
     FcitxMessagesSetMessageCount(msgDown, 0);
 
     for (i = 0; i < FcitxMessagesGetMessageCount(input->msgAuxUp) ; i ++) {
-        FcitxMessagesAddMessageAtLast(msgUp, FcitxMessagesGetMessageType(input->msgAuxUp, i), "%s", FcitxMessagesGetMessageString(input->msgAuxUp, i));
+        FcitxMessagesAddMessageStringsAtLast(msgUp, FcitxMessagesGetMessageType(input->msgAuxUp, i), FcitxMessagesGetMessageString(input->msgAuxUp, i));
         extraLength += strlen(FcitxMessagesGetMessageString(input->msgAuxUp, i));
     }
 
     for (i = 0; i < FcitxMessagesGetMessageCount(input->msgPreedit) ; i ++)
-        FcitxMessagesAddMessageAtLast(msgUp, FcitxMessagesGetMessageType(input->msgPreedit, i), "%s", FcitxMessagesGetMessageString(input->msgPreedit, i));
+        FcitxMessagesAddMessageStringsAtLast(msgUp, FcitxMessagesGetMessageType(input->msgPreedit, i), FcitxMessagesGetMessageString(input->msgPreedit, i));
 
     for (i = 0; i < FcitxMessagesGetMessageCount(input->msgAuxDown) ; i ++)
-        FcitxMessagesAddMessageAtLast(msgDown, FcitxMessagesGetMessageType(input->msgAuxDown, i), "%s", FcitxMessagesGetMessageString(input->msgAuxDown, i));
+        FcitxMessagesAddMessageStringsAtLast(msgDown, FcitxMessagesGetMessageType(input->msgAuxDown, i), FcitxMessagesGetMessageString(input->msgAuxDown, i));
 
     FcitxCandidateWord* candWord = NULL;
 
@@ -724,12 +800,11 @@ int FcitxUINewMessageToOldStyleMessage(FcitxInstance* instance, FcitxMessages* m
 
         unsigned int mod = FcitxCandidateWordGetModifier(input->candList);
 
-        FcitxMessagesAddMessageAtLast(msgDown, MSG_INDEX, "%s%s%s%s%s",
-                                      (mod & FcitxKeyState_Super) ? "M-" : "",
-                                      (mod & FcitxKeyState_Ctrl) ? "C-" : "",
-                                      (mod & FcitxKeyState_Alt) ? "A-" : "",
-                                      (mod & FcitxKeyState_Shift) ? "S-" : "",
-                                      strTemp);
+        FcitxMessagesAddMessageStringsAtLast(
+            msgDown, MSG_INDEX, (mod & FcitxKeyState_Super) ? "M-" : "",
+            (mod & FcitxKeyState_Ctrl) ? "C-" : "",
+            (mod & FcitxKeyState_Alt) ? "A-" : "",
+            (mod & FcitxKeyState_Shift) ? "S-" : "", strTemp);
 
         FcitxMessageType type = candWord->wordType;
 
@@ -740,12 +815,13 @@ int FcitxUINewMessageToOldStyleMessage(FcitxInstance* instance, FcitxMessages* m
         )
             type = MSG_FIRSTCAND;
 
-        FcitxMessagesAddMessageAtLast(msgDown, type, "%s", candWord->strWord);
+        FcitxMessagesAddMessageStringsAtLast(msgDown, type, candWord->strWord);
 
-        if (candWord->strExtra && strlen(candWord->strExtra) != 0)
-            FcitxMessagesAddMessageAtLast(msgDown, candWord->extraType, "%s", candWord->strExtra);
+        if (candWord->strExtra && strlen(candWord->strExtra))
+            FcitxMessagesAddMessageStringsAtLast(msgDown, candWord->extraType,
+                                                 candWord->strExtra);
 
-        FcitxMessagesAddMessageAtLast(msgDown, MSG_OTHER, " ");
+        FcitxMessagesAddMessageStringsAtLast(msgDown, MSG_OTHER, " ");
     }
 
     return extraLength;
@@ -821,10 +897,10 @@ char* FcitxUICandidateWordToCString(FcitxInstance* instance)
         len += strlen(strTemp);
         len += strlen(candWord->strWord);
 
-        if (candWord->strExtra && strlen(candWord->strExtra) != 0)
+        if (candWord->strExtra && strlen(candWord->strExtra))
             len += strlen(candWord->strExtra);
 
-        len ++;
+        len++;
     }
 
     char *result = fcitx_utils_malloc0(sizeof(char) * (len + 1));
@@ -841,7 +917,7 @@ char* FcitxUICandidateWordToCString(FcitxInstance* instance)
         strcat(result, strTemp);
         strcat(result, candWord->strWord);
 
-        if (candWord->strExtra && strlen(candWord->strExtra) != 0)
+        if (candWord->strExtra && strlen(candWord->strExtra))
             strcat(result, candWord->strExtra);
 
         strcat(result, " ");
