@@ -392,6 +392,24 @@ extern "C" {
      **/
     void FcitxMessagesAddMessageAtLast(FcitxMessages* message, FcitxMessageType type, const char *fmt, ...);
 
+    /**
+     * add a message string at last, cat strings version
+     *
+     * @param message message
+     * @param type message type
+     * @return void
+     **/
+    void FcitxMessagesAddMessageVStringAtLast(FcitxMessages *message,
+                                                  FcitxMessageType type,
+                                                  size_t n, const char **strs);
+#define FcitxMessagesAddMessageStringsAtLast(message, type, strs...) do { \
+        const char *__msg_str_lst[] = {strs};                           \
+        size_t __msg_str_count = sizeof(__msg_str_lst) / sizeof(char*); \
+        FcitxMessagesAddMessageVStringAtLast(message, type,         \
+                                                 __msg_str_count,       \
+                                                 __msg_str_lst);        \
+    } while (0)
+
     /**
      * set a message string at position
      *
@@ -413,6 +431,31 @@ extern "C" {
      * @return void
      **/
     void FcitxMessagesSetMessageText(FcitxMessages* message, int position, const char* fmt, ...);
+
+    /**
+     * set only message string, it doesn't insert string into position,
+     * it concat all the string in strs, then replace the string at
+     * position
+     *
+     * @param message message
+     * @param position position
+     * @param n strs length
+     * @param strs string array
+     * @return void
+     *
+     * @since 4.2.7
+     **/
+    void FcitxMessagesSetMessageTextVString(FcitxMessages *message,
+                                            int position, size_t n,
+                                            const char **strs);
+#define FcitxMessagesSetMessageTextStrings(message, position, strs...) do { \
+        const char *__msg_str_lst[] = {strs};                           \
+        size_t __msg_str_count = sizeof(__msg_str_lst) / sizeof(char*); \
+        FcitxMessagesSetMessageTextVString(message, position,       \
+                                               __msg_str_count,         \
+                                               __msg_str_lst);          \
+    } while (0)
+
     /**
      * concat a string to message string at position
      *
@@ -441,6 +484,26 @@ extern "C" {
      * @return void
      **/
     void FcitxMessagesSetMessageV(FcitxMessages* message, int position, int type, const char* fmt, va_list ap);
+    /**
+     * set message string cat strings version
+     *
+     * @param message message
+     * @param position position
+     * @param type message type
+     * @param n number of strings
+     * @param strs list of strings
+     * @return void
+     **/
+    void FcitxMessagesSetMessageStringsReal(FcitxMessages *message,
+                                            int position, int type,
+                                            size_t n, const char **strs);
+#define FcitxMessagesSetMessageStrings(message, position, type, strs...) do { \
+        const char *__msg_str_lst[] = {strs};                           \
+        size_t __msg_str_count = sizeof(__msg_str_lst) / sizeof(char*); \
+        FcitxMessagesSetMessageStringsReal(message, position, type,     \
+                                           __msg_str_count, __msg_str_lst); \
+    } while (0)
+
     /**
      * set message count
      *
@@ -543,6 +606,14 @@ extern "C" {
      * @return void
      **/
     void FcitxUICloseInputWindow(struct _FcitxInstance* instance);
+    /**
+     * refresh a status info, since it changed outside the toggle function
+     *
+     * @param instance fcitx instance
+     * @param name status name
+     * @return void
+     **/
+    void FcitxUIRefreshStatus(struct _FcitxInstance* instance, const char* name);
     /**
      * toggle a user interface status
      *
