@@ -35,7 +35,7 @@
 #include "fcitx-utils/utils.h"
 #include "fcitx-utils/log.h"
 #include "fcitx-config/xdg.h"
-#include "module/spell/spell.h"
+#include "module/spell/fcitx-spell.h"
 
 #include "AutoEng.h"
 #define case_autoeng_replace case ' ': case '\'': case '-': case '_':   \
@@ -48,6 +48,7 @@ typedef enum {
     AECM_ALT,
     AECM_CTRL,
     AECM_SHIFT,
+    _AECM_COUNT
 } AutoEngChooseModifier;
 
 typedef struct {
@@ -60,7 +61,7 @@ typedef struct {
 } FcitxAutoEngConfig;
 
 typedef struct _FcitxAutoEngState {
-    UT_array* autoEng;
+    UT_array *autoEng;
     char *buf;
     int index;
     size_t buff_size;
@@ -77,7 +78,7 @@ static const unsigned int cmodtable[] = {
     FcitxKeyState_Shift
 };
 
-static const UT_icd autoeng_icd = { sizeof(AUTO_ENG), 0, 0, 0 };
+static const UT_icd autoeng_icd = { sizeof(AUTO_ENG), NULL, NULL, NULL };
 
 CONFIG_BINDING_BEGIN(FcitxAutoEngConfig);
 CONFIG_BINDING_REGISTER("Auto English", "ChooseModifier", chooseModifier);
@@ -162,7 +163,7 @@ FCITX_DEFINE_PLUGIN(fcitx_autoeng, module ,FcitxModule) = {
 static void
 AutoEngSetBuffLen(FcitxAutoEngState* autoEngState, size_t len)
 {
-    int size = (len / MAX_USER_INPUT + 1) * MAX_USER_INPUT;
+    unsigned int size = (len / MAX_USER_INPUT + 1) * MAX_USER_INPUT;
     if (autoEngState->buf) {
         if (autoEngState->buff_size != size) {
             autoEngState->buf = realloc(autoEngState->buf, size);
@@ -265,11 +266,9 @@ AutoEngCheckPreedit(FcitxAutoEngState *autoEngState)
     char *preedit;
     input = FcitxInstanceGetInputState(autoEngState->owner);
     preedit = FcitxUIMessagesToCString(FcitxInputStateGetPreedit(input));
-    if (preedit && *fcitx_utils_get_ascii_end(preedit)) {
-        free(preedit);
-        return false;
-    }
-    return true;
+    boolean res = !(preedit && *fcitx_utils_get_ascii_end(preedit));
+    free(preedit);
+    return res;
 }
 
 void *AutoEngCreate(FcitxInstance *instance)
@@ -302,12 +301,71 @@ static INPUT_RETURN_VALUE
 AutoEngCheckSelect(FcitxAutoEngState *autoEngState,
                    FcitxKeySym sym, unsigned int state)
 {
-    FcitxCandidateWordList *candList = FcitxInputStateGetCandidateList(
+    FcitxInstance *instance = autoEngState->owner;
+    FcitxCandidateWordList *cand_list = FcitxInputStateGetCandidateList(
         FcitxInstanceGetInputState(autoEngState->owner));
-    int iKey = FcitxCandidateWordCheckChooseKey(candList, sym, state);
-    if (iKey >= 0)
-        return FcitxCandidateWordChooseByIndex(candList, iKey);
-    return 0;
+    if (!FcitxCandidateWordGetListSize(cand_list))
+        return IRV_TO_PROCESS;
+    FcitxInputState *input = FcitxInstanceGetInputState(instance);
+    FcitxGlobalConfig *fc = FcitxInstanceGetGlobalConfig(instance);
+    int key;
+    FcitxCandidateWord *cand_word;
+    if (FcitxHotkeyIsHotKey(sym, state, fc->nextWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetNext(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(cand_list, 0);
+            cand_word = FcitxCandidateWordGetFirst(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state, fc->prevWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetPrev(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(
+                cand_list, FcitxCandidateWordPageCount(cand_list) - 1);
+            cand_word = FcitxCandidateWordGetLast(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigPrevPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (!FcitxCandidateWordGoPrevPage(cand_list)) {
+            FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+            return IRV_TO_PROCESS;
+        }
+        cand_word = FcitxCandidateWordGetCurrentWindow(cand_list) +
+            FcitxCandidateWordGetCurrentWindowSize(cand_list) - 1;
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigNextPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (!FcitxCandidateWordGoNextPage(cand_list)) {
+            FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+            return IRV_TO_PROCESS;
+        }
+        cand_word = FcitxCandidateWordGetCurrentWindow(cand_list);
+    } else if ((key = FcitxCandidateWordCheckChooseKey(cand_list,
+                                                       sym, state)) >= 0) {
+        return FcitxCandidateWordChooseByIndex(cand_list, key);
+    } else {
+        return IRV_TO_PROCESS;
+    }
+    FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+    AutoEngSetBuff(autoEngState, cand_word->strWord, '\0');
+    FcitxMessages *client_preedit = FcitxInputStateGetClientPreedit(input);
+    FcitxMessages *preedit = FcitxInputStateGetPreedit(input);
+    FcitxMessagesSetMessageCount(client_preedit, 0);
+    FcitxMessagesAddMessageStringsAtLast(client_preedit, MSG_INPUT,
+                                         autoEngState->buf);
+    FcitxMessagesSetMessageCount(preedit, 0);
+    FcitxMessagesAddMessageStringsAtLast(preedit, MSG_INPUT,
+                                         autoEngState->buf);
+    FcitxInputStateSetClientCursorPos(input, autoEngState->index);
+    return IRV_FLAG_UPDATE_INPUT_WINDOW;
 }
 
 static void
@@ -331,46 +389,45 @@ static boolean PreInputProcessAutoEng(void* arg, FcitxKeySym sym,
         return false;
 
     FcitxKeySym keymain = FcitxHotkeyPadToMain(sym);
-    if (autoEngState->active) {
-        if ((*retval = AutoEngCheckSelect(autoEngState, sym, state))) {
-            return true;
-        } else if (FcitxHotkeyIsHotKeySimple(keymain, state)) {
-            *retval = AutoEngPushKey(autoEngState, keymain);
-        } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_BACKSPACE)) {
-            AutoEngSetBuffLen(autoEngState, --autoEngState->index);
-            if (autoEngState->index == 0) {
-                ResetAutoEng(autoEngState);
-                *retval = IRV_CLEAN;
-            } else {
+    if (!autoEngState->active) {
+        if (FcitxHotkeyIsHotKeySimple(sym, state)) {
+            AutoEngSetBuff(autoEngState,
+                           FcitxInputStateGetRawInputBuffer(input), keymain);
+            if (SwitchToEng(autoEngState, autoEngState->buf)) {
                 *retval = IRV_DISPLAY_MESSAGE;
+                FcitxInputStateSetShowCursor(input, false);
+                autoEngState->active = true;
+                ShowAutoEngMessage(autoEngState, retval);
+                return true;
             }
-        } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
-            AutoEngCommit(autoEngState);
-            ResetAutoEng(autoEngState);
-            *retval = IRV_FLAG_UPDATE_INPUT_WINDOW | IRV_FLAG_RESET_INPUT;
-        } else if (FcitxHotkeyIsHotkeyCursorMove(sym, state)) {
-            *retval = IRV_DO_NOTHING;
-            return true;
-        } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ESCAPE)) {
-            *retval = IRV_CLEAN;
-            return true;
         }
-        ShowAutoEngMessage(autoEngState, retval);
+        return false;
+    }
+    if ((*retval = AutoEngCheckSelect(autoEngState, sym, state))) {
+        return true;
+    } else if (FcitxHotkeyIsHotKeySimple(keymain, state)) {
+        *retval = AutoEngPushKey(autoEngState, keymain);
+    } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_BACKSPACE)) {
+        AutoEngSetBuffLen(autoEngState, --autoEngState->index);
+        if (autoEngState->index == 0) {
+            ResetAutoEng(autoEngState);
+            *retval = IRV_CLEAN;
+        } else {
+            *retval = IRV_DISPLAY_MESSAGE;
+        }
+    } else if (FcitxHotkeyIsHotkeyCursorMove(sym, state)) {
+        *retval = IRV_DO_NOTHING;
+        return true;
+    } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
+        AutoEngCommit(autoEngState);
+        ResetAutoEng(autoEngState);
+        *retval = IRV_FLAG_UPDATE_INPUT_WINDOW | IRV_FLAG_RESET_INPUT;
+    } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ESCAPE)) {
+        *retval = IRV_CLEAN;
         return true;
     }
-    if (FcitxHotkeyIsHotKeySimple(sym, state)) {
-        AutoEngSetBuff(autoEngState, FcitxInputStateGetRawInputBuffer(input),
-                       keymain);
-
-        if (SwitchToEng(autoEngState, autoEngState->buf)) {
-            *retval = IRV_DISPLAY_MESSAGE;
-            FcitxInputStateSetShowCursor(input, false);
-            autoEngState->active = true;
-            ShowAutoEngMessage(autoEngState, retval);
-            return true;
-        }
-    }
-    return false;
+    ShowAutoEngMessage(autoEngState, retval);
+    return true;
 }
 
 boolean PostInputProcessAutoEng(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retval)
@@ -435,8 +492,8 @@ LoadAutoEngConfig(FcitxAutoEngConfig *config)
     FcitxConfigFile *cfile = FcitxConfigParseConfigFileFp(fp, configDesc);
     FcitxAutoEngConfigConfigBind(config, cfile, configDesc);
     FcitxConfigBindSync(&config->gconfig);
-    if (config->chooseModifier > AECM_CTRL)
-        config->chooseModifier = AECM_CTRL;
+    if (fcitx_unlikely(config->chooseModifier >= _AECM_COUNT))
+        config->chooseModifier = _AECM_COUNT - 1;
 
     if (fp)
         fclose(fp);
@@ -497,8 +554,9 @@ boolean SwitchToEng(FcitxAutoEngState *autoEngState, const char *str)
 }
 
 static INPUT_RETURN_VALUE
-AutoEngGetCandWordCb(FcitxAutoEngState *autoEngState, const char *commit)
+AutoEngGetCandWordCb(void *arg, const char *commit)
 {
+    FcitxAutoEngState *autoEngState = arg;
     INPUT_RETURN_VALUE res = IRV_DO_NOTHING;
     if (autoEngState->config.maxKeep == 0 &&
         !autoEngState->config.selectAddSpace) {
@@ -524,10 +582,11 @@ AutoEngGetSpellHint(FcitxAutoEngState *autoEngState)
     FcitxCandidateWordList *candList;
     if (autoEngState->config.disableSpell)
         return;
-    candList = InvokeVaArgs(autoEngState->owner, FCITX_SPELL, GET_CANDWORDS,
-                            NULL, autoEngState->buf, NULL,
-                            (void*)(long)autoEngState->config.maxHintLength,
-                            "en", "cus", AutoEngGetCandWordCb, autoEngState);
+    candList = FcitxSpellGetCandWords(autoEngState->owner, NULL,
+                                      autoEngState->buf, NULL,
+                                      autoEngState->config.maxHintLength,
+                                      "en", "cus", AutoEngGetCandWordCb,
+                                      autoEngState);
     if (candList) {
         FcitxInputState *input = FcitxInstanceGetInputState(autoEngState->owner);
         FcitxCandidateWordList *iList = FcitxInputStateGetCandidateList(input);
@@ -561,24 +620,25 @@ ShowAutoEngMessage(FcitxAutoEngState *autoEngState, INPUT_RETURN_VALUE *retval)
         FcitxInputStateSetRawInputBufferSize(input, buff_len);
     }
     if (buff_len > AUTOENG_MAX_PREEDIT) {
-        FcitxMessagesAddMessageAtLast(FcitxInputStateGetPreedit(input),
-                                      MSG_INPUT, "%s", autoEngState->buf +
-                                      buff_len - AUTOENG_MAX_PREEDIT);
+        FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
+                                             MSG_INPUT, autoEngState->buf +
+                                             buff_len - AUTOENG_MAX_PREEDIT);
         FcitxInputStateSetCursorPos(input, AUTOENG_MAX_PREEDIT);
     } else {
-        FcitxMessagesAddMessageAtLast(FcitxInputStateGetPreedit(input),
-                                      MSG_INPUT, "%s", autoEngState->buf);
+        FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
+                                             MSG_INPUT, autoEngState->buf);
         FcitxInputStateSetCursorPos(input, autoEngState->index);
     }
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetClientPreedit(input),
-                                  MSG_INPUT, "%s", autoEngState->buf);
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetClientPreedit(input),
+                                         MSG_INPUT, autoEngState->buf);
 
     FcitxInputStateSetClientCursorPos(input, autoEngState->index);
     FcitxInputStateSetShowCursor(input, true);
 
     AutoEngGetSpellHint(autoEngState);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxDown(input),
-                                  MSG_TIPS, _("Press Enter to input text"));
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetAuxDown(input),
+                                         MSG_TIPS,
+                                         _("Press Enter to input text"));
     *retval |= IRV_FLAG_UPDATE_INPUT_WINDOW;
 }
 
