@@ -49,8 +49,7 @@ static void* DBusCreate(FcitxInstance* instance);
 static void DBusSetFD(void* arg);
 static void DBusProcessEvent(void* arg);
 static void DBusDestroy(void* arg);
-static void* DBusGetConnection(void* arg, FcitxModuleFunctionArg args);
-static void* DBusGetPrivateConnection(void* arg, FcitxModuleFunctionArg args);
+DECLARE_ADDFUNCTIONS(DBus)
 
 FCITX_DEFINE_PLUGIN(fcitx_dbus, module, FcitxModule) = {
     DBusCreate,
@@ -60,7 +59,8 @@ FCITX_DEFINE_PLUGIN(fcitx_dbus, module, FcitxModule) = {
     NULL
 };
 
-DBusHandlerResult DBusModuleFilter(DBusConnection* connection, DBusMessage* msg, void* user_data)
+DBusHandlerResult
+DBusModuleFilter(DBusConnection* connection, DBusMessage* msg, void* user_data)
 {
     FCITX_UNUSED(connection);
 
@@ -80,7 +80,6 @@ DBusHandlerResult DBusModuleFilter(DBusConnection* connection, DBusMessage* msg,
 void* DBusCreate(FcitxInstance* instance)
 {
     FcitxDBus *dbusmodule = (FcitxDBus*) fcitx_utils_malloc0(sizeof(FcitxDBus));
-    FcitxAddon* dbusaddon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_DBUS_NAME);
     dbusmodule->owner = instance;
 
     DBusError err;
@@ -98,7 +97,8 @@ void* DBusCreate(FcitxInstance* instance)
     int retry = 0;
     DBusConnection* conn = NULL;
     char* servicename = NULL;
-    asprintf(&servicename, "%s-%d", FCITX_DBUS_SERVICE, fcitx_utils_get_display_number());
+    asprintf(&servicename, "%s-%d", FCITX_DBUS_SERVICE,
+             fcitx_utils_get_display_number());
 
     /* do session dbus initialize */
     do {
@@ -130,8 +130,9 @@ void* DBusCreate(FcitxInstance* instance)
         if (!dbus_connection_add_filter(conn, DBusModuleFilter, dbusmodule, NULL))
             break;
 
-        if (!dbus_connection_set_watch_functions(conn, DBusAddWatch, DBusRemoveWatch,
-                NULL, &dbusmodule->watches, NULL)) {
+        if (!dbus_connection_set_watch_functions(conn, DBusAddWatch,
+                                                 DBusRemoveWatch, NULL,
+                                                 &dbusmodule->watches, NULL)) {
             FcitxLog(WARNING, "Add Watch Function Error");
             dbus_error_free(&err);
             dbus_error_init(&err);
@@ -210,7 +211,8 @@ void* DBusCreate(FcitxInstance* instance)
         privconn = dbus_connection_open(dbusmodule->daemon.address, &err);
 
         if (dbus_error_is_set(&err)) {
-            FcitxLog(ERROR, "Private dbus daemon connection error (%s)", err.message);
+            FcitxLog(ERROR, "Private dbus daemon connection error (%s)",
+                     err.message);
             break;
         }
 
@@ -237,15 +239,17 @@ void* DBusCreate(FcitxInstance* instance)
         if (!dbus_connection_add_filter(privconn, DBusModuleFilter, dbusmodule, NULL))
             break;
 
-        if (!dbus_connection_set_watch_functions(privconn, DBusAddWatch, DBusRemoveWatch,
-                NULL, &dbusmodule->watches, NULL)) {
+        if (!dbus_connection_set_watch_functions(privconn, DBusAddWatch,
+                                                 DBusRemoveWatch, NULL,
+                                                 &dbusmodule->watches, NULL)) {
             FcitxLog(WARNING, "Add Watch Function Error");
             break;
         }
 
         char* addressFile = NULL;
         char* localMachineId = dbus_get_local_machine_id();
-        asprintf(&addressFile, "%s-%d", localMachineId, fcitx_utils_get_display_number());
+        asprintf(&addressFile, "%s-%d", localMachineId,
+                 fcitx_utils_get_display_number());
         dbus_free(localMachineId);
 
         FILE* fp = FcitxXDGGetFileUserWithPrefix("dbus", addressFile, "w", NULL);
@@ -284,12 +288,10 @@ void* DBusCreate(FcitxInstance* instance)
         }
     }
 
-    AddFunction(dbusaddon, DBusGetConnection);
-    AddFunction(dbusaddon, DBusGetPrivateConnection);
+    FcitxDBusAddFunctions(instance);
     dbus_error_free(&err);
 
     dbusmodule->serviceName = servicename;
-
     return dbusmodule;
 
 dbus_init_failed:
@@ -315,18 +317,6 @@ void DBusDestroy(void* arg) {
     free(dbusmodule);
 }
 
-void* DBusGetConnection(void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxDBus* dbusmodule = (FcitxDBus*)arg;
-    return dbusmodule->conn;
-}
-
-void* DBusGetPrivateConnection(void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxDBus* dbusmodule = (FcitxDBus*)arg;
-    return dbusmodule->privconn;
-}
-
 void DBusSetFD(void* arg)
 {
     FcitxDBus* dbusmodule = (FcitxDBus*) arg;
@@ -352,4 +342,4 @@ void DBusProcessEvent(void* arg)
     DBusProcessEventForConnection(dbusmodule->privconn);
 }
 
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-dbus-addfunctions.h"
