@@ -33,24 +33,39 @@
 #include "fcitx-config/xdg.h"
 #include "fcitx-utils/log.h"
 
-typedef struct _IMSelector {
+#define SELECTOR_COUNT 9
+
+typedef struct _IMSelector IMSelector;
+
+typedef struct _SelectorHandle {
+    int idx;
+    boolean global;
+    IMSelector* imselector;
+} SelectorHandle;
+
+struct _IMSelector {
     FcitxGenericConfig gconfig;
     FcitxHotkey localKey[2];
     FcitxHotkey globalKey[2];
+    FcitxHotkey clearLocalKey[2];
+    FcitxHotkey selectorKey[2][SELECTOR_COUNT][2];
+    SelectorHandle handle[2][SELECTOR_COUNT];
     boolean triggered;
     boolean global;
     FcitxInstance* owner;
-} IMSelector;
+};
 
 static void* IMSelectorCreate(FcitxInstance* instance);
 static boolean IMSelectorPreFilter(void* arg, FcitxKeySym sym,
-                                    unsigned int state,
-                                    INPUT_RETURN_VALUE *retval
+                                   unsigned int state,
+                                   INPUT_RETURN_VALUE *retval
                                    );
 static  void IMSelectorReset(void* arg);
 static  void IMSelectorReload(void* arg);
 static INPUT_RETURN_VALUE IMSelectorLocalTrigger(void* arg);
 static INPUT_RETURN_VALUE IMSelectorGlobalTrigger(void* arg);
+static INPUT_RETURN_VALUE IMSelectorClearLocal(void* arg);
+static INPUT_RETURN_VALUE IMSelectorSelect(void* arg);
 static boolean LoadIMSelectorConfig(IMSelector* imselector);
 static void SaveIMSelectorConfig(IMSelector* imselector);
 static void IMSelectorGetCands(IMSelector* imselector);
@@ -66,6 +81,25 @@ FCITX_DEFINE_PLUGIN(fcitx_imselector, module, FcitxModule) = {
 CONFIG_BINDING_BEGIN(IMSelector)
 CONFIG_BINDING_REGISTER("IMSelector", "LocalInputMethodSelectKey", localKey)
 CONFIG_BINDING_REGISTER("IMSelector", "GlobalInputMethodSelectKey", globalKey)
+CONFIG_BINDING_REGISTER("IMSelector", "ClearLocal", clearLocalKey)
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM1", selectorKey[1][0])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM2", selectorKey[1][1])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM3", selectorKey[1][2])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM4", selectorKey[1][3])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM5", selectorKey[1][4])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM6", selectorKey[1][5])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM7", selectorKey[1][6])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM8", selectorKey[1][7])
+CONFIG_BINDING_REGISTER("GlobalSelector", "IM9", selectorKey[1][8])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM1", selectorKey[0][0])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM2", selectorKey[0][1])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM3", selectorKey[0][2])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM4", selectorKey[0][3])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM5", selectorKey[0][4])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM6", selectorKey[0][5])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM7", selectorKey[0][6])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM8", selectorKey[0][7])
+CONFIG_BINDING_REGISTER("LocalSelector", "IM9", selectorKey[0][8])
 CONFIG_BINDING_END()
 
 void* IMSelectorCreate(FcitxInstance* instance)
@@ -93,6 +127,43 @@ void* IMSelectorCreate(FcitxInstance* instance)
     hkhk.hotkey = imselector->globalKey;
     FcitxInstanceRegisterHotkeyFilter(instance, hkhk);
 
+    hkhk.arg = imselector;
+    hkhk.hotkeyhandle = IMSelectorClearLocal;
+    hkhk.hotkey = imselector->clearLocalKey;
+    FcitxInstanceRegisterHotkeyFilter(instance, hkhk);
+
+    /* this key is ignore the very first input method which is for inactive */
+#define _ADD_HANDLE(X, ISGLOBAL) \
+    do { \
+        SelectorHandle* handle = &imselector->handle[ISGLOBAL][X - 1]; \
+        handle->global = ISGLOBAL; \
+        handle->idx = X; \
+        handle->imselector = imselector; \
+        hkhk.arg = handle; \
+        hkhk.hotkeyhandle = IMSelectorSelect; \
+        hkhk.hotkey = imselector->selectorKey[ISGLOBAL][X - 1]; \
+        FcitxInstanceRegisterHotkeyFilter(instance, hkhk); \
+    } while(0);
+
+    _ADD_HANDLE(1, false);
+    _ADD_HANDLE(2, false);
+    _ADD_HANDLE(3, false);
+    _ADD_HANDLE(4, false);
+    _ADD_HANDLE(5, false);
+    _ADD_HANDLE(6, false);
+    _ADD_HANDLE(7, false);
+    _ADD_HANDLE(8, false);
+    _ADD_HANDLE(9, false);
+    _ADD_HANDLE(1, true);
+    _ADD_HANDLE(2, true);
+    _ADD_HANDLE(3, true);
+    _ADD_HANDLE(4, true);
+    _ADD_HANDLE(5, true);
+    _ADD_HANDLE(6, true);
+    _ADD_HANDLE(7, true);
+    _ADD_HANDLE(8, true);
+    _ADD_HANDLE(9, true);
+
     FcitxIMEventHook resethk;
     resethk.arg = imselector;
     resethk.func = IMSelectorReset;
@@ -108,21 +179,13 @@ boolean IMSelectorPreFilter(void* arg, FcitxKeySym sym, unsigned int state, INPU
     FcitxGlobalConfig* fc = FcitxInstanceGetGlobalConfig(instance);
     if (!imselector->triggered)
         return false;
-
-    const FcitxHotkey* hkPrevPage = FcitxInstanceGetContextHotkey(instance, CONTEXT_ALTERNATIVE_PREVPAGE_KEY);
-    if (hkPrevPage == NULL)
-        hkPrevPage = fc->hkPrevPage;
-
-    const FcitxHotkey* hkNextPage = FcitxInstanceGetContextHotkey(instance, CONTEXT_ALTERNATIVE_NEXTPAGE_KEY);
-    if (hkNextPage == NULL)
-        hkNextPage = fc->hkNextPage;
-
     FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
-
-    if (FcitxHotkeyIsHotKey(sym, state, hkPrevPage)) {
+    if (FcitxHotkeyIsHotKey(sym, state,
+                            FcitxConfigPrevPageKey(instance, fc))) {
         FcitxCandidateWordGoPrevPage(candList);
         *retval = IRV_DISPLAY_MESSAGE;
-    } else if (FcitxHotkeyIsHotKey(sym, state, hkNextPage)) {
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigNextPageKey(instance, fc))) {
         FcitxCandidateWordGoNextPage(candList);
         *retval = IRV_DISPLAY_MESSAGE;
     } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_SPACE)) {
@@ -134,9 +197,9 @@ boolean IMSelectorPreFilter(void* arg, FcitxKeySym sym, unsigned int state, INPU
             *retval = FcitxCandidateWordChooseByIndex(candList, iKey);
     } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ESCAPE)) {
         *retval = IRV_CLEAN;
-    } else {
-        *retval = IRV_DO_NOTHING;
     }
+    if (*retval == IRV_TO_PROCESS)
+        *retval = IRV_DO_NOTHING;
     return true;
 }
 
@@ -158,6 +221,34 @@ INPUT_RETURN_VALUE IMSelectorLocalTrigger(void* arg)
     return IRV_DISPLAY_MESSAGE;
 }
 
+INPUT_RETURN_VALUE IMSelectorClearLocal(void* arg)
+{
+    IMSelector* imselector = arg;
+    FcitxInstance* instance = imselector->owner;
+    FcitxInstanceSetLocalIMName(instance, FcitxInstanceGetCurrentIC(instance), NULL);
+    return IRV_CLEAN;
+}
+
+INPUT_RETURN_VALUE IMSelectorSelect(void* arg)
+{
+    SelectorHandle* handle = arg;
+    IMSelector* imselector = handle->imselector;
+    FcitxInstance* instance = imselector->owner;
+    FcitxIM* im = FcitxInstanceGetIMByIndex(instance, handle->idx);
+    if (!im)
+        return IRV_TO_PROCESS;
+    if (handle->global) {
+        FcitxInstanceSwitchIMByIndex(instance, handle->idx);
+    }
+    else {
+        FcitxInstanceSetLocalIMName(instance, FcitxInstanceGetCurrentIC(instance), im->uniqueName);
+
+        if (FcitxInstanceGetCurrentState(instance) != IS_ACTIVE)
+            FcitxInstanceEnableIM(instance, FcitxInstanceGetCurrentIC(instance), false);
+    }
+    return IRV_CLEAN;
+}
+
 INPUT_RETURN_VALUE IMSelectorGetCand(void* arg, FcitxCandidateWord* candWord)
 {
     IMSelector* imselector = arg;
@@ -200,16 +291,20 @@ void IMSelectorGetCands(IMSelector* imselector)
     if (!ic)
         return;
 
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, "%s", imselector->global ? _("Select global input method: ") : _("Select local input method: "));
+    FcitxMessages *aux_up = FcitxInputStateGetAuxUp(input);
+    FcitxMessagesAddMessageStringsAtLast(aux_up, MSG_TIPS, imselector->global ?
+                                         _("Select global input method: ") :
+                                         _("Select local input method: "));
     if (ic2->imname) {
         int idx = FcitxInstanceGetIMIndexByName(instance, ic2->imname);
-        FcitxIM* im = (FcitxIM*) utarray_eltptr(imes, idx);
+        FcitxIM *im = fcitx_array_eltptr(imes, idx);
         if (im) {
-            FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Current local input method is %s"), im->strName);
+            FcitxMessagesAddMessageStringsAtLast(aux_up, MSG_TIPS,
+                _("Current local input method is "), im->strName);
         }
-    }
-    else {
-        FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("No local input method"));
+    } else {
+        FcitxMessagesAddMessageStringsAtLast(aux_up, MSG_TIPS,
+                                             _("No local input method"));
     }
     for (pim = (FcitxIM *) utarray_front(imes);
             pim != NULL;
