@@ -50,9 +50,9 @@
 
 struct _FcitxPuncState;
 typedef struct _WidePunc {
-    char            ASCII;
-    char            strWidePunc[MAX_PUNC_NO][MAX_PUNC_LENGTH * UTF8_MAX_LENGTH + 1];
-    unsigned        iCount: 2;
+    char ASCII;
+    char strWidePunc[MAX_PUNC_NO][MAX_PUNC_LENGTH * UTF8_MAX_LENGTH + 1];
+    unsigned iCount: 2;
 } WidePunc;
 
 typedef struct _PuncWhich {
@@ -74,8 +74,6 @@ static void FreePunc(struct _FcitxPuncState* puncState);
 static void* PuncCreate(FcitxInstance* instance);
 static boolean PuncPreFilter(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retVal);
 static boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retVal);
-static void* PuncGetPunc(void* a, FcitxModuleFunctionArg arg);
-static void* PuncGetPunc2(void* a, FcitxModuleFunctionArg arg);
 static void TogglePuncState(void *arg);
 static boolean GetPuncState(void *arg);
 static void ReloadPunc(void *arg);
@@ -89,6 +87,7 @@ static void* PuncWhichAlloc(void* arg);
 static void* PuncWhichCopy(void* arg, void* data, void* src);
 static void PuncWhichFree(void* arg, void* data);
 
+DECLARE_ADDFUNCTIONS(Punc)
 
 typedef struct _FcitxPuncState {
     char cLastIsAutoConvert;
@@ -110,7 +109,6 @@ FCITX_DEFINE_PLUGIN(fcitx_punc, module, FcitxModule) = {
 void* PuncCreate(FcitxInstance* instance)
 {
     FcitxPuncState* puncState = fcitx_utils_malloc0(sizeof(FcitxPuncState));
-    FcitxAddon* puncaddon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_PUNC_NAME);
     puncState->owner = instance;
     LoadPuncDict(puncState);
     FcitxKeyFilterHook hk;
@@ -141,24 +139,28 @@ void* PuncCreate(FcitxInstance* instance)
 
     FcitxInstanceRegisterInputUnFocusHook(instance, hook);
 
-    FcitxInstanceWatchContext(instance, CONTEXT_IM_LANGUAGE, PuncLanguageChanged, puncState);
+    FcitxInstanceWatchContext(instance, CONTEXT_IM_LANGUAGE,
+                              PuncLanguageChanged, puncState);
 
     FcitxProfile* profile = FcitxInstanceGetProfile(instance);
     FcitxUIRegisterStatus(instance, puncState, "punc",
-                          profile->bUseWidePunc ? _("Full width punct") :  _("Latin punct"),
-                          _("Toggle Full Width Punctuation"), TogglePuncState, GetPuncState);
+                          profile->bUseWidePunc ? _("Full width punct") :
+                          _("Latin punct"),
+                          _("Toggle Full Width Punctuation"), TogglePuncState,
+                          GetPuncState);
 
-    puncState->slot = FcitxInstanceAllocDataForIC(instance, PuncWhichAlloc, PuncWhichCopy, PuncWhichFree, puncState);
+    puncState->slot = FcitxInstanceAllocDataForIC(instance, PuncWhichAlloc,
+                                                  PuncWhichCopy, PuncWhichFree,
+                                                  puncState);
 
-    AddFunction(puncaddon, PuncGetPunc);
-    AddFunction(puncaddon, PuncGetPunc2);
+    FcitxPuncAddFunctions(instance);
     return puncState;
 }
 
-void* PuncWhichAlloc(void* arg)
+void* PuncWhichAlloc(void *arg)
 {
-    FcitxPunc* puncState = arg;
-    PuncWhich* which = fcitx_utils_new(PuncWhich);
+    FcitxPunc *puncState = arg;
+    PuncWhich *which = fcitx_utils_new(PuncWhich);
     which->lastPunc = puncState->curPunc;
     which->bitset = fcitx_bitset_new(256);
     return which;
@@ -166,8 +168,9 @@ void* PuncWhichAlloc(void* arg)
 
 void* PuncWhichCopy(void* arg, void* data, void* src)
 {
-    PuncWhich* which = data;
-    PuncWhich* whichsrc = src;
+    FCITX_UNUSED(arg);
+    PuncWhich *which = data;
+    PuncWhich *whichsrc = src;
     which->lastPunc = whichsrc->lastPunc;
     memcpy(which->bitset, whichsrc->bitset, fcitx_bitset_size(256));
     return data;
@@ -175,7 +178,8 @@ void* PuncWhichCopy(void* arg, void* data, void* src)
 
 void PuncWhichFree(void* arg, void* data)
 {
-    PuncWhich* which = data;
+    FCITX_UNUSED(arg);
+    PuncWhich *which = data;
     free(which->bitset);
     free(data);
 }
@@ -197,28 +201,17 @@ void PuncLanguageChanged(void* arg, const void* value)
     FcitxUISetStatusVisable (puncState->owner, "punc",  puncState->curPunc != NULL) ;
 }
 
-void* PuncGetPunc(void* a, FcitxModuleFunctionArg arg)
+static void
+PuncGetPunc2(FcitxPuncState *puncState, int key, char **p1, char **p2)
 {
-    FcitxPuncState* puncState = (FcitxPuncState*) a;
-    int *key = arg.args[0];
-    return GetPunc(puncState, *key);
-}
-
-
-void* PuncGetPunc2(void* a, FcitxModuleFunctionArg arg)
-{
-    FcitxPuncState* puncState = (FcitxPuncState*) a;
-    int *key = arg.args[0];
-    char** p1 = arg.args[1];
-    char** p2 = arg.args[2];
-    int             iIndex = 0;
-    WidePunc       *curPunc = puncState->curPunc;
+    int iIndex = 0;
+    WidePunc *curPunc = puncState->curPunc;
 
     if (!curPunc)
-        return NULL;
+        return;
 
     while (curPunc[iIndex].ASCII) {
-        if (curPunc[iIndex].ASCII == *key) {
+        if (curPunc[iIndex].ASCII == key) {
             if (p1)
                 *p1 = curPunc[iIndex].strWidePunc[0];
             if (curPunc[iIndex].iCount > 1 && p2)
@@ -227,8 +220,6 @@ void* PuncGetPunc2(void* a, FcitxModuleFunctionArg arg)
         }
         iIndex++;
     }
-
-    return NULL;
 }
 
 void ResetPunc(void* arg)
@@ -253,15 +244,14 @@ void ResetPuncWhichStatus(void* arg)
     fcitx_bitset_clear(puncWhich->bitset);
 }
 
-boolean PuncPreFilter(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retVal)
+boolean PuncPreFilter(void* arg, FcitxKeySym sym, unsigned int state,
+                      INPUT_RETURN_VALUE* retVal)
 {
-    if (!FcitxHotkeyIsHotKeySimple(sym, state))
-        return false;
-
-    FcitxPuncState* puncState = (FcitxPuncState*) arg;
-    if (!FcitxHotkeyIsHotKeyDigit(sym, state) && !IsHotKeyPunc(sym, state))
+    FCITX_UNUSED(retVal);
+    FcitxPuncState *puncState = (FcitxPuncState*)arg;
+    if (FcitxHotkeyIsHotKeySimple(sym, state) &&
+        !FcitxHotkeyIsHotKeyDigit(sym, state) && !IsHotKeyPunc(sym, state))
         puncState->bLastIsNumber = false;
-
     return false;
 }
 
@@ -278,10 +268,26 @@ boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN
     if (*retVal != IRV_TO_PROCESS)
         return false;
 
+    FcitxCandidateWordList *candList = FcitxInputStateGetCandidateList(input);
+    if (FcitxCandidateWordGetListSize(candList) != 0) {
+        if (FcitxCandidateWordGetHasGoneToNextPage(candList) &&
+            FcitxHotkeyIsHotKey(sym, state,
+                                FcitxConfigPrevPageKey(instance, config))) {
+            return false;
+        }
+        /*
+         * comparing with upper case, if paging is occupied,
+         * punc will not let next page pass
+         */
+        if (FcitxHotkeyIsHotKey(sym, state,
+                                FcitxConfigNextPageKey(instance, config))) {
+            return false;
+        }
+    }
+
     FcitxKeySym origsym = sym;
     sym = FcitxHotkeyPadToMain(sym);
     if (profile->bUseWidePunc) {
-
         if (puncState->bLastIsNumber && config->bEngPuncAfterNumber
                 && (FcitxHotkeyIsHotKey(origsym, state, FCITX_PERIOD)
                     || FcitxHotkeyIsHotKey(origsym, state, FCITX_SEMICOLON)
@@ -302,7 +308,7 @@ boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN
         FcitxInputStateGetOutputString(input)[0] = '\0';
         INPUT_RETURN_VALUE ret = IRV_TO_PROCESS;
         if (!FcitxInputStateGetIsInRemind(input))
-            ret = FcitxCandidateWordChooseByIndex(FcitxInputStateGetCandidateList(input), 0);
+            ret = FcitxCandidateWordChooseByTotalIndex(FcitxInputStateGetCandidateList(input), 0);
 
         /* if there is nothing to commit */
         if (ret == IRV_TO_PROCESS) {
@@ -342,12 +348,10 @@ boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN
             puncState->cLastIsAutoConvert = 0;
             *retVal = IRV_DO_NOTHING;
             return true;
-        } else if (FcitxHotkeyIsHotKeySimple(sym, state)) {
-            if (FcitxHotkeyIsHotKeyDigit(sym, state))
-                puncState->bLastIsNumber = true;
-            else {
-                puncState->bLastIsNumber = false;
-            }
+        } else if (FcitxHotkeyIsHotKeyDigit(sym, state)) {
+            puncState->bLastIsNumber = true;
+        } else {
+            puncState->bLastIsNumber = false;
         }
     }
     puncState->cLastIsAutoConvert = 0;
@@ -516,25 +520,27 @@ static inline void SetPuncWhich(FcitxPuncState* puncState, WidePunc* punc)
  * 根据字符得到相应的标点符号
  * 如果该字符不在标点符号集中，则返回NULL
  */
-char           *GetPunc(FcitxPuncState* puncState, int iKey)
+static char*
+GetPunc(FcitxPuncState *puncState, int iKey)
 {
-    int             iIndex = 0;
-    char           *pPunc;
-    WidePunc       *curPunc = puncState->curPunc;
+    int iIndex = 0;
+    char *pPunc;
+    WidePunc *curPunc = puncState->curPunc;
 
     if (!curPunc)
-        return (char *) NULL;
+        return NULL;
 
     while (curPunc[iIndex].ASCII) {
         if (curPunc[iIndex].ASCII == iKey) {
-            pPunc = curPunc[iIndex].strWidePunc[GetPuncWhich(puncState, &curPunc[iIndex])];
+            pPunc = curPunc[iIndex].strWidePunc[GetPuncWhich(puncState,
+                                                             &curPunc[iIndex])];
             SetPuncWhich(puncState, &curPunc[iIndex]);
             return pPunc;
         }
         iIndex++;
     }
 
-    return (char *) NULL;
+    return NULL;
 }
 
 void TogglePuncState(void* arg)
@@ -545,8 +551,9 @@ void TogglePuncState(void* arg)
     profile->bUseWidePunc = !profile->bUseWidePunc;
 
     FcitxUISetStatusString(puncState->owner, "punc",
-                           profile->bUseWidePunc ? _("Full width punct") :  _("Latin punct"),
-                          _("Toggle Full Width Punctuation"));
+                           profile->bUseWidePunc ? _("Full width punct") :
+                           _("Latin punct"),
+                           _("Toggle Full Width Punctuation"));
     FcitxProfileSave(profile);
 }
 
@@ -591,6 +598,4 @@ boolean IsHotKeyPunc(FcitxKeySym sym, unsigned int state)
 
     return false;
 }
-
-
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-punc-addfunctions.h"
