@@ -37,9 +37,9 @@
 #include "fcitx/frontend.h"
 #include "fcitx/candidate.h"
 #include "fcitx/context.h"
-#include "module/punc/punc.h"
-#include "module/lua/luamod.h"
-#include "module/spell/spell.h"
+#include "module/punc/fcitx-punc.h"
+#include "module/lua/fcitx-lua.h"
+#include "module/spell/fcitx-spell.h"
 
 #include "fcitx/ime.h"
 
@@ -61,6 +61,7 @@ typedef enum {
     QPCM_ALT,
     QPCM_CTRL,
     QPCM_SHIFT,
+    _QPCM_COUNT
 } QuickPhraseChooseModifier;
 
 typedef struct {
@@ -94,7 +95,7 @@ static void FreeQuickPhrase(void* arg);
 static void ReloadQuickPhrase(void* arg);
 static INPUT_RETURN_VALUE QuickPhraseDoInput(void* arg, FcitxKeySym sym, int state);
 static INPUT_RETURN_VALUE QuickPhraseGetCandWords(QuickPhraseState* qpstate);
-static UT_icd qp_icd = {sizeof(QUICK_PHRASE), NULL, NULL, NULL};
+static UT_icd qp_icd = { sizeof(QUICK_PHRASE), NULL, NULL, NULL };
 static void ShowQuickPhraseMessage(QuickPhraseState *qpstate);
 static INPUT_RETURN_VALUE QuickPhraseGetCandWord(void* arg, FcitxCandidateWord* candWord);
 static INPUT_RETURN_VALUE QuickPhraseGetLuaCandWord(void* arg, FcitxCandidateWord* candWord);
@@ -109,8 +110,8 @@ static boolean QuickPhrasePreFilter(void* arg, FcitxKeySym sym,
                                     INPUT_RETURN_VALUE *retval
     );
 static void QuickPhraseReset(void* arg);
-static void* QuickPhraseLaunch(void* arg, FcitxModuleFunctionArg args);
 static void _QuickPhraseLaunch(QuickPhraseState* qpstate);
+DECLARE_ADDFUNCTIONS(QuickPhrase)
 
 FCITX_DEFINE_PLUGIN(fcitx_quickphrase, module, FcitxModule) = {
     QuickPhraseCreate,
@@ -164,9 +165,9 @@ int PhraseCmpA(const void* a, const void* b)
     }
 }
 
-void * QuickPhraseCreate(FcitxInstance *instance)
+void *QuickPhraseCreate(FcitxInstance *instance)
 {
-    QuickPhraseState *qpstate = fcitx_utils_malloc0(sizeof(QuickPhraseState));
+    QuickPhraseState *qpstate = fcitx_utils_new(QuickPhraseState);
     qpstate->owner = instance;
     qpstate->enabled = false;
 
@@ -178,7 +179,7 @@ void * QuickPhraseCreate(FcitxInstance *instance)
     LoadQuickPhrase(qpstate);
 
     FcitxKeyFilterHook hk;
-    hk.arg = qpstate ;
+    hk.arg = qpstate;
     hk.func = QuickPhrasePostFilter;
     FcitxInstanceRegisterPostInputFilter(instance, hk);
 
@@ -190,29 +191,23 @@ void * QuickPhraseCreate(FcitxInstance *instance)
     resethk.func = QuickPhraseReset;
     FcitxInstanceRegisterResetInputHook(instance, resethk);
 
-    FcitxInstanceRegisterWatchableContext(instance, CONTEXT_DISABLE_QUICKPHRASE, FCT_Boolean, FCF_ResetOnInputMethodChange);
-
-    FcitxAddon* addon = FcitxAddonsGetAddonByName(
-        FcitxInstanceGetAddons(instance),
-        FCITX_QUICKPHRASE_NAME);
-    AddFunction(addon, QuickPhraseLaunch);
+    FcitxInstanceRegisterWatchableContext(instance, CONTEXT_DISABLE_QUICKPHRASE,
+                                          FCT_Boolean,
+                                          FCF_ResetOnInputMethodChange);
 
+    FcitxQuickPhraseAddFunctions(instance);
     return qpstate;
 }
 
-void* QuickPhraseLaunch(void* arg, FcitxModuleFunctionArg args)
+static void
+QuickPhraseLaunch(QuickPhraseState *qpstate, int key, boolean useDup,
+                  boolean append)
 {
-    int* key = args.args[0];
-    boolean* useDup = args.args[1];
-    boolean* append = args.args[2];
-    QuickPhraseState *qpstate = (QuickPhraseState*) arg;
-    qpstate->curTriggerKey[0].sym = *key;
-    qpstate->useDupKeyInput = *useDup;
-    qpstate->append = *append;
+    qpstate->curTriggerKey[0].sym = key;
+    qpstate->useDupKeyInput = useDup;
+    qpstate->append = append;
     _QuickPhraseLaunch(qpstate);
     FcitxUIUpdateInputWindow(qpstate->owner);
-
-    return (void*) true;
 }
 
 void QuickPhraseFillKeyString(QuickPhraseState* qpstate, char c[2])
@@ -239,8 +234,8 @@ void _QuickPhraseLaunch(QuickPhraseState* qpstate)
     ShowQuickPhraseMessage(qpstate);
 
     if (c[0]) {
-        FcitxKeySym s = qpstate->curTriggerKey[0].sym;
-        char *strTemp = InvokeVaArgs(qpstate->owner, FCITX_PUNC, GETPUNC, &s);
+        int s = qpstate->curTriggerKey[0].sym;
+        char *strTemp = FcitxPuncGetPunc(qpstate->owner, &s);
         const char* full = strTemp ? strTemp : c;
         FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxDown(input), MSG_TIPS, _("Space for %s Enter for %s") , full, c);
     }
@@ -321,6 +316,7 @@ void FreeQuickPhrase(void *arg)
         return;
 
     utarray_free(qpstate->quickPhrases);
+    qpstate->quickPhrases = NULL;
 }
 
 void ShowQuickPhraseMessage(QuickPhraseState *qpstate)
@@ -331,9 +327,14 @@ void ShowQuickPhraseMessage(QuickPhraseState *qpstate)
     FcitxInputStateSetCursorPos(input, strlen(qpstate->buffer));
     FcitxInputStateSetClientCursorPos(input, strlen(qpstate->buffer) + strlen(c));
     FcitxInstanceCleanInputWindowUp(qpstate->owner);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxUp(input), MSG_TIPS, _("Quick Phrase: %s"), (qpstate->append) ? c : "");
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetPreedit(input), MSG_INPUT, "%s", qpstate->buffer);
-    FcitxMessagesAddMessageAtLast(FcitxInputStateGetClientPreedit(input), MSG_INPUT, "%s%s", (qpstate->append) ? c : "", qpstate->buffer);
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetAuxUp(input),
+                                         MSG_TIPS, _("Quick Phrase: "),
+                                         (qpstate->append) ? c : "");
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetPreedit(input),
+                                         MSG_INPUT, qpstate->buffer);
+    FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetClientPreedit(input),
+                                         MSG_INPUT, (qpstate->append) ? c : "",
+                                         qpstate->buffer);
 }
 
 boolean QuickPhrasePreFilter(void *arg, FcitxKeySym sym,
@@ -355,9 +356,8 @@ boolean QuickPhrasePreFilter(void *arg, FcitxKeySym sym,
             ((qpstate->useDupKeyInput &&
               FcitxHotkeyIsHotKey(keymain, state, qpstate->curTriggerKey)) ||
              FcitxHotkeyIsHotKey(keymain, state, FCITX_SPACE))) {
-            FcitxKeySym s = qpstate->curTriggerKey[0].sym;
-            char *strTemp = InvokeVaArgs(qpstate->owner, FCITX_PUNC,
-                                         GETPUNC, &s);
+            int s = qpstate->curTriggerKey[0].sym;
+            char *strTemp = FcitxPuncGetPunc(qpstate->owner, &s);
             strcpy(FcitxInputStateGetOutputString(input), strTemp ? strTemp : c);
             *retval = IRV_COMMIT_STRING;
         } else {
@@ -380,10 +380,12 @@ boolean QuickPhrasePreFilter(void *arg, FcitxKeySym sym,
             *retval = QuickPhraseGetCandWords(qpstate);
         }
     } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
-        if (strlen(qpstate->buffer) > 0) {
+        size_t len = strlen(qpstate->buffer);
+        if (len > 0) {
             if (qpstate->append) {
-                sprintf(FcitxInputStateGetOutputString(input),
-                        "%s%s", c, qpstate->buffer);
+                fcitx_utils_cat_str(FcitxInputStateGetOutputString(input),
+                                    2, (const char*[]){c, qpstate->buffer},
+                                    (size_t[]){strlen(c), len});
             } else {
                 strcpy(FcitxInputStateGetOutputString(input),
                        qpstate->buffer);
@@ -402,9 +404,9 @@ boolean QuickPhrasePreFilter(void *arg, FcitxKeySym sym,
     if (*retval == IRV_DISPLAY_MESSAGE) {
         FcitxMessagesSetMessageCount(FcitxInputStateGetAuxDown(input), 0);
         if (!FcitxCandidateWordPageCount(FcitxInputStateGetCandidateList(input)))
-            FcitxMessagesAddMessageAtLast(FcitxInputStateGetAuxDown(input),
-                                          MSG_TIPS, "%s",
-                                          _("Press Enter to input text"));
+            FcitxMessagesAddMessageStringsAtLast(FcitxInputStateGetAuxDown(input),
+                                                 MSG_TIPS,
+                                                 _("Press Enter to input text"));
     }
     return true;
 }
@@ -462,38 +464,70 @@ void QuickPhraseReset(void* arg)
     qpstate->append = false;
     memset(qpstate->curTriggerKey, 0, sizeof(FcitxHotkey) * 2);
 }
-INPUT_RETURN_VALUE QuickPhraseDoInput(void* arg, FcitxKeySym sym, int state)
+
+static INPUT_RETURN_VALUE
+QuickPhraseDoInput(void* arg, FcitxKeySym sym, int state)
 {
-    QuickPhraseState *qpstate = (QuickPhraseState*) arg;
-    FcitxInputState *input = FcitxInstanceGetInputState(qpstate->owner);
-    FcitxGlobalConfig* fc = FcitxInstanceGetGlobalConfig(qpstate->owner);
-    int retVal = IRV_TO_PROCESS;
-
-    const FcitxHotkey* hkPrevPage = FcitxInstanceGetContextHotkey(qpstate->owner, CONTEXT_ALTERNATIVE_PREVPAGE_KEY);
-    if (hkPrevPage == NULL)
-        hkPrevPage = fc->hkPrevPage;
-
-    const FcitxHotkey* hkNextPage = FcitxInstanceGetContextHotkey(qpstate->owner, CONTEXT_ALTERNATIVE_NEXTPAGE_KEY);
-    if (hkNextPage == NULL)
-        hkNextPage = fc->hkNextPage;
-
+    QuickPhraseState *qpstate = (QuickPhraseState*)arg;
+    FcitxInstance *instance = qpstate->owner;
+    FcitxInputState *input = FcitxInstanceGetInputState(instance);
+    FcitxGlobalConfig *fc = FcitxInstanceGetGlobalConfig(instance);
     int iKey = -1;
-    FcitxCandidateWordList *candList = FcitxInputStateGetCandidateList(input);
-    if (FcitxHotkeyIsHotKey(sym, state, hkPrevPage)) {
-        if (FcitxCandidateWordGoPrevPage(candList))
-            retVal = IRV_DISPLAY_MESSAGE;
-    } else if (FcitxHotkeyIsHotKey(sym, state, hkNextPage)) {
-        if (FcitxCandidateWordGoNextPage(candList))
-            retVal = IRV_DISPLAY_MESSAGE;
-    } else if ((iKey = FcitxCandidateWordCheckChooseKey(candList,
-                                                        sym, state)) >= 0) {
-        retVal = FcitxCandidateWordChooseByIndex(candList, iKey);
+    FcitxCandidateWordList *cand_list = FcitxInputStateGetCandidateList(input);
+    if (!FcitxCandidateWordGetListSize(cand_list))
+        return IRV_TO_PROCESS;
+    FcitxCandidateWord *cand_word;
+    if (FcitxHotkeyIsHotKey(sym, state, fc->nextWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetNext(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(cand_list, 0);
+            cand_word = FcitxCandidateWordGetFirst(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state, fc->prevWord)) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        cand_word = FcitxCandidateWordGetPrev(cand_list, cand_word);
+        if (!cand_word) {
+            FcitxCandidateWordSetPage(
+                cand_list, FcitxCandidateWordPageCount(cand_list) - 1);
+            cand_word = FcitxCandidateWordGetLast(cand_list);
+        } else {
+            FcitxCandidateWordSetFocus(
+                cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+        }
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigPrevPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (!FcitxCandidateWordGoPrevPage(cand_list)) {
+            FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+            return IRV_TO_PROCESS;
+        }
+        cand_word = FcitxCandidateWordGetCurrentWindow(cand_list) +
+            FcitxCandidateWordGetCurrentWindowSize(cand_list) - 1;
+    } else if (FcitxHotkeyIsHotKey(sym, state,
+                                   FcitxConfigNextPageKey(instance, fc))) {
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        if (!FcitxCandidateWordGoNextPage(cand_list)) {
+            FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+            return IRV_TO_PROCESS;
+        }
+        cand_word = FcitxCandidateWordGetCurrentWindow(cand_list);
+    } else if ((iKey = FcitxCandidateWordCheckChooseKey(cand_list, sym,
+                                                        state)) >= 0) {
+        return FcitxCandidateWordChooseByIndex(cand_list, iKey);
     } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_SPACE)) {
-        if (FcitxCandidateWordPageCount(candList) != 0)
-            retVal = FcitxCandidateWordChooseByIndex(candList, 0);
+        cand_word = FcitxCandidateWordGetFocus(cand_list, true);
+        return FcitxCandidateWordChooseByTotalIndex(
+            cand_list, FcitxCandidateWordGetIndex(cand_list, cand_word));
+    } else {
+        return IRV_TO_PROCESS;
     }
 
-    return retVal;
+    FcitxCandidateWordSetType(cand_word, MSG_CANDIATE_CURSOR);
+    return IRV_FLAG_UPDATE_INPUT_WINDOW;
 }
 
 static void
@@ -517,14 +551,13 @@ QuickPhraseGetSpellHint(QuickPhraseState* qpstate)
     char* search;
     char* needfree = NULL;
     if (qpstate->append) {
-        asprintf(&search, "%s%s", c, qpstate->buffer);
+        fcitx_utils_alloc_cat_str(search, c, qpstate->buffer);
         needfree = search;
     } else {
         search = qpstate->buffer;
     }
-    new_list = InvokeVaArgs(qpstate->owner, FCITX_SPELL, GET_CANDWORDS,
-                            NULL, search, NULL, (void*)(long)space_left,
-                            "en", "cus", NULL, NULL);
+    new_list = FcitxSpellGetCandWords(qpstate->owner, NULL, search, NULL,
+                                      space_left, "en", "cus", NULL, NULL);
     if (new_list) {
         FcitxCandidateWordMerge(cand_list, new_list, -1);
         FcitxCandidateWordFreeList(new_list);
@@ -544,18 +577,14 @@ INPUT_RETURN_VALUE QuickPhraseGetCandWords(QuickPhraseState* qpstate)
     int iLastQuickPhrase;
     int iFirstQuickPhrase;
     FcitxInstanceCleanInputWindowDown(qpstate->owner);
-
     FcitxCandidateWordSetPageSize(candList, config->iMaxCandWord);
     FcitxCandidateWordSetChooseAndModifier(
         candList, DIGIT_STR_CHOOSE, cmodtable[qpstate->config.chooseModifier]);
 
     pKey = &searchKey;
 
-    {
-        char *text = qpstate->buffer;
-        InvokeVaArgs(qpstate->owner, FCITX_LUA, CALLCOMMAND,
-                     text, QuickPhraseGetLuaCandWord, qpstate);
-    }
+    FcitxLuaCallCommand(qpstate->owner, qpstate->buffer,
+                        QuickPhraseGetLuaCandWord, qpstate);
 
     do {
         if (!qpstate->quickPhrases)
@@ -576,10 +605,10 @@ INPUT_RETURN_VALUE QuickPhraseGetCandWords(QuickPhraseState* qpstate)
             break;
         }
 
-        for (currentQuickPhrase = (QUICK_PHRASE*)utarray_eltptr(qpstate->quickPhrases,
-                                                                iFirstQuickPhrase);
+        for (currentQuickPhrase = fcitx_array_eltptr(qpstate->quickPhrases,
+                                                     iFirstQuickPhrase);
              currentQuickPhrase != NULL;
-             currentQuickPhrase = (QUICK_PHRASE*) utarray_next(qpstate->quickPhrases, currentQuickPhrase)) {
+             currentQuickPhrase = (QUICK_PHRASE*)utarray_next(qpstate->quickPhrases, currentQuickPhrase)) {
             if (!strncmp(qpstate->buffer, currentQuickPhrase->strCode,
                          iInputLen)) {
                 QuickPhraseCand *qpcand = fcitx_utils_new(QuickPhraseCand);
@@ -588,8 +617,9 @@ INPUT_RETURN_VALUE QuickPhraseGetCandWords(QuickPhraseState* qpstate)
                 candWord.callback = QuickPhraseGetCandWord;
                 candWord.owner = qpstate;
                 candWord.priv = qpcand;
-                candWord.strExtra = strdup(currentQuickPhrase->strCode +
-                                           iInputLen);
+                fcitx_utils_alloc_cat_str(candWord.strExtra, " ",
+                                          currentQuickPhrase->strCode +
+                                          iInputLen);
                 candWord.strWord = strdup(currentQuickPhrase->strPhrase);
                 candWord.wordType = MSG_OTHER;
                 candWord.extraType = MSG_CODE;
@@ -658,8 +688,8 @@ boolean LoadQuickPhraseConfig(QuickPhraseConfig *qpconfig)
 
     QuickPhraseConfigConfigBind(qpconfig, cfile, configDesc);
     FcitxConfigBindSync(&qpconfig->gconfig);
-    if (qpconfig->chooseModifier > QPCM_CTRL)
-        qpconfig->chooseModifier = QPCM_CTRL;
+    if (fcitx_unlikely(qpconfig->chooseModifier >= _QPCM_COUNT))
+        qpconfig->chooseModifier = _QPCM_COUNT - 1;
 
     if (fp)
         fclose(fp);
@@ -680,4 +710,4 @@ void SaveQuickPhraseConfig(QuickPhraseConfig* qpconfig)
         fclose(fp);
 }
 
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-quickphrase-addfunctions.h"
