@@ -4,11 +4,6 @@
  * original file is licensed under GPLv2+
  */
 
-#if defined(__linux__)
-#include <endian.h>
-#else
-#include <sys/endian.h>
-#endif
 #include <stdint.h>
 #include <ctype.h>
 #include <libintl.h>
@@ -16,6 +11,11 @@
 #include <fcitx-utils/utils.h>
 #include <fcitx-config/xdg.h>
 #include <fcitx/fcitx.h>
+#if defined(__linux__) || defined(__GLIBC__)
+#include <endian.h>
+#else
+#include <sys/endian.h>
+#endif
 #include "charselectdata.h"
 
 /* constants for hangul (de)composition, see UAX #15 */
@@ -38,24 +38,20 @@ typedef struct _UnicodeSet {
     UT_hash_handle hh;
 } UnicodeSet;
 
-const UT_icd int16_icd = {sizeof(int16_t), NULL, NULL, NULL};
-const UT_icd pidx_icd = {sizeof(CharSelectDataIndex*), NULL, NULL, NULL};
+static const UT_icd int16_icd = { sizeof(int16_t), NULL, NULL, NULL };
 
-static const char JAMO_L_TABLE[][4] =
-    {
+static const char JAMO_L_TABLE[][4] = {
         "G", "GG", "N", "D", "DD", "R", "M", "B", "BB",
         "S", "SS", "", "J", "JJ", "C", "K", "T", "P", "H"
     };
 
-static const char JAMO_V_TABLE[][4] =
-    {
+static const char JAMO_V_TABLE[][4] = {
         "A", "AE", "YA", "YAE", "EO", "E", "YEO", "YE", "O",
         "WA", "WAE", "OE", "YO", "U", "WEO", "WE", "WI",
         "YU", "EU", "YI", "I"
     };
 
-static const char JAMO_T_TABLE[][4] =
-    {
+static const char JAMO_T_TABLE[][4] = {
         "", "G", "GG", "GS", "N", "NJ", "NH", "D", "L", "LG", "LM",
         "LB", "LS", "LT", "LP", "LH", "M", "B", "BS",
         "S", "SS", "NG", "J", "C", "K", "T", "P", "H"
@@ -221,347 +217,6 @@ uint32_t CharSelectDataGetDetailIndex(CharSelectData* charselect, uint16_t unico
     return 0;
 }
 
-#if 0
-
-QList<QChar> KCharSelectData::blockContents(int block)
-{
-    if(!openDataFile()) {
-        return QList<QChar>();
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t offsetBegin = FromLittleEndian32(data+20);
-    const uint32_t offsetEnd = FromLittleEndian32(data+24);
-
-    int max = ((offsetEnd - offsetBegin) / 4) - 1;
-
-    QList<QChar> res;
-
-    if(block > max)
-        return res;
-
-    uint16_t unicodeBegin = FromLittleEndian16(data + offsetBegin + block*4);
-    uint16_t unicodeEnd = FromLittleEndian16(data + offsetBegin + block*4 + 2);
-
-    while(unicodeBegin < unicodeEnd) {
-        res.append(unicodeBegin);
-        unicodeBegin++;
-    }
-    res.append(unicodeBegin); // Be carefull when unicodeEnd==0xffff
-
-    return res;
-}
-
-QList<int> KCharSelectData::sectionContents(int section)
-{
-    if(!openDataFile()) {
-        return QList<int>();
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t offsetBegin = FromLittleEndian32(data+28);
-    const uint32_t offsetEnd = FromLittleEndian32(data+32);
-
-    int max = ((offsetEnd - offsetBegin) / 4) - 1;
-
-    QList<int> res;
-
-    if(section > max)
-        return res;
-
-    for(int i = 0; i <= max; i++) {
-        const uint16_t currSection = FromLittleEndian16(data + offsetBegin + i*4);
-        if(currSection == section) {
-            res.append( FromLittleEndian16(data + offsetBegin + i*4 + 2) );
-        }
-    }
-
-    return res;
-}
-
-UT_array* KCharSelectData::sectionList()
-{
-    if(!openDataFile()) {
-        return fcitx_utils_new_string_list();
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t stringBegin = FromLittleEndian32(data+24);
-    const uint32_t stringEnd = FromLittleEndian32(data+28);
-
-    const char* data = dataFile.constData();
-    UT_array* list;
-    uint32_t i = stringBegin;
-    while(i < stringEnd) {
-        list.append(i18nc("KCharSelect section name", data + i));
-        i += strlen(data + i) + 1;
-    }
-
-    return list;
-}
-
-QString KCharSelectData::block(CharSelectData* charselect, uint16_t unicode)
-{
-    return blockName(blockIndex(c));
-}
-
-QString KCharSelectData::section(CharSelectData* charselect, uint16_t unicode)
-{
-    return sectionName(sectionIndex(blockIndex(c)));
-}
-
-int KCharSelectData::blockIndex(CharSelectData* charselect, uint16_t unicode)
-{
-    if(!openDataFile()) {
-        return 0;
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t offsetBegin = FromLittleEndian32(data+20);
-    const uint32_t offsetEnd = FromLittleEndian32(data+24);
-    const uint16_t unicode = c.unicode();
-
-    int max = ((offsetEnd - offsetBegin) / 4) - 1;
-
-    int i = 0;
-
-    while (unicode > FromLittleEndian16(data + offsetBegin + i*4 + 2) && i < max) {
-        i++;
-    }
-
-    return i;
-}
-
-int KCharSelectData::sectionIndex(int block)
-{
-    if(!openDataFile()) {
-        return 0;
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t offsetBegin = FromLittleEndian32(data+28);
-    const uint32_t offsetEnd = FromLittleEndian32(data+32);
-
-    int max = ((offsetEnd - offsetBegin) / 4) - 1;
-
-    for(int i = 0; i <= max; i++) {
-        if( FromLittleEndian16(data + offsetBegin + i*4 + 2) == block) {
-            return FromLittleEndian16(data + offsetBegin + i*4);
-        }
-    }
-
-    return 0;
-}
-
-QString KCharSelectData::blockName(int index)
-{
-    if(!openDataFile()) {
-        return QString();
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t stringBegin = FromLittleEndian32(data+16);
-    const uint32_t stringEnd = FromLittleEndian32(data+20);
-
-    uint32_t i = stringBegin;
-    int currIndex = 0;
-
-    const char* data = dataFile.constData();
-    while(i < stringEnd && currIndex < index) {
-        i += strlen(data + i) + 1;
-        currIndex++;
-    }
-
-    return i18nc("KCharselect unicode block name", data + i);
-}
-
-QString KCharSelectData::sectionName(int index)
-{
-    if(!openDataFile()) {
-        return QString();
-    }
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t stringBegin = FromLittleEndian32(data+24);
-    const uint32_t stringEnd = FromLittleEndian32(data+28);
-
-    uint32_t i = stringBegin;
-    int currIndex = 0;
-
-    const char* data = dataFile.constData();
-    while(i < stringEnd && currIndex < index) {
-        i += strlen(data + i) + 1;
-        currIndex++;
-    }
-
-    return i18nc("KCharselect unicode section name", data + i);
-}
-
-QChar::Category KCharSelectData::category(CharSelectData* charselect, uint16_t unicode)
-{
-    if(!openDataFile()) {
-        return c.category();
-    }
-
-    uint16_t unicode = c.unicode();
-
-
-    const char* data = charselect->dataFile;
-    const uint32_t offsetBegin = FromLittleEndian32(data+4);
-    const uint32_t offsetEnd = FromLittleEndian32(data+8);
-
-    int min = 0;
-    int mid;
-    int max = ((offsetEnd - offsetBegin) / 6) - 1;
-    QString s;
-
-    while (max >= min) {
-        mid = (min + max) / 2;
-        const uint16_t midUnicode = FromLittleEndian16(data + offsetBegin + mid*6);
-        if (unicode > midUnicode)
-            min = mid + 1;
-        else if (unicode < midUnicode)
-            max = mid - 1;
-        else {
-            uint32_t offset = FromLittleEndian32(data + offsetBegin + mid*6 + 2);
-            const uint8_t categoryCode = * (uint8_t *)(data + offset);
-            return QChar::Category(categoryCode);
-        }
-    }
-
-    return c.category();
-}
-
-bool KCharSelectData::isPrint(CharSelectData* charselect, uint16_t unicode)
-{
-    QChar::Category cat = category(c);
-    return !(cat == QChar::Other_Control || cat == QChar::Other_NotAssigned);
-}
-
-bool KCharSelectData::isDisplayable(CharSelectData* charselect, uint16_t unicode)
-{
-    // Qt internally uses U+FDD0 and U+FDD1 to mark the beginning and the end of frames.
-    // They should be seen as non-printable characters, as trying to display them leads
-    //  to a crash caused by a Qt "noBlockInString" assertion.
-    if(c == 0xFDD0 || c == 0xFDD1)
-        return false;
-
-    return !isIgnorable(c) && isPrint(c);
-}
-
-bool KCharSelectData::isIgnorable(CharSelectData* charselect, uint16_t unicode)
-{
-    /*
-     * According to the Unicode standard, Default Ignorable Code Points
-     * should be ignored unless explicitly supported. For example, U+202E
-     * RIGHT-TO-LEFT-OVERRIDE ir printable according to Qt, but displaying
-     * it gives the undesired effect of all text being turned RTL. We do not
-     * have a way to "explicitly" support it, so we will treat it as
-     * non-printable.
-     *
-     * There is a list of these on
-     * http://unicode.org/Public/UNIDATA/DerivedCoreProperties.txt under the
-     * property Default_Ignorable_Code_Point.
-     */
-
-    //NOTE: not very nice to hardcode these here; is it worth it to modify
-    //      the binary data file to hold them?
-    return c == 0x00AD || c == 0x034F || c == 0x115F || c == 0x1160 ||
-           c == 0x17B4 || c == 0x17B5 || (c >= 0x180B && c <= 0x180D) ||
-           (c >= 0x200B && c <= 0x200F) || (c >= 0x202A && c <= 0x202E) ||
-           (c >= 0x2060 && c <= 0x206F) || c == 0x3164 ||
-           (c >= 0xFE00 && c <= 0xFE0F) || c == 0xFEFF || c == 0xFFA0 ||
-           (c >= 0xFFF0 && c <= 0xFFF8);
-}
-
-bool KCharSelectData::isCombining(const QChar &c)
-{
-    return section(c) == i18nc("KCharSelect section name", "Combining Diacritical Marks");
-    //FIXME: this is an imperfect test. There are many combining characters
-    //       that are outside of this section. See Grapheme_Extend in
-    //       http://www.unicode.org/Public/UNIDATA/DerivedCoreProperties.txt
-}
-
-QString KCharSelectData::display(const QChar &c, const QFont &font)
-{
-    if (!isDisplayable(c)) {
-        return QString("<b>") + i18n("Non-printable") + "</b>";
-    } else {
-        QString s = QString("<font size=\"+4\" face=\"") + font.family() + "\">";
-        if (isCombining(c)) {
-            s += displayCombining(c);
-        } else {
-            s += "&#" + QString::number(c.unicode()) + ';';
-        }
-        s += "</font>";
-        return s;
-    }
-}
-
-QString KCharSelectData::displayCombining(const QChar &c)
-{
-    /*
-     * The purpose of this is to make it easier to see how a combining
-     * character affects the text around it.
-     * The initial plan was to use U+25CC DOTTED CIRCLE for this purpose,
-     * as seen in pdfs from Unicode, but there seem to be a lot of alignment
-     * problems with that.
-     *
-     * Eventually, it would be nice to determine whether the character
-     * combines to the left or to the right, etc.
-     */
-    QString s = "&nbsp;&#" + QString::number(c.unicode()) + ";&nbsp;" +
-                " (ab&#" + QString::number(c.unicode()) + ";c)";
-    return s;
-}
-
-QString KCharSelectData::categoryText(QChar::Category category)
-{
-    switch (category) {
-    case QChar::Other_Control: return i18n("Other, Control");
-    case QChar::Other_Format: return i18n("Other, Format");
-    case QChar::Other_NotAssigned: return i18n("Other, Not Assigned");
-    case QChar::Other_PrivateUse: return i18n("Other, Private Use");
-    case QChar::Other_Surrogate: return i18n("Other, Surrogate");
-    case QChar::Letter_Lowercase: return i18n("Letter, Lowercase");
-    case QChar::Letter_Modifier: return i18n("Letter, Modifier");
-    case QChar::Letter_Other: return i18n("Letter, Other");
-    case QChar::Letter_Titlecase: return i18n("Letter, Titlecase");
-    case QChar::Letter_Uppercase: return i18n("Letter, Uppercase");
-    case QChar::Mark_SpacingCombining: return i18n("Mark, Spacing Combining");
-    case QChar::Mark_Enclosing: return i18n("Mark, Enclosing");
-    case QChar::Mark_NonSpacing: return i18n("Mark, Non-Spacing");
-    case QChar::Number_DecimalDigit: return i18n("Number, Decimal Digit");
-    case QChar::Number_Letter: return i18n("Number, Letter");
-    case QChar::Number_Other: return i18n("Number, Other");
-    case QChar::Punctuation_Connector: return i18n("Punctuation, Connector");
-    case QChar::Punctuation_Dash: return i18n("Punctuation, Dash");
-    case QChar::Punctuation_Close: return i18n("Punctuation, Close");
-    case QChar::Punctuation_FinalQuote: return i18n("Punctuation, Final Quote");
-    case QChar::Punctuation_InitialQuote: return i18n("Punctuation, Initial Quote");
-    case QChar::Punctuation_Other: return i18n("Punctuation, Other");
-    case QChar::Punctuation_Open: return i18n("Punctuation, Open");
-    case QChar::Symbol_Currency: return i18n("Symbol, Currency");
-    case QChar::Symbol_Modifier: return i18n("Symbol, Modifier");
-    case QChar::Symbol_Math: return i18n("Symbol, Math");
-    case QChar::Symbol_Other: return i18n("Symbol, Other");
-    case QChar::Separator_Line: return i18n("Separator, Line");
-    case QChar::Separator_Paragraph: return i18n("Separator, Paragraph");
-    case QChar::Separator_Space: return i18n("Separator, Space");
-    default: return i18n("Unknown");
-    }
-}
-
-#endif
-
-
 char* CharSelectDataName(CharSelectData* charselect, uint16_t unicode)
 {
     char* result = NULL;
@@ -584,7 +239,10 @@ char* CharSelectDataName(CharSelectData* charselect, uint16_t unicode)
             VIndex = (SIndex % NCount) / TCount;
             TIndex = SIndex % TCount;
 
-            asprintf(&result, "HANGUL SYLLABLE %s%s%s", JAMO_L_TABLE[LIndex], JAMO_V_TABLE[VIndex], JAMO_T_TABLE[TIndex]);
+            fcitx_utils_alloc_cat_str(result, "HANGUL SYLLABLE ",
+                                      JAMO_L_TABLE[LIndex],
+                                      JAMO_V_TABLE[VIndex],
+                                      JAMO_T_TABLE[TIndex]);
         } else if (unicode >= 0xD800 && unicode <= 0xDB7F)
             result = strdup(_("<Non Private Use High Surrogate>"));
         else if (unicode >= 0xDB80 && unicode <= 0xDBFF)
@@ -728,7 +386,7 @@ UT_array* CharSelectDataFind(CharSelectData* charselect, const char* needle)
     char* simplified = Simplified(needle);
     UT_array* searchStrings = SplitString(simplified);
 
-    if(strlen(simplified) == 1) {
+    if (strlen(simplified) == 1) {
         // search for hex representation of the character
         utarray_clear(searchStrings);
         char* format = FormatCode(simplified[0], 4, "U+");
@@ -810,12 +468,18 @@ UnicodeSet* InsertResult(UnicodeSet* set, uint16_t unicode) {
 
 UnicodeSet* CharSelectDataGetMatchingChars(CharSelectData* charselect, const char* s)
 {
-    UnicodeSet* result = NULL;
-
-    CharSelectDataIndex** pos = utarray_custom_bsearch(s, charselect->indexList, 0, index_search_cmp);
-    CharSelectDataIndex** last = utarray_custom_bsearch(s, charselect->indexList, 0, index_search_a_cmp);
-
-    while (pos != last && strncasecmp(s, (*pos)->key, strlen(s)) == 0) {
+    UnicodeSet *result = NULL;
+    size_t s_l = strlen(s);
+    CharSelectDataIndex **pos;
+    CharSelectDataIndex **last;
+    pos = utarray_custom_bsearch(s, charselect->indexList, 0, index_search_cmp);
+    last = utarray_custom_bsearch(s, charselect->indexList,
+                                  0, index_search_a_cmp);
+    if (!pos)
+        return NULL;
+    if (!last)
+        last = (CharSelectDataIndex**)utarray_back(charselect->indexList);
+    while (pos != last && strncasecmp(s, (*pos)->key, s_l) == 0) {
         utarray_foreach (c, (*pos)->items, uint16_t) {
             result = InsertResult(result, *c);
         }
@@ -1103,7 +767,7 @@ void CharSelectDataCreateIndex(CharSelectData* charselect)
          }
      }
 
-     utarray_new(charselect->indexList, &pidx_icd);
+     utarray_new(charselect->indexList, fcitx_ptr_icd);
 
      CharSelectDataIndex* idx = charselect->index;
      while(idx) {
