@@ -35,10 +35,10 @@
 #include "fcitx-config/xdg.h"
 #include "fcitx/hook.h"
 #include "fcitx-utils/utils.h"
-#include "module/x11/x11stuff.h"
+#include "module/x11/fcitx-x11.h"
 #include "ui/cairostuff/cairostuff.h"
 #include "ui/cairostuff/font.h"
-#include "ui/classic/classicuiinterface.h"
+#include "ui/classic/fcitx-classicui.h"
 
 #define VK_FILE "vk.conf"
 
@@ -279,7 +279,7 @@ VKWindow* CreateVKWindow(FcitxVKState* vkstate)
 
     vs = VKFindARGBVisual(vkstate);
     VKInitWindowAttribute(vkstate, &vs, &cmap, &attrib, &attribmask, &depth);
-    vkWindow->dpy = InvokeVaArgs(vkstate->owner, FCITX_X11, GETDISPLAY);
+    vkWindow->dpy = FcitxX11GetDisplay(vkstate->owner);
 
     vkWindow->fontSize = 12;
     vkWindow->defaultFont = strdup("sans");
@@ -301,8 +301,7 @@ VKWindow* CreateVKWindow(FcitxVKState* vkstate)
 
     VKSetWindowProperty(vkstate, vkWindow->window, FCITX_WINDOW_DOCK, strWindowName);
 
-    InvokeVaArgs(vkstate->owner, FCITX_X11, ADDXEVENTHANDLER,
-                 VKWindowEventHandler, vkWindow);
+    FcitxX11AddXEventHandler(vkstate->owner, VKWindowEventHandler, vkWindow);
 
     return vkWindow;
 }
@@ -340,8 +339,8 @@ cairo_surface_t* LoadVKImage(VKWindow* vkWindow)
     FcitxVKState* vkstate = vkWindow->owner;
     boolean fallback = true;
     char vkimage[] = "keyboard.png";
-    cairo_surface_t *image = InvokeVaArgs(vkstate->owner, FCITX_CLASSIC_UI,
-                                          LOADIMAGE, vkimage, &fallback);
+    cairo_surface_t *image = FcitxClassicUILoadImage(vkstate->owner,
+                                                     vkimage, &fallback);
 
     if (image)
         return image;
@@ -367,15 +366,15 @@ void DestroyVKWindow(VKWindow* vkWindow)
 
 void DrawVKWindow(VKWindow* vkWindow)
 {
-    int             i;
-    int             iPos;
+    int i;
+    int iPos;
     cairo_t *cr;
     FcitxVKState *vkstate = vkWindow->owner;
     VKS *vks = vkstate->vks;
 
-    FcitxConfigColor *fontColor = InvokeVaArgs(vkstate->owner, FCITX_CLASSIC_UI,
-                                               GETKEYBOARDFONTCOLOR);
-    char **font = InvokeVaArgs(vkstate->owner, FCITX_CLASSIC_UI, GETFONT);
+    FcitxConfigColor *fontColor;
+    fontColor = FcitxClassicUIGetKeyboardFontColor(vkstate->owner);
+    char **font = FcitxClassicUIGetFont(vkstate->owner);
 
     if (!fontColor || !font) {
         fontColor = &blackColor;
@@ -420,7 +419,8 @@ void DrawVKWindow(VKWindow* vkWindow)
         iPos += 24;
     }
 
-    _CAIRO_DESTROY(cr);
+    cairo_destroy(cr);
+    cairo_surface_flush(vkWindow->surface);
 }
 
 /*
@@ -712,8 +712,7 @@ void SwitchVK(FcitxVKState *vkstate)
     if (vkstate->bVK) {
         int             x, y;
         int dwidth, dheight;
-        InvokeVaArgs(vkstate->owner, FCITX_X11, GETSCREENSIZE,
-                     &dwidth, &dheight);
+        FcitxX11GetScreenSize(vkstate->owner, &dwidth, &dheight);
 
         if (!FcitxUISupportMainWindow(instance)) {
             x = dwidth / 2 - VK_WINDOW_WIDTH / 2;
@@ -766,26 +765,25 @@ VKInitWindowAttribute(FcitxVKState* vkstate, Visual ** vs, Colormap * cmap,
                       XSetWindowAttributes * attrib,
                       unsigned long *attribmask, int *depth)
 {
-    InvokeVaArgs(vkstate->owner, FCITX_X11, INITWINDOWATTR,
-                 vs, cmap, attrib, attribmask, depth);
+    FcitxX11InitWindowAttribute(vkstate->owner,
+                                vs, cmap, attrib, attribmask, depth);
 }
 
 Visual * VKFindARGBVisual(FcitxVKState* vkstate)
 {
-    return InvokeVaArgs(vkstate->owner, FCITX_X11, FINDARGBVISUAL);
+    return FcitxX11FindARGBVisual(vkstate->owner);
 }
 
 void VKSetWindowProperty(FcitxVKState* vkstate, Window window, FcitxXWindowType type, char *windowTitle)
 {
-    InvokeVaArgs(vkstate->owner, FCITX_X11, SETWINDOWPROP,
-                 &window, &type, windowTitle);
+    FcitxX11SetWindowProp(vkstate->owner, &window, &type, windowTitle);
 }
 
 boolean
 VKMouseClick(FcitxVKState* vkstate, Window window, int *x, int *y)
 {
     boolean bMoved = false;
-    InvokeVaArgs(vkstate->owner, FCITX_X11, MOUSECLICK, &window, x, y, &bMoved);
+    FcitxX11MouseClick(vkstate->owner, &window, x, y, &bMoved);
     return bMoved;
 }
 
