@@ -20,7 +20,6 @@
 
 #include "config.h"
 #include "fcitx/fcitx.h"
-
 #include <limits.h>
 #include <unistd.h>
 #include <X11/extensions/Xrender.h>
@@ -38,100 +37,93 @@
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/utils.h"
 #include "fcitx/instance.h"
-#include "x11stuff.h"
+#include "fcitx/fcitx.h"
+#include "x11stuff-internal.h"
+#include "x11selection.h"
 #include "xerrorhandler.h"
+#include "frontend/xim/fcitx-xim.h"
 
 static void* X11Create(FcitxInstance* instance);
 static void X11SetFD(void* arg);
 static void X11ProcessEvent(void *arg);
-static void* X11GetDisplay(void* x11priv, FcitxModuleFunctionArg arg);
-static void* X11AddEventHandler(void* x11priv, FcitxModuleFunctionArg arg);
-static void* X11RemoveEventHandler(void* x11priv, FcitxModuleFunctionArg arg);
-static void* X11FindARGBVisual(void* arg, FcitxModuleFunctionArg args);
-static void* X11InitWindowAttribute(void* arg, FcitxModuleFunctionArg args);
-static void* X11SetWindowProperty(void* arg, FcitxModuleFunctionArg args);
-static void* X11GetScreenSize(void *arg, FcitxModuleFunctionArg args);
-static void* X11MouseClick(void *arg, FcitxModuleFunctionArg args);
-static void* X11AddCompositeHandler(void* arg, FcitxModuleFunctionArg args);
-static void* X11ScreenGeometry(void* arg, FcitxModuleFunctionArg args);
-static void* X11GetDPI(void* arg, FcitxModuleFunctionArg args);
-static boolean X11InitComposite(FcitxX11* x11priv);
+static void X11Destroy(void* arg);
+static boolean X11InitComposite(FcitxX11 *x11priv);
+static void X11InitAtoms(FcitxX11 *x11priv);
 static void X11HandlerComposite(FcitxX11* x11priv, boolean enable);
 static boolean X11GetCompositeManager(FcitxX11* x11priv);
 static void X11InitScreen(FcitxX11* x11priv);
-static void X11ProcessEventReal(void* arg, FcitxModuleFunctionArg args);
 
 static inline boolean RectIntersects(FcitxRect rt1, FcitxRect rt2);
 static inline int RectWidth(FcitxRect r);
 static inline int RectHeight(FcitxRect r);
 
-const UT_icd handler_icd = {sizeof(FcitxXEventHandler), 0, 0, 0};
-const UT_icd comphandler_icd = {sizeof(FcitxCompositeChangedHandler), 0, 0, 0};
+DECLARE_ADDFUNCTIONS(X11)
+
+static const UT_icd handler_icd = {
+    sizeof(FcitxXEventHandler), NULL, NULL, NULL
+};
+static const UT_icd comphandler_icd = {
+    sizeof(FcitxCompositeChangedHandler), NULL, NULL, NULL
+};
 
 FCITX_DEFINE_PLUGIN(fcitx_x11, module, FcitxModule) = {
     X11Create,
     X11SetFD,
     X11ProcessEvent,
-    NULL,
+    X11Destroy,
     NULL
 };
 
 void* X11Create(FcitxInstance* instance)
 {
-    FcitxX11* x11priv = fcitx_utils_malloc0(sizeof(FcitxX11));
-    FcitxAddon* x11addon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_X11_NAME);
+    FcitxX11 *x11priv = fcitx_utils_new(FcitxX11);
     x11priv->dpy = XOpenDisplay(NULL);
-    x11priv->xim = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), "fcitx-xim");
     if (x11priv->dpy == NULL)
         return NULL;
 
     x11priv->owner = instance;
     x11priv->iScreen = DefaultScreen(x11priv->dpy);
-    x11priv->windowTypeAtom = XInternAtom(x11priv->dpy, "_NET_WM_WINDOW_TYPE", False);
-    x11priv->typeMenuAtom = XInternAtom(x11priv->dpy, "_NET_WM_WINDOW_TYPE_MENU", False);
-    x11priv->typeDialogAtom = XInternAtom(x11priv->dpy, "_NET_WM_WINDOW_TYPE_DIALOG", False);
-    x11priv->typeDockAtom = XInternAtom(x11priv->dpy, "_NET_WM_WINDOW_TYPE_DOCK", False);
-    x11priv->pidAtom = XInternAtom(x11priv->dpy, "_NET_WM_PID", False);
+    x11priv->rootWindow = DefaultRootWindow(x11priv->dpy);
+    x11priv->eventWindow = XCreateWindow(x11priv->dpy, x11priv->rootWindow,
+                                         0, 0, 1, 1, 0, 0, InputOnly,
+                                         CopyFromParent, 0, NULL);
+    X11InitAtoms(x11priv);
 
     utarray_init(&x11priv->handlers, &handler_icd);
     utarray_init(&x11priv->comphandlers, &comphandler_icd);
 
-    /* ensure the order ! */
-    AddFunction(x11addon, X11GetDisplay);
-    AddFunction(x11addon, X11AddEventHandler);
-    AddFunction(x11addon, X11RemoveEventHandler);
-    AddFunction(x11addon, X11FindARGBVisual);
-    AddFunction(x11addon, X11InitWindowAttribute);
-    AddFunction(x11addon, X11SetWindowProperty);
-    AddFunction(x11addon, X11GetScreenSize);
-    AddFunction(x11addon, X11MouseClick);
-    AddFunction(x11addon, X11AddCompositeHandler);
-    AddFunction(x11addon, X11ScreenGeometry);
-    AddFunction(x11addon, X11ProcessEventReal);
-    AddFunction(x11addon, X11GetDPI);
+    FcitxX11AddFunctions(instance);
 
 #ifdef HAVE_XFIXES
     int ignore;
-    if (XFixesQueryExtension (x11priv->dpy,
-                              &x11priv->xfixesEventBase,
-                              &ignore))
+    if (XFixesQueryExtension(x11priv->dpy, &x11priv->xfixesEventBase,
+                             &ignore))
         x11priv->hasXfixes = true;
 #endif
-
+    X11InitSelection(x11priv);
     X11InitComposite(x11priv);
-
     X11InitScreen(x11priv);
 
     XWindowAttributes attr;
-    XGetWindowAttributes(x11priv->dpy, DefaultRootWindow(x11priv->dpy), &attr);
+    XGetWindowAttributes(x11priv->dpy, x11priv->rootWindow, &attr);
     if ((attr.your_event_mask & StructureNotifyMask) != StructureNotifyMask) {
-        XSelectInput(x11priv->dpy, DefaultRootWindow(x11priv->dpy), attr.your_event_mask | StructureNotifyMask);
+        XSelectInput(x11priv->dpy, x11priv->rootWindow,
+                     attr.your_event_mask | StructureNotifyMask);
     }
 
     InitXErrorHandler(x11priv);
     return x11priv;
 }
 
+void X11Destroy(void* arg)
+{
+    FcitxX11* x11priv = (FcitxX11*)arg;
+    UnsetXErrorHandler();
+    if (x11priv->eventWindow) {
+        XDestroyWindow(x11priv->dpy, x11priv->eventWindow);
+    }
+}
+
 void X11SetFD(void* arg)
 {
     FcitxX11* x11priv = (FcitxX11*)arg;
@@ -151,6 +143,30 @@ void X11DelayedCompositeTest(void* arg)
         X11HandlerComposite(x11priv, true);
 }
 
+#ifdef HAVE_XFIXES
+static boolean
+X11ProcessXFixesEvent(FcitxX11 *x11priv, XEvent *xevent)
+{
+    switch (xevent->type - x11priv->xfixesEventBase) {
+    case XFixesSelectionNotify:
+        X11ProcessXFixesSelectionNotifyEvent(
+            x11priv, (XFixesSelectionNotifyEvent*)xevent);
+        return true;
+    }
+    return false;
+}
+
+static void
+X11CompManagerSelectionNotify(FcitxX11 *x11priv, Atom selection, int subtype,
+                              X11SelectionNotify *notify)
+{
+    FCITX_UNUSED(selection);
+    FCITX_UNUSED(subtype);
+    FCITX_UNUSED(notify);
+    X11HandlerComposite(x11priv, X11GetCompositeManager(x11priv));
+}
+#endif
+
 static void
 X11ProcessEventRealInternal(FcitxX11 *x11priv)
 {
@@ -163,24 +179,33 @@ X11ProcessEventRealInternal(FcitxX11 *x11priv)
     while (XPending(x11priv->dpy)) {
         XNextEvent(x11priv->dpy, &event);
         if (XFilterEvent(&event, None) == False) {
-            if (event.type == DestroyNotify) {
+            switch (event.type) {
+            case DestroyNotify:
                 if (event.xany.window == x11priv->compManager)
                     X11HandlerComposite(x11priv, false);
-            } else if (event.type == ClientMessage) {
+                break;
+            case ClientMessage:
                 if (event.xclient.data.l[1] == x11priv->compManagerAtom) {
                     if (X11GetCompositeManager(x11priv))
                         X11HandlerComposite(x11priv, true);
                 }
-            } else if (event.type == ConfigureNotify) {
-                if (event.xconfigure.window == DefaultRootWindow(x11priv->dpy))
+                break;
+            case ConfigureNotify:
+                if (event.xconfigure.window == x11priv->rootWindow)
                     X11InitScreen(x11priv);
-            }
+                break;
+            case SelectionNotify:
+                X11ProcessSelectionNotifyEvent(x11priv,
+                                               (XSelectionEvent*)&event);
+                break;
+            default:
 #ifdef HAVE_XFIXES
-            else if (event.type - x11priv->xfixesEventBase == XFixesSelectionNotify) {
-                boolean result = X11InitComposite(x11priv);
-                X11HandlerComposite(x11priv, result);
-            }
+                if (x11priv->hasXfixes &&
+                    X11ProcessXFixesEvent(x11priv, &event))
+                    break;
 #endif
+                break;
+            }
 
             FcitxXEventHandler* handler;
             for (handler = (FcitxXEventHandler*)utarray_front(&x11priv->handlers);
@@ -192,95 +217,93 @@ X11ProcessEventRealInternal(FcitxX11 *x11priv)
     }
 }
 
-
-void X11ProcessEventReal(void* arg, FcitxModuleFunctionArg args)
-{
-    X11ProcessEventRealInternal((FcitxX11*)arg);
-}
-
-void X11ProcessEvent(void *arg)
+static void
+X11ProcessEvent(void *arg)
 {
     FcitxX11 *x11priv = (FcitxX11*)arg;
     X11ProcessEventRealInternal(x11priv);
-    FcitxModuleInvokeVaArgs(x11priv->xim, 0);
+    FcitxXimConsumeQueue(x11priv->owner);
 }
 
-void* X11GetDisplay(void* arg, FcitxModuleFunctionArg args)
+static void
+X11AddEventHandler(FcitxX11 *x11priv, FcitxX11XEventHandler event_handler,
+                   void *data)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    return x11priv->dpy;
-}
-
-void* X11AddEventHandler(void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    FcitxXEventHandler handler;
-    handler.eventHandler = args.args[0];
-    handler.instance = args.args[1];
+    FcitxXEventHandler handler = {
+        .eventHandler = event_handler,
+        .instance = data
+    };
     utarray_push_back(&x11priv->handlers, &handler);
-    return NULL;
 }
 
-void* X11AddCompositeHandler(void* arg, FcitxModuleFunctionArg args)
+static void
+X11AddCompositeHandler(FcitxX11 *x11priv,
+                       FcitxX11CompositeHandler comp_handler, void *data)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    FcitxCompositeChangedHandler handler;
-    handler.eventHandler = args.args[0];
-    handler.instance = args.args[1];
+    FcitxCompositeChangedHandler handler = {
+        .eventHandler = comp_handler,
+        .instance = data
+    };
     utarray_push_back(&x11priv->comphandlers, &handler);
-    return NULL;
 }
 
-void* X11RemoveEventHandler(void* arg, FcitxModuleFunctionArg args)
+static void
+X11RemoveEventHandler(FcitxX11 *x11priv, void *data)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    FcitxXEventHandler* handler;
-    int i = 0;
-    for (i = 0 ;
-            i < utarray_len(&x11priv->handlers);
-            i ++) {
-        handler = (FcitxXEventHandler*) utarray_eltptr(&x11priv->handlers, i);
-        if (handler->instance == args.args[0])
-            break;
+    FcitxXEventHandler *handler;
+    int i;
+    for (i = 0;i < utarray_len(&x11priv->handlers);i++) {
+        handler = (FcitxXEventHandler*)utarray_eltptr(&x11priv->handlers, i);
+        if (handler->instance == data) {
+            utarray_erase(&x11priv->handlers, i, 1);
+            return;
+        }
     }
-    utarray_erase(&x11priv->handlers, i, 1);
-    return NULL;
 }
 
-boolean X11InitComposite(FcitxX11* x11priv)
+static void
+X11InitAtoms(FcitxX11 *x11priv)
 {
-    char *atom_names = NULL;
+    char *atom_names;
+    x11priv->windowTypeAtom = XInternAtom(x11priv->dpy, "_NET_WM_WINDOW_TYPE",
+                                          False);
+    x11priv->typeMenuAtom = XInternAtom(x11priv->dpy,
+                                        "_NET_WM_WINDOW_TYPE_MENU", False);
+    x11priv->typeDialogAtom = XInternAtom(x11priv->dpy,
+                                          "_NET_WM_WINDOW_TYPE_DIALOG", False);
+    x11priv->typeDockAtom = XInternAtom(x11priv->dpy,
+                                        "_NET_WM_WINDOW_TYPE_DOCK", False);
+    x11priv->pidAtom = XInternAtom(x11priv->dpy, "_NET_WM_PID", False);
+    x11priv->utf8Atom = XInternAtom(x11priv->dpy, "UTF8_STRING", False);
+    x11priv->stringAtom = XInternAtom(x11priv->dpy, "STRING", False);
+    x11priv->compTextAtom = XInternAtom(x11priv->dpy, "COMPOUND_TEXT", False);
     asprintf(&atom_names, "_NET_WM_CM_S%d", x11priv->iScreen);
     x11priv->compManagerAtom = XInternAtom(x11priv->dpy, atom_names, False);
-
     free(atom_names);
+}
 
+static boolean
+X11InitComposite(FcitxX11* x11priv)
+{
 #ifdef HAVE_XFIXES
-    if (x11priv->hasXfixes) {
-       XFixesSelectSelectionInput (x11priv->dpy,
-                                   DefaultRootWindow(x11priv->dpy),
-                                   x11priv->compManagerAtom,
-                                   XFixesSetSelectionOwnerNotifyMask |
-                                   XFixesSelectionWindowDestroyNotifyMask |
-                                   XFixesSelectionClientCloseNotifyMask);
-    }
+    X11SelectionNotifyRegisterInternal(
+        x11priv, x11priv->compManagerAtom, x11priv,
+        X11CompManagerSelectionNotify, NULL, NULL, NULL);
 #endif
-
     return X11GetCompositeManager(x11priv);
 }
 
-void*
-X11FindARGBVisual(void* arg, FcitxModuleFunctionArg args)
+static Visual*
+X11FindARGBVisual(FcitxX11 *x11priv)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
     XVisualInfo *xvi;
     XVisualInfo temp;
-    int         nvi;
-    int         i;
-    XRenderPictFormat   *format;
-    Visual      *visual;
-    Display*    dpy = x11priv->dpy;
-    int         scr = x11priv->iScreen;
+    int nvi;
+    int i;
+    XRenderPictFormat *format;
+    Visual *visual;
+    Display *dpy = x11priv->dpy;
+    int scr = x11priv->iScreen;
 
     if (x11priv->compManager == None)
         return NULL;
@@ -288,11 +311,12 @@ X11FindARGBVisual(void* arg, FcitxModuleFunctionArg args)
     temp.screen = scr;
     temp.depth = 32;
     temp.class = TrueColor;
-    xvi = XGetVisualInfo(dpy,  VisualScreenMask | VisualDepthMask | VisualClassMask, &temp, &nvi);
+    xvi = XGetVisualInfo(dpy, VisualScreenMask | VisualDepthMask |
+                         VisualClassMask, &temp, &nvi);
     if (!xvi)
-        return 0;
+        return NULL;
     visual = 0;
-    for (i = 0; i < nvi; i++) {
+    for (i = 0;i < nvi;i++) {
         format = XRenderFindVisualFormat(dpy, xvi[i].visual);
         if (format->type == PictTypeDirect && format->direct.alphaMask) {
             visual = xvi[i].visual;
@@ -304,31 +328,25 @@ X11FindARGBVisual(void* arg, FcitxModuleFunctionArg args)
     return visual;
 }
 
-void *
-X11InitWindowAttribute(void* arg, FcitxModuleFunctionArg args)
+static void
+X11InitWindowAttribute(FcitxX11 *x11priv, Visual **vs, Colormap *cmap,
+                       XSetWindowAttributes *attrib, unsigned long *attribmask,
+                       int *depth)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    Visual ** vs = args.args[0];
-    Colormap * cmap = args.args[1];
-    XSetWindowAttributes * attrib = args.args[2];
-    unsigned long *attribmask = args.args[3];
-    int *depth = args.args[4];
-    Display* dpy = x11priv->dpy;
+    Display *dpy = x11priv->dpy;
     int iScreen = x11priv->iScreen;
     attrib->bit_gravity = NorthWestGravity;
     attrib->backing_store = WhenMapped;
     attrib->save_under = True;
     if (*vs) {
-        *cmap =
-            XCreateColormap(dpy, RootWindow(dpy, iScreen), *vs, AllocNone);
-
+        *cmap = XCreateColormap(dpy, RootWindow(dpy, iScreen), *vs, AllocNone);
         attrib->override_redirect = True;       // False;
         attrib->background_pixel = 0;
         attrib->border_pixel = 0;
         attrib->colormap = *cmap;
-        *attribmask =
-            (CWBackPixel | CWBorderPixel | CWOverrideRedirect | CWSaveUnder |
-             CWColormap | CWBitGravity | CWBackingStore);
+        *attribmask = (CWBackPixel | CWBorderPixel | CWOverrideRedirect |
+                       CWSaveUnder | CWColormap | CWBitGravity |
+                       CWBackingStore);
         *depth = 32;
     } else {
         *cmap = DefaultColormap(dpy, iScreen);
@@ -336,23 +354,18 @@ X11InitWindowAttribute(void* arg, FcitxModuleFunctionArg args)
         attrib->override_redirect = True;       // False;
         attrib->background_pixel = 0;
         attrib->border_pixel = 0;
-        *attribmask = (CWBackPixel | CWBorderPixel | CWOverrideRedirect | CWSaveUnder
-                       | CWBitGravity | CWBackingStore);
+        *attribmask = (CWBackPixel | CWBorderPixel | CWOverrideRedirect |
+                       CWSaveUnder | CWBitGravity | CWBackingStore);
         *depth = DefaultDepth(dpy, iScreen);
     }
-    return NULL;
 }
 
-void* X11SetWindowProperty(void* arg, FcitxModuleFunctionArg args)
+static void
+X11SetWindowProperty(FcitxX11 *x11priv, Window window, FcitxXWindowType type,
+                     char *windowTitle)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    Atom* wintype = NULL;
-
-    Window window = *(Window*) args.args[0];
-    FcitxXWindowType *type = args.args[1];
-    char *windowTitle = args.args[2];
-
-    switch (*type) {
+    Atom *wintype = NULL;
+    switch (type) {
     case FCITX_WINDOW_DIALOG:
         wintype = &x11priv->typeDialogAtom;
         break;
@@ -367,11 +380,12 @@ void* X11SetWindowProperty(void* arg, FcitxModuleFunctionArg args)
         break;
     }
     if (wintype)
-        XChangeProperty(x11priv->dpy, window, x11priv->windowTypeAtom, XA_ATOM, 32, PropModeReplace, (void *) wintype, 1);
+        XChangeProperty(x11priv->dpy, window, x11priv->windowTypeAtom,
+                        XA_ATOM, 32, PropModeReplace, (void*)wintype, 1);
 
     pid_t pid = getpid();
     XChangeProperty(x11priv->dpy, window, x11priv->pidAtom, XA_CARDINAL, 32,
-                    PropModeReplace, (unsigned char *)&pid, 1);
+                    PropModeReplace, (unsigned char*)&pid, 1);
 
     char res_name[] = "fcitx";
     char res_class[] = "fcitx";
@@ -383,54 +397,43 @@ void* X11SetWindowProperty(void* arg, FcitxModuleFunctionArg args)
     if (windowTitle) {
         XTextProperty   tp;
         memset(&tp, 0, sizeof(XTextProperty));
-        Xutf8TextListToTextProperty(x11priv->dpy, &windowTitle, 1, XUTF8StringStyle, &tp);
+        Xutf8TextListToTextProperty(x11priv->dpy, &windowTitle, 1,
+                                    XUTF8StringStyle, &tp);
         if (tp.value) {
             XSetWMName(x11priv->dpy, window, &tp);
             XFree(tp.value);
         }
     }
-
-    return NULL;
 }
 
-void* X11GetScreenSize(void* arg, FcitxModuleFunctionArg args)
+static void
+X11GetScreenSize(FcitxX11 *x11priv, int *width, int *height)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    int* width = args.args[0];
-    int* height = args.args[1];
-
-    if (width != NULL)
-        (*width) = RectWidth(x11priv->rects[x11priv->defaultScreen]);
-    if (height != NULL)
-        (*height) = RectHeight(x11priv->rects[x11priv->defaultScreen]);
-
-    return NULL;
+    if (width) {
+        *width = RectWidth(x11priv->rects[x11priv->defaultScreen]);
+    }
+    if (height) {
+        *height = RectHeight(x11priv->rects[x11priv->defaultScreen]);
+    }
 }
 
-void*
-X11MouseClick(void *arg, FcitxModuleFunctionArg args)
+static void
+X11MouseClick(FcitxX11 *x11priv, Window window, int *x, int *y, boolean *bMoved)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    Window window = *(Window*) args.args[0];
-    int *x = args.args[1];
-    int *y = args.args[2];
-    XEvent          evtGrabbed;
-    boolean           *bMoved = args.args[3];
-
+    XEvent evtGrabbed;
     // To motion the window
     while (1) {
         XMaskEvent(x11priv->dpy,
                    PointerMotionMask | ButtonReleaseMask | ButtonPressMask,
                    &evtGrabbed);
         if (ButtonRelease == evtGrabbed.xany.type) {
-            if (Button1 == evtGrabbed.xbutton.button)
+            if (Button1 == evtGrabbed.xbutton.button) {
                 break;
+            }
         } else if (MotionNotify == evtGrabbed.xany.type) {
-            static Time     LastTime;
-
+            static Time LastTime;
             if (evtGrabbed.xmotion.time - LastTime < 20)
                 continue;
-
             XMoveWindow(x11priv->dpy, window, evtGrabbed.xmotion.x_root - *x,
                         evtGrabbed.xmotion.y_root - *y);
             XRaiseWindow(x11priv->dpy, window);
@@ -439,11 +442,8 @@ X11MouseClick(void *arg, FcitxModuleFunctionArg args)
             LastTime = evtGrabbed.xmotion.time;
         }
     }
-
     *x = evtGrabbed.xmotion.x_root - *x;
     *y = evtGrabbed.xmotion.y_root - *y;
-
-    return NULL;
 }
 
 void X11HandlerComposite(FcitxX11* x11priv, boolean enable)
@@ -568,7 +568,7 @@ void X11InitScreen(FcitxX11* x11priv)
 #endif // HAVE_XINERAMA
 }
 
-inline
+static inline
 boolean RectIntersects(FcitxRect rt1, FcitxRect rt2)
 {
     int l1 = rt1.x1;
@@ -608,13 +608,13 @@ boolean RectIntersects(FcitxRect rt1, FcitxRect rt2)
     return true;
 }
 
-inline
+static inline
 int RectWidth(FcitxRect r)
 {
     return r.x2 - r.x1 + 1;
 }
 
-inline
+static inline
 int RectHeight(FcitxRect r)
 {
     return r.y2 - r.y1 + 1;
@@ -635,13 +635,9 @@ int PointToRect(int x, int y, FcitxRect r)
     return dx + dy;
 }
 
-void* X11ScreenGeometry(void* arg, FcitxModuleFunctionArg args)
+static void
+X11ScreenGeometry(FcitxX11 *x11priv, int x, int y, FcitxRect *rect)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
-    int x = *(int*) args.args[0];
-    int y = *(int*) args.args[1];
-    FcitxRect* rect = (FcitxRect*) args.args[2];
-
     int closestScreen = -1;
     int shortestDistance = INT_MAX;
     int i;
@@ -657,16 +653,14 @@ void* X11ScreenGeometry(void* arg, FcitxModuleFunctionArg args)
         closestScreen = x11priv->defaultScreen;
 
     *rect = x11priv->rects[closestScreen];
-
-    return NULL;
 }
 
-void* X11GetDPI(void* arg, FcitxModuleFunctionArg args)
+static void
+X11GetDPI(FcitxX11 *x11priv, int *_i, double *_d)
 {
-    FcitxX11* x11priv = (FcitxX11*)arg;
     if (!x11priv->dpi) {
-        char* v = XGetDefault(x11priv->dpy, "Xft", "dpi");
-        char* e = NULL;
+        char *v = XGetDefault(x11priv->dpy, "Xft", "dpi");
+        char *e = NULL;
         double value;
         if (v) {
             value = strtod(v, &e);
@@ -690,15 +684,12 @@ void* X11GetDPI(void* arg, FcitxModuleFunctionArg args)
         FcitxLog(DEBUG, "DPI: %d %lf", x11priv->dpi, x11priv->dpif);
     }
 
-    if (args.args[0]) {
-        int* i = (int*) args.args[0];
-        *i = x11priv->dpi;
+    if (_i) {
+        *_i = x11priv->dpi;
     }
-    if (args.args[1]) {
-        double* d = (double*) args.args[1];
-        *d = x11priv->dpif;
+    if (_d) {
+        *_d = x11priv->dpif;
     }
-    return NULL;
 }
 
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-x11-addfunctions.h"
