@@ -19,18 +19,15 @@
  ***************************************************************************/
 
 #include <libxml/parser.h>
+#include <fcitx/fcitx.h>
 
 #include "rules.h"
 #define XMLCHAR_CAST (const char*)
 
-static void RulesHandlerStartElement(void *ctx,
-                              const xmlChar *name,
-                              const xmlChar **atts);
-static void RulesHandlerEndElement(void *ctx,
-                const xmlChar *name);
-static void RulesHandlerCharacters(void *ctx,
-                const xmlChar *ch,
-                int len);
+static void RulesHandlerStartElement(void *ctx, const xmlChar *name,
+                                     const xmlChar **atts);
+static void RulesHandlerEndElement(void *ctx, const xmlChar *name);
+static void RulesHandlerCharacters(void *ctx, const xmlChar *ch, int len);
 static void FcitxXkbLayoutInfoInit(void* arg);
 static void FcitxXkbVariantInfoInit(void* arg);
 static void FcitxXkbModelInfoInit(void* arg);
@@ -49,16 +46,27 @@ static void FcitxXkbOptionInfoFree(void* arg);
 static inline FcitxXkbLayoutInfo* FindByName(FcitxXkbRules* rules, const char* name);
 static void MergeRules(FcitxXkbRules* rules, FcitxXkbRules* rulesextra);
 
-static const UT_icd layout_icd = {sizeof(FcitxXkbLayoutInfo), FcitxXkbLayoutInfoInit, FcitxXkbLayoutInfoCopy, FcitxXkbLayoutInfoFree };
-static const UT_icd variant_icd = {sizeof(FcitxXkbVariantInfo), FcitxXkbVariantInfoInit, FcitxXkbVariantInfoCopy, FcitxXkbVariantInfoFree };
-static const UT_icd model_icd = {sizeof(FcitxXkbModelInfo), FcitxXkbModelInfoInit, FcitxXkbModelInfoCopy, FcitxXkbModelInfoFree };
-static const UT_icd option_group_icd = {sizeof(FcitxXkbOptionGroupInfo), FcitxXkbOptionGroupInfoInit, FcitxXkbOptionGroupInfoCopy, FcitxXkbOptionGroupInfoFree };
-static const UT_icd option_icd = {sizeof(FcitxXkbOptionInfo), FcitxXkbOptionInfoInit, FcitxXkbOptionInfoCopy, FcitxXkbOptionInfoFree };
-#define COPY_IF_NOT_NULL(x) ((x)?(strdup(x)):NULL)
-#define utarray_clone(t, f) do { \
-        utarray_new((t), (f)->icd); \
-        utarray_concat((t), (f)); \
-    } while(0)
+static const UT_icd layout_icd = {
+    sizeof(FcitxXkbLayoutInfo), FcitxXkbLayoutInfoInit,
+    FcitxXkbLayoutInfoCopy, FcitxXkbLayoutInfoFree
+};
+static const UT_icd variant_icd = {
+    sizeof(FcitxXkbVariantInfo), FcitxXkbVariantInfoInit,
+    FcitxXkbVariantInfoCopy, FcitxXkbVariantInfoFree
+};
+static const UT_icd model_icd = {
+    sizeof(FcitxXkbModelInfo), FcitxXkbModelInfoInit,
+    FcitxXkbModelInfoCopy, FcitxXkbModelInfoFree
+};
+static const UT_icd option_group_icd = {
+    sizeof(FcitxXkbOptionGroupInfo), FcitxXkbOptionGroupInfoInit,
+    FcitxXkbOptionGroupInfoCopy, FcitxXkbOptionGroupInfoFree
+};
+static const UT_icd option_icd = {
+    sizeof(FcitxXkbOptionInfo), FcitxXkbOptionInfoInit,
+    FcitxXkbOptionInfoCopy, FcitxXkbOptionInfoFree
+};
+#define COPY_IF_NOT_NULL(x) ((x) ? (strdup(x)) : NULL)
 
 boolean StringEndsWith(const char* str, const char* suffix)
 {
@@ -82,7 +90,7 @@ FcitxXkbRules* FcitxXkbReadRules(const char* file)
 
     xmlInitParser();
 
-    FcitxXkbRules* rules = (FcitxXkbRules*) fcitx_utils_malloc0(sizeof(FcitxXkbRules));
+    FcitxXkbRules* rules = fcitx_utils_new(FcitxXkbRules);
     utarray_new(rules->layoutInfos, &layout_icd);
     utarray_new(rules->modelInfos, &model_icd);
     utarray_new(rules->optionGroupInfos, &option_group_icd);
@@ -95,10 +103,12 @@ FcitxXkbRules* FcitxXkbReadRules(const char* file)
     xmlSAXUserParseFile(&handle, &ruleshandler, file);
     utarray_free(ruleshandler.path);
 
-    if (strcmp (file + strlen(file) - 4, ".xml") == 0) {
-        char* extra = strndup (file, strlen (file) - 4), *extrafile;
-        asprintf(&extrafile, "%s.extras.xml", extra);
-        FcitxXkbRules* rulesextra = (FcitxXkbRules*) fcitx_utils_malloc0(sizeof(FcitxXkbRules));
+    size_t extra_len = strlen(file) - strlen(".xml");
+    if (strcmp(file + extra_len, ".xml") == 0) {
+        char extrafile[extra_len + strlen(".extras.xml") + 1];
+        memcpy(extrafile, file, extra_len);
+        memcpy(extrafile + extra_len, ".extras.xml", sizeof(".extras.xml"));
+        FcitxXkbRules *rulesextra = fcitx_utils_new(FcitxXkbRules);
         utarray_new(rulesextra->layoutInfos, &layout_icd);
         utarray_new(rulesextra->modelInfos, &model_icd);
         utarray_new(rulesextra->optionGroupInfos, &option_group_icd);
@@ -106,9 +116,6 @@ FcitxXkbRules* FcitxXkbReadRules(const char* file)
         ruleshandler.path = fcitx_utils_new_string_list();
         xmlSAXUserParseFile(&handle, &ruleshandler, extrafile);
         utarray_free(ruleshandler.path);
-        free(extra);
-        free(extrafile);
-
         MergeRules(rules, rulesextra);
     }
 
@@ -137,9 +144,8 @@ void MergeRules(FcitxXkbRules* rules, FcitxXkbRules* rulesextra)
     utarray_concat(rules->optionGroupInfos, rulesextra->optionGroupInfos);
 
     FcitxXkbLayoutInfo* layoutInfo;
-    UT_icd icd = {sizeof(FcitxXkbLayoutInfo*), 0, 0, 0};
     UT_array toAdd;
-    utarray_init(&toAdd, &icd);
+    utarray_init(&toAdd, fcitx_ptr_icd);
     for (layoutInfo = (FcitxXkbLayoutInfo*) utarray_front(rulesextra->layoutInfos);
          layoutInfo != NULL;
          layoutInfo = (FcitxXkbLayoutInfo*) utarray_next(rulesextra->layoutInfos, layoutInfo))
@@ -153,9 +159,9 @@ void MergeRules(FcitxXkbRules* rules, FcitxXkbRules* rulesextra)
             utarray_push_back(&toAdd, &layoutInfo);
     }
 
-    int i;
-    for(i = 0; i < utarray_len(&toAdd); i ++) {
-        FcitxXkbLayoutInfo* p = *(FcitxXkbLayoutInfo**) utarray_eltptr(&toAdd, i);
+    unsigned int i;
+    for(i = 0;i < utarray_len(&toAdd);i++) {
+        FcitxXkbLayoutInfo* p = *(FcitxXkbLayoutInfo**)utarray_eltptr(&toAdd, i);
         utarray_push_back(rules->layoutInfos, p);
     }
 
@@ -241,8 +247,7 @@ FcitxXkbLayoutInfo* FindByName(FcitxXkbRules* rules, const char* name) {
 }
 
 /* code borrow from kde-workspace/kcontrol/keyboard/xkb_rules.cpp */
-void RulesHandlerStartElement(void *ctx,
-                              const xmlChar *name,
+void RulesHandlerStartElement(void *ctx, const xmlChar *name,
                               const xmlChar **atts)
 {
     FcitxXkbRulesHandler* ruleshandler = (FcitxXkbRulesHandler*) ctx;
@@ -287,9 +292,9 @@ void RulesHandlerStartElement(void *ctx,
     free(strPath);
 }
 
-void RulesHandlerEndElement(void *ctx,
-                const xmlChar *name)
+void RulesHandlerEndElement(void *ctx, const xmlChar *name)
 {
+    FCITX_UNUSED(name);
     FcitxXkbRulesHandler* ruleshandler = (FcitxXkbRulesHandler*) ctx;
     utarray_pop_back(ruleshandler->path);
 }
@@ -450,8 +455,8 @@ void FcitxXkbLayoutInfoCopy(void* dst, const void* src)
     FcitxXkbLayoutInfo* layoutInfoSrc = (FcitxXkbLayoutInfo*) src;
     layoutInfoDst->name = COPY_IF_NOT_NULL(layoutInfoSrc->name);
     layoutInfoDst->description = COPY_IF_NOT_NULL(layoutInfoSrc->description);
-    utarray_clone(layoutInfoDst->languages, layoutInfoSrc->languages);
-    utarray_clone(layoutInfoDst->variantInfos, layoutInfoSrc->variantInfos);
+    layoutInfoDst->languages = utarray_clone(layoutInfoSrc->languages);
+    layoutInfoDst->variantInfos = utarray_clone(layoutInfoSrc->variantInfos);
 }
 
 void FcitxXkbModelInfoCopy(void* dst, const void* src)
@@ -470,7 +475,7 @@ void FcitxXkbOptionGroupInfoCopy(void* dst, const void* src)
     optionGroupInfoDst->name = COPY_IF_NOT_NULL(optionGroupInfoSrc->name);
     optionGroupInfoDst->description = COPY_IF_NOT_NULL(optionGroupInfoSrc->description);
     optionGroupInfoDst->exclusive = optionGroupInfoSrc->exclusive;
-    utarray_clone(optionGroupInfoDst->optionInfos, optionGroupInfoSrc->optionInfos);
+    optionGroupInfoDst->optionInfos = utarray_clone(optionGroupInfoSrc->optionInfos);
 }
 
 void FcitxXkbOptionInfoCopy(void* dst, const void* src)
@@ -487,6 +492,5 @@ void FcitxXkbVariantInfoCopy(void* dst, const void* src)
     FcitxXkbVariantInfo* variantInfoSrc = (FcitxXkbVariantInfo*) src;
     variantInfoDst->name = COPY_IF_NOT_NULL(variantInfoSrc->name);
     variantInfoDst->description = COPY_IF_NOT_NULL(variantInfoSrc->description);
-    utarray_clone(variantInfoDst->languages, variantInfoSrc->languages);
+    variantInfoDst->languages = utarray_clone(variantInfoSrc->languages);
 }
-
