@@ -23,6 +23,7 @@
 #include <dirent.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <limits.h>
 #include <libintl.h>
 
 #include <X11/Xlib.h>
@@ -34,7 +35,7 @@
 #include "fcitx/hook.h"
 #include "fcitx/addon.h"
 #include "fcitx/module.h"
-#include "module/x11/x11stuff.h"
+#include "module/x11/fcitx-x11.h"
 #include "fcitx-config/xdg.h"
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/utils.h"
@@ -48,35 +49,29 @@
 #define XKB_RULES_XML_FILE "/usr/share/X11/xkb/rules/evdev.xml"
 #endif
 
-#define GROUP_CHANGE_MASK \
-    ( XkbGroupStateMask | XkbGroupBaseMask | XkbGroupLatchMask | XkbGroupLockMask )
+#define GROUP_CHANGE_MASK (XkbGroupStateMask | XkbGroupBaseMask |       \
+                           XkbGroupLatchMask | XkbGroupLockMask)
 
-static void* FcitxXkbGetRules(void* arg, FcitxModuleFunctionArg args);
-static void* FcitxXkbLayoutExists(void* arg, FcitxModuleFunctionArg args);
+DECLARE_ADDFUNCTIONS(Xkb)
 static void FcitxXkbGetCurrentLayoutInternal(FcitxXkb *xkb,
                                              char **layout, char **variant);
-static void* FcitxXkbGetCurrentLayout(void* arg, FcitxModuleFunctionArg args);
-static void* FcitxXkbGetLayoutOverride(void* arg, FcitxModuleFunctionArg args);
-static void* FcitxXkbSetLayoutOverride(void* arg, FcitxModuleFunctionArg args);
-static void* FcitxXkbSetDefaultLayout(void* arg, FcitxModuleFunctionArg args);
 static void* FcitxXkbCreate(struct _FcitxInstance* instance);
 static void FcitxXkbDestroy(void*);
 static void FcitxXkbReloadConfig(void*);
 static void FcitxXkbInitDefaultLayout (FcitxXkb* xkb);
-static Bool FcitxXkbSetRules (FcitxXkb* xkb,
-             const char *rules_file, const char *model,
-             const char *all_layouts, const char *all_variants,
-             const char *all_options);
-static Bool FcitxXkbUpdateProperties (FcitxXkb* xkb,
-                     const char *rules_file, const char *model,
-                     const char *all_layouts, const char *all_variants,
-                     const char *all_options);
+static Bool FcitxXkbSetRules(FcitxXkb* xkb, const char *rules_file,
+                             const char *model, const char *all_layouts,
+                             const char *all_variants, const char *all_options);
+static Bool FcitxXkbUpdateProperties(FcitxXkb* xkb, const char *rules_file,
+                                     const char *model, const char *all_layouts,
+                                     const char *all_variants,
+                                     const char *all_options);
 static void FcitxXkbApplyCustomScript(FcitxXkb* xkb);
 
 #if 0
-static char* FcitxXkbGetCurrentLayout (FcitxXkb* xkb);
-static char* FcitxXkbGetCurrentModel (FcitxXkb* xkb);
-static char* FcitxXkbGetCurrentOption (FcitxXkb* xkb);
+static char* FcitxXkbGetCurrentLayout(FcitxXkb* xkb);
+static char* FcitxXkbGetCurrentModel(FcitxXkb* xkb);
+static char* FcitxXkbGetCurrentOption(FcitxXkb* xkb);
 #endif
 static boolean
 FcitxXkbSetLayoutByName(FcitxXkb *xkb,
@@ -84,9 +79,9 @@ FcitxXkbSetLayoutByName(FcitxXkb *xkb,
 static void
 FcitxXkbRetrieveCloseGroup(FcitxXkb *xkb);
 static boolean FcitxXkbSetLayout  (FcitxXkb* xkb,
-               const char *layouts,
-               const char *variants,
-               const char *options);
+                                   const char *layouts,
+                                   const char *variants,
+                                   const char *options);
 static int FcitxXkbGetCurrentGroup (FcitxXkb* xkb);
 static void FcitxXkbIMKeyboardLayoutChanged(void* arg, const void* value);
 static int FcitxXkbFindLayoutIndex(FcitxXkb* xkb, const char* layout, const char* variant);
@@ -122,9 +117,9 @@ boolean FcitxXkbSupported(FcitxXkb* xkb, int* xkbOpcode)
     int major = XkbMajorVersion;
     int minor = XkbMinorVersion;
 
-    if (!XkbLibraryVersion(&major, &minor))
-    {
-        FcitxLog(WARNING, "Xlib XKB extension %d.%d != %d %d", major, minor, XkbMajorVersion, XkbMinorVersion);
+    if (!XkbLibraryVersion(&major, &minor)) {
+        FcitxLog(WARNING, "Xlib XKB extension %d.%d != %d %d",
+                 major, minor, XkbMajorVersion, XkbMinorVersion);
         return false;
     }
 
@@ -133,142 +128,136 @@ boolean FcitxXkbSupported(FcitxXkb* xkb, int* xkbOpcode)
     int opcode_rtrn;
     int error_rtrn;
     int xkb_opcode;
-    if( ! XkbQueryExtension(xkb->dpy, &opcode_rtrn, &xkb_opcode, &error_rtrn, &major, &minor)) {
-        FcitxLog(WARNING, "Xlib XKB extension %d.%d != %d %d", major, minor, XkbMajorVersion, XkbMinorVersion);
+    if (!XkbQueryExtension(xkb->dpy, &opcode_rtrn, &xkb_opcode,
+                           &error_rtrn, &major, &minor)) {
+        FcitxLog(WARNING, "Xlib XKB extension %d.%d != %d %d",
+                 major, minor, XkbMajorVersion, XkbMinorVersion);
         return false;
     }
 
-    if( xkbOpcode != NULL ) {
+    if (xkbOpcode != NULL) {
         *xkbOpcode = xkb_opcode;
     }
 
     return true;
 }
 
+static inline void
+FcitxXkbClearVarDefsRec(XkbRF_VarDefsRec *vdp)
+{
+    fcitx_utils_free(vdp->model);
+    fcitx_utils_free(vdp->layout);
+    fcitx_utils_free(vdp->variant);
+    fcitx_utils_free(vdp->options);
+}
 
 static char* FcitxXkbGetRulesName(FcitxXkb* xkb)
 {
     XkbRF_VarDefsRec vd;
     char *tmp = NULL;
 
-    if (XkbRF_GetNamesProp(xkb->dpy, &tmp, &vd) && tmp != NULL ) {
-        return strdup(tmp);
+    /* the four strings in vd are either %NULL or strdup'ed */
+    if (XkbRF_GetNamesProp(xkb->dpy, &tmp, &vd)) {
+        FcitxXkbClearVarDefsRec(&vd);
+        return tmp;
     }
-
     return NULL;
 }
 
 
 static char* FcitxXkbFindXkbRulesFile(FcitxXkb* xkb)
 {
-    char* rulesFile = NULL;
-    char* rulesName = FcitxXkbGetRulesName(xkb);
+    char *rulesFile = NULL;
+    char *rulesName = FcitxXkbGetRulesName(xkb);
 
-    if ( rulesName != NULL ) {
+    if (rulesName) {
         if (rulesName[0] == '/') {
-            asprintf(&rulesFile, "%s.xml", rulesName);
-        }
-        else {
-            char* xkbParentDir = NULL;
-
-            const char* base = XLIBDIR;
-
+            fcitx_utils_alloc_cat_str(rulesFile, rulesName, ".xml");
+        } else {
             int count = 0, i = 0;
-            while(base[i]) {
+            const char* base = XLIBDIR;
+            while (base[i]) {
                 if (base[i] == '/')
                     count++;
-                i ++;
+                i++;
             }
 
-            if( count >= 3 ) {
-                // .../usr/lib/X11 -> /usr/share/X11/xkb vs .../usr/X11/lib -> /usr/X11/share/X11/xkb
-                const char* delta = StringEndsWith(base, "X11") ? "/../../share/X11" : "/../share/X11";
-                char* dirPath;
-                asprintf(&dirPath, "%s%s", base, delta);
-
-                DIR* dir = opendir(dirPath);
-                if( dir ) {
-                    closedir(dir);
-                    xkbParentDir = realpath(dirPath, NULL);
-                    free(dirPath);
-                }
-                else {
-                    free(dirPath);
-                    asprintf(&dirPath, "%s/X11", base);
-                    DIR* dir = opendir(dirPath);
-                    if( dir ) {
-                        closedir(dir);
-                        xkbParentDir = realpath(dirPath, NULL);
+            if (count >= 3) {
+                // .../usr/lib/X11 -> /usr/share/X11/xkb vs
+                // .../usr/X11/lib -> /usr/X11/share/X11/xkb
+                const char* delta = StringEndsWith(base, "X11") ?
+                    "/../../share/X11" : "/../share/X11";
+                char *tmppath;
+                fcitx_utils_alloc_cat_str(tmppath, base, delta,
+                                          "/xkb/rules/", rulesName, ".xml");
+                if(fcitx_utils_isreg(tmppath)) {
+                    rulesFile = realpath(tmppath, NULL);
+                    free(tmppath);
+                } else {
+                    fcitx_utils_alloc_cat_str(tmppath, base, "/X11/xkb/rules/",
+                                              rulesName, ".xml");
+                    if(fcitx_utils_isreg(tmppath)) {
+                        rulesFile = realpath(tmppath, NULL);
+                        free(tmppath);
                     }
-                    free(dirPath);
                 }
             }
-
-            if( xkbParentDir == NULL || strlen(xkbParentDir) == 0 ) {
-                xkbParentDir = strdup("/usr/share/X11");
+            if(!rulesFile) {
+                fcitx_utils_alloc_cat_str(rulesFile,
+                                          "/usr/share/X11/xkb/rules/",
+                                          rulesName, ".xml");
             }
-
-            asprintf(&rulesFile, "%s/xkb/rules/%s.xml", xkbParentDir, rulesName);
-            fcitx_utils_free(xkbParentDir);
         }
+        free(rulesName);
+    } else {
+        return strdup(XKB_RULES_XML_FILE);
     }
-
-    fcitx_utils_free(rulesName);
-
-    if (rulesFile == NULL)
-        rulesFile = strdup(XKB_RULES_XML_FILE);
-
     return rulesFile;
 }
 
 static void
-FcitxXkbInitDefaultLayout (FcitxXkb* xkb)
+FcitxXkbInitDefaultLayout(FcitxXkb* xkb)
 {
     Display* dpy = xkb->dpy;
-    XkbRF_VarDefsRec vd;
-    char *tmp = NULL;
+        XkbRF_VarDefsRec vd;
 
-    if (xkb->defaultLayouts) fcitx_utils_free_string_list(xkb->defaultLayouts);
-    if (xkb->defaultModels) fcitx_utils_free_string_list(xkb->defaultModels);
-    if (xkb->defaultOptions) fcitx_utils_free_string_list(xkb->defaultOptions);
-    if (xkb->defaultVariants) fcitx_utils_free_string_list(xkb->defaultVariants);
+    utarray_clear(xkb->defaultLayouts);
+    utarray_clear(xkb->defaultModels);
+    utarray_clear(xkb->defaultOptions);
+    utarray_clear(xkb->defaultVariants);
 
-    if (!XkbRF_GetNamesProp(dpy, &tmp, &vd) || !tmp)
-    {
-        FcitxLog(WARNING, "Couldn't interpret %s property", _XKB_RF_NAMES_PROP_ATOM);
+    /* XkbRF_GetNamesProp won't fill the second parameter if it is NULL */
+    if (!XkbRF_GetNamesProp(dpy, NULL, &vd)) {
+        FcitxLog(WARNING, "Couldn't interpret %s property",
+                 _XKB_RF_NAMES_PROP_ATOM);
         return;
     }
-    if (!vd.model || !vd.layout) {
-        FcitxLog (WARNING, "Could not get group layout from X property");
-        return;
+    /* Print warning only, in order to free vd correctly. */
+    if (!vd.model || !vd.layout)
+        FcitxLog(WARNING, "Could not get group layout from X property");
+    if (vd.layout) {
+        fcitx_utils_append_split_string(xkb->defaultLayouts, vd.layout, ",");
+    }
+    if (vd.model) {
+        fcitx_utils_append_split_string(xkb->defaultModels, vd.model, ",");
+    }
+    if (vd.options) {
+        fcitx_utils_append_split_string(xkb->defaultOptions, vd.options, ",");
     }
-    if (vd.layout)
-        xkb->defaultLayouts = fcitx_utils_split_string (vd.layout, ',');
-    else
-        xkb->defaultLayouts = fcitx_utils_new_string_list();
-    if (vd.model)
-        xkb->defaultModels = fcitx_utils_split_string (vd.model, ',');
-    else
-        xkb->defaultModels = fcitx_utils_new_string_list();
-    if (vd.options)
-        xkb->defaultOptions = fcitx_utils_split_string (vd.options, ',');
-    else
-        xkb->defaultOptions = fcitx_utils_new_string_list();
     if (vd.variant) {
-        xkb->defaultVariants = fcitx_utils_split_string (vd.variant, ',');
+        fcitx_utils_append_split_string(xkb->defaultVariants, vd.variant, ",");
     }
-    else
-        xkb->defaultVariants = fcitx_utils_new_string_list();
+
+    FcitxXkbClearVarDefsRec(&vd);
 }
 
 static Bool
-FcitxXkbSetRules (FcitxXkb* xkb,
-             const char *rules_file, const char *model,
-             const char *all_layouts, const char *all_variants,
-             const char *all_options)
+FcitxXkbSetRules(FcitxXkb* xkb, const char *rules_file, const char *model,
+                 const char *all_layouts, const char *all_variants,
+                 const char *all_options)
 {
-    Display* dpy = xkb->dpy;
-    char *rulesPath;
+    Display *dpy = xkb->dpy;
+    char *prefix;
     XkbRF_RulesPtr rules = NULL;
     XkbRF_VarDefsRec rdefs;
     XkbComponentNamesRec rnames;
@@ -277,39 +266,38 @@ FcitxXkbSetRules (FcitxXkb* xkb,
     if (!rules_file)
         return False;
 
-    if (rules_file[0] != '/')
-        asprintf(&rulesPath, "./rules/%s", rules_file);
-    else
-        rulesPath = strdup(rules_file);
-    rules = XkbRF_Load (rulesPath, "C", True, True);
+    if (rules_file[0] != '/') {
+        prefix = "./rules/";
+    } else {
+        prefix = "";
+    }
+    char *rules_path1;
+    fcitx_utils_alloc_cat_str(rules_path1, prefix, rules_file);
+    rules = XkbRF_Load(rules_path1, "C", True, True);
+    free(rules_path1);
     if (rules == NULL) {
-        free (rulesPath);
-
-        rulesPath = FcitxXkbFindXkbRulesFile(xkb);
-        if (strcmp (rulesPath + strlen(rulesPath) - 4, ".xml") == 0) {
-            char* old = rulesPath;
-            rulesPath = strndup (rulesPath,
-                                 strlen (rulesPath) - 4);
-            free(old);
+        char *rulesPath = FcitxXkbFindXkbRulesFile(xkb);
+        size_t rulesBaseLen = strlen(rulesPath) - strlen(".xml");
+        if (strcmp(rulesPath + rulesBaseLen, ".xml") == 0) {
+            rulesPath[rulesBaseLen] = '\0';
         }
-        rules = XkbRF_Load (rulesPath, "C", True, True);
-    }
-    if (rules == NULL) {
+        rules = XkbRF_Load(rulesPath, "C", True, True);
         free(rulesPath);
-        return False;
     }
+    if (rules == NULL)
+        return False;
 
-    memset (&rdefs, 0, sizeof (XkbRF_VarDefsRec));
-    memset (&rnames, 0, sizeof (XkbComponentNamesRec));
-    rdefs.model = model ? strdup (model) : NULL;
-    rdefs.layout = all_layouts ? strdup (all_layouts) : NULL;
-    rdefs.variant = all_variants ? strdup (all_variants) : NULL;
-    rdefs.options = all_options ? strdup (all_options) : NULL;
-    XkbRF_GetComponents (rules, &rdefs, &rnames);
-    xkbDesc = XkbGetKeyboardByName (dpy, XkbUseCoreKbd, &rnames,
-                                XkbGBN_AllComponentsMask,
-                                XkbGBN_AllComponentsMask &
-                                (~XkbGBN_GeometryMask), True);
+    memset(&rdefs, 0, sizeof(XkbRF_VarDefsRec));
+    memset(&rnames, 0, sizeof(XkbComponentNamesRec));
+    rdefs.model = model ? strdup(model) : NULL;
+    rdefs.layout = all_layouts ? strdup(all_layouts) : NULL;
+    rdefs.variant = all_variants ? strdup(all_variants) : NULL;
+    rdefs.options = all_options ? strdup(all_options) : NULL;
+    XkbRF_GetComponents(rules, &rdefs, &rnames);
+    xkbDesc = XkbGetKeyboardByName(dpy, XkbUseCoreKbd, &rnames,
+                                   XkbGBN_AllComponentsMask,
+                                   XkbGBN_AllComponentsMask &
+                                   (~XkbGBN_GeometryMask), True);
 
     Bool result = True;
     if (!xkbDesc) {
@@ -317,30 +305,28 @@ FcitxXkbSetRules (FcitxXkb* xkb,
         result = False;
     }
     else {
-       char* tempstr = strdup(rules_file);
-       XkbRF_SetNamesProp (dpy, tempstr, &rdefs);
-       free (tempstr);
+        char* tempstr = strdup(rules_file);
+        XkbRF_SetNamesProp(dpy, tempstr, &rdefs);
+        free (tempstr);
     }
-    free (rulesPath);
-    free (rdefs.model);
-    free (rdefs.layout);
-    free (rdefs.variant);
-    free (rdefs.options);
+    free(rdefs.model);
+    free(rdefs.layout);
+    free(rdefs.variant);
+    free(rdefs.options);
 
     return result;
 }
 
 static Bool
-FcitxXkbUpdateProperties (FcitxXkb* xkb,
-                     const char *rules_file, const char *model,
-                     const char *all_layouts, const char *all_variants,
-                     const char *all_options)
+FcitxXkbUpdateProperties(FcitxXkb* xkb, const char *rules_file,
+                         const char *model, const char *all_layouts,
+                         const char *all_variants, const char *all_options)
 {
     Display *dpy = xkb->dpy;
     int len;
     char *pval;
     char *next;
-    Atom rules_atom;
+    static Atom rules_atom = None;
     Window root_window;
 
     len = (rules_file ? strlen (rules_file) : 0);
@@ -354,7 +340,8 @@ FcitxXkbUpdateProperties (FcitxXkb* xkb,
     }
     len += 5; /* trailing NULs */
 
-    rules_atom = XInternAtom (dpy, _XKB_RF_NAMES_PROP_ATOM, False);
+    if (rules_atom == None)
+        rules_atom = XInternAtom (dpy, _XKB_RF_NAMES_PROP_ATOM, False);
     root_window = XDefaultRootWindow (dpy);
     pval = next = (char*) fcitx_utils_malloc0 (sizeof(char) *(len + 1));
     if (!pval) {
@@ -392,8 +379,8 @@ FcitxXkbUpdateProperties (FcitxXkb* xkb,
     }
 
     XChangeProperty (dpy, root_window,
-                    rules_atom, XA_STRING, 8, PropModeReplace,
-                    (unsigned char *) pval, len);
+                     rules_atom, XA_STRING, 8, PropModeReplace,
+                     (unsigned char *) pval, len);
     XSync(dpy, False);
 
     return True;
@@ -402,8 +389,11 @@ FcitxXkbUpdateProperties (FcitxXkb* xkb,
 static void
 FcitxXkbGetCurrentLayoutInternal(FcitxXkb *xkb, char **layout, char **variant)
 {
-    char* const * layoutName = (char* const*) utarray_eltptr(xkb->defaultLayouts, FcitxXkbGetCurrentGroup(xkb));
-    char* const * pVariantName = (char* const*) utarray_eltptr(xkb->defaultVariants, FcitxXkbGetCurrentGroup(xkb));
+    int cur_group = FcitxXkbGetCurrentGroup(xkb);
+    char* const *layoutName = fcitx_array_eltptr(xkb->defaultLayouts,
+                                                 cur_group);
+    char* const *pVariantName = fcitx_array_eltptr(xkb->defaultVariants,
+                                                   cur_group);
     if (layoutName)
         *layout = strdup(*layoutName);
     else
@@ -414,17 +404,10 @@ FcitxXkbGetCurrentLayoutInternal(FcitxXkb *xkb, char **layout, char **variant)
         *variant = NULL;
 }
 
-static void*
-FcitxXkbGetCurrentLayout (void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxXkbGetCurrentLayoutInternal((FcitxXkb*)arg,
-                                     args.args[0], args.args[1]);
-    return NULL;
-}
 #if 0
 
-static char *
-FcitxXkbGetCurrentModel (FcitxXkb* xkb)
+static char*
+FcitxXkbGetCurrentModel(FcitxXkb* xkb)
 {
     if (xkb->defaultModels == NULL) {
         return NULL;
@@ -432,8 +415,8 @@ FcitxXkbGetCurrentModel (FcitxXkb* xkb)
     return fcitx_utils_join_string_list(xkb->defaultModels, ',');
 }
 
-static char *
-FcitxXkbGetCurrentOption (FcitxXkb* xkb)
+static char*
+FcitxXkbGetCurrentOption(FcitxXkb* xkb)
 {
     if (xkb->defaultOptions == NULL) {
         return NULL;
@@ -444,26 +427,24 @@ FcitxXkbGetCurrentOption (FcitxXkb* xkb)
 
 /* This SHOULD be _SetLayouts_ .... */
 static boolean
-FcitxXkbSetLayout  (FcitxXkb* xkb,
-               const char *layouts,
-               const char *variants,
-               const char *options)
+FcitxXkbSetLayout(FcitxXkb* xkb, const char *layouts,
+                  const char *variants, const char *options)
 {
-    boolean retval;
+    boolean retval = False;
     char *layouts_line;
     char *options_line;
     char *variants_line;
     char *model_line;
 
-    if (xkb->defaultLayouts == NULL) {
+    if (utarray_len(xkb->defaultLayouts) == 0) {
         FcitxLog(WARNING, "Your system seems not to support XKB.");
         return False;
     }
 
     if (layouts == NULL)
-        layouts_line = fcitx_utils_join_string_list (xkb->defaultLayouts, ',');
+        layouts_line = fcitx_utils_join_string_list(xkb->defaultLayouts, ',');
     else
-        layouts_line = strdup (layouts);
+        layouts_line = strdup(layouts);
 
     if (variants == NULL)
         variants_line = fcitx_utils_join_string_list(xkb->defaultVariants, ',');
@@ -477,32 +458,36 @@ FcitxXkbSetLayout  (FcitxXkb* xkb,
 
     model_line = fcitx_utils_join_string_list(xkb->defaultModels, ',');
 
-    retval = FcitxXkbSetRules(xkb,
-                              FcitxXkbGetRulesName(xkb), model_line,
-                              layouts_line, variants_line, options_line);
-    FcitxXkbUpdateProperties(xkb,
-                             FcitxXkbGetRulesName(xkb), model_line,
-                             layouts_line, variants_line, options_line);
-    free (layouts_line);
-    free (variants_line);
-    free (options_line);
-    free (model_line);
+    char* rulesName = FcitxXkbGetRulesName(xkb);
+    if (rulesName) {
+        retval = FcitxXkbSetRules(xkb,
+                                  rulesName, model_line,
+                                  layouts_line, variants_line, options_line);
+        FcitxXkbUpdateProperties(xkb,
+                                 rulesName, model_line,
+                                 layouts_line, variants_line, options_line);
+        free(rulesName);
+    }
+    free(layouts_line);
+    free(variants_line);
+    free(options_line);
+    free(model_line);
 
     return retval;
 }
 
 static int
-FcitxXkbGetCurrentGroup (FcitxXkb* xkb)
+FcitxXkbGetCurrentGroup(FcitxXkb* xkb)
 {
     Display *dpy = xkb->dpy;
     XkbStateRec state;
 
-    if (xkb->defaultLayouts == NULL) {
+    if (utarray_len(xkb->defaultLayouts) == 0) {
         FcitxLog(WARNING, "Your system seems not to support XKB.");
         return 0;
     }
 
-    if (XkbGetState (dpy, XkbUseCoreKbd, &state) != Success) {
+    if (XkbGetState(dpy, XkbUseCoreKbd, &state) != Success) {
         FcitxLog(WARNING, "Could not get state");
         return 0;
     }
@@ -510,12 +495,14 @@ FcitxXkbGetCurrentGroup (FcitxXkb* xkb)
     return state.group;
 }
 
-static void FcitxXkbAddNewLayout(FcitxXkb* xkb, const char* layoutString, const char* variantString)
+static void FcitxXkbAddNewLayout(FcitxXkb* xkb, const char* layoutString,
+                                 const char* variantString)
 {
     if (!layoutString)
         return;
 
-    while (utarray_len(xkb->defaultVariants) < utarray_len(xkb->defaultLayouts)) {
+    while (utarray_len(xkb->defaultVariants) <
+           utarray_len(xkb->defaultLayouts)) {
         const char* dummy = "";
         utarray_push_back(xkb->defaultVariants, &dummy);
     }
@@ -574,13 +561,13 @@ FcitxXkbRetrieveCloseGroup(FcitxXkb *xkb)
 
 static void FcitxXkbIMKeyboardLayoutChanged(void* arg, const void* value)
 {
-    /* fcitx-keyboard will be useless, if fcitx-xkb doesn't change the layout for them */
-    FcitxXkb* xkb = (FcitxXkb*) arg;
-    FcitxIM* currentIM = FcitxInstanceGetCurrentIM(xkb->owner);
+    /* fcitx-keyboard won't be able to set layout,
+     * if fcitx-xkb doesn't change the layout for it. */
+    FcitxXkb *xkb = (FcitxXkb*) arg;
+    FcitxIM *currentIM = FcitxInstanceGetCurrentIM(xkb->owner);
     if (xkb->config.bIgnoreInputMethodLayoutRequest
         && (!currentIM || strncmp(currentIM->uniqueName, "fcitx-keyboard",
-                                  strlen("fcitx-keyboard")) != 0))
-    {
+                                  strlen("fcitx-keyboard")) != 0)) {
         FcitxXkbRetrieveCloseGroup(xkb);
         return;
     }
@@ -600,9 +587,9 @@ static void FcitxXkbIMKeyboardLayoutChanged(void* arg, const void* value)
         else {
             const char* layout = (const char*) value;
             if (layout) {
-                UT_array* s = fcitx_utils_split_string(layout, ',');
-                char** pLayoutString = (char**) utarray_eltptr(s, 0);
-                char** pVariantString = (char**) utarray_eltptr(s, 1);
+                s = fcitx_utils_split_string(layout, ',');
+                char  **pLayoutString = (char**)utarray_eltptr(s, 0);
+                char **pVariantString = (char**)utarray_eltptr(s, 1);
                 layoutString = (pLayoutString)? *pLayoutString: NULL;
                 variantString = (pVariantString)? *pVariantString: NULL;
             }
@@ -613,10 +600,10 @@ static void FcitxXkbIMKeyboardLayoutChanged(void* arg, const void* value)
         }
         if (!FcitxXkbSetLayoutByName(xkb, layoutString, variantString))
             FcitxXkbRetrieveCloseGroup(xkb);
-        if (s)
+        if (s) {
             fcitx_utils_free_string_list(s);
-    }
-    else {
+        }
+    } else {
         FcitxXkbRetrieveCloseGroup(xkb);
     }
 }
@@ -628,13 +615,13 @@ static int FcitxXkbFindLayoutIndex(FcitxXkb* xkb, const char* layout, const char
     if (layout == NULL)
         return -1;
 
-    int i = 0;
-    for (i = 0; i < utarray_len(xkb->defaultLayouts); i++ )
-    {
-        layoutName = (char**) utarray_eltptr(xkb->defaultLayouts, i);
-        pVariantName = (char**) utarray_eltptr(xkb->defaultVariants, i);
+    unsigned int i = 0;
+    for (i = 0;i < utarray_len(xkb->defaultLayouts);i++) {
+        layoutName = (char**)utarray_eltptr(xkb->defaultLayouts, i);
+        pVariantName = (char**)utarray_eltptr(xkb->defaultVariants, i);
         variantName = pVariantName ? *pVariantName : NULL;
-        if (strcmp(*layoutName, layout) == 0 && fcitx_utils_strcmp_empty(variantName, variant) == 0) {
+        if (strcmp(*layoutName, layout) == 0 &&
+            fcitx_utils_strcmp_empty(variantName, variant) == 0) {
             return i;
         }
     }
@@ -648,25 +635,36 @@ FcitxXkbSaveCloseGroup(FcitxXkb *xkb)
     char *tmpvariant;
     FcitxXkbGetCurrentLayoutInternal(xkb, &tmplayout, &tmpvariant);
     if (!tmplayout) {
-        if (tmpvariant)
-            free(tmpvariant);
+        fcitx_utils_free(tmpvariant);
         return;
     }
-    if (xkb->closeLayout)
-        free(xkb->closeLayout);
+    fcitx_utils_free(xkb->closeLayout);
+    fcitx_utils_free(xkb->closeVariant);
     xkb->closeLayout = tmplayout;
-    if (xkb->closeVariant)
-        free(xkb->closeVariant);
     xkb->closeVariant = tmpvariant;
 }
 
+static void
+FcitxXkbGetDefaultXmodmap(FcitxXkb *xkb)
+{
+    static char *home = NULL;
+    if (fcitx_likely(xkb->defaultXmodmapPath))
+        return;
+    if (fcitx_likely(!home)) {
+        home = getenv("HOME");
+        if (fcitx_unlikely(!home)) {
+            return;
+        }
+    }
+    fcitx_utils_alloc_cat_str(xkb->defaultXmodmapPath, home, "/.Xmodmap");
+}
+
 static void* FcitxXkbCreate(FcitxInstance* instance)
 {
-    FcitxAddon* addon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), "fcitx-xkb");
-    FcitxXkb* xkb = (FcitxXkb*) fcitx_utils_malloc0(sizeof(FcitxXkb));
+    FcitxXkb *xkb = fcitx_utils_new(FcitxXkb);
     xkb->owner = instance;
     do {
-        xkb->dpy = InvokeVaArgs(xkb->owner, FCITX_X11, GETDISPLAY);
+        xkb->dpy = FcitxX11GetDisplay(xkb->owner);
         if (!xkb->dpy)
             break;
 
@@ -681,16 +679,21 @@ static void* FcitxXkbCreate(FcitxInstance* instance)
         xkb->rules = FcitxXkbReadRules(rulesPath);
         free(rulesPath);
 
+        xkb->defaultLayouts = fcitx_utils_new_string_list();
+        xkb->defaultModels = fcitx_utils_new_string_list();
+        xkb->defaultOptions = fcitx_utils_new_string_list();
+        xkb->defaultVariants = fcitx_utils_new_string_list();
+
         FcitxXkbInitDefaultLayout(xkb);
         FcitxXkbSaveCloseGroup(xkb);
 
         int eventMask = XkbNewKeyboardNotifyMask | XkbStateNotifyMask;
         XkbSelectEvents(xkb->dpy, XkbUseCoreKbd, eventMask, eventMask);
 
-        InvokeVaArgs(xkb->owner, FCITX_X11, ADDXEVENTHANDLER,
-                     FcitxXkbEventHandler, xkb);
+        FcitxX11AddXEventHandler(xkb->owner, FcitxXkbEventHandler, xkb);
 
-        FcitxInstanceWatchContext(instance, CONTEXT_IM_KEYBOARD_LAYOUT, FcitxXkbIMKeyboardLayoutChanged, xkb);
+        FcitxInstanceWatchContext(instance, CONTEXT_IM_KEYBOARD_LAYOUT,
+                                  FcitxXkbIMKeyboardLayoutChanged, xkb);
 
         FcitxIMEventHook hk;
         hk.arg = xkb;
@@ -703,21 +706,13 @@ static void* FcitxXkbCreate(FcitxInstance* instance)
         hk.func = FcitxXkbCurrentStateChangedTriggerOn;
         FcitxInstanceRegisterTriggerOnHook(instance, hk);
 
-        AddFunction(addon, FcitxXkbGetRules);
-        AddFunction(addon, FcitxXkbGetCurrentLayout);
-        AddFunction(addon, FcitxXkbLayoutExists);
-        AddFunction(addon, FcitxXkbGetLayoutOverride);
-        AddFunction(addon, FcitxXkbSetLayoutOverride);
-        AddFunction(addon, FcitxXkbSetDefaultLayout);
-
+        FcitxXkbAddFunctions(instance);
         if (xkb->config.bOverrideSystemXKBSettings)
             FcitxXkbSetLayout(xkb, NULL, NULL, NULL);
-
         return xkb;
     } while (0);
 
     free(xkb);
-
     return NULL;
 }
 
@@ -728,7 +723,8 @@ static boolean FcitxXkbEventHandler(void* arg, XEvent* event)
     if (event->type == xkb->xkbOpcode) {
         XkbEvent *xkbEvent = (XkbEvent*) event;
 
-        if (xkbEvent->any.xkb_type == XkbStateNotify && xkbEvent->state.changed & GROUP_CHANGE_MASK) {
+        if (xkbEvent->any.xkb_type == XkbStateNotify &&
+            xkbEvent->state.changed & GROUP_CHANGE_MASK) {
             FcitxIM* currentIM = FcitxInstanceGetCurrentIM(xkb->owner);
             if (FcitxInstanceGetCurrentStatev2(xkb->owner) != IS_ACTIVE
                 || (xkb->config.bIgnoreInputMethodLayoutRequest
@@ -738,7 +734,10 @@ static boolean FcitxXkbEventHandler(void* arg, XEvent* event)
                 FcitxXkbSaveCloseGroup(xkb);
         }
 
-        if (xkbEvent->any.xkb_type == XkbNewKeyboardNotify) {
+        if (xkbEvent->any.xkb_type == XkbNewKeyboardNotify
+            && xkbEvent->new_kbd.serial != xkb->lastSerial
+        ) {
+            xkb->lastSerial = xkbEvent->new_kbd.serial;
             XSync(xkb->dpy, False);
             FcitxUIUpdateInputWindow(xkb->owner);
             FcitxXkbInitDefaultLayout(xkb);
@@ -771,14 +770,13 @@ static void FcitxXkbDestroy(void* arg)
     FcitxXkbRetrieveCloseGroup(xkb);
     XSync(xkb->dpy, False);
     FcitxXkbRulesFree(xkb->rules);
-    fcitx_utils_free_string_list (xkb->defaultVariants);
-    xkb->defaultVariants = NULL;
-    fcitx_utils_free_string_list (xkb->defaultLayouts);
-    xkb->defaultLayouts = NULL;
-    fcitx_utils_free_string_list (xkb->defaultModels);
-    xkb->defaultModels = NULL;
-    fcitx_utils_free_string_list (xkb->defaultOptions);
-    xkb->defaultOptions = NULL;
+    fcitx_utils_free_string_list(xkb->defaultVariants);
+    fcitx_utils_free_string_list(xkb->defaultLayouts);
+    fcitx_utils_free_string_list(xkb->defaultModels);
+    fcitx_utils_free_string_list(xkb->defaultOptions);
+    fcitx_utils_free(xkb->closeLayout);
+    fcitx_utils_free(xkb->closeVariant);
+    fcitx_utils_free(xkb->defaultXmodmapPath);
     free(xkb);
 }
 
@@ -791,41 +789,36 @@ static void FcitxXkbReloadConfig(void* arg)
         FcitxXkbSetLayout(xkb, NULL, NULL, NULL);
 }
 
-
-static void* FcitxXkbGetRules(void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxXkb* xkb = (FcitxXkb*) arg;
-    return xkb->rules;
-}
-
-static void* FcitxXkbLayoutExists(void* arg, FcitxModuleFunctionArg args)
-{
-    FcitxXkb* xkb = (FcitxXkb*) arg;
-    int idx = FcitxXkbFindLayoutIndex(xkb, args.args[0], args.args[1]);
-    boolean* result = args.args[2];
-    *result = (idx >= 0);
-    return NULL;
-}
-
 static void FcitxXkbApplyCustomScript(FcitxXkb* xkb)
 {
-    if (!xkb->config.bOverrideSystemXKBSettings)
+    FcitxXkbConfig *config = &xkb->config;
+    if (!config->bOverrideSystemXKBSettings || !config->xmodmapCommand ||
+        !config->xmodmapCommand[0])
         return;
 
-    if (!(xkb->config.xmodmapCommand && xkb->config.xmodmapCommand[0]
-        && xkb->config.customXModmapScript && xkb->config.customXModmapScript[0]))
-        return;
-
-    char* customXModmapScript = NULL;
-    FcitxXDGGetFileUserWithPrefix("data", xkb->config.customXModmapScript, NULL, &customXModmapScript);
+    char *to_free = NULL;
+    char *xmodmap_script = NULL;
+    if (!config->customXModmapScript || !config->customXModmapScript[0]) {
+        if (strcmp(config->xmodmapCommand, "xmodmap") == 0) {
+            FcitxXkbGetDefaultXmodmap(xkb);
+            if (!(xkb->defaultXmodmapPath &&
+                  fcitx_utils_isreg(xkb->defaultXmodmapPath)))
+                return;
+            xmodmap_script = xkb->defaultXmodmapPath;
+        }
+    } else {
+        FcitxXDGGetFileUserWithPrefix("data", config->customXModmapScript,
+                                      NULL, &to_free);
+        xmodmap_script = to_free;
+    }
 
     char* args[] = {
-        xkb->config.xmodmapCommand,
-        customXModmapScript,
-        0
+        config->xmodmapCommand,
+        xmodmap_script,
+        NULL
     };
     fcitx_utils_start_process(args);
-    free(customXModmapScript);
+    fcitx_utils_free(to_free);
 }
 
 static inline void LayoutOverrideFree(LayoutOverride* item) {
@@ -843,12 +836,13 @@ void LoadLayoutOverride(FcitxXkb* xkb)
 
     /* clean all old configuration */
     while (xkb->layoutOverride) {
-        LayoutOverride* cur = xkb->layoutOverride;
+        LayoutOverride *cur = xkb->layoutOverride;
         HASH_DEL(xkb->layoutOverride, cur);
         LayoutOverrideFree(cur);
     }
 
-    FILE *fp = FcitxXDGGetFileUserWithPrefix("data", "layout_override", "r", NULL);
+    FILE *fp = FcitxXDGGetFileUserWithPrefix("data", "layout_override",
+                                             "r", NULL);
     if (!fp)
         return;
 
@@ -858,9 +852,9 @@ void LoadLayoutOverride(FcitxXkb* xkb)
         trimedbuf = fcitx_utils_trim(buf);
 
         UT_array* s = fcitx_utils_split_string(trimedbuf, ',');
-        char** pIMString = (char**) utarray_eltptr(s, 0);
-        char** pLayoutString = (char**) utarray_eltptr(s, 1);
-        char** pVariantString = (char**) utarray_eltptr(s, 2);
+        char** pIMString = (char**)utarray_eltptr(s, 0);
+        char** pLayoutString = (char**)utarray_eltptr(s, 1);
+        char** pVariantString = (char**)utarray_eltptr(s, 2);
         char* imString = (pIMString)? *pIMString: NULL;
         char* layoutString = (pLayoutString)? *pLayoutString: NULL;
         char* variantString = (pVariantString)? *pVariantString: NULL;
@@ -880,7 +874,8 @@ void LoadLayoutOverride(FcitxXkb* xkb)
         override->layout = strdup(layoutString);
         override->variant= variantString? strdup(variantString) : NULL;
 
-        HASH_ADD_KEYPTR(hh, xkb->layoutOverride, override->im, strlen(override->im), override);
+        HASH_ADD_KEYPTR(hh, xkb->layoutOverride,
+                        override->im, strlen(override->im), override);
     }
 
     if (buf)
@@ -897,12 +892,13 @@ boolean LoadXkbConfig(FcitxXkb* xkb)
     if (configDesc == NULL)
         return false;
 
-    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-xkb.config", "r", NULL);
+    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-xkb.config",
+                                             "r", NULL);
 
-    if (!fp)
-    {
-        if (errno == ENOENT)
+    if (!fp) {
+        if (errno == ENOENT) {
             SaveXkbConfig(xkb);
+        }
     }
     FcitxConfigFile *cfile = FcitxConfigParseConfigFileFp(fp, configDesc);
     FcitxXkbConfigConfigBind(&xkb->config, cfile, configDesc);
@@ -945,34 +941,26 @@ void SaveXkbConfig(FcitxXkb* xkb)
     SaveLayoutOverride(xkb);
 }
 
-
-void* FcitxXkbGetLayoutOverride(void* arg, FcitxModuleFunctionArg args)
+static void
+FcitxXkbGetLayoutOverride(FcitxXkb *xkb, const char *imname, char **layout,
+                          char **variant)
 {
-    FcitxXkb* xkb = arg;
-    const char* imname = args.args[0];
-    char** layout = args.args[1];
-    char** variant = args.args[2];
-    LayoutOverride* item = NULL;
+    LayoutOverride *item = NULL;
     HASH_FIND_STR(xkb->layoutOverride, imname, item);
     if (item) {
         *layout = item->layout;
         *variant = item->variant;
-    }
-    else {
+    } else {
         *layout = NULL;
         *variant = NULL;
     }
-    return NULL;
 }
 
-
-void* FcitxXkbSetLayoutOverride(void* arg, FcitxModuleFunctionArg args)
+static void
+FcitxXkbSetLayoutOverride(FcitxXkb *xkb, const char *imname, const char *layout,
+                          const char* variant)
 {
-    FcitxXkb* xkb = arg;
-    const char* imname = args.args[0];
-    const char* layout = args.args[1];
-    const char* variant = args.args[2];
-    LayoutOverride* item = NULL;
+    LayoutOverride *item = NULL;
     HASH_FIND_STR(xkb->layoutOverride, imname, item);
     if (item) {
         HASH_DEL(xkb->layoutOverride, item);
@@ -985,22 +973,17 @@ void* FcitxXkbSetLayoutOverride(void* arg, FcitxModuleFunctionArg args)
         item->im = strdup(imname);
         item->layout = strdup(layout);
         item->variant= (variant && variant[0] != '\0')? strdup(variant) : NULL;
-
-        HASH_ADD_KEYPTR(hh, xkb->layoutOverride, item->im, strlen(item->im), item);
+        HASH_ADD_KEYPTR(hh, xkb->layoutOverride, item->im,
+                        strlen(item->im), item);
     }
-
     SaveLayoutOverride(xkb);
     FcitxXkbCurrentStateChanged(xkb);
-
-    return NULL;
 }
 
-void* FcitxXkbSetDefaultLayout(void* arg, FcitxModuleFunctionArg args)
+static void
+FcitxXkbSetDefaultLayout(FcitxXkb *xkb, const char *layout, const char *variant)
 {
-    FcitxXkb* xkb = arg;
-    const char* layout = args.args[0];
-    const char* variant = args.args[1];
-    LayoutOverride* item = NULL;
+    LayoutOverride *item = NULL;
     HASH_FIND_STR(xkb->layoutOverride, "default", item);
     if (item) {
         HASH_DEL(xkb->layoutOverride, item);
@@ -1012,12 +995,11 @@ void* FcitxXkbSetDefaultLayout(void* arg, FcitxModuleFunctionArg args)
         item->im = strdup("default");
         item->layout = strdup(layout);
         item->variant= (variant && variant[0] != '\0')? strdup(variant) : NULL;
-
-        HASH_ADD_KEYPTR(hh, xkb->layoutOverride, item->im, strlen(item->im), item);
+        HASH_ADD_KEYPTR(hh, xkb->layoutOverride, item->im,
+                        strlen(item->im), item);
     }
-
     SaveLayoutOverride(xkb);
     FcitxXkbCurrentStateChanged(xkb);
-
-    return NULL;
 }
+
+#include "fcitx-xkb-addfunctions.h"
