@@ -19,16 +19,14 @@
  ***************************************************************************/
 
 #include <dbus/dbus.h>
-
 #include "config.h"
 
 #include "libintl.h"
-#include "module/xkb/xkb.h"
-#include "module/xkb/rules.h"
-#include "module/dbus/dbusstuff.h"
+#include "module/xkb/fcitx-xkb.h"
+#include "module/dbus/fcitx-dbus.h"
 #include "fcitx-utils/log.h"
 #include "fcitx/module.h"
-#include <im/keyboard/isocodes.h>
+#include "im/keyboard/isocodes.h"
 
 static void* FcitxXkbDBusCreate(struct _FcitxInstance* instance);
 typedef struct _FcitxXkbDBus {
@@ -45,6 +43,16 @@ FCITX_DEFINE_PLUGIN(fcitx_xkbdbus, module, FcitxModule) = {
     NULL
 };
 
+
+static inline DBusMessage* FcitxXkbDBusUnknownMethod(DBusMessage *msg)
+{
+    DBusMessage* reply = dbus_message_new_error_printf(msg,
+                                                       DBUS_ERROR_UNKNOWN_METHOD,
+                                                       "No such method with signature (%s)",
+                                                       dbus_message_get_signature(msg));
+    return reply;
+}
+
 static DBusHandlerResult FcitxXkbDBusEventHandler (DBusConnection  *connection,
                                                    DBusMessage     *message,
                                                    void            *user_data);
@@ -84,21 +92,24 @@ void* FcitxXkbDBusCreate(FcitxInstance* instance)
     FcitxXkbDBus* xkbdbus = fcitx_utils_new(FcitxXkbDBus);
     xkbdbus->owner = instance;
     do {
-        DBusConnection *conn = InvokeVaArgs(instance, FCITX_DBUS,
-                                            GETCONNECTION);
-        if (conn == NULL) {
+        DBusConnection *conn = FcitxDBusGetConnection(instance);
+        DBusConnection *privconn = FcitxDBusGetPrivConnection(instance);
+        if (conn == NULL && privconn == NULL) {
             FcitxLog(ERROR, "DBus Not initialized");
             break;
         }
 
         DBusObjectPathVTable fcitxIPCVTable = {NULL, &FcitxXkbDBusEventHandler, NULL, NULL, NULL, NULL };
 
-        if (!dbus_connection_register_object_path(conn,  FCITX_XKB_PATH,
-                &fcitxIPCVTable, xkbdbus)) {
-            FcitxLog(ERROR, "No memory");
-            break;
+        if (conn) {
+            dbus_connection_register_object_path(conn, FCITX_XKB_PATH, &fcitxIPCVTable, xkbdbus);
         }
-        FcitxXkbRules *rules = InvokeVaArgs(instance, FCITX_XKB, GETRULES);
+
+        if (privconn) {
+            dbus_connection_register_object_path(privconn, FCITX_XKB_PATH, &fcitxIPCVTable, xkbdbus);
+        }
+
+        FcitxXkbRules *rules = FcitxXkbGetRules(instance);
 
         if (!rules)
             break;
@@ -113,7 +124,9 @@ void* FcitxXkbDBusCreate(FcitxInstance* instance)
     return NULL;
 }
 
-void FcitxXkbDBusAppendLayout(DBusMessageIter* sub, const char* layout, const char* variant, const char* description, const char* lang)
+void FcitxXkbDBusAppendLayout(DBusMessageIter* sub, const char* layout,
+                              const char* variant, const char* description,
+                              const char* lang)
 {
     DBusMessageIter ssub;
     if (!lang)
@@ -155,36 +168,32 @@ void FcitxXkbDBusGetLayouts(FcitxXkbDBus* xkbdbus, DBusMessage* message)
             layoutInfo != NULL;
             layoutInfo = (FcitxXkbLayoutInfo*) utarray_next(rules->layoutInfos, layoutInfo))
         {
-            char* description;
-            asprintf(&description, "%s",
-                     dgettext("xkeyboard-config", layoutInfo->description));
-
+            char* description = dgettext("xkeyboard-config",
+                                         layoutInfo->description);
             GET_LANG;
-            FcitxXkbDBusAppendLayout(&sub, layoutInfo->name, "", description, lang);
-            free(description);
+            FcitxXkbDBusAppendLayout(&sub, layoutInfo->name, "",
+                                     description, lang);
             FcitxXkbVariantInfo* variantInfo;
             for (variantInfo = (FcitxXkbVariantInfo*) utarray_front(layoutInfo->variantInfos);
                 variantInfo != NULL;
                 variantInfo = (FcitxXkbVariantInfo*) utarray_next(layoutInfo->variantInfos, variantInfo))
             {
-                char* description;
-                asprintf(&description, "%s - %s",
-                        dgettext("xkeyboard-config", layoutInfo->description),
-                        dgettext("xkeyboard-config", variantInfo->description));
-
+                char *description;
+                fcitx_utils_alloc_cat_str(
+                    description,
+                    dgettext("xkeyboard-config", layoutInfo->description),
+                    " - ",
+                    dgettext("xkeyboard-config", variantInfo->description));
                 GET_LANG;
-                FcitxXkbDBusAppendLayout(&sub, layoutInfo->name, variantInfo->name, description, lang);
+                FcitxXkbDBusAppendLayout(&sub, layoutInfo->name,
+                                         variantInfo->name, description, lang);
                 free(description);
             }
         }
     }
     else {
-        char* description;
-        asprintf(&description, "%s",
-                dgettext("xkeyboard-config", "English (US)"));
-
+        char *description = dgettext("xkeyboard-config", "English (US)");
         FcitxXkbDBusAppendLayout(&sub, "us", "", description, "en");
-        free(description);
     }
     dbus_message_iter_close_container(&iter, &sub);
 }
@@ -194,66 +203,63 @@ DBusHandlerResult FcitxXkbDBusEventHandler (DBusConnection  *connection,
                                             void            *user_data)
 {
     FcitxXkbDBus* xkbdbus = user_data;
+    DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    DBusMessage *reply = NULL;
+    boolean flush = false;
     if (dbus_message_is_method_call(message, DBUS_INTERFACE_INTROSPECTABLE, "Introspect")) {
-        DBusMessage *reply = dbus_message_new_method_return(message);
+        reply = dbus_message_new_method_return(message);
 
         dbus_message_append_args(reply, DBUS_TYPE_STRING, &introspection_xml, DBUS_TYPE_INVALID);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(message, FCITX_XKB_INTERFACE, "GetLayouts")) {
-        DBusMessage *reply = dbus_message_new_method_return(message);
+        reply = dbus_message_new_method_return(message);
         FcitxXkbDBusGetLayouts(xkbdbus, reply);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        dbus_connection_flush(connection);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(message, FCITX_XKB_INTERFACE, "SetLayoutForIM")) {
         DBusError error;
         dbus_error_init(&error);
         char* im, *layout, *variant;
         if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &im, DBUS_TYPE_STRING, &layout, DBUS_TYPE_STRING, &variant, DBUS_TYPE_INVALID)) {
-            InvokeVaArgs(xkbdbus->owner, FCITX_XKB, SETLAYOUTOVERRIDE,
-                         im, layout, variant);
+            FcitxXkbSetLayoutOverride(xkbdbus->owner, im, layout, variant);
+            reply = dbus_message_new_method_return(message);
+        } else {
+            reply = FcitxXkbDBusUnknownMethod(message);
         }
-        DBusMessage *reply = dbus_message_new_method_return(message);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        dbus_connection_flush(connection);
-        return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(message, FCITX_XKB_INTERFACE, "SetDefaultLayout")) {
         DBusError error;
         dbus_error_init(&error);
         char *layout, *variant;
         if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &layout, DBUS_TYPE_STRING, &variant, DBUS_TYPE_INVALID)) {
-            InvokeVaArgs(xkbdbus->owner, FCITX_XKB, SETDEFAULTLAYOUT,
-                           layout, variant);
+            FcitxXkbSetDefaultLayout(xkbdbus->owner, layout, variant);
+            reply = dbus_message_new_method_return(message);
+        } else {
+            reply = FcitxXkbDBusUnknownMethod(message);
         }
-        DBusMessage *reply = dbus_message_new_method_return(message);
-        dbus_connection_send(connection, reply, NULL);
-        dbus_message_unref(reply);
-        dbus_connection_flush(connection);
-        return DBUS_HANDLER_RESULT_HANDLED;
     }  else if (dbus_message_is_method_call(message, FCITX_XKB_INTERFACE, "GetLayoutForIM")) {
         DBusError error;
         dbus_error_init(&error);
         char* im = NULL, *layout = NULL, *variant = NULL;
-        if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &im, DBUS_TYPE_INVALID)) {
-            InvokeVaArgs(xkbdbus->owner, FCITX_XKB, GETLAYOUTOVERRIDE,
-                         im, &layout, &variant);
+        if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING,
+                                  &im, DBUS_TYPE_INVALID)) {
+            FcitxXkbGetLayoutOverride(xkbdbus->owner, im, &layout, &variant);
 
             if (!layout)
                 layout = "";
             if (!variant)
                 variant = "";
 
-            DBusMessage *reply = dbus_message_new_method_return(message);
+            reply = dbus_message_new_method_return(message);
             dbus_message_append_args(reply, DBUS_TYPE_STRING, &layout, DBUS_TYPE_STRING, &variant, DBUS_TYPE_INVALID);
-            dbus_connection_send(connection, reply, NULL);
-            dbus_message_unref(reply);
+        } else {
+            reply = FcitxXkbDBusUnknownMethod(message);
+        }
+    }
+
+    if (reply) {
+        dbus_connection_send(connection, reply, NULL);
+        dbus_message_unref(reply);
+        if (flush) {
             dbus_connection_flush(connection);
         }
-        return DBUS_HANDLER_RESULT_HANDLED;
+        result = DBUS_HANDLER_RESULT_HANDLED;
     }
-    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    return result;
 }
