@@ -30,12 +30,12 @@
 #include "fcitx/frontend.h"
 #include "fcitx/configfile.h"
 #include "fcitx/instance.h"
+#include "fcitx/candidate.h"
 #include "fcitx-utils/utils.h"
 
 #include "InputWindow.h"
 #include "classicui.h"
 #include "skin.h"
-#include "module/x11/x11stuff.h"
 #include "MainWindow.h"
 #include "fcitx-utils/log.h"
 #include <cairo/cairo.h>
@@ -113,10 +113,10 @@ InputWindow* CreateInputWindow(FcitxClassicUI *classicui)
     inputWindow->owner = classicui;
     InitInputWindow(inputWindow);
 
-    InvokeVaArgs(classicui->owner, FCITX_X11, ADDXEVENTHANDLER,
-                 InputWindowEventHandler, inputWindow);
-    InvokeVaArgs(classicui->owner, FCITX_X11, ADDCOMPOSITEHANDLER,
-                   ReloadInputWindow, inputWindow);
+    FcitxX11AddXEventHandler(classicui->owner,
+                             InputWindowEventHandler, inputWindow);
+    FcitxX11AddCompositeHandler(classicui->owner,
+                                ReloadInputWindow, inputWindow);
 
     inputWindow->msgUp = FcitxMessagesNew();
     inputWindow->msgDown = FcitxMessagesNew();
@@ -167,13 +167,23 @@ void DrawInputWindow(InputWindow* inputWindow)
 {
     int lastW = inputWindow->iInputWindowWidth, lastH = inputWindow->iInputWindowHeight;
     int cursorPos = FcitxUINewMessageToOldStyleMessage(inputWindow->owner->owner, inputWindow->msgUp, inputWindow->msgDown);
-    DrawInputBar(inputWindow->skin, inputWindow, cursorPos, inputWindow->msgUp, inputWindow->msgDown, &inputWindow->iInputWindowHeight , &inputWindow->iInputWindowWidth);
+    FcitxInputState* input = FcitxInstanceGetInputState(inputWindow->owner->owner);
+    FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
+    FcitxCandidateLayoutHint layout = FcitxCandidateWordGetLayoutHint(candList);
+
+    boolean vertical = inputWindow->owner->bVerticalList;
+    if (layout == CLH_Vertical)
+        vertical = true;
+    else if (layout == CLH_Horizontal)
+        vertical = false;
+
+    DrawInputBar(inputWindow->skin, inputWindow, vertical, cursorPos, inputWindow->msgUp, inputWindow->msgDown, &inputWindow->iInputWindowHeight , &inputWindow->iInputWindowWidth);
 
     /* Resize Window will produce Expose Event, so there is no need to draw right now */
     if (lastW != inputWindow->iInputWindowWidth || lastH != inputWindow->iInputWindowHeight) {
         _CAIRO_SETSIZE(inputWindow->cs_x_input_bar,
-                                    inputWindow->iInputWindowWidth,
-                                    inputWindow->iInputWindowHeight);
+                       inputWindow->iInputWindowWidth,
+                       inputWindow->iInputWindowHeight);
         MoveInputWindowInternal(inputWindow);
         XResizeWindow(
             inputWindow->dpy,
@@ -188,7 +198,8 @@ void DrawInputWindow(InputWindow* inputWindow)
     cairo_rectangle(c, 0, 0, inputWindow->iInputWindowWidth, inputWindow->iInputWindowHeight);
     cairo_clip(c);
     cairo_paint(c);
-    _CAIRO_DESTROY(c);
+    cairo_destroy(c);
+    cairo_surface_flush(inputWindow->cs_x_input_bar);
 
     XFlush(inputWindow->dpy);
 }
@@ -235,14 +246,14 @@ void ReloadInputWindow(void* arg, boolean enabled)
     InputWindow* inputWindow = (InputWindow*) arg;
     boolean visable = WindowIsVisable(inputWindow->dpy, inputWindow->window);
     int i = 0;
-    _CAIRO_DESTROY(inputWindow->c_back);
+    cairo_destroy(inputWindow->c_back);
     inputWindow->c_back = NULL;
 
     for (i = 0 ; i < 7; i ++) {
-        _CAIRO_DESTROY(inputWindow->c_font[i]);
+        cairo_destroy(inputWindow->c_font[i]);
         inputWindow->c_font[i] = NULL;
     }
-    _CAIRO_DESTROY(inputWindow->c_cursor);
+    cairo_destroy(inputWindow->c_cursor);
     inputWindow->c_cursor = NULL;
 
     cairo_surface_destroy(inputWindow->cs_input_bar);
