@@ -40,7 +40,6 @@
 
 #include "MainWindow.h"
 #include "fcitx-utils/log.h"
-#include "module/x11/x11stuff.h"
 #include "classicui.h"
 #include "skin.h"
 #include "MenuWindow.h"
@@ -102,8 +101,8 @@ void InitMainWindow(MainWindow* mainWindow)
                                     MAIN_BAR_MAX_WIDTH,
                                     MAIN_BAR_MAX_HEIGHT);
     mainWindow->cs_main_bar = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
-                              MAIN_BAR_MAX_WIDTH,
-                              MAIN_BAR_MAX_HEIGHT);
+                                                         MAIN_BAR_MAX_WIDTH,
+                                                         MAIN_BAR_MAX_HEIGHT);
 
     XChangeWindowAttributes(dpy, mainWindow->window, attribmask, &attrib);
     XSelectInput(dpy, mainWindow->window, ExposureMask | ButtonPressMask | ButtonReleaseMask  | PointerMotionMask | LeaveWindowMask);
@@ -119,10 +118,10 @@ MainWindow* CreateMainWindow(FcitxClassicUI* classicui)
     mainWindow->owner = classicui;
     InitMainWindow(mainWindow);
 
-    InvokeVaArgs(classicui->owner, FCITX_X11, ADDXEVENTHANDLER,
-                 MainWindowEventHandler, mainWindow);
-    InvokeVaArgs(classicui->owner, FCITX_X11, ADDCOMPOSITEHANDLER,
-                 ReloadMainWindow, mainWindow);
+    FcitxX11AddXEventHandler(classicui->owner,
+                             MainWindowEventHandler, mainWindow);
+    FcitxX11AddCompositeHandler(classicui->owner,
+                                ReloadMainWindow, mainWindow);
     return mainWindow;
 }
 
@@ -137,6 +136,7 @@ void DrawMainWindow(MainWindow* mainWindow)
     FcitxClassicUI* classicui = mainWindow->owner;
     FcitxSkin *sc = &mainWindow->owner->skin;
     FcitxInstance *instance = mainWindow->owner->owner;
+    FcitxInputContext2* ic2 = (FcitxInputContext2*) FcitxInstanceGetCurrentIC(classicui->owner);
 
     FcitxLog(DEBUG, _("DRAW MainWindow"));
 
@@ -170,7 +170,8 @@ void DrawMainWindow(MainWindow* mainWindow)
         privstat->avail = false;
     }
 
-    if (mainWindow->owner->hideMainWindow == HM_SHOW || (mainWindow->owner->hideMainWindow == HM_AUTO && (FcitxInstanceGetCurrentState(mainWindow->owner->owner) == IS_ACTIVE))) {
+    if (mainWindow->owner->hideMainWindow == HM_SHOW
+        || (mainWindow->owner->hideMainWindow == HM_AUTO && ((ic2 && ic2->switchBySwitchKey) || FcitxInstanceGetCurrentState(mainWindow->owner->owner) == IS_ACTIVE))) {
         SkinImage* activeIcon = LoadImage(sc, sc->skinMainBar.active, false);
         cairo_t *c;
 
@@ -227,17 +228,17 @@ void DrawMainWindow(MainWindow* mainWindow)
                             continue;
 
                         boolean active = status->getCurrentStatus(status->arg);
-                        char* path;
-                        if (active)
-                            asprintf(&path, "%s_active.png", status->name);
-                        else
-                            asprintf(&path, "%s_inactive.png", status->name);
+                        char *path;
+                        fcitx_utils_alloc_cat_str(path, status->name, active ?
+                                                  "_active.png" :
+                                                  "_inactive.png");
                         SkinImage* statusicon = LoadImage(sc, path, false);
                         free(path);
                         if (statusicon == NULL)
                             continue;
                         privstat->avail = true;
-                        DrawImage(c, statusicon->image, sp->x, sp->y, privstat->mouse);
+                        DrawImage(c, statusicon->image, sp->x, sp->y,
+                                  privstat->mouse);
                         UpdateStatusGeometry(privstat, statusicon, sp->x, sp->y);
                     }
 
@@ -247,18 +248,24 @@ void DrawMainWindow(MainWindow* mainWindow)
                         if (privstat == NULL)
                             continue;
 
-                        const char* icon = compstatus->getIconName(compstatus->arg);
-                        char* path;
-                        if (icon[0] != '/')
-                            asprintf(&path, "%s.png", compstatus->name);
-                        else
-                            path = strdup(icon);
-                        SkinImage* statusicon = LoadImage(sc, path, false);
-                        free(path);
+                        const char* icon = compstatus->getIconName(
+                            compstatus->arg);
+                        const char *path;
+                        char *tmpstr = NULL;
+                        if (icon[0] != '/') {
+                            fcitx_utils_alloc_cat_str(tmpstr,
+                                                      compstatus->name, ".png");
+                            path = tmpstr;
+                        } else {
+                            path = icon;
+                        }
+                        SkinImage *statusicon = LoadImage(sc, path, false);
+                        fcitx_utils_free(tmpstr);
                         if (statusicon == NULL)
                             continue;
                         privstat->avail = true;
-                        DrawImage(c, statusicon->image, sp->x, sp->y, privstat->mouse);
+                        DrawImage(c, statusicon->image, sp->x, sp->y,
+                                  privstat->mouse);
                         UpdateStatusGeometry(privstat, statusicon, sp->x, sp->y);
                     }
                 }
@@ -300,37 +307,39 @@ void DrawMainWindow(MainWindow* mainWindow)
                 ) {
                 if (!compstatus->visible)
                     continue;
-                const char* icon = compstatus->getIconName(compstatus->arg);
-                char* path;
+                const char *icon = compstatus->getIconName(compstatus->arg);
+                char *tmpstr = NULL;
+                const char *path;
                 if (icon[0] != '/') {
-                    if (icon[0] == '\0')
-                        asprintf(&path, "%s", compstatus->shortDescription);
-                    else
-                        asprintf(&path, "%s.png", icon);
+                    if (icon[0] == '\0') {
+                        path = compstatus->shortDescription;
+                    } else {
+                        fcitx_utils_alloc_cat_str(tmpstr, icon, ".png");
+                        path = tmpstr;
+                    }
+                } else {
+                    path = icon;
                 }
-                else
-                    path = strdup(icon);
                 SkinImage* statusicon = NULL;
                 if (icon[0] != '\0')
                     statusicon = LoadImage(sc, path, false);
                 if (statusicon == NULL || statusicon->textIcon) {
                     if (activeIcon && icon[0] == '\0') {
-                        statusicon = LoadImageWithText(classicui, sc, path, compstatus->shortDescription,
-                                                       cairo_image_surface_get_width(activeIcon->image),
-                                                       cairo_image_surface_get_height(activeIcon->image),
-                                                       true
-                                                      );
+                        statusicon = LoadImageWithText(
+                            classicui, sc, path, compstatus->shortDescription,
+                            cairo_image_surface_get_width(activeIcon->image),
+                            cairo_image_surface_get_height(activeIcon->image),
+                            true);
                     }
-                }
-                else {
+                } else {
                     if (icon[0] == '/' && activeIcon) {
-                        ResizeSurface(&statusicon->image,
-                                        cairo_image_surface_get_width(activeIcon->image),
-                                        cairo_image_surface_get_height(activeIcon->image));
+                        ResizeSurface(
+                            &statusicon->image,
+                            cairo_image_surface_get_width(activeIcon->image),
+                            cairo_image_surface_get_height(activeIcon->image));
                     }
                 }
-                free(path);
-
+                fcitx_utils_free(tmpstr);
                 if (statusicon == NULL)
                     continue;
 
@@ -349,10 +358,8 @@ void DrawMainWindow(MainWindow* mainWindow)
                     continue;
                 boolean active = status->getCurrentStatus(status->arg);
                 char *path;
-                if (active)
-                    asprintf(&path, "%s_active.png", status->name);
-                else
-                    asprintf(&path, "%s_inactive.png", status->name);
+                fcitx_utils_alloc_cat_str(path, status->name, active ?
+                                          "_active.png" : "_inactive.png");
                 SkinImage* statusicon = LoadImage(sc, path, false);
                 if (statusicon == NULL || statusicon->textIcon) {
                     if (activeIcon) {
@@ -410,18 +417,22 @@ void DrawMainWindow(MainWindow* mainWindow)
                     continue;
                 if (!compstatus->visible)
                     continue;
-                const char* icon = compstatus->getIconName(compstatus->arg);
-                char* path;
+                const char *icon = compstatus->getIconName(compstatus->arg);
+                const char *path;
+                char *tmpstr = NULL;
                 if (icon[0] != '/') {
-                    if (icon[0] == '\0')
-                        asprintf(&path, "%s", compstatus->shortDescription);
-                    else
-                        asprintf(&path, "%s.png", compstatus->name);
+                    if (icon[0] == '\0') {
+                        path = compstatus->shortDescription;
+                    } else {
+                        fcitx_utils_alloc_cat_str(tmpstr,
+                                                  compstatus->name, ".png");
+                        path = tmpstr;
+                    }
+                } else {
+                    path = icon;
                 }
-                else
-                    path = strdup(icon);
-                SkinImage* statusicon = LoadImage(sc, path, false);
-                free(path);
+                SkinImage *statusicon = LoadImage(sc, path, false);
+                fcitx_utils_free(tmpstr);
                 if (statusicon == NULL)
                     continue;
                 privstat->avail = true;
@@ -431,9 +442,8 @@ void DrawMainWindow(MainWindow* mainWindow)
             }
 
             for (status = (FcitxUIStatus*) utarray_front(uistats);
-                    status != NULL;
-                    status = (FcitxUIStatus*) utarray_next(uistats, status)
-                ) {
+                 status != NULL;
+                 status = (FcitxUIStatus*) utarray_next(uistats, status)) {
                 FcitxClassicUIStatus* privstat = GetPrivateStatus(status);
                 if (privstat == NULL)
                     continue;
@@ -442,10 +452,8 @@ void DrawMainWindow(MainWindow* mainWindow)
                 /* reset status */
                 boolean active =  status->getCurrentStatus(status->arg);
                 char *path;
-                if (active)
-                    asprintf(&path, "%s_active.png", status->name);
-                else
-                    asprintf(&path, "%s_inactive.png", status->name);
+                fcitx_utils_alloc_cat_str(path, status->name, active ?
+                                          "_active.png" : "_inactive.png");
                 SkinImage* statusicon = LoadImage(sc, path, false);
                 free(path);
                 if (statusicon == NULL)
@@ -457,10 +465,8 @@ void DrawMainWindow(MainWindow* mainWindow)
             }
         }
 
-        _CAIRO_DESTROY(c);
-        _CAIRO_SETSIZE(mainWindow->cs_x_main_bar,
-                                    width,
-                                    height);
+        cairo_destroy(c);
+        _CAIRO_SETSIZE(mainWindow->cs_x_main_bar, width, height);
 
         c = cairo_create(mainWindow->cs_x_main_bar);
         cairo_set_operator(c, CAIRO_OPERATOR_SOURCE);
@@ -468,7 +474,8 @@ void DrawMainWindow(MainWindow* mainWindow)
         cairo_rectangle(c, 0, 0, width, height);
         cairo_clip(c);
         cairo_paint(c);
-        _CAIRO_DESTROY(c);
+        cairo_destroy(c);
+        cairo_surface_flush(mainWindow->cs_x_main_bar);
 
         XMapRaised(mainWindow->dpy, mainWindow->window);
     } else
