@@ -31,7 +31,6 @@
 #include "fcitx/fcitx.h"
 #include "fcitx/ui.h"
 #include "fcitx/module.h"
-#include "module/x11/x11stuff.h"
 
 #include "classicui.h"
 #include "classicuiinterface.h"
@@ -71,11 +70,10 @@ static void ClassicUISuspend(void *arg);
 static void ClassicUIResume(void *arg);
 
 static FcitxConfigFileDesc* GetClassicUIDesc();
-static void ClassicUIMainWindowSizeHint(void *arg, int* x, int* y, int* w, int* h);
+static void ClassicUIMainWindowSizeHint(void *arg, int* x, int* y,
+                                        int* w, int* h);
 
-static void* ClassicUILoadImage(void *arg, FcitxModuleFunctionArg args);
-static void* ClassicUIGetKeyBoardFontColor(void* arg, FcitxModuleFunctionArg args);
-static void* ClassicUIGetFont(void *arg, FcitxModuleFunctionArg args);
+DECLARE_ADDFUNCTIONS(ClassicUI)
 
 FCITX_DEFINE_PLUGIN(fcitx_classic_ui, ui, FcitxUI) = {
     ClassicUICreate,
@@ -102,8 +100,8 @@ FCITX_DEFINE_PLUGIN(fcitx_classic_ui, ui, FcitxUI) = {
 
 void* ClassicUICreate(FcitxInstance* instance)
 {
-    FcitxAddon* classicuiaddon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_CLASSIC_UI_NAME);
-    FcitxClassicUI* classicui = fcitx_utils_malloc0(sizeof(FcitxClassicUI));
+    FcitxAddon *classicuiaddon = Fcitx_ClassicUI_GetAddon(instance);
+    FcitxClassicUI *classicui = fcitx_utils_new(FcitxClassicUI);
     classicui->owner = instance;
     if (!LoadClassicUIConfig(classicui)) {
         free(classicui);
@@ -113,13 +111,13 @@ void* ClassicUICreate(FcitxInstance* instance)
         free(classicui);
         return NULL;
     }
-    classicui->dpy = InvokeVaArgs(instance, FCITX_X11, GETDISPLAY);
+    classicui->dpy = FcitxX11GetDisplay(instance);
     if (classicui->dpy == NULL) {
         free(classicui);
         return NULL;
     }
 
-    InvokeVaArgs(instance, FCITX_X11, GETDPI, &classicui->dpi, NULL);
+    FcitxX11GetDPI(instance, &classicui->dpi, NULL);
     if (classicui->dpi <= 0)
         classicui->dpi = 96;
 
@@ -145,7 +143,6 @@ void* ClassicUICreate(FcitxInstance* instance)
     classicui->mainMenu.priv = classicui;
     classicui->mainMenu.mark = -1;
 
-
     classicui->inputWindow = CreateInputWindow(classicui);
     classicui->mainWindow = CreateMainWindow(classicui);
     classicui->trayWindow = CreateTrayWindow(classicui);
@@ -159,18 +156,13 @@ void* ClassicUICreate(FcitxInstance* instance)
 
     DisplaySkin(classicui, classicui->skinType);
 
-    /* ensure order ! */
-    AddFunction(classicuiaddon, ClassicUILoadImage);
-    AddFunction(classicuiaddon, ClassicUIGetKeyBoardFontColor);
-    AddFunction(classicuiaddon, ClassicUIGetFont);
-
+    FcitxClassicUIAddFunctions(instance);
     return classicui;
 }
 
 void ClassicUISetWindowProperty(FcitxClassicUI* classicui, Window window, FcitxXWindowType type, char *windowTitle)
 {
-    InvokeVaArgs(classicui->owner, FCITX_X11, SETWINDOWPROP,
-                 &window, &type, windowTitle);
+    FcitxX11SetWindowProp(classicui->owner, &window, &type, windowTitle);
 }
 
 static void ClassicUIInputReset(void *arg)
@@ -231,16 +223,16 @@ static void ClassicUIRegisterStatus(void *arg, FcitxUIStatus* status)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     FcitxSkin* sc = &classicui->skin;
-    status->uipriv[classicui->isfallback] = fcitx_utils_malloc0(sizeof(FcitxClassicUIStatus));
-    char* activename, *inactivename;
-    asprintf(&activename, "%s_active.png", status->name);
-    asprintf(&inactivename, "%s_inactive.png", status->name);
+    status->uipriv[classicui->isfallback] = fcitx_utils_new(FcitxClassicUIStatus);
+    char *name;
 
-    LoadImage(sc, activename, false);
-    LoadImage(sc, inactivename, false);
+    fcitx_utils_alloc_cat_str(name, status->name, "_active.png");
+    LoadImage(sc, name, false);
+    free(name);
 
-    free(activename);
-    free(inactivename);
+    fcitx_utils_alloc_cat_str(name, status->name, "_inactive.png");
+    LoadImage(sc, name, false);
+    free(name);
 }
 
 static void ClassicUIOnInputFocus(void *arg)
@@ -283,7 +275,9 @@ void ActivateWindow(Display *dpy, int iScreen, Window window)
 
     memset(&ev, 0, sizeof(ev));
 
-    Atom _NET_ACTIVE_WINDOW = XInternAtom(dpy, "_NET_ACTIVE_WINDOW", False);
+    static Atom _NET_ACTIVE_WINDOW;
+    if (_NET_ACTIVE_WINDOW == None)
+        _NET_ACTIVE_WINDOW = XInternAtom(dpy, "_NET_ACTIVE_WINDOW", False);
 
     ev.xclient.type = ClientMessage;
     ev.xclient.window = window;
@@ -299,14 +293,13 @@ void ActivateWindow(Display *dpy, int iScreen, Window window)
 
 void GetScreenSize(FcitxClassicUI* classicui, int* width, int* height)
 {
-    InvokeVaArgs(classicui->owner, FCITX_X11, GETSCREENSIZE, width, height);
+    FcitxX11GetScreenSize(classicui->owner, width, height);
 }
 
 FcitxRect GetScreenGeometry(FcitxClassicUI* classicui, int x, int y)
 {
     FcitxRect result = { 0, 0 , 0 , 0 };
-    InvokeVaArgs(classicui->owner, FCITX_X11, GETSCREENGEOMETRY,
-                 &x, &y, &result);
+    FcitxX11GetScreenGeometry(classicui->owner, &x, &y, &result);
     return result;
 }
 
@@ -358,8 +351,7 @@ boolean
 ClassicUIMouseClick(FcitxClassicUI* classicui, Window window, int *x, int *y)
 {
     boolean bMoved = false;
-    InvokeVaArgs(classicui->owner, FCITX_X11, MOUSECLICK,
-                 &window, x, y, &bMoved);
+    FcitxX11MouseClick(classicui->owner, &window, x, y, &bMoved);
     return bMoved;
 }
 
@@ -504,13 +496,13 @@ ClassicUIInitWindowAttribute(FcitxClassicUI *classicui, Visual **vs,
                              Colormap *cmap, XSetWindowAttributes *attrib,
                              unsigned long *attribmask, int *depth)
 {
-    InvokeVaArgs(classicui->owner, FCITX_X11, INITWINDOWATTR,
-                   vs, cmap, attrib, attribmask, depth);
+    FcitxX11InitWindowAttribute(classicui->owner, vs, cmap, attrib,
+                                attribmask, depth);
 }
 
 Visual * ClassicUIFindARGBVisual(FcitxClassicUI* classicui)
 {
-    return InvokeVaArgs(classicui->owner, FCITX_X11, FINDARGBVISUAL);
+    return FcitxX11FindARGBVisual(classicui->owner);
 }
 
 void ClassicUIMainWindowSizeHint(void* arg, int* x, int* y, int* w, int* h)
@@ -534,30 +526,6 @@ void ClassicUIMainWindowSizeHint(void* arg, int* x, int* y, int* w, int* h)
 
 }
 
-void* ClassicUILoadImage(void *arg, FcitxModuleFunctionArg args)
-{
-    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    char *name = args.args[0];
-    boolean *fallback = args.args[1];
-    SkinImage* image = LoadImage(&classicui->skin, name, *fallback);
-    if (image == NULL)
-        return NULL;
-    else
-        return image->image;
-}
-
-void* ClassicUIGetKeyBoardFontColor(void *arg, FcitxModuleFunctionArg args)
-{
-    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    return &classicui->skin.skinKeyboard.keyColor;
-}
-
-void* ClassicUIGetFont(void *arg, FcitxModuleFunctionArg args)
-{
-    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    return &classicui->font;
-}
-
 void ReloadConfigClassicUI(void* arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
@@ -619,12 +587,11 @@ void ResizeSurface(cairo_surface_t** surface, int w, int h)
     cairo_rectangle(c, 0, 0, ow, oh);
     cairo_clip(c);
     cairo_paint(c);
-    _CAIRO_DESTROY(c);
+    cairo_destroy(c);
 
     cairo_surface_destroy(*surface);
 
     *surface = newsurface;
 }
 
-
-// kate: indent-mode cstyle; space-indent on; indent-width 0;
+#include "fcitx-classic-ui-addfunctions.h"
