@@ -25,18 +25,19 @@
 #include <ctype.h>
 #include <limits.h>
 #include <libintl.h>
+#include <sys/socket.h>
+#include <dbus/dbus.h>
+
 #include "config.h"
 #include "fcitx/ui.h"
 #include "fcitx-utils/log.h"
-#include <dbus/dbus.h>
-#include "module/dbus/dbusstuff.h"
+#include "module/dbus/fcitx-dbus.h"
 #include "fcitx/instance.h"
 #include "fcitx/module.h"
 #include "fcitx/frontend.h"
 #include "fcitx/hook.h"
 #include "fcitx-utils/utils.h"
 #include "fcitx/candidate.h"
-#include <sys/socket.h>
 
 #define FCITX_KIMPANEL_INTERFACE "org.kde.kimpanel.inputmethod"
 #define FCITX_KIMPANEL_PATH "/kimpanel"
@@ -110,6 +111,7 @@ const char * kimpanel_introspection_xml =
 
 typedef struct _FcitxKimpanelUI {
     FcitxInstance* owner;
+    FcitxAddon* addon;
     DBusConnection* conn;
     int iOffsetY;
     int iOffsetX;
@@ -122,6 +124,7 @@ typedef struct _FcitxKimpanelUI {
     int lastUpdateW;
     int lastUpdateH;
     int lastCursor;
+    boolean hasSetLookupTable;
 } FcitxKimpanelUI;
 
 static void* KimpanelCreate(FcitxInstance* instance);
@@ -147,6 +150,13 @@ static void KimUpdateSpotLocation(FcitxKimpanelUI* kimpanel, int x, int y);
 static void KimSetSpotRect(FcitxKimpanelUI* kimpanel, int x, int y, int w, int h);
 static void KimShowLookupTable(FcitxKimpanelUI* kimpanel, boolean toShow);
 static void KimUpdateLookupTable(FcitxKimpanelUI* kimpanel, char *labels[], int nLabel, char *texts[], int nText, boolean has_prev, boolean has_next);
+static void KimSetLookupTable(FcitxKimpanelUI* kimpanel,
+                              char *labels[], int nLabel,
+                              char *texts[], int nText,
+                              boolean has_prev,
+                              boolean has_next,
+                              int cursor,
+                              int layout);
 static void KimUpdatePreeditText(FcitxKimpanelUI* kimpanel, char *text);
 static void KimUpdateAux(FcitxKimpanelUI* kimpanel, char *text);
 static void KimUpdatePreeditCaret(FcitxKimpanelUI* kimpanel, int position);
@@ -192,16 +202,16 @@ FCITX_DEFINE_PLUGIN(fcitx_kimpanel_ui, ui, FcitxUI) = {
 };
 
 
-static void SetIMMenu(FcitxIM* pim, char** prop)
+static void SetIMMenu(FcitxIM *pim, char** prop)
 {
     const char *icon = "";
-    if (strncmp(pim->uniqueName, "fcitx-keyboard-", strlen("fcitx-keyboard-")) != 0)
+    if (strncmp(pim->uniqueName, "fcitx-keyboard-",
+                strlen("fcitx-keyboard-")) != 0)
         icon = pim->strIconName;
-
-    if (icon[0] == '\0' || icon[0] == '/')
-        asprintf(prop, "/Fcitx/im/%s:%s:%s:%s", pim->uniqueName, pim->strName, icon, pim->strName);
-    else
-        asprintf(prop, "/Fcitx/im/%s:%s:fcitx-%s:%s", pim->uniqueName, pim->strName, icon, pim->strName);
+    fcitx_utils_alloc_cat_str(*prop, "/Fcitx/im/", pim->uniqueName, ":",
+                              pim->strName,
+                              (icon[0] == '\0' || icon[0] == '/') ?
+                              ":" : ":fcitx-", icon, ":", pim->strName);
 }
 
 static void SetIMIcon(FcitxInstance* instance, char** prop)
@@ -250,21 +260,27 @@ static void SetIMIcon(FcitxInstance* instance, char** prop)
         description = _("Input Method Disabled");
     }
     /* add fcitx- prefix, unless icon name is an absolute path */
-    if (icon[0] == '\0' || icon[0] == '/')
-        asprintf(prop, "/Fcitx/im:%s:%s:%s", imname, icon, description);
-    else
-        asprintf(prop, "/Fcitx/im:%s:fcitx-%s:%s", imname, icon, description);
+
+    char *icon_prefix;
+    if (icon[0] == '\0' || icon[0] == '/') {
+        icon_prefix = ":";
+    } else {
+        icon_prefix = ":fcitx-";
+    }
+    fcitx_utils_alloc_cat_str(*prop, "/Fcitx/im:", imname, icon_prefix, icon,
+                              ":", description);
 }
 
 void* KimpanelCreate(FcitxInstance* instance)
 {
     FcitxKimpanelUI *kimpanel = fcitx_utils_malloc0(sizeof(FcitxKimpanelUI));
 
+    kimpanel->addon = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), "fcitx-kimpanel-ui");
     kimpanel->lastCursor = -2;
     kimpanel->version = 1;
     kimpanel->iCursorPos = 0;
     kimpanel->owner = instance;
-    kimpanel->conn = InvokeVaArgs(instance, FCITX_DBUS, GETCONNECTION);
+    kimpanel->conn = FcitxDBusGetConnection(instance);
 
     DBusError err;
     dbus_error_init(&err);
@@ -296,7 +312,8 @@ void* KimpanelCreate(FcitxInstance* instance)
                            "type='signal',"
                            "interface='" DBUS_INTERFACE_DBUS "',"
                            "path='" DBUS_PATH_DBUS "',"
-                           "member='NameOwnerChanged'",
+                           "member='NameOwnerChanged',"
+                           "arg0='org.kde.impanel'",
                            &err);
 
         dbus_connection_flush(kimpanel->conn);
@@ -328,8 +345,8 @@ void* KimpanelCreate(FcitxInstance* instance)
 
         DBusPendingCall* call = NULL;
         dbus_bool_t reply = dbus_connection_send_with_reply(kimpanel->conn, message,
-                            &call,
-                            0);
+                                                            &call,
+                                                            0);
         if (reply == TRUE) {
             dbus_pending_call_set_notify(call,
                                          KimpanelServiceExistCallback,
@@ -355,8 +372,9 @@ void KimpanelRegisterAllStatus(FcitxKimpanelUI* kimpanel)
     UT_array* uistats = FcitxInstanceGetUIStats(instance);
     UT_array* uicompstats = FcitxInstanceGetUIComplexStats(instance);
     char **prop = fcitx_utils_malloc0(sizeof(char*) * (2 + utarray_len(uistats) + utarray_len(uicompstats)));
-    asprintf(&prop[0], "/Fcitx/logo:%s:%s:%s", _("Fcitx"), "fcitx", _("Fcitx"));
 
+    char *fcitx = _("Fcitx");
+    fcitx_utils_alloc_cat_str(prop[0], "/Fcitx/logo:", fcitx, ":fcitx:", fcitx);
     SetIMIcon(instance, &prop[1]);
 
     int count = 2;
@@ -373,8 +391,8 @@ void KimpanelRegisterAllStatus(FcitxKimpanelUI* kimpanel)
 
     FcitxUIStatus *status;
     for (status = (FcitxUIStatus *) utarray_front(uistats);
-            status != NULL;
-            status = (FcitxUIStatus *) utarray_next(uistats, status)) {
+         status != NULL;
+         status = (FcitxUIStatus *) utarray_next(uistats, status)) {
         if (!status->visible)
             continue;
         prop[count] = Status2String(status);
@@ -402,6 +420,8 @@ void KimpanelSetIMStatus(FcitxKimpanelUI* kimpanel)
 void KimpanelInputReset(void* arg)
 {
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
+    if (kimpanel->addon != FcitxInstanceGetCurrentUI(kimpanel->owner))
+        return;
     KimpanelSetIMStatus(kimpanel);
 }
 
@@ -463,11 +483,14 @@ void KimpanelMoveInputWindow(void* arg)
 
 void KimpanelRegisterMenu(void* arg, FcitxUIMenu* menu)
 {
-    return ;
+    FCITX_UNUSED(arg);
+    FCITX_UNUSED(menu);
+    return;
 }
 
 void KimpanelRegisterStatus(void* arg, FcitxUIStatus* status)
 {
+    FCITX_UNUSED(status);
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     KimpanelRegisterAllStatus(kimpanel);
     return ;
@@ -475,22 +498,20 @@ void KimpanelRegisterStatus(void* arg, FcitxUIStatus* status)
 
 void KimpanelRegisterComplexStatus(void* arg, FcitxUIComplexStatus* status)
 {
-    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
+    FCITX_UNUSED(status);
+    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*)arg;
     KimpanelRegisterAllStatus(kimpanel);
     return ;
 }
 
 char* Status2String(FcitxUIStatus* status)
 {
-    char *result = NULL;
-    asprintf(&result, "/Fcitx/%s:%s:fcitx-%s-%s:%s",
-             status->name,
-             status->shortDescription,
-             status->name,
-             ((status->getCurrentStatus(status->arg)) ? "active" : "inactive"),
-             status->longDescription
-            );
-
+    char *result;
+    fcitx_utils_alloc_cat_str(result, "/Fcitx/", status->name, ":",
+                              status->shortDescription, ":fcitx-", status->name,
+                              ((status->getCurrentStatus(status->arg)) ?
+                               "-active:" : "-inactive:"),
+                              status->longDescription);
     return result;
 }
 
@@ -498,28 +519,12 @@ char* Status2String(FcitxUIStatus* status)
 char* ComplexStatus2String(FcitxUIComplexStatus* status)
 {
     const char* iconName = status->getIconName(status->arg);
-
-    const char* templ;
-    switch(iconName[0]) {
-        case '/':
-            templ = "/Fcitx/%s:%s:%s:%s";
-            break;
-        case '\0':
-            templ = "/Fcitx/%s:%s:%s:%s";
-            break;
-        default:
-            templ = "/Fcitx/%s:%s:fcitx-%s:%s";
-            break;
-    }
-
-    char *result = NULL;
-    asprintf(&result, templ,
-             status->name,
-             status->shortDescription,
-             iconName,
-             status->longDescription
-            );
-
+    char *result;
+    fcitx_utils_alloc_cat_str(result, "/Fcitx/", status->name, ":",
+                              status->shortDescription,
+                              (iconName[0] == '\0' || iconName[0] == '/') ?
+                              ":" : ":fcitx-", iconName, ":",
+                              status->longDescription);
     return result;
 }
 
@@ -528,12 +533,17 @@ void KimpanelShowInputWindow(void* arg)
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     FcitxInstance* instance = kimpanel->owner;
     FcitxInputState* input = FcitxInstanceGetInputState(instance);
+    FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
     kimpanel->iCursorPos = FcitxUINewMessageToOldStyleMessage(instance, kimpanel->messageUp, kimpanel->messageDown);
     FcitxMessages* messageDown = kimpanel->messageDown;
     FcitxMessages* messageUp = kimpanel->messageUp;
     FcitxLog(DEBUG, "KimpanelShowInputWindow");
     KimpanelMoveInputWindow(kimpanel);
 
+    boolean hasPrev = FcitxCandidateWordHasPrev(candList);
+    boolean hasNext = FcitxCandidateWordHasNext(candList);
+    FcitxCandidateLayoutHint layout = FcitxCandidateWordGetLayoutHint(candList);
+
     int n = FcitxMessagesGetMessageCount(messageDown);
     int nLabels = 0;
     int nTexts = 0;
@@ -590,13 +600,10 @@ void KimpanelShowInputWindow(void* arg)
         if (nTexts == 0) {
             KimShowLookupTable(kimpanel, false);
         } else {
-            KimUpdateLookupTable(kimpanel,
-                                 label,
-                                 nLabels,
-                                 text,
-                                 nTexts,
-                                 FcitxCandidateWordHasPrev(FcitxInputStateGetCandidateList(input)),
-                                 FcitxCandidateWordHasNext(FcitxInputStateGetCandidateList(input)));
+            if (kimpanel->hasSetLookupTable)
+                KimSetLookupTable(kimpanel, label, nLabels, text, nTexts, hasPrev, hasNext, pos, layout);
+            else
+                KimUpdateLookupTable(kimpanel, label, nLabels, text, nTexts, hasPrev, hasNext);
             KimShowLookupTable(kimpanel, true);
         }
         for (i = 0; i < nTexts; i++)
@@ -604,16 +611,15 @@ void KimpanelShowInputWindow(void* arg)
         for (i = 0; i < nLabels; i++)
             free(label[i]);
     } else {
-        KimUpdateLookupTable(kimpanel,
-                             NULL,
-                             0,
-                             NULL,
-                             0,
-                             FcitxCandidateWordHasPrev(FcitxInputStateGetCandidateList(input)),
-                             FcitxCandidateWordHasNext(FcitxInputStateGetCandidateList(input)));
+        if (kimpanel->hasSetLookupTable)
+            KimSetLookupTable(kimpanel, NULL, 0, NULL, 0, hasNext, hasNext, pos, layout);
+        else
+            KimUpdateLookupTable(kimpanel, NULL, 0, NULL, 0, hasPrev, hasNext);
         KimShowLookupTable(kimpanel, false);
     }
-    KimUpdateLookupTableCursor(kimpanel, pos);
+
+    if (!kimpanel->hasSetLookupTable)
+        KimUpdateLookupTableCursor(kimpanel, pos);
 
     n = FcitxMessagesGetMessageCount(messageUp);
     char aux[MESSAGE_MAX_LENGTH] = "";
@@ -654,6 +660,7 @@ void KimpanelShowInputWindow(void* arg)
 
 void KimpanelUpdateStatus(void* arg, FcitxUIStatus* status)
 {
+    FCITX_UNUSED(status);
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     KimpanelRegisterAllStatus(kimpanel);
     return ;
@@ -661,14 +668,18 @@ void KimpanelUpdateStatus(void* arg, FcitxUIStatus* status)
 
 void KimpanelUpdateComplexStatus(void* arg, FcitxUIComplexStatus* status)
 {
+    FCITX_UNUSED(status);
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     KimpanelRegisterAllStatus(kimpanel);
     return ;
 }
 
-static DBusHandlerResult KimpanelDBusEventHandler(DBusConnection *connection, DBusMessage *msg, void *arg)
+static DBusHandlerResult
+KimpanelDBusEventHandler(DBusConnection *connection,
+                         DBusMessage *msg, void *arg)
 {
-    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
+    FCITX_UNUSED(connection);
+    FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*)arg;
 
     if (dbus_message_is_method_call(msg, DBUS_INTERFACE_INTROSPECTABLE, "Introspect")) {
         DBusMessage *reply = dbus_message_new_method_return(msg);
@@ -733,19 +744,27 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
         DBusError error;
         dbus_error_init(&error);
         if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &s0 , DBUS_TYPE_INVALID)) {
-            size_t len = strlen("/Fcitx/");
-            if (strlen(s0) > len) {
-                s0 += len;
+            if (strlen(s0) > strlen("/Fcitx/")) {
+                s0 += strlen("/Fcitx/");
                 if (strcmp("logo", s0) == 0) {
-                    size_t len = 3;
-                    char **prop = fcitx_utils_malloc0(len * sizeof(char*));
-                    asprintf(&prop[0], "/Fcitx/logo/toggle:%s::%s", _("Toggle Input Method"), _("Toggle Input Method"));
-                    asprintf(&prop[1], "/Fcitx/logo/configureim:%s:configure:%s", _("Configure Current Input Method"), _("Configure Current Input Method"));
-                    asprintf(&prop[2], "/Fcitx/logo/restart:%s:view-refresh:%s",_("Restart"), _("Restart"));
-                    KimExecMenu(kimpanel, prop, len );
-                    while (len --)
-                        free(prop[len]);
-                    free(prop);
+                    char *prop[3];
+                    char *trans_str = _("Toggle Input Method");
+                    fcitx_utils_alloc_cat_str(prop[0], "/Fcitx/logo/toggle:",
+                                              trans_str, "::", trans_str);
+                    trans_str = _("Configure Current Input Method");
+                    fcitx_utils_alloc_cat_str(prop[1],
+                                              "/Fcitx/logo/configureim:",
+                                              trans_str, ":configure:",
+                                              trans_str);
+                    trans_str = _("Restart");
+                    fcitx_utils_alloc_cat_str(prop[2], "/Fcitx/logo/restart:",
+                                              trans_str, ":view-refresh:",
+                                              trans_str);
+                    KimExecMenu(kimpanel, prop, 3);
+                    int i;
+                    for (i = 0;i < 3;i++) {
+                        free(prop[i]);
+                    }
                 } else if (strncmp("logo/", s0, strlen("logo/")) == 0) {
                     s0 += strlen("logo/");
                     if (strcmp(s0, "toggle") == 0)
@@ -774,8 +793,8 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
                     size_t len = utarray_len(imes);
                     char **prop = fcitx_utils_malloc0(len * sizeof(char*));
                     for (pim = (FcitxIM *) utarray_front(imes);
-                            pim != NULL;
-                            pim = (FcitxIM *) utarray_next(imes, pim)) {
+                         pim != NULL;
+                         pim = (FcitxIM *) utarray_next(imes, pim)) {
 
                         SetIMMenu(pim, &prop[index]);
                         index ++;
@@ -796,27 +815,30 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
                         sscanf(pos, "%d", &index);
                         if (menup)
                             menup->MenuAction(menup, index);
-                    }
-                    else {
-                        FcitxUIMenu* menu = FcitxUIGetMenuByStatusName(instance, s0);
+                    } else {
+                        FcitxUIMenu *menu = FcitxUIGetMenuByStatusName(instance, s0);
                         if (menu) {
                             menu->UpdateMenu(menu);
 
-                            int i = 0, index = 0;
-                            char **prop = fcitx_utils_malloc0(utarray_len(&menu->shell) * sizeof(char*));
-                            for (i = 0; i < utarray_len(&menu->shell); i++) {
-                                if (GetMenuItem(menu, i)->type == MENUTYPE_SIMPLE) {
-                                    asprintf(&prop[index], "/Fcitx/%s/%d:%s::%s", s0, index, GetMenuItem(menu, i)->tipstr, GetMenuItem(menu, i)->tipstr);
-                                    index ++;
+                            unsigned int i = 0, index = 0;
+                            unsigned int len = utarray_len(&menu->shell);
+                            char *prop[len];
+                            for (i = 0; i < len; i++) {
+                                FcitxMenuItem *menu_item = GetMenuItem(menu, i);
+                                if (menu_item->type == MENUTYPE_SIMPLE) {
+                                    asprintf(&prop[index],
+                                             "/Fcitx/%s/%d:%s::%s", s0, index,
+                                             menu_item->tipstr,
+                                             menu_item->tipstr);
+                                    index++;
                                 }
                             }
                             KimExecMenu(kimpanel, prop, index);
-                            while (index --)
+                            while (index--)
                                 free(prop[index]);
-                            free(prop);
-                        }
-                        else
+                        } else {
                             FcitxUIUpdateStatus(instance, s0);
+                        }
                     }
                 }
             }
@@ -835,6 +857,7 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
         kimpanel->version = 2;
         KimpanelReset(kimpanel);
         KimpanelRegisterAllStatus(kimpanel);
+        KimpanelIntrospect(kimpanel);
         return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_signal(msg, "org.kde.impanel", "Exit")) {
         FcitxLog(DEBUG, "Exit");
@@ -863,8 +886,8 @@ DBusHandlerResult KimpanelDBusFilter(DBusConnection* connection, DBusMessage* ms
                                   DBUS_TYPE_INVALID)) {
             /* old die and no new one */
             if (strcmp(service, "org.kde.impanel") == 0
-                    && strlen(oldowner) > 0
-                    && strlen(newowner) == 0)
+                && strlen(oldowner) > 0
+                && strlen(newowner) == 0)
                 FcitxUISwitchToFallback(instance);
             /*
              * since if new rise, it will send PanelCreated,
@@ -911,6 +934,7 @@ void KimExecMenu(FcitxKimpanelUI* kimpanel, char *props[], int n)
     dbus_uint32_t serial = 0; // unique number to associate replies with requests
     DBusMessage* msg;
     DBusMessageIter args;
+    int i;
 
     // create a signal and check for errors
     msg = dbus_message_new_signal(FCITX_KIMPANEL_PATH, // object name of the signal
@@ -921,17 +945,11 @@ void KimExecMenu(FcitxKimpanelUI* kimpanel, char *props[], int n)
         return;
     }
 
-    if (n == -1) {
-        n = 0;
-        while (*(props[n]) != 0) {
-            if (!fcitx_utf8_check_string(props[n]))
-                return;
-            n++;
-        }
-
+    for (i = 0; i < n; i++) {
+        if (!fcitx_utf8_check_string(props[i]))
+            return;
     }
 
-    int i;
     // append arguments onto signal
     dbus_message_iter_init_append(msg, &args);
     DBusMessageIter sub;
@@ -959,6 +977,7 @@ void KimRegisterProperties(FcitxKimpanelUI* kimpanel, char *props[], int n)
     dbus_uint32_t serial = 0; // unique number to associate replies with requests
     DBusMessage* msg;
     DBusMessageIter args;
+    int i;
 
     // create a signal and check for errors
     msg = dbus_message_new_signal(FCITX_KIMPANEL_PATH, // object name of the signal
@@ -969,16 +988,11 @@ void KimRegisterProperties(FcitxKimpanelUI* kimpanel, char *props[], int n)
         return;
     }
 
-    if (n == -1) {
-        n = 0;
-        while (*(props[n]) != 0) {
-            if (!fcitx_utf8_check_string(props[n]))
-                return;
-            n++;
-        }
+    for (i = 0; i < n; i++) {
+        if (!fcitx_utf8_check_string(props[i]))
+            return;
     }
 
-    int i;
     // append arguments onto signal
     dbus_message_iter_init_append(msg, &args);
     DBusMessageIter sub;
@@ -1297,6 +1311,82 @@ void KimUpdateLookupTable(FcitxKimpanelUI* kimpanel, char *labels[], int nLabel,
 
 }
 
+void KimSetLookupTable(FcitxKimpanelUI* kimpanel,
+                       char *labels[], int nLabel,
+                       char *texts[], int nText,
+                       boolean has_prev,
+                       boolean has_next,
+                       int cursor,
+                       int layout
+                      )
+{
+    int i;
+    dbus_uint32_t serial = 0; // unique number to associate replies with requests
+    DBusMessage* msg;
+    DBusMessageIter args;
+
+    // create a signal and check for errors
+    msg = dbus_message_new_method_call("org.kde.impanel",
+                                       "/org/kde/impanel",
+                                       "org.kde.impanel2",
+                                       "SetLookupTable"); // name of the signal
+    if (NULL == msg) {
+        FcitxLog(DEBUG, "Message Null");
+        return;
+    }
+    for (i = 0; i < nLabel; i++) {
+        if (!fcitx_utf8_check_string(labels[i]))
+            return;
+    }
+    for (i = 0; i < nText; i++) {
+        if (!fcitx_utf8_check_string(texts[i]))
+            return;
+    }
+    DBusMessageIter subLabel;
+    DBusMessageIter subText;
+    DBusMessageIter subAttrs;
+    // append arguments onto signal
+    dbus_message_iter_init_append(msg, &args);
+    dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &subLabel);
+    for (i = 0; i < nLabel; i++) {
+        if (!dbus_message_iter_append_basic(&subLabel, DBUS_TYPE_STRING, &labels[i])) {
+            FcitxLog(DEBUG, "Out Of Memory!");
+        }
+    }
+    dbus_message_iter_close_container(&args, &subLabel);
+
+    dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &subText);
+    for (i = 0; i < nText; i++) {
+        if (!dbus_message_iter_append_basic(&subText, DBUS_TYPE_STRING, &texts[i])) {
+            FcitxLog(DEBUG, "Out Of Memory!");
+        }
+    }
+    dbus_message_iter_close_container(&args, &subText);
+
+    char *attr = "";
+    dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &subAttrs);
+    for (i = 0; i < nLabel; i++) {
+        if (!dbus_message_iter_append_basic(&subAttrs, DBUS_TYPE_STRING, &attr)) {
+            FcitxLog(DEBUG, "Out Of Memory!");
+        }
+    }
+    dbus_message_iter_close_container(&args, &subAttrs);
+
+    dbus_message_iter_append_basic(&args, DBUS_TYPE_BOOLEAN, &has_prev);
+    dbus_message_iter_append_basic(&args, DBUS_TYPE_BOOLEAN, &has_next);
+    dbus_message_iter_append_basic(&args, DBUS_TYPE_INT32, &cursor);
+    dbus_message_iter_append_basic(&args, DBUS_TYPE_INT32, &layout);
+
+    // send the message and flush the connection
+    if (!dbus_connection_send(kimpanel->conn, msg, &serial)) {
+        FcitxLog(DEBUG, "Out Of Memory!");
+    }
+
+    // free the message
+    dbus_message_unref(msg);
+
+}
+
 void KimUpdatePreeditCaret(FcitxKimpanelUI* kimpanel, int position)
 {
 
@@ -1455,22 +1545,22 @@ void KimSetSpotRect(FcitxKimpanelUI* kimpanel, int x, int y, int w, int h)
 
     // create a signal and check for errors
     msg = dbus_message_new_method_call("org.kde.impanel",
-                                  "/org/kde/impanel",
-                                  "org.kde.impanel2",
-                                  "SetSpotRect"); // name of the signal
+                                       "/org/kde/impanel",
+                                       "org.kde.impanel2",
+                                       "SetSpotRect"); // name of the signal
     if (NULL == msg) {
         FcitxLog(DEBUG, "Message Null");
         return;
     }
 
     if (!dbus_message_append_args(
-        msg,
-        DBUS_TYPE_INT32, &x,
-        DBUS_TYPE_INT32, &y,
-        DBUS_TYPE_INT32, &w,
-        DBUS_TYPE_INT32, &h,
-        DBUS_TYPE_INVALID
-    )) {
+            msg,
+            DBUS_TYPE_INT32, &x,
+            DBUS_TYPE_INT32, &y,
+            DBUS_TYPE_INT32, &w,
+            DBUS_TYPE_INT32, &h,
+            DBUS_TYPE_INVALID
+            )) {
         FcitxLog(DEBUG, "Out Of Memory!");
     }
 
@@ -1518,18 +1608,17 @@ void KimUpdateScreen(FcitxKimpanelUI* kimpanel, int id)
 
 int CalKimCursorPos(FcitxKimpanelUI *kimpanel)
 {
-    size_t             i = 0;
-    int             iChar;
-    int             iCount = 0;
-    FcitxMessages* messageUp = kimpanel->messageUp;
+    int i = 0;
+    unsigned int iChar;
+    int iCount = 0;
+    FcitxMessages *messageUp = kimpanel->messageUp;
 
-    const char      *p1;
-    const char      *pivot;
+    const char *p1;
+    const char *pivot;
 
     iChar = kimpanel->iCursorPos;
 
-
-    for (i = 0; i < FcitxMessagesGetMessageCount(messageUp) ; i++) {
+    for (i = 0;i < FcitxMessagesGetMessageCount(messageUp);i++) {
         if (kimpanel->iCursorPos && iChar) {
             p1 = pivot = FcitxMessagesGetMessageString(messageUp, i);
             while (*p1 && p1 < pivot + iChar) {
@@ -1579,13 +1668,13 @@ void KimpanelIntrospect(FcitxKimpanelUI* kimpanel)
 
     DBusPendingCall* call = NULL;
     dbus_bool_t reply = dbus_connection_send_with_reply(kimpanel->conn, message,
-                        &call,
-                        0);
+                                                        &call,
+                                                        0);
     if (reply == TRUE) {
         dbus_pending_call_set_notify(call,
-                                    KimpanelIntrospectCallback,
-                                    kimpanel,
-                                    NULL);
+                                     KimpanelIntrospectCallback,
+                                     kimpanel,
+                                     NULL);
     }
     dbus_connection_flush(kimpanel->conn);
     dbus_message_unref(message);
@@ -1602,8 +1691,11 @@ void KimpanelIntrospectCallback(DBusPendingCall* call, void* data)
         DBusError error;
         dbus_error_init(&error);
         if (dbus_message_get_args(msg, &error, DBUS_TYPE_STRING, &s , DBUS_TYPE_INVALID)) {
-            if (strstr(s, "org.kde.impanel2"))
+            if (strstr(s, "org.kde.impanel2")) {
                 kimpanel->version = 2;
+                if (strstr(s, "SetLookupTable"))
+                    kimpanel->hasSetLookupTable = true;
+            }
         }
         dbus_message_unref(msg);
         dbus_error_free(&error);
@@ -1618,6 +1710,7 @@ void KimpanelSuspend(void* arg)
 {
     FcitxKimpanelUI* kimpanel = (FcitxKimpanelUI*) arg;
     kimpanel->version = 1;
+    kimpanel->hasSetLookupTable = false;
 }
 
 
@@ -1635,11 +1728,12 @@ void KimpanelDestroy(void* arg)
     dbus_connection_flush(kimpanel->conn);
 
     dbus_bus_remove_match(kimpanel->conn,
-                        "type='signal',"
-                        "interface='" DBUS_INTERFACE_DBUS "',"
-                        "path='" DBUS_PATH_DBUS "',"
-                        "member='NameOwnerChanged'",
-                        NULL);
+                          "type='signal',"
+                          "interface='" DBUS_INTERFACE_DBUS "',"
+                          "path='" DBUS_PATH_DBUS "',"
+                          "member='NameOwnerChanged',"
+                          "arg0='org.kde.impanel'",
+                          NULL);
 
     dbus_connection_flush(kimpanel->conn);
 
